// 
// SysMemRes.c
// 
//
// This program:
//
// 	Initialises the system software managers.
//
// 	Creates a nonrelocatable block of memory for a window record.
//
// 	Loads a window template ('WIND') resource and creates a window.
//
// 	Loads a purgeable 'PICT' resource and a non-purgeable 'STR ' resource and draws
//		them in the window.
//
// 	Checks if any error codes were generated as a result to Memory Manager and Resource
//		Manager routines.
//
// 	Terminates when the mouse button is clicked.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Resources.h>

//  defines

#define	rWindowResourceID		128
#define rStringResourceID		128
#define rPictureResourceID	128

//  function prototypes

void	main								(void);
void	doInitManagers			(void);
void	doNewWindow					(void);
void	doDrawPictAndString	(void);

//  main

void  main(void)
{	
	doInitManagers();
	doNewWindow();
	doDrawPictAndString();
	while(!Button()) ;
}

//  initManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();
}

//  doNewWindow

void  doNewWindow(void)
{
	WindowPtr windowPtr;
	Ptr				windowRecPtr;

	windowRecPtr = NewPtr(sizeof(WindowRecord));
	if(windowRecPtr == NULL)
	{
		SysBeep(10);
		ExitToShell();
	}

	windowPtr = GetNewWindow(rWindowResourceID,windowRecPtr,(WindowPtr) -1);
	if(windowPtr == NULL)
	{
		SysBeep(10);
		ExitToShell();
	}

	SetPort(windowPtr);
	TextFont(systemFont);
	
}

//  doDrawPictAndString

void  doDrawPictAndString(void)
{
	PicHandle			pictureHdl;
	StringHandle  stringHdl;
	Rect					pictureRect;
	SInt16				resourceError;
	OSErr					memoryError;			

	pictureHdl = GetPicture(rPictureResourceID);
	if(pictureHdl == NULL)
	{
		SysBeep(10);
		ExitToShell();
	}

	SetRect(&pictureRect,148,25,353,170);

	HNoPurge((Handle) pictureHdl);
	DrawPicture(pictureHdl,&pictureRect);
	HPurge((Handle) pictureHdl);

	stringHdl = GetString(rStringResourceID);
	if(stringHdl == NULL)
	{
		SysBeep(10);
		ExitToShell();
	}

	MoveTo(105,210);
	DrawString(*stringHdl);

	ReleaseResource((Handle) pictureHdl);
	ReleaseResource((Handle) stringHdl);

	resourceError = ResError();
	if(resourceError == noErr)
	{
		MoveTo(162,240);
		DrawString("\pNo Resource Manager errors");
	}	

	memoryError = MemError();
	if(memoryError == noErr)
	{
		MoveTo(165,255);
		DrawString("\pNo Memory Manager errors");
	}	
}

// 

