// 
// Menus.c
// 
// 
// This program:
//
// 	Opens a window.
//
// 	Creates these pull-down menus: Apple, File, Edit, Font, Size and Special.
//
//		The Apple menu includes an "About" menu item for the program.
//
//		The second menu item in the Special menu contains a submenu.
//
//		A "Help" menu item for the program is appended to the Help menu.
//
// 	Creates a pop-up menu in the window. 
//
// 	Displays text in the window indicating the menu selection made by the user.
//
// The implementation of the Size menu is nominal only.  The current size is indicated
// with a checkmark; however, the number of sizes shown is not font-dependent and there
// is no "Other" item.
//
// Because the primary purpose of the program is to demonstrate menu creation and
// handling, no code is included to update and activate/deactivate the window or to
// respond to events which are not relevant to the demonstration. 
//
// The program is terminated by selecting Quit from the File menu, by pressing the
// keyboard equivalent for that item (Command-Q), or by clicking in the window's go-away
// box. 
//  
// The program utilises the following resources:
//
// 	A 'WIND' resource (purgeable) (initially not visible).
//
// 	An 'MBAR' resource (preload, non-purgeable).
//
// 	'MENU' resources for the drop-down, hierarchical and pop-up menus (all preload, 
//		all non-purgeable).
//
// 	A 'CNTL' resource for the pop-up menu (purgeable).
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Balloons.h>
#include <Devices.h>

//  defines

#define mApple					128
#define  iAbout					1
#define mFile						129
#define  iQuit					11
#define mEdit						130		
#define  iUndo					1
#define  iCut						3
#define  iCopy					4
#define  iPaste					5
#define  iClear					6
#define mFont						131
#define mStyle					132
#define  iPlain					1
#define  iBold					3
#define  iItalic				4
#define  iUnderline			5
#define  iOutline				6
#define  iShadow				7
#define mSize						133
#define  iTen						1
#define  iTwelve				2
#define  iEighteen			3
#define  iTwentyFour		4
#define mSpecial				134		
#define  iFirstItem			1
#define  hmSecondItem		100
#define   siFirstSub		1
#define   siSecondSub		2
#define pControlResID		128
#define  pSydney				1
#define  pNewYork				2
#define  pLondon				3
#define  pRome					4
#define rWindowResource	128
	
//  global variables

Boolean	gDone;
SInt16	gCurrentFont =	1;
Style		gCurrentStyle =	0;
SInt16	gCurrentSize =	2;

//  function prototypes

void		main											(void);
void		doInitManagers						(void);
void		doGetMenus								(WindowPtr);
void		doEvents									(EventRecord *);
void		doMouseDown								(EventRecord *);
SInt16	doCheckForControlAndValue	(EventRecord *,WindowPtr);
void		doAdjustMenus							(void);
void		doMenuChoice							(SInt32);
void		doPopupMenuChoice					(SInt16);
void		doAppleMenu								(SInt16);
void		doFileMenu								(SInt16);
void		doEditMenu								(SInt16);
void		doFontMenu								(SInt16);
void		doStyleMenu								(SInt16);
void		doSizeMenu								(SInt16);
void		doSpecialMenu							(SInt16);
void		doSubMenus								(SInt16);
void		doHelpMenu								(SInt16);
void		drawItemString						(Str255);

//  main

void  main(void)
{
	EventRecord	eventRec;
	WindowPtr 	windowPtr;

	//  initialise managers

	doInitManagers();
	
	//  open a window
		
	if(!(windowPtr = GetNewWindow(rWindowResource,NULL,(WindowPtr) -1)))
	{
		SysBeep(10);
		ExitToShell();
	}

	SetPort(windowPtr);

	//  set up menu bar and menus, then show window and pop-up menu
	
	doGetMenus(windowPtr);
	ShowWindow(windowPtr);
	DrawControls(windowPtr);

	//  event loop 			

	gDone = false;

	while(!gDone)
	{
		if(WaitNextEvent(everyEvent,&eventRec,180,NULL))
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();	
	FlushEvents(everyEvent,0);
}

//  doGetMenus

void  doGetMenus(WindowPtr windowPtr)
{
	Handle				menubarHdl;
	MenuHandle		menuHdl;
	OSErr					osErr;
	ControlHandle	popupControlHdl;
	
	menubarHdl=GetNewMBar(128);
	if(menubarHdl == NULL)
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl != NULL)
		AppendResMenu(menuHdl,'DRVR');
	else
		ExitToShell();

	menuHdl = GetMenuHandle(mFont);
	if(menuHdl != NULL)
		AppendResMenu(menuHdl,'FONT');
	else
		ExitToShell();

	menuHdl = GetMenu(hmSecondItem);
	if(menuHdl != NULL)
		InsertMenu(menuHdl,hierMenu);
	else
		ExitToShell();

	osErr = HMGetHelpMenuHandle(&menuHdl);
	if(osErr == noErr)
		AppendMenu(menuHdl,"\pMenus Help");
	else
		ExitToShell();
	
	popupControlHdl = GetNewControl(pControlResID,windowPtr);
	if(popupControlHdl == NULL)
		ExitToShell();

	doFontMenu(gCurrentFont);
	doStyleMenu(gCurrentStyle);
	doSizeMenu(gCurrentSize);
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	SInt8	charCode;

	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
			{
				doAdjustMenus();
				doMenuChoice(MenuKey(charCode));
			}
			break;

		case updateEvt:
			BeginUpdate((WindowPtr)eventRecPtr->message);
			EndUpdate((WindowPtr)eventRecPtr->message);
			break;

		case osEvt:
			HiliteMenu(0);
			break;
	}
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode, popupItem;
	SInt32		menuChoice;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);
	
	switch(partCode)
	{
		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inMenuBar:
			doAdjustMenus();
			menuChoice = MenuSelect(eventRecPtr->where);
			doMenuChoice(menuChoice);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			else
			{			
				popupItem = doCheckForControlAndValue(eventRecPtr,windowPtr);
				if(popupItem)
					doPopupMenuChoice(popupItem);
			}
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where))
				gDone = true;
			break;
	}
}

//  doCheckForControlAndValue

SInt16  doCheckForControlAndValue(EventRecord *eventRecPtr,WindowPtr windowPtr)
{
	ControlHandle	controlHdl;
	SInt16				startControlValue = 0, finishControlValue = 0;
	
	SetPort(windowPtr);
	GlobalToLocal(&eventRecPtr->where);
	
	if(FindControl(eventRecPtr->where,windowPtr,&controlHdl))
	{
		startControlValue = GetControlValue(controlHdl);
		TrackControl(controlHdl,eventRecPtr->where,(ControlActionUPP) -1);
		finishControlValue = GetControlValue(controlHdl);
	}

	if(finishControlValue != startControlValue)
		return finishControlValue;
	else
		return 0;
}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	// Adjust menus here.
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16		menuID, menuItem;
		
	menuID	 = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			doAppleMenu(menuItem);
			break;

		case mFile:
			doFileMenu(menuItem);
			break;
			
		case mEdit:
			doEditMenu(menuItem);
			break;

		case mFont:
			doFontMenu(menuItem);
			break;

		case mStyle:
			doStyleMenu(menuItem);
			break;

		case mSize:
			doSizeMenu(menuItem);
			break;

		case mSpecial:
			doSpecialMenu(menuItem);
			break;

		case hmSecondItem:
			doSubMenus(menuItem);
			break;

		case kHMHelpMenuID:
			doHelpMenu(menuItem);
			break;
	}

	HiliteMenu(0);
}

//  doPopupMenuChoice

void  doPopupMenuChoice(SInt16 popupItem)
{
	switch(popupItem)
	{
		case pSydney:
			drawItemString("\pSydney");
			break;

		case pNewYork:
			drawItemString("\pNew York");
			break;

		case pLondon:
			drawItemString("\pLondon");
			break;

		case pRome:
			drawItemString("\pRome");
			break;
	}
}

//  doAppleMenu

void	doAppleMenu(SInt16 menuItem)
{
	Str255	itemName;
	SInt16	daDriverRefNum;
	
	if(menuItem == iAbout)
		drawItemString("\pAbout Menus");
	else
	{
		GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
		daDriverRefNum = OpenDeskAcc(itemName);
	}
}

//  doFileMenu

void	doFileMenu(SInt16 menuItem)
{
	if(menuItem  == iQuit)
		gDone = true;
}

//  doEditMenu

void	doEditMenu(SInt16 menuItem)
{
	switch(menuItem)
	{
		case iUndo:
			drawItemString("\pUndo");
			break;

		case iCut:
			drawItemString("\pCut");
			break;

		case iCopy:
			drawItemString("\pCopy");
			break;

		case iPaste:
			drawItemString("\pPaste");
			break;

		case iClear:
			drawItemString("\pClear");
			break;
	}
}

//  doFontMenu

void	doFontMenu(SInt16 menuItem)
{
	MenuHandle	fontMenuHdl;
	Str255			fontName;
	SInt16			fontNumber;
	
	fontMenuHdl = GetMenuHandle(mFont);
	
	CheckItem(fontMenuHdl,gCurrentFont,false);
	CheckItem(fontMenuHdl,menuItem,true);

	gCurrentFont = menuItem;	
			
	GetMenuItemText(fontMenuHdl,menuItem,fontName);
	GetFNum(fontName,&fontNumber);
	TextFont(fontNumber);

	drawItemString(fontName);
}

//  doStyleMenu

void	doStyleMenu(SInt16 menuItem)
{
	MenuHandle	styleMenuHdl;
	
	switch(menuItem)
	{
		case iPlain:
			gCurrentStyle = 0;
			break;

		case iBold:
			if(gCurrentStyle & bold)
				gCurrentStyle -= bold;
			else
				gCurrentStyle |= bold;
			break;

		case iItalic:
			if(gCurrentStyle & italic)
				gCurrentStyle -= italic;
			else
				gCurrentStyle |= italic;
			break;

		case iUnderline:
			if(gCurrentStyle & underline)
				gCurrentStyle -= underline;
			else
				gCurrentStyle |= underline;
			break;

		case iOutline:
			if(gCurrentStyle & outline)
				gCurrentStyle -= outline;
			else
				gCurrentStyle |= outline;
			break;

		case iShadow:
			if(gCurrentStyle & shadow)
				gCurrentStyle -= shadow;
			else
				gCurrentStyle |= shadow;
			break;
	}

	styleMenuHdl = GetMenuHandle(mStyle);

	CheckItem(styleMenuHdl,iPlain,		gCurrentStyle == 0);
	CheckItem(styleMenuHdl,iBold,			gCurrentStyle & bold);
	CheckItem(styleMenuHdl,iItalic,		gCurrentStyle & italic);
	CheckItem(styleMenuHdl,iUnderline,gCurrentStyle & underline);
	CheckItem(styleMenuHdl,iOutline,	gCurrentStyle & outline);
	CheckItem(styleMenuHdl,iShadow,		gCurrentStyle & shadow);
	
	TextFace(gCurrentStyle);

	drawItemString("\pStyle change");
}

//  doSizeMenu

void	doSizeMenu(SInt16 menuItem)
{	
	MenuHandle	sizeMenuHdl;	

	switch(menuItem)
	{
		case iTen:
			TextSize(10);
			break;

		case iTwelve:
			TextSize(12);
			break;

		case iEighteen:
			TextSize(18);
			break;

		case iTwentyFour:
			TextSize(24);
			break;
	}

	sizeMenuHdl = GetMenuHandle(mSize);

	CheckItem(sizeMenuHdl,gCurrentSize,false);
	CheckItem(sizeMenuHdl,menuItem,true);
	
	gCurrentSize = menuItem;
	
	drawItemString("\pSize change");
}

//  doSpecialMenu

void	doSpecialMenu(SInt16 menuItem)
{
	if(menuItem == iFirstItem)
		drawItemString("\pFirst Item");
}

//  doSubMenus

void  doSubMenus(SInt16 menuItem)
{
	switch(menuItem)
	{
		case siFirstSub:
			drawItemString("\pSubitem 1");
			break;

		case siSecondSub:
			drawItemString("\pSubitem 2");
			break;
	}
}

//  doHelpMenu

void	doHelpMenu(SInt16 menuItem)
{
	MenuHandle	helpMenuHdl;
	SInt16			origHelpItems, numItems;

	HMGetHelpMenuHandle(&helpMenuHdl);

	numItems = CountMItems(helpMenuHdl);
	origHelpItems = numItems - 1;

	if(menuItem > origHelpItems)
		drawItemString("\pMenus Help");
}

//  drawItemString

void	drawItemString(Str255 eventString)
{
	RgnHandle	tempRegion;
	WindowPtr	windowPtr;
	Rect			scrollBox;

	windowPtr = FrontWindow();
	tempRegion = NewRgn();

	scrollBox = windowPtr->portRect;
	scrollBox.top = scrollBox.top + 50;
	
	ScrollRect(&scrollBox,0,-24,tempRegion);
	DisposeRgn(tempRegion);
	
	MoveTo(8,286);
	DrawString(eventString);
}

// 


