// 
// Controls1.c
// 
// 
// This program opens a zoomDocProc window containing:
//
// 	A pop-up menu.
//
// 	Three radio buttons.
//
// 	Two checkboxes.
//
// 	One button.
//
// 	Vertical and horizontal scroll bars.
//
// The pop-up menu, radio buttons, checkboxes, and button work correctly except that the
// control values are not used for any specific purpose.
//
// The scroll bars are moved and resized when the user resizes or zooms the window;
// however, no action is taken when the scroll box is moved or the scroll arrows or gray
// areas are clicked.
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit menus and a pop-up
//		menu (preload, non-purgeable).  
//
// 	A 'WIND' resource (purgeable) (initially not visible).  
//
// 	'CNTL' resources for the pop-up menu, radio buttons, checkboxes, button and
//		scroll bars (preload, purgeable) (initially visible).  
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch,
//		and is32BitCompatible flags set.	  
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>

//  defines

#define rMenubar		128
#define rNewWindow	128
#define mApple			128
#define  iAbout			1
#define mFile				129
#define  iQuit			11
#define mEdit				130
#define cTimeZone		128
#define  pSydney		1
#define  pNewYork		2
#define  pLondon		3
#define  pRome			4
#define cRed				129
#define cWhite			130
#define cBlue				131
#define cShowgrid		132
#define cShowrulers	133
#define cButton			134
#define cVScrollbar	135
#define cHScrollbar	136
#define MAXLONG			0x7FFFFFFF

//  typedefs

typedef struct
{
	ControlHandle	popupControlHdl;
	ControlHandle redHdl;
	ControlHandle whiteHdl;
	ControlHandle blueHdl;
	ControlHandle showGridHdl;
	ControlHandle showRulersHdl;
	ControlHandle okButtonHdl;
	ControlHandle vScrollbarHdl;
	ControlHandle hScrollbarHdl;
} DocRec;

typedef DocRec **DocRecHandle;

//  global variables

Boolean	gDone;
Boolean	gInBackground;

//  function prototypes

void	main										(void);
void	doInitManagers					(void);
void	doGetControls						(WindowPtr);
void	doEvents								(EventRecord *);
void	doMouseDown							(EventRecord *);
void	doMenuChoice						(SInt32);
void	doInContent							(EventRecord *,WindowPtr);
void	doPopupMenuChoice				(SInt16);
void	doControls							(ControlHandle,DocRecHandle);
void	doUpdate								(EventRecord *);
void	doActivate							(EventRecord *);
void	doActivateWindow				(WindowPtr,Boolean);
void	doOSEvent								(EventRecord *);
void	doAdjustScrollBars			(WindowPtr);
void	doEraseGrowIcon					(WindowPtr);

//  main

void  main(void)
{
	Handle				menubarHdl;
	MenuHandle		menuHdl;
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	EventRecord		eventRec;

	//  initialise managers

	doInitManagers();

	//  set up menu bar and menus
	
	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');

	//  open a window

	if(!(windowPtr = GetNewWindow(rNewWindow,NULL,(WindowPtr)-1)))
		ExitToShell();
	SetPort(windowPtr);
	
	//  get block for document record, assign handle to window record refCon field

	if(!(docRecHdl = (DocRecHandle) NewHandle(sizeof(DocRec))))
		ExitToShell();

	SetWRefCon(windowPtr,(SInt32) docRecHdl);

	//  get controls, adjust scroll bars and show window
		
	doGetControls(windowPtr);
	doAdjustScrollBars(windowPtr);

	ShowWindow(windowPtr);

	//  enter eventLoop

	gDone = false;

	while(!gDone)
	{
		if(WaitNextEvent(everyEvent,&eventRec,MAXLONG,NULL))
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();	
	FlushEvents(everyEvent,0);
}

//  doGetControls

void  doGetControls(WindowPtr windowPtr)
{
	DocRecHandle docRecHdl;

	docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));

	if(!((*docRecHdl)->popupControlHdl = GetNewControl(cTimeZone,windowPtr)))
		ExitToShell();
		
	if(!((*docRecHdl)->redHdl = GetNewControl(cRed,windowPtr)))
		ExitToShell();
	if(!((*docRecHdl)->whiteHdl = GetNewControl(cWhite,windowPtr)))		
		ExitToShell();
	if(!((*docRecHdl)->blueHdl = GetNewControl(cBlue,windowPtr)))
		ExitToShell()
		;
	if(!((*docRecHdl)->showGridHdl = GetNewControl(cShowgrid,windowPtr)))
		ExitToShell();
	if(!((*docRecHdl)->showRulersHdl = GetNewControl(cShowrulers,windowPtr)))
		ExitToShell();
		
	if(!((*docRecHdl)->okButtonHdl = GetNewControl(cButton,windowPtr)))
		ExitToShell();
		
	if(!((*docRecHdl)->vScrollbarHdl = GetNewControl(cVScrollbar,windowPtr)))
		ExitToShell();
	if(!((*docRecHdl)->hScrollbarHdl = GetNewControl(cHScrollbar,windowPtr)))
		ExitToShell();
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case activateEvt:
			doActivate(eventRecPtr);
			break;

		case osEvt:
			doOSEvent(eventRecPtr);
			HiliteMenu(0);
			break;
	}
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;
	Rect			growRect;
	SInt32		newSize;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);
	
	switch(partCode)
	{
		case inMenuBar:
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			else
				doInContent(eventRecPtr,windowPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where) == true)
				gDone = true;
			break;

		case inGrow:
			growRect = qd.screenBits.bounds;
			growRect.top 	= 200; 
			growRect.left = 275;
			newSize = GrowWindow(windowPtr,eventRecPtr->where,&growRect);
			if(newSize != 0)
			{
				doEraseGrowIcon(windowPtr);
				SizeWindow(windowPtr,LoWord(newSize),HiWord(newSize),true);
				doAdjustScrollBars(windowPtr);
			}
			break;

		case inZoomIn:
		case inZoomOut:
			if(TrackBox(windowPtr,eventRecPtr->where,partCode))
			{
				SetPort(windowPtr);
				EraseRect(&windowPtr->portRect);
				ZoomWindow(windowPtr,partCode,false);
				InvalRect(&windowPtr->portRect);
				doAdjustScrollBars(windowPtr);
			}
			break;
	}
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;
	 		
	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
				SysBeep(10);
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;

		case mFile:
			if(menuItem == iQuit)
				gDone = true;
			break;
	}

	HiliteMenu(0);
}

//  doInContent

void  doInContent(EventRecord *eventRecPtr,WindowPtr windowPtr)
{
	ControlHandle	controlHdl;
	SInt16				controlValue;
	DocRecHandle	docRecHdl;

	GlobalToLocal(&eventRecPtr->where);

	if(FindControl(eventRecPtr->where,windowPtr,&controlHdl))
	{
		docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));
		if(controlHdl == (*docRecHdl)->popupControlHdl)
		{
			TrackControl(controlHdl,eventRecPtr->where,(ControlActionUPP) -1);
			controlValue = GetControlValue(controlHdl);
			doPopupMenuChoice(controlValue);
		}
		else
		{
			if(TrackControl(controlHdl,eventRecPtr->where,NULL))
				doControls(controlHdl,docRecHdl);
		}
	}	
}

//  doPopupMenuChoice

void  doPopupMenuChoice(SInt16 controlValue)
{
	switch(controlValue)
	{
		case pSydney:
			// Action as appropriate.
			break;

		case pNewYork:
			// Action as appropriate.
			break;

		case pLondon:
			// Action as appropriate.
			break;

		case pRome:
			// Action as appropriate.
			break;
	}

	SysBeep(10);
}

//  doControls

void  doControls(ControlHandle controlHdl,DocRecHandle docRecHdl)
{	
	if(controlHdl == (*docRecHdl)->redHdl || controlHdl == (*docRecHdl)->whiteHdl ||
	 								 controlHdl == (*docRecHdl)->blueHdl)
	{
		SetControlValue((*docRecHdl)->redHdl,0);
		SetControlValue((*docRecHdl)->whiteHdl,0);
		SetControlValue((*docRecHdl)->blueHdl,0);
		SetControlValue(controlHdl,1);
	}
	else if(controlHdl == (*docRecHdl)->showGridHdl || 
												controlHdl == (*docRecHdl)->showRulersHdl)
	{
		SetControlValue(controlHdl,!GetControlValue(controlHdl));
	}
	else if(controlHdl == (*docRecHdl)->vScrollbarHdl || 
												controlHdl == (*docRecHdl)->hScrollbarHdl)
	{
		// Do scroll bars handling.
	}
	else
	{
		// Must be button.  Do button handling.
	}

	SysBeep(10);
}

//  doUpdate

void  doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;

	windowPtr = (WindowPtr) eventRecPtr->message;

	BeginUpdate(windowPtr);

	if(!EmptyRgn(windowPtr->visRgn))
	{
		SetPort(windowPtr);
		UpdateControls(windowPtr,windowPtr->visRgn);
		DrawGrowIcon(windowPtr);
	}

	EndUpdate(windowPtr);
}

//  doActivate

void  doActivate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventRecPtr->message;

	becomingActive = ((eventRecPtr->modifiers & activeFlag) == activeFlag);

	doActivateWindow(windowPtr,becomingActive);
}

//  doActivateWindow

void  doActivateWindow(WindowPtr windowPtr,Boolean becomingActive)
{	
	DocRecHandle	docRecHdl;
	SInt16				hiliteState;
		
	docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));

	if(becomingActive)
		hiliteState = 0;
	else
		hiliteState = 255;

	HiliteControl((*docRecHdl)->popupControlHdl,hiliteState);
	HiliteControl((*docRecHdl)->redHdl,					hiliteState);
	HiliteControl((*docRecHdl)->whiteHdl,				hiliteState);
	HiliteControl((*docRecHdl)->blueHdl,				hiliteState);
	HiliteControl((*docRecHdl)->showGridHdl,		hiliteState);
	HiliteControl((*docRecHdl)->showRulersHdl,	hiliteState);
	HiliteControl((*docRecHdl)->okButtonHdl,		hiliteState);
	HiliteControl((*docRecHdl)->vScrollbarHdl,	hiliteState);
	HiliteControl((*docRecHdl)->hScrollbarHdl,	hiliteState);
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
	switch((eventRecPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:
			DrawGrowIcon(FrontWindow());
			gInBackground = (eventRecPtr->message & resumeFlag) == 0;
			doActivateWindow(FrontWindow(),!gInBackground);
			break;
				
		case mouseMovedMessage:
			break;
	}
}

//  doAdjustScrollBars

void	doAdjustScrollBars(WindowPtr windowPtr)
{
	Rect					winRect;
	DocRecHandle	docRecHdl;

	docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));

	winRect = windowPtr->portRect;

	HideControl((*docRecHdl)->vScrollbarHdl);
	HideControl((*docRecHdl)->hScrollbarHdl);

	MoveControl((*docRecHdl)->vScrollbarHdl,winRect.right - 15,winRect.top - 1);
	MoveControl((*docRecHdl)->hScrollbarHdl,winRect.left -1,winRect.bottom -15);

	SizeControl((*docRecHdl)->vScrollbarHdl,16, winRect.bottom - 13);
	SizeControl((*docRecHdl)->hScrollbarHdl, winRect.right - 13,16);

	ShowControl((*docRecHdl)->vScrollbarHdl);
	ShowControl((*docRecHdl)->hScrollbarHdl);

	DrawGrowIcon(windowPtr);
}

//  doEraseGrowIcon

void  doEraseGrowIcon(WindowPtr windowPtr)
{
	Rect	growBoxRect;
	
	SetPort(windowPtr);

	growBoxRect = windowPtr->portRect;
	growBoxRect.left = growBoxRect.right - 15;
	growBoxRect.top = growBoxRect.bottom - 15;
	EraseRect(&growBoxRect);
}	

// 
