// 
// Controls2.c
// 
// 
// This program:
//
// 	Opens a noGrowDocProc window with a horizontal scrollbar.
//
// 	Allows the user to horizontally scroll a picture within the window using the
//		scroll box, the scroll arrows and the gray area.
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File and Edit (preload, non-
//		purgeable).  
//
// 	A 'WIND' resource (purgeable) (initially visible).  
//
// 	An 'CNTL' resource for the horizontal scroll bar (purgeable).  
//
// 	A 'PICT' resource containing the picture to be scrolled (non-purgeable).  
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch,
//		and is32BitCompatible flags set.	  
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>

//  defines

#define rMenubar		128
#define rNewWindow	128
#define rPicture		128
#define mApple			128
#define  iAbout			1
#define mFile				129
#define  iQuit			11
#define mEdit				130
#define cHScrollbar	128
#define MAXLONG			0x7FFFFFFF

//  typedefs

typedef struct
{
	ControlHandle hScrollbarHdl;
} DocRec;

typedef DocRec **DocRecHandle;

//  global variables

Boolean		gDone;
Boolean		gInBackground;
Rect			gPictRect;
PicHandle	gPictureHdl;

//  function prototypes

void					main								(void);
void					doInitManagers			(void);
void					doGetControl				(WindowPtr);
void					doGetPicture				(void);
void					doEvents						(EventRecord *);
void					doMouseDown					(EventRecord *);
void					doUpdate						(EventRecord *);
void					doActivate					(EventRecord *);
void					doActivateWindow		(WindowPtr,Boolean);
void					doOSEvent						(EventRecord *);
void					doMenuChoice				(SInt32);
void					doInContent					(EventRecord *,WindowPtr);
void					doScrollBars				(ControlPartCode,WindowPtr,ControlHandle,Point);
pascal void		actionProcedure			(ControlHandle,ControlPartCode);
void					doMoveScrollBox			(ControlHandle,SInt16);

//  main

void  main(void)
{
	Handle				menubarHdl;
	MenuHandle		menuHdl;
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	EventRecord		eventRec;

	//  initialise managers

	doInitManagers();
	
	//  set up menu bar and menus
	
	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');

	//  open a window

	if(!(windowPtr = GetNewWindow(rNewWindow,NULL,(WindowPtr)-1)))
		ExitToShell();

	SetPort(windowPtr);

	//  get block for document record, assign handle to window record refCon field

	docRecHdl = (DocRecHandle) NewHandle(sizeof(DocRec));
	SetWRefCon(windowPtr,(SInt32) docRecHdl);

	//  get controls

	doGetControl(windowPtr);

	//  get picture

	doGetPicture();

	//  enter eventLoop

	gDone = false;

	while(!gDone)
	{
		if(WaitNextEvent(everyEvent,&eventRec,MAXLONG,NULL))
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();	
	FlushEvents(everyEvent,0);
}

//  doGetControl

void  doGetControl(WindowPtr windowPtr)
{
	DocRecHandle docRecHdl;

	docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));

	(*docRecHdl)->hScrollbarHdl = GetNewControl(cHScrollbar,windowPtr);
}

//  doGetPicture

void  doGetPicture(void)
{
	gPictureHdl = GetPicture(rPicture);

	gPictRect = (*gPictureHdl)->picFrame;

	gPictRect.right -=  gPictRect.left;
	gPictRect.left = 0;
	gPictRect.bottom -= gPictRect.top;
	gPictRect.top = 0;
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case activateEvt:
			doActivate(eventRecPtr);
			break;

		case osEvt:
			doOSEvent(eventRecPtr);
			HiliteMenu(0);
			break;
	}
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);

	switch(partCode)
	{
		case inMenuBar:
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			else
				doInContent(eventRecPtr,windowPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where) == true)
				gDone = true;
			break;
	}
}

//  doUpdate

void  doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	DocRecHandle	docRecHdl;

	windowPtr = (WindowPtr)eventRecPtr->message;
	docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));

	BeginUpdate(windowPtr);

	if(!EmptyRgn(windowPtr->visRgn))
	{
		SetPort(windowPtr);
		UpdateControls(windowPtr,windowPtr->visRgn);

		SetOrigin(GetControlValue((*docRecHdl)->hScrollbarHdl),0);
		DrawPicture(gPictureHdl,&gPictRect);
		SetOrigin(0,0);
	}

	EndUpdate(windowPtr);
}

//  doActivate

void  doActivate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventRecPtr->message;

	becomingActive = ((eventRecPtr->modifiers & activeFlag) == activeFlag);

	doActivateWindow(windowPtr,becomingActive);
}

//  doActivateWindow

void  doActivateWindow(WindowPtr windowPtr,Boolean becomingActive)
{	
	DocRecHandle	docRecHdl;

	docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));

	if(becomingActive)
		HiliteControl((*docRecHdl)->hScrollbarHdl,0);
	else
		HiliteControl((*docRecHdl)->hScrollbarHdl,255);
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
	switch((eventRecPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:
			gInBackground = (eventRecPtr->message & resumeFlag) == 0;
			doActivateWindow(FrontWindow(),!gInBackground);
			break;

		case mouseMovedMessage:
			break;
	}
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
				SysBeep(10);
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;
			
		case mFile:
			if(menuItem == iQuit)
				gDone = true;
			break;
	}

	HiliteMenu(0);
}

//  doIncontent

void  doInContent(EventRecord *eventRecPtr,WindowPtr windowPtr)
{
	Point						mouseXY;
	ControlPartCode	partCode;
	ControlHandle		controlHdl;

	mouseXY = eventRecPtr->where;
	GlobalToLocal(&mouseXY);

	if(partCode = FindControl(mouseXY,windowPtr,&controlHdl))
		doScrollBars(partCode,windowPtr,controlHdl,mouseXY);
}

//  doScrollBars

void  doScrollBars(ControlPartCode partCode,WindowPtr windowPtr,ControlHandle controlHdl,
									 Point mouseXY)
{
	DocRecHandle	docRecHdl;
	SInt16				oldControlValue;
	SInt16				scrollDistance;
	RgnHandle			updateRegion;

	docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));

	switch(partCode)
	{
		case kControlIndicatorPart:
			oldControlValue = GetControlValue(controlHdl);
			if(TrackControl(controlHdl,mouseXY,NULL))
			{
				scrollDistance = oldControlValue - GetControlValue(controlHdl);
				if(scrollDistance != 0)
				{
					if(controlHdl == (*docRecHdl)->hScrollbarHdl)
					{
						updateRegion = NewRgn();
						ScrollRect(&gPictRect,scrollDistance,0,updateRegion);
						InvalRgn(updateRegion);
						DisposeRgn(updateRegion);
					}
					else
					{
						// Vertical scroll bar scroll box handling here.
					}
				}
			}
			break;

		case kControlUpButtonPart:
		case kControlDownButtonPart:
		case kControlPageUpPart:
		case kControlPageDownPart:
			if(controlHdl == (*docRecHdl)->hScrollbarHdl)
				TrackControl(controlHdl,mouseXY,&actionProcedure);
			else
				// Vertical scroll via horizontal scrolling action procedure here.
			break;
	}
}

//  actionProcedure

pascal void  actionProcedure(ControlHandle controlHdl,ControlPartCode partCode)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	SInt16				scrollDistance;
	SInt16 				controlValue;
	RgnHandle			updateRegion;

	if(partCode)
	{
		windowPtr = (*controlHdl)->contrlOwner;
		docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));

		switch(partCode)
		{
			case kControlUpButtonPart:
			case kControlDownButtonPart:
				scrollDistance = 2;
				break;

			case kControlPageUpPart:
			case kControlPageDownPart:
				scrollDistance = (windowPtr->portRect.right - windowPtr->portRect.left) - 10;
				break;
		}

		if((partCode == kControlDownButtonPart) || (partCode == kControlPageDownPart))
			scrollDistance = -scrollDistance;

		controlValue = GetControlValue(controlHdl);
		if(((controlValue == GetControlMaximum(controlHdl)) && scrollDistance < 0) || 
			 ((controlValue == GetControlMinimum(controlHdl)) && scrollDistance > 0))
			return;

		doMoveScrollBox(controlHdl,scrollDistance);

		updateRegion = NewRgn();
		ScrollRect(&gPictRect,scrollDistance,0,updateRegion);
		InvalRgn(updateRegion);
		DisposeRgn(updateRegion);

		if(scrollDistance == 2 || scrollDistance == -2)
			BeginUpdate(windowPtr);

		SetOrigin(GetControlValue((*docRecHdl)->hScrollbarHdl),0);
		DrawPicture(gPictureHdl,&gPictRect);
		SetOrigin(0,0);

		if(scrollDistance == 2 || scrollDistance == -2)
			EndUpdate(windowPtr);
	}
}

//  doMoveScrollBox

void doMoveScrollBox(ControlHandle controlHdl,SInt16 scrollDistance)
{
	SInt16		oldControlValue, controlValue, controlMax;

	oldControlValue = GetControlValue(controlHdl);
	controlMax = GetControlMaximum(controlHdl);

	controlValue = oldControlValue - scrollDistance;
	
	if(controlValue < 0)
		controlValue = 0;
	else if(controlValue > controlMax)
		controlValue = controlMax;

	SetControlValue(controlHdl,controlValue);
}

// 
