// 
// DialogsAndAlerts.c
// 
// 
// This program:
//
// 	Opens a window for the purposes of displaying text and for proving correct window
//		updating and activation/deactivation in the presence of alert and dialog boxes.
//
// 	Allows the user to invoke a demonstration alert, a modal dialog box, a movable
//		modal dialog box and a modeless dialog box via the Demonstration menu.
//
// The modal dialog box contains three checkboxes.
//
// The movable modal dialog box contains three radio buttons.
//
// The modeless dialog box contains an icon and an editable text item.  The editable text
// item is supported by the Edit menu Cut, Copy, Paste and Clear commands.
//
// The alert box and modal dialog use an application-defined event filter function.
//
// An application-defined function is used to draw the bold outline around the default
// button in the modal, movable modal and modeless dialog boxes.
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Demonstration and Help
//		menus (preload, non-purgeable).
//
// 	A 'WIND' resource (purgeable) (initially visible).
//
// 	An 'ALRT' resource (purgeable).
//
// 	'DLOG' resources (purgeable) (initially not visible) and associated 'DITL' 
//		resources (purgeable).
//
// 	'dctb' resources (purgeable) to force the Dialog Manager to create colour graphics
//		ports for the movable modal and modeless dialog boxes. 
//
// 	A 'cicn' resource (purgeable).
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents and doesActivateOnFGSwitch,
//		and is32BitCompatible flags set.	
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <Palettes.h>
#include <LowMem.h>

//  defines

#define mApple					128
#define  iAbout					1
#define mFile						129
#define  iClose					4
#define  iQuit					11
#define mEdit						130
#define  iCut						3
#define  iCopy					4
#define  iPaste					5
#define  iClear					6
#define mDemonstration	131
#define  iAlert					1
#define  iModal					2
#define  iMovable				3
#define  iModeless			4
#define rMenubar				128
#define rNewWindow			128
#define rAlert					128
#define  iOK						1
#define  iCancel				2
#define  iUserItem			3
#define rModal					129
#define  iGridSnap			4
#define  iShowGrid			5
#define  iShowRulers		6
#define rMovable				130
#define  iCharcoal			4
#define  iOilPaint			5
#define  iWaterColour		6
#define rModeless				131
#define  iSearch				1
#define  iEditText			4

#define kMovableModal		1
#define kModeless				2

#define kReturn					(SInt8) 0x0D
#define kEnter					(SInt8) 0x03
#define kEscape					(SInt8) 0x1B
#define kPeriod					(SInt8) 0x2E

#define MAXLONG					0x7FFFFFFF

//  typedefs

typedef struct
{
	ControlHandle	vScrollbarHdl;
	ControlHandle hScrollbarHdl;
} docRec, *docRecPointer, **docRecHandle;

//  global variables

WindowPtr	gWindowPtr;
SInt32		gSleepTime;
Boolean		gDone;
Boolean		gInBackground;
Boolean		gGridSnap				= false;
Boolean		gShowGrid				= false;
Boolean		gShowRule				= false;	
SInt16		gBrushType			= iCharcoal;
SInt16		gOldBrushType		= iCharcoal;
DialogPtr gModelessDlgPtr	= NULL;

//  function prototypes

void			main											(void);
void			doInitManagers						(void);
void			eventLoop									(void);
void			doIdle										(EventRecord *);
void			doOSEvent									(EventRecord *);
void			doMouseDown								(EventRecord *);
void			doKeyDown									(EventRecord *);
void			doKeyDownDocument					(EventRecord *);
void			doKeyDownMovableModal			(EventRecord *);
void			doKeyDownModeless					(EventRecord *);
void			doUpdate									(EventRecord *);
void			doUpdateDocument					(EventRecord *);
void			doUpdateMovableOrModeless	(EventRecord *);
void			doUpDateModeless					(EventRecord *);
void			doActivate								(EventRecord *);
void			doActivateDocument				(WindowPtr,Boolean);
void			doActivateMovableModal		(WindowPtr,Boolean);
void			doActivateModeless				(WindowPtr,Boolean);
void			doEvents									(EventRecord *);
void			doAdjustMenus							(void);
void			doMenuChoice							(SInt32);
void			doEditMenu								(SInt16);
void			doDemonstrationMenu				(SInt16);
Boolean		doModalDialog							(void);
Boolean		doMovableModalDialog			(void);
Boolean		doModelessDialog					(void);
void			doInContent								(EventRecord *);
void			doItemHitMovableModal			(DialogPtr,SInt16);
void			doItemHitModeless					(DialogPtr);
void			doHideModeless						(void);
void			invalidateScrollBarArea		(WindowPtr);

pascal Boolean	eventFilter								(DialogPtr,EventRecord *,SInt16 *);
pascal void			drawDefaultButtonOutline	(DialogPtr,SInt16);

//  main

void  main(void)
{
	Handle				menubarHdl;
	MenuHandle		menuHdl;
	docRecHandle	docRecHdl;
	
	//  initialise managers

	doInitManagers();

	//  set up menu bar and menus

	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');

	//  open window

	if(!(gWindowPtr = GetNewWindow(rNewWindow,NULL,(WindowPtr)-1)))
		ExitToShell();

	if(!(docRecHdl = (docRecHandle) NewHandle(sizeof(docRec))))
		ExitToShell();

	SetWRefCon(gWindowPtr,(SInt32) docRecHdl);

	//  enter eventLoop
	
	eventLoop();
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();
	FlushEvents(everyEvent,0);
}

//  eventLoop

void  eventLoop(void)
{
	EventRecord		eventRec;
	Boolean				gotEvent;		

	gSleepTime = MAXLONG;	

	gDone = false;
	
	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventRec,gSleepTime,NULL);

		if(gotEvent)
			doEvents(&eventRec);
		else
			doIdle(&eventRec);
	}
}

//  doIdle

void  doIdle(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		dialogType;
	SInt16		itemHit;

	windowPtr = FrontWindow();

	if(((WindowPeek) windowPtr)->windowKind == dialogKind)
	{
		dialogType = ((WindowPeek) windowPtr)->refCon;
	
		if(dialogType == kModeless)
			DialogSelect(eventRecPtr,&(DialogPtr) windowPtr,&itemHit);
	}
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			doKeyDown(eventRecPtr);
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case activateEvt:
			doActivate(eventRecPtr);
			break;

		case osEvt:
			doOSEvent(eventRecPtr);
			HiliteMenu(0);
			break;
	}
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;
	Rect			growRect;
	SInt32		newSize;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);

	switch(partCode)
	{
		case inMenuBar:
			doAdjustMenus();
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
			{
				if(((WindowPeek) (FrontWindow()))->refCon == kMovableModal)
					SysBeep(10);
				else
					SelectWindow(windowPtr);
			}
			else
				doInContent(eventRecPtr);
			break;

		case inDrag:
			if((((WindowPeek) FrontWindow())->refCon == kMovableModal) && 
				 (((WindowPeek) windowPtr)->refCon != kMovableModal))
			{
				SysBeep(10);
				return;
			}
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where))
				doHideModeless();
			break;

		case inGrow:
			growRect = qd.screenBits.bounds;
			growRect.top 	= 80; 
			growRect.left = 160;
			newSize = GrowWindow(windowPtr,eventRecPtr->where,&growRect);
			if(newSize != 0)
			{
				invalidateScrollBarArea(windowPtr);
				SizeWindow(windowPtr,LoWord(newSize),HiWord(newSize),true);
				invalidateScrollBarArea(windowPtr);
			}
			break;
	}
}

//  doKeyDown

void  doKeyDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16 		dialogType;

	windowPtr = FrontWindow();

	if(((WindowPeek) windowPtr)->windowKind == dialogKind)
	{
		dialogType = ((WindowPeek) windowPtr)->refCon;

		switch(dialogType)
		{
			case kMovableModal:
				doKeyDownMovableModal(eventRecPtr);
				break;

			case kModeless:
				doKeyDownModeless(eventRecPtr);
				break;
		}
	}																	
	else if(((WindowPeek) windowPtr)->windowKind == userKind)
		doKeyDownDocument(eventRecPtr);
}

//  doKeyDownDocument

void  doKeyDownDocument(EventRecord *eventRecPtr)
{
	SInt8	charCode;

	charCode = eventRecPtr->message & charCodeMask;

	if((eventRecPtr->modifiers & cmdKey) != 0)
	{
		doAdjustMenus();
		doMenuChoice(MenuKey(charCode));
	}
}

//  doKeyDownMovableModal

void  doKeyDownMovableModal(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt8			charCode;
	SInt16		itemType;
	Handle		itemHandle;
	Rect			itemRect;
	SInt32		finalTicks;

	windowPtr = FrontWindow();
	charCode = eventRecPtr->message & charCodeMask;

	if((charCode == (SInt8) kReturn) || (charCode == (SInt8) kEnter))
	{
		GetDialogItem((DialogPtr) windowPtr,iOK,&itemType,&itemHandle,&itemRect);
		HiliteControl((ControlHandle) itemHandle,kControlButtonPart);
		Delay(8,&finalTicks);
		HiliteControl((ControlHandle) itemHandle,0);
		DisposeDialog((DialogPtr) windowPtr);
	}
	else if((charCode == (SInt8) kEscape) || 
		 ((eventRecPtr->modifiers & cmdKey) && (charCode == (SInt8) kPeriod)))
	{
		GetDialogItem((DialogPtr) windowPtr,iCancel,&itemType,&itemHandle,&itemRect);
		HiliteControl((ControlHandle) itemHandle,kControlButtonPart);
		Delay(8,&finalTicks);
		HiliteControl((ControlHandle) itemHandle,0);
		gBrushType = gOldBrushType;
		DisposeDialog((DialogPtr) windowPtr);			
	}
}

//  doKeyDownModeless

void  doKeyDownModeless(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt8			charCode;
	SInt16		itemType;
	Handle		itemHandle;
	Rect			itemRect;
	SInt32		finalTicks;
	DialogPtr	dialogPtr;	
	SInt16		itemHit;

	windowPtr = FrontWindow();
	charCode = eventRecPtr->message & charCodeMask;

	if((charCode == (SInt8) kReturn) || (charCode == (SInt8) kEnter))
	{
		GetDialogItem((DialogPtr) windowPtr,iSearch,&itemType,&itemHandle,&itemRect);
		HiliteControl((ControlHandle) itemHandle,kControlButtonPart);
		Delay(8,&finalTicks);
		HiliteControl((ControlHandle) itemHandle,0);
		doItemHitModeless((DialogPtr) windowPtr);
	}
	else
	{
		dialogPtr = (DialogPtr) windowPtr;
		DialogSelect(eventRecPtr,&dialogPtr,&itemHit);
	}
}

//  doUpdate

void  doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16 		dialogType;

	windowPtr = (WindowPtr) eventRecPtr->message;

	if(((WindowPeek) windowPtr)->windowKind == dialogKind)
	{
		dialogType = ((WindowPeek) windowPtr)->refCon;
		
		if(dialogType == kMovableModal || dialogType == kModeless)
			doUpdateMovableOrModeless(eventRecPtr);
	}
	else if(((WindowPeek) windowPtr)->windowKind == userKind)
		doUpdateDocument(eventRecPtr);
}

//  doUpdateDocument

void  doUpdateDocument(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	Rect 			paintRect;
	Pattern		fillPattern;

	windowPtr = (WindowPtr) eventRecPtr->message;

	BeginUpdate(windowPtr);

	if(!EmptyRgn(windowPtr->visRgn))
	{
		SetPort(windowPtr);

		EraseRgn(windowPtr->visRgn);

		paintRect = windowPtr->portRect;
		paintRect.right -= 15;
		paintRect.bottom -= 15;
		GetIndPattern(&fillPattern,0,16);
		FillRect(&paintRect,&fillPattern);

		DrawGrowIcon(windowPtr);
	}	

	EndUpdate(windowPtr);
}

//  doUpdateMovableOrModeless

void  doUpdateMovableOrModeless(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;

	windowPtr = (WindowPtr) eventRecPtr->message;

	BeginUpdate(windowPtr);
	UpdateDialog(windowPtr,windowPtr->visRgn);
	EndUpdate(windowPtr);
}

//  doActivate

void  doActivate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		dialogType;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventRecPtr->message;
	becomingActive = (eventRecPtr->modifiers & activeFlag) == activeFlag;

	if(((WindowPeek) windowPtr)->windowKind == dialogKind)
	{
		dialogType = ((WindowPeek) windowPtr)->refCon;

		if(dialogType == kMovableModal)
			doActivateMovableModal(windowPtr,becomingActive);	
		else if(dialogType == kModeless)
			doActivateModeless(windowPtr,becomingActive);
	}
	else if(((WindowPeek) windowPtr)->windowKind == userKind)
		doActivateDocument(windowPtr,becomingActive);
}

//  doActivateDocument

void  doActivateDocument(WindowPtr windowPtr,Boolean becomingActive)
{
	if(becomingActive)
		doAdjustMenus();

	DrawGrowIcon(windowPtr);
}

//  doActivateMovableModal

void  doActivateMovableModal(WindowPtr windowPtr,Boolean becomingActive)
{
	SInt16 	a, itemType;
	Handle 	itemHdl;
	Rect 		itemRect;

	if(becomingActive)
	{
		for(a=iOK;a<(iWaterColour+1);a++) 
		{
			if(a != iUserItem)
			{
				GetDialogItem((DialogPtr) windowPtr,a,&itemType,&itemHdl,&itemRect);
				HiliteControl((ControlHandle) itemHdl,0);
			}
		}

		drawDefaultButtonOutline((DialogPtr) windowPtr,iOK);
		doAdjustMenus();
	}
	else
	{
		for(a=iOK;a<(iWaterColour+1);a++) 
		{
			if(a != iUserItem)
			{
				GetDialogItem((DialogPtr) windowPtr,a,&itemType,&itemHdl,&itemRect);
				HiliteControl((ControlHandle) itemHdl,255);
			}
		}
		drawDefaultButtonOutline((DialogPtr) windowPtr,iOK);
	}
}

//  doActivateModeless

void  doActivateModeless(WindowPtr windowPtr,Boolean becomingActive)
{
	SInt16 	itemType;
	Handle 	itemHdl;
	Rect 		itemRect;

	if(becomingActive)
	{
		GetDialogItem((DialogPtr) windowPtr,iSearch,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,0);

		drawDefaultButtonOutline((DialogPtr) windowPtr,iSearch);
		SelectDialogItemText((DialogPtr) windowPtr,iEditText,0,32767);
		gSleepTime =  LMGetCaretTime();
		doAdjustMenus();
	}
	else
	{
		GetDialogItem((DialogPtr) windowPtr,iSearch,&itemType,&itemHdl,&itemRect);
		HiliteControl((ControlHandle) itemHdl,255);
		
		drawDefaultButtonOutline((DialogPtr) windowPtr,iSearch);
		SelectDialogItemText((DialogPtr) windowPtr,iEditText,0,0);
		gSleepTime = MAXLONG;	
	}
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
	SInt16		dialogType;
	WindowPtr	windowPtr;
	
	windowPtr = FrontWindow();

	switch((eventRecPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:

			gInBackground = (eventRecPtr->message & resumeFlag) == 0;

			if(((WindowPeek) windowPtr)->windowKind == dialogKind)
			{
				dialogType = ((WindowPeek) windowPtr)->refCon;
				
				if(dialogType == kMovableModal)
					doActivateMovableModal(windowPtr,!gInBackground);
				else if(dialogType == kModeless)
					doActivateModeless(windowPtr,!gInBackground);
			}
			else if(((WindowPeek) windowPtr)->windowKind == userKind)
				doActivateDocument(windowPtr,!gInBackground);

			break;

		case mouseMovedMessage:
		break;
	}
}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	WindowPtr		windowPtr;
	SInt16			dialogType;
	MenuHandle	menuHdl;

	windowPtr = FrontWindow();

	if(((WindowPeek) windowPtr)->windowKind == dialogKind)
	{
		dialogType = ((WindowPeek) windowPtr)->refCon;

		switch(dialogType)
		{
			case kMovableModal:
				menuHdl = GetMenuHandle(mFile);
				DisableItem(menuHdl,0);
				menuHdl = GetMenuHandle(mEdit);
				DisableItem(menuHdl,0);		
				menuHdl = GetMenuHandle(mDemonstration);
				DisableItem(menuHdl,0);		
				EnableItem(menuHdl,4);
				break;

			case kModeless:
				menuHdl = GetMenuHandle(mFile);
				EnableItem(menuHdl,0);
				EnableItem(menuHdl,4);
				menuHdl = GetMenuHandle(mEdit);
				EnableItem(menuHdl,0);		
				menuHdl = GetMenuHandle(mDemonstration);
				EnableItem(menuHdl,0);		
				DisableItem(menuHdl,4);
				break;
		}
	}
	else if(((WindowPeek) windowPtr)->windowKind == userKind)
	{
		menuHdl = GetMenuHandle(mFile);
		EnableItem(menuHdl,0);
		DisableItem(menuHdl,4);
		menuHdl = GetMenuHandle(mEdit);
		DisableItem(menuHdl,0);
		menuHdl = GetMenuHandle(mDemonstration);
		EnableItem(menuHdl,0);		
		EnableItem(menuHdl,4);
	}

	DrawMenuBar();
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
				SysBeep(10);
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;

		case mFile:
			if(menuItem == iQuit)
				gDone = true;
			else if(menuItem == iClose)
				doHideModeless();
			break;

		case mEdit:
			doEditMenu(menuItem);
			break;

		case mDemonstration:
			doDemonstrationMenu(menuItem);
			break;
	}

	HiliteMenu(0);
}

//  doEditMenu

void	doEditMenu(SInt16 menuItem)
{
	WindowPtr	windowPtr;
	SInt16		dialogType;

	windowPtr = FrontWindow();

	if(((WindowPeek) windowPtr)->windowKind == dialogKind)
	{
		dialogType = ((WindowPeek) windowPtr)->refCon;
		
		if(dialogType == kModeless)
		{
			switch(menuItem)
			{
				case iCut:
					DialogCut((DialogPtr) windowPtr);
					break;

				case iCopy:
					DialogCopy((DialogPtr) windowPtr);				
					break;

				case iPaste:
					DialogPaste((DialogPtr) windowPtr);
					break;

				case iClear:
					DialogDelete((DialogPtr) windowPtr);
					break;
			}
		}
	}
}

//  doDemonstrationMenu

void	doDemonstrationMenu(SInt16 menuItem)
{
	WindowPtr	windowPtr;
	Rect			theRect;
	
	switch(menuItem)
	{
		case iAlert:
			windowPtr = FrontWindow();
			if(GetAlrtStage() > 0)
			{
				if(windowPtr && ((WindowPeek) windowPtr)->windowKind != dialogKind)
				{
					SetRect(&theRect,windowPtr->portRect.right - 15,windowPtr->portRect.bottom - 15,
									windowPtr->portRect.right,windowPtr->portRect.bottom);
					InvalRect(&theRect);
					doActivateDocument(windowPtr,false);
				}
				else if(windowPtr && ((WindowPeek) windowPtr)->windowKind == dialogKind)
					doActivateModeless(windowPtr,false);
			}
			NoteAlert(rAlert,(ModalFilterUPP) &eventFilter);
			break;

		case iModal:
			windowPtr = FrontWindow();
			if(windowPtr && ((WindowPeek) windowPtr)->windowKind != dialogKind)
			{
				SetRect(&theRect,windowPtr->portRect.right - 15,windowPtr->portRect.bottom - 15,
								windowPtr->portRect.right,windowPtr->portRect.bottom);
				InvalRect(&theRect);
				doActivateDocument(windowPtr,false);
			}
			else if(windowPtr && ((WindowPeek) windowPtr)->windowKind == dialogKind)
				doActivateModeless(windowPtr,false);

			if(!doModalDialog())
			{
				SysBeep(10);
				ExitToShell();
			}
			break;

		case iMovable:
			if(!doMovableModalDialog())
			{
				SysBeep(10);
				ExitToShell();
			}
			break;

		case iModeless:
			if(!doModelessDialog())
			{
				SysBeep(10);
				ExitToShell();
			}
			break;
	}
}

//  doModalDialog

Boolean  doModalDialog(void)
{
	DialogPtr		modalDlgPtr;
	SInt16			itemType, itemHit;
	Handle			itemHdl;
	Rect				itemRect;
	
	if(!(modalDlgPtr = GetNewDialog(rModal,NULL,(WindowPtr) -1)))
		return(false);

	GetDialogItem(modalDlgPtr,iUserItem,&itemType,&itemHdl,&itemRect);
	SetDialogItem(modalDlgPtr,iUserItem,itemType,(Handle) &drawDefaultButtonOutline,&itemRect);

	GetDialogItem(modalDlgPtr,iGridSnap,&itemType,&itemHdl,&itemRect);
	SetControlValue((ControlHandle) itemHdl,gGridSnap);

	GetDialogItem(modalDlgPtr,iShowGrid,&itemType,&itemHdl,&itemRect);
	SetControlValue((ControlHandle) itemHdl,gShowGrid);

	GetDialogItem(modalDlgPtr,iShowRulers,&itemType,&itemHdl,&itemRect);
	SetControlValue((ControlHandle) itemHdl,gShowRule);

	ShowWindow(modalDlgPtr);

	do
	{
		ModalDialog((ModalFilterUPP) &eventFilter,&itemHit);
		GetDialogItem(modalDlgPtr,itemHit,&itemType,&itemHdl,&itemRect);
		SetControlValue((ControlHandle) itemHdl,!GetControlValue((ControlHandle) itemHdl));
	} while((itemHit != iOK) && (itemHit != iCancel));

	if(itemHit == iOK)
	{
		GetDialogItem(modalDlgPtr,iGridSnap,&itemType,&itemHdl,&itemRect);
		gGridSnap = GetControlValue((ControlHandle) itemHdl);

		GetDialogItem(modalDlgPtr,iShowGrid,&itemType,&itemHdl,&itemRect);
		gShowGrid = GetControlValue((ControlHandle) itemHdl);

		GetDialogItem(modalDlgPtr,iShowRulers,&itemType,&itemHdl,&itemRect);
		gShowRule = GetControlValue((ControlHandle) itemHdl);				
	}

	DisposeDialog(modalDlgPtr);

	return(true);
}

//  doMovableModalDialog

Boolean  doMovableModalDialog(void)
{
	DialogPtr		modalDlgPtr;
	SInt16			itemType;
	Handle			itemHdl;
	Rect				itemRect;
	
	if(!(modalDlgPtr = GetNewDialog(rMovable,NULL,(WindowPtr) -1)))
		return(false);

	SetWRefCon(modalDlgPtr,(SInt32) kMovableModal);

	GetDialogItem(modalDlgPtr,iUserItem,&itemType,&itemHdl,&itemRect);
	SetDialogItem(modalDlgPtr,iUserItem,itemType,(Handle) &drawDefaultButtonOutline,&itemRect);

	GetDialogItem(modalDlgPtr,gBrushType,&itemType,&itemHdl,&itemRect);
	SetControlValue((ControlHandle) itemHdl,1);

	ShowWindow(modalDlgPtr);

	gOldBrushType = gBrushType;

	return(true);
}

//  doModelessDialog

Boolean  doModelessDialog(void)
{
	SInt16	itemType;
	Handle	itemHdl;
	Rect		itemRect;

	if(gModelessDlgPtr == NULL)
	{
		if(!(gModelessDlgPtr = GetNewDialog(rModeless,NULL,(WindowPtr) -1)))
			return(false);

		SetWRefCon(gModelessDlgPtr,(SInt32) kModeless);

		GetDialogItem(gModelessDlgPtr,iUserItem,&itemType,&itemHdl,&itemRect);
		SetDialogItem(gModelessDlgPtr,iUserItem,itemType,(Handle) &drawDefaultButtonOutline,
									&itemRect);

		ShowWindow(gModelessDlgPtr);
		SelectDialogItemText(gModelessDlgPtr,iEditText,0,32767);
	}
	else
	{
		ShowWindow(gModelessDlgPtr);
		SelectWindow(gModelessDlgPtr);
	}

	return(true);
}

//  doInContent

void  doInContent(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		dialogType;
	DialogPtr	dialogPtr;
	SInt16		itemHit;

	windowPtr = FrontWindow();
	dialogType = ((WindowPeek) windowPtr)->refCon;

	if(((WindowPeek) windowPtr)->windowKind == dialogKind)
	{
		dialogType = ((WindowPeek) windowPtr)->refCon;
	
		if(dialogType == kMovableModal)
		{
			if(DialogSelect(eventRecPtr,&dialogPtr,&itemHit))
				doItemHitMovableModal(dialogPtr,itemHit);
		}
		else if(dialogType == kModeless)
		{
			if(DialogSelect(eventRecPtr,&dialogPtr,&itemHit))
				doItemHitModeless(dialogPtr);
		}
	}
	else if(((WindowPeek) windowPtr)->windowKind == userKind)			
	{			
		// Handle clicks in document content region here.
	}
}

//  doItemHitMovableModal

void  doItemHitMovableModal(DialogPtr dialogPtr,SInt16 itemHit)
{
	SInt16	a, itemType;
	Handle	itemHdl;
	Rect		itemRect;

	if(itemHit == iCharcoal || itemHit == iOilPaint || itemHit == iWaterColour)
	{
		for(a=iCharcoal;a<(iWaterColour+1);a++)
		{
			GetDialogItem(dialogPtr,a,&itemType,&itemHdl,&itemRect);
			SetControlValue((ControlHandle) itemHdl,0);
		}

		GetDialogItem(dialogPtr,itemHit,&itemType,&itemHdl,&itemRect);
		SetControlValue((ControlHandle) itemHdl,1);
		gBrushType = itemHit;
	}
	else if(itemHit == iOK || itemHit == iCancel)
	{
		if(itemHit == iCancel)
			gBrushType = gOldBrushType;
		DisposeDialog(dialogPtr);
	}
}

//  doItemHitModeless

void  doItemHitModeless(DialogPtr dialogPtr)
{
	WindowPtr	oldPort;
	Rect			printRect, itemRect;
	SInt16		itemType;
	Handle		itemHdl;
	Str255		itemString;

	GetPort(&oldPort);
	SetPort(gWindowPtr);

	SetRect(&printRect,15,13,369,36);

	PenMode(patBic);
	PaintRect(&printRect);	

	GetDialogItem(dialogPtr,iEditText,&itemType,&itemHdl,&itemRect);
	GetDialogItemText(itemHdl,itemString);
	MoveTo(20,29);
	DrawString("\pSearch string:  ");
	DrawString(itemString);
	
	PenNormal();
	SetPort(oldPort);
}

//  doHideModeless

void  doHideModeless(void)
{
	WindowPtr	windowPtr;
	SInt16		dialogType;

	windowPtr = FrontWindow();

	if(((WindowPeek) windowPtr)->windowKind == dialogKind)
	{
		dialogType = ((WindowPeek) windowPtr)->refCon;

		if(dialogType == kModeless)
		{
			HideWindow(windowPtr);
			InvalRgn(gWindowPtr->visRgn);
			gSleepTime = MAXLONG;
		}
	}
}

//  invalidateScrollBarArea

void  invalidateScrollBarArea(WindowPtr windowPtr)
{
	Rect	tempRect;

	SetPort(windowPtr);

	tempRect = windowPtr->portRect;
	tempRect.left = tempRect.right - 15;
	InvalRect(&tempRect);

	tempRect = windowPtr->portRect;
	tempRect.top = tempRect.bottom - 15;
	InvalRect(&tempRect);
}

//  eventFilter

pascal Boolean  eventFilter(DialogPtr dialogPtr,EventRecord *eventRecPtr,SInt16 *itemHit)
{
	SInt8		charCode;
	SInt16	itemType;
	Handle	itemHandle;
	Rect		itemRect;
	SInt32	finalTicks;
	SInt16	handledEvent;

	handledEvent = false;

	if((eventRecPtr->what == updateEvt) && ((WindowPtr) eventRecPtr->message != dialogPtr))
	{
		doUpdate(eventRecPtr);
	}
	else
	{
		switch(eventRecPtr->what)
		{
			case keyDown:
			case autoKey:
				charCode = eventRecPtr->message & charCodeMask;
				if((charCode == (SInt8) kReturn) || (charCode == (SInt8) kEnter))
				{
					GetDialogItem(dialogPtr,iOK,&itemType,&itemHandle,&itemRect);
					HiliteControl((ControlHandle) itemHandle,kControlButtonPart);
					Delay(8,&finalTicks);
					HiliteControl((ControlHandle) itemHandle,0);
					handledEvent = true;
					*itemHit = iOK;
				}
				if((charCode == (SInt8) kEscape) || 
					 ((eventRecPtr->modifiers & cmdKey) && (charCode == (SInt8) kPeriod)))
				{
					GetDialogItem(dialogPtr,iCancel,&itemType,&itemHandle,&itemRect);
					HiliteControl((ControlHandle) itemHandle,kControlButtonPart
					);
					Delay(8,&finalTicks);
					HiliteControl((ControlHandle) itemHandle,0);
					handledEvent = true;
					*itemHit = iCancel;
				}
				//	Other keyboard equivalents handled here.
				break;

			// Disk-inserted and other events handled here.
		}
	}

	return(handledEvent);
}

//  drawDefaultButtonOutline

pascal void  drawDefaultButtonOutline(DialogPtr dialogPtr,SInt16 theItem)
{
	WindowPtr	oldPort;
	PenState	oldPenState;
	SInt16		itemType;
	Handle		itemHandle;
	Rect			itemRect;
	CGrafPtr	cgrafPtr;
	Boolean		isColour;
	SInt8			buttonOval;
	RGBColor	backColour;
	RGBColor	foreSaveColour;
	RGBColor	newForeColour;
	Boolean		newGray;
	GDHandle	targetDevice;

	GetPort(&oldPort);
	GetPenState(&oldPenState);

	GetDialogItem(dialogPtr,iOK,&itemType,&itemHandle,&itemRect);
	SetPort((*(ControlHandle) itemHandle)->contrlOwner);
	InsetRect(&itemRect,-4,-4);

	cgrafPtr = (CGrafPtr) (*(ControlHandle) itemHandle)->contrlOwner;

	if((cgrafPtr->portVersion >> 14) && 0x00000003)
		isColour = true;
	else
		isColour = false;

	buttonOval = (itemRect.bottom - itemRect.top) / 2 + 2;

	if((*(ControlHandle) itemHandle)->contrlHilite == 255)
	{
		newGray = false;

		if(isColour)
		{
			GetBackColor(&backColour);
			GetForeColor(&foreSaveColour);
			newForeColour = foreSaveColour;
			targetDevice = GetMainDevice();
			newGray = GetGray(targetDevice,&backColour,&newForeColour);
		};

		if(newGray)
			RGBForeColor(&newForeColour);
		else
			PenPat(&qd.gray);

		PenSize(3,3);
		FrameRoundRect(&itemRect,buttonOval,buttonOval);

		if(isColour)
			RGBForeColor(&foreSaveColour);
	}
	else
	{
		PenPat(&qd.black);
		PenSize(3,3);
		FrameRoundRect(&itemRect,buttonOval,buttonOval);
	}

	SetPenState(&oldPenState);
	SetPort(oldPort);
}

// 