// 
// GDevice.c
// 
// 
// This program opens a small window, gets a handle to the GDevice record for the main
// device and displays some information obtained from that record.
//
// The program will run only on Macintoshes with Color QuickDraw.
//
// The program utilises an 'ALRT' resource, a 'DITL' resource, and a 'WIND' resource.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Gestalt.h>
#include <LowMem.h>

//  global variables

#define rAlert	128
#define rWindow	128

//  function prototypes

void	main	(void);

//  main

void  main(void)
{
	OSErr					osErr;
	SInt32				response;
	WindowPtr 		windowPtr;
	GDHandle			deviceHdl;
	SInt16				deviceType, bytesPerRow;
	Rect					theRect;
	Str255				string;
	PixMapHandle	pixMapHdl;

	//  initialise managers

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	InitCursor();
	
	//  check for Color QuickDraw
	
	osErr = Gestalt(gestaltQuickdrawVersion,&response);
	if(response < gestalt8BitQD)
	{
		ParamText("\pThis program will run only on Macintoshes with Color QuickDraw",NULL,
							NULL,NULL);
		StopAlert(rAlert,NULL);
		ExitToShell();
	}

	//  open a window

	if(!(windowPtr = GetNewWindow(128,NULL,(WindowPtr) -1)))
		ExitToShell();

	SetPort(windowPtr);
	TextSize(10);

	//  get handle to GDevice record for main device

	deviceHdl = LMGetMainDevice();

	//  print some information from the GDevice record

	MoveTo(10,20);
	deviceType = (**deviceHdl).gdType;
	switch(deviceType)
	{
		case 0:
			DrawString("\pIndexed device with variable CLUT.");
			break;

		case 1:
			DrawString("\pIndexed device with fixed CLUT.");
			break;

		case 2:
			DrawString("\pDirect device.");
			break;
	}

	MoveTo(10,40);
	theRect = (**deviceHdl).gdRect;
	DrawString("\pBoundary rectangle top = ");
	NumToString((SInt32) theRect.top,string);
	DrawString(string);

	MoveTo(10,55);
	DrawString("\pBoundary rectangle left = ");
	NumToString((SInt32) theRect.left,string);
	DrawString(string);

	MoveTo(10,70);
	DrawString("\pBoundary rectangle bottom = ");
	NumToString((SInt32) theRect.bottom,string);
	DrawString(string);

	MoveTo(10,85);
	DrawString("\pBoundary rectangle right = ");
	NumToString((SInt32) theRect.right,string);
	DrawString(string);

	MoveTo(10,105);
	pixMapHdl = (**deviceHdl).gdPMap;
	DrawString("\pPixel depth = ");
	NumToString((SInt32) (**pixMapHdl).pixelSize,string);
	DrawString(string);

	MoveTo(10,120);
	bytesPerRow = (**pixMapHdl).rowBytes & 0x7FFF;
	DrawString("\pBytes per row = ");
	NumToString((SInt32) bytesPerRow,string);
	DrawString(string);

	MoveTo(10,135);
	DrawString("\pTotal pixel image bytes = ");
	NumToString(((SInt32) bytesPerRow) * theRect.bottom,string);
	DrawString(string);

	MoveTo(10,155);
	if((**pixMapHdl).hRes == 0x00480000)
		DrawString("\pResolution = 72 dpi");

	MoveTo(10,175);
	if(BitTst(&(**deviceHdl).gdFlags,screenActive))
		DrawString("\pDevice is the current device");
	else
		DrawString("\pDevice is not the current device");

	while(!Button()) ;
}

// 

