// 
// GWorldPicCursIcon.c
// 
// 
// This program:
//
// 	Opens a window in which the results of various drawing operations are displayed,
//		and in which regions are established for a cursor shape change demonstration.
//
// 	Demonstrates offscreen graphics world, picture, cursor, animated cursor, and icon
//		operations as a result of the user choosing items from a Demonstration menu.
//
// 	Quits when the user chooses Quit or clicks the window's close box.
//
// The program utilises the following resources:
//
// 	'MBAR' resource and associated 'MENU' resources (preload, non-purgeable).
//
// 	A 'WIND' resource (purgeable) (initially visible). 
//
// 	An 'acur' resource (purgeable).
//
// 	'CURS' resources associated with the 'acur' resource (purgeable).
//
// 	An 'ALRT' resource (purgeable) and associated 'DITL' resource (purgeable) for an
//		About GWorldPicCursIcon alert box, which is used to demonstrate the display of
//		icons in alert boxes.
//
// 	'ICON', 'cicn', and 'SICN' resources (purgeable) for the display of icons in menu
//		items and the About GWorldPicCursIcon alert box.
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents and is32BitCompatible flags 
//		set.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <QDOffscreen.h>
#include <Resources.h>
#include <PictUtils.h>
#include <Gestalt.h>

//  defines

#define mApple										128
#define  iAbout										1
#define mFile											129
#define  iQuit										11
#define  mDemonstration						131
#define  iWithoutOffScreenGWorld	1
#define  iWithOffScreenGWorld			2
#define  iPicture									3
#define  iCursor									4
#define  iAnimatedCursor					5
#define  iIcon										6
#define rAlert										128
#define rMenubar									128
#define rWindow										128
#define rBeachBallCursor					128
#define rIcon											257
#define kBeachBallTickInterval		5
#define MAXLONG										0x7FFFFFFF

#define topLeft(r)								(((Point *) &(r))[0])
#define botRight(r)								(((Point *) &(r))[1])

//  typedefs

typedef struct
{
	SInt16			numberOfFrames;
	SInt16			whichFrame;
	CursHandle	frame[];
} animCurs, *animCursPtr, **animCursHandle;

//  global variables

Boolean					gDone;
WindowPtr				gWindowPtr;
SInt32					gSleepTime;
RgnHandle				gCursorRegion;
Boolean					gInBackground;
Boolean					gCursorRegionsActive = false;
animCursHandle	gAnimCursHdl;
Boolean					gAnimCursActive = false;
SInt16					gAnimCursTickInterval;
SInt32					gAnimCursLastTick;

//  function prototypes

void		main											(void);
void		doInitManagers						(void);
void		eventLoop									(void);
void		doEvents									(EventRecord *);
void		doMouseDown								(EventRecord *);
void		doOSEvent									(EventRecord *);
void		doMenuChoice							(SInt32);
void		doDemonstrationMenu				(SInt16);
void		doIdle										(void);
void		doWithoutOffScreenGWorld	(void);
void		doWithOffScreenGWorld			(void);
void		doGWorldDrawing						(void);
void		doPicture									(void);
void		doCursor									(void);
void		changeCursor							(WindowPtr,RgnHandle);
void		doAnimCursor							(void);
Boolean	getAnimCursor							(SInt16,SInt16);
void		spinAnimCursor						(void);
void		releaseAnimCursor					(void);
void		doIcon										(void);

//  main

void  main(void)
{
	Handle			menubarHdl;
	MenuHandle	menuHdl;
	
	//  initialise managers

	doInitManagers();

	//  set up menu bar and menus
	
	if(!(menubarHdl = GetNewMBar(rMenubar)))
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();
	if(!(menuHdl = GetMenuHandle(mApple)))
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');

	//  open window

	if(!(gWindowPtr = GetNewWindow(rWindow,NULL,(WindowPtr)-1)))
		ExitToShell();

	SetPort(gWindowPtr);

	TextSize(10);

	//  enter event loop

	eventLoop();
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();
	FlushEvents(everyEvent,0);
}

//  eventLoop

void  eventLoop(void)
{
	EventRecord	eventRec;
	Boolean			gotEvent;

	gDone = false;
	gSleepTime = MAXLONG;
	gCursorRegion = NULL;
	
	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventRec,gSleepTime,gCursorRegion);
		if(gotEvent)
			doEvents(&eventRec);
		else
			doIdle();
	}
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{		
	WindowPtr	windowPtr;
	SInt8			charCode;

	windowPtr = (WindowPtr) eventRecPtr->message;
	
	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
				doMenuChoice(MenuKey(charCode));
			break;
		
		case updateEvt:
			BeginUpdate(windowPtr);
			EndUpdate(windowPtr);
			break;

		case osEvt:
			doOSEvent(eventRecPtr);
			break;
	}
}
	
//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);

	switch(partCode)
	{
		case inMenuBar:
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where) == true)
				gDone = true;
			break;
	}
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
		switch((eventRecPtr->message >> 24) & 0x000000FF)
		{
			case suspendResumeMessage:
				if((eventRecPtr->message & resumeFlag) == 1)
					gInBackground = false;
				else
					gInBackground = true;
				break;
				
			case mouseMovedMessage:
				if(gCursorRegionsActive)
					changeCursor(gWindowPtr,gCursorRegion);
				break;
		}
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	if(gAnimCursActive == true)
	{
		gAnimCursActive = false;
		SetCursor(&qd.arrow);
		releaseAnimCursor();
		gSleepTime = MAXLONG;
	}
	if(gCursorRegionsActive == true)
	{
		gCursorRegionsActive = false;
		DisposeRgn(gCursorRegion);
		gCursorRegion = NULL;
	}

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
					Alert(rAlert,NULL);
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;

		case mFile:
			if(menuItem == iQuit)
				gDone = true;
			break;

		case mDemonstration:
			doDemonstrationMenu(menuItem);
			break;
	}
	
	HiliteMenu(0);
}

//  doDemonstrationMenu

void	doDemonstrationMenu(SInt16 menuItem)
{
	switch(menuItem)
	{
		case iWithoutOffScreenGWorld:
			doWithoutOffScreenGWorld();
			break;			

		case iWithOffScreenGWorld:
			doWithOffScreenGWorld();
			break;

		case iPicture:
			doPicture();
			break;

		case iCursor:
			doCursor();
			break;

		case iAnimatedCursor:
			doAnimCursor();
			break;

		case iIcon:
			doIcon();
			break;
	}
}

//  doIdle

void  doIdle(void)
{
	if(gAnimCursActive == true)
		spinAnimCursor();
}

//  doWithoutOffScreenGWorld

void  doWithoutOffScreenGWorld(void)
{
	BackColor(whiteColor);
	FillRect(&(gWindowPtr->portRect),&qd.white);

	doGWorldDrawing();
}

//  doWithOffScreenGWorld

void  doWithOffScreenGWorld(void)
{
	CGrafPtr			windowPortPtr;
	GDHandle			deviceHdl;
	QDErr					qdErr;
	GWorldPtr			gworldPortPtr;
	PixMapHandle	gworldPixMapHdl;
	Boolean				lockPixResult;
	Rect					sourceRect, destRect;	

	BackColor(whiteColor);
	FillRect(&(gWindowPtr->portRect),&qd.white);

	ForeColor(blackColor);
	MoveTo(130,140);
	DrawString("\pPlease Wait.  Drawing in offscreen graphics port.");	

	SetCursor(*(GetCursor(watchCursor)));

	GetGWorld(&windowPortPtr,&deviceHdl);

	qdErr = NewGWorld(&gworldPortPtr,0,&gWindowPtr->portRect,NULL,NULL,0);
	if(gworldPortPtr == NULL || qdErr != noErr)
	{
		SysBeep(10);
		return;
	}

	SetGWorld(gworldPortPtr,NULL);

	gworldPixMapHdl = GetGWorldPixMap(gworldPortPtr);
	if(!(lockPixResult = LockPixels(gworldPixMapHdl)))
	{
		SysBeep(10);
		return;
	}

	EraseRect(&(gworldPortPtr->portRect));	

	doGWorldDrawing();

	SetGWorld(windowPortPtr,deviceHdl);

	sourceRect = gworldPortPtr->portRect;
	destRect = windowPortPtr->portRect;

	CopyBits(&((GrafPtr) gworldPortPtr)->portBits,&((GrafPtr) windowPortPtr)->portBits,
					 &sourceRect,&destRect,srcCopy,NULL);

	if(QDError() != noErr)
		SysBeep(10);

	UnlockPixels(gworldPixMapHdl);
	DisposeGWorld(gworldPortPtr);	

	SetCursor(&qd.arrow);
}

//  doGWorldDrawing

void  doGWorldDrawing(void)
{
	SInt16	a, b, c;
	Rect		theRect;

	PenPat(&qd.black);
	PenSize(1,1);

	for(a=0;a<8;a++)
		for(b=12;b<463;b+=30)
			for(c=5;c<276;c+=18)
			{
				SetRect(&theRect,b+a,c+a,b+28-a,c+16-a);
				if(a < 3)								 ForeColor(redColor);
				else if(a > 2 && a < 6)  ForeColor(greenColor);
				else if(a > 5)					 ForeColor(blueColor);
				FrameRect(&theRect);
			}
}

//  doPicture

void  doPicture(void)
{
	Rect						pictureRect;	
	OpenCPicParams	picParams;
	PicHandle				pictureHdl;
	PolyHandle			trianglePoly;
	PictInfo				pictInfo;
	Str255					pictInfoString;

	BackColor(whiteColor);
	FillRect(&(gWindowPtr->portRect),&qd.white);

	pictureRect = gWindowPtr->portRect;
	InsetRect(&pictureRect,50,50);

	picParams.srcRect	= pictureRect;
	picParams.hRes		= 0x00480000;
	picParams.vRes		= 0x00480000;
	picParams.version = -2;

	pictureHdl = OpenCPicture(&picParams);

	ClipRect(&gWindowPtr->portRect);

	ForeColor(blueColor);
	FillRect(&pictureRect,&qd.dkGray);	
	ForeColor(yellowColor);
	FillOval(&pictureRect,&qd.gray);

	trianglePoly = OpenPoly();
	MoveTo(pictureRect.left,pictureRect.bottom);
	LineTo(pictureRect.left + ((pictureRect.right - pictureRect.left) / 2),pictureRect.top);
	LineTo(pictureRect.right,pictureRect.bottom);
	ClosePoly();

	PenPat(&qd.black);
	ForeColor(redColor);
	PaintPoly(trianglePoly);
	KillPoly(trianglePoly);

	ForeColor(blackColor);
	TextSize(30);
	TextFont(systemFont);
	MoveTo(115,230);
	DrawString("\pRecorded Picture");
	ForeColor(whiteColor);
	MoveTo(112,227);
	DrawString("\pRecorded Picture");

	ClosePicture();

	DrawPicture(pictureHdl,&pictureRect);

	SetWTitle(gWindowPtr,"\pClick Mouse for Picture Information");

	while(!Button()) ;

	FillRect(&(gWindowPtr->portRect),&qd.white);
	SetWTitle(gWindowPtr,"\pOffscreen Graphics Worlds, Pictures and Cursors");

	TextFont(1);
	TextSize(10);

	GetPictInfo(pictureHdl,&pictInfo,returnPalette,8,systemMethod,0);
	ForeColor(blackColor);
	MoveTo(180,50);
	DrawString("\pSome Picture Information:");

	MoveTo(180,80);
	DrawString("\pTextStrings: ");
	NumToString(pictInfo.textCount,pictInfoString);
	DrawString(pictInfoString);

	MoveTo(180,95);
	DrawString("\pRectangles: ");
	NumToString(pictInfo.rectCount,pictInfoString);
	DrawString(pictInfoString);

	MoveTo(180,110);
	DrawString("\pRound Rectangles: ");
	NumToString(pictInfo.rRectCount,pictInfoString);
	DrawString(pictInfoString);

	MoveTo(180,125);
	DrawString("\pOvals: ");
	NumToString(pictInfo.ovalCount,pictInfoString);
	DrawString(pictInfoString);

	MoveTo(180,140);
	DrawString("\pArcs: ");
	NumToString(pictInfo.arcCount,pictInfoString);
	DrawString(pictInfoString);

	MoveTo(180,155);
	DrawString("\pPolygons: ");
	NumToString(pictInfo.polyCount,pictInfoString);
	DrawString(pictInfoString);

	MoveTo(180,170);
	DrawString("\pUnique Fonts: ");
	NumToString(pictInfo.uniqueFonts,pictInfoString);
	DrawString(pictInfoString);

	KillPicture(pictureHdl);

	TextFont(1);
	TextSize(10);
}

//  doCursor

void  doCursor(void)
{
	Rect		cursorRect;
	SInt16	a;

	BackColor(whiteColor);
	FillRect(&(gWindowPtr->portRect),&qd.white);

	cursorRect = gWindowPtr->portRect;
	PenPat(&qd.gray);
	PenSize(1,1);
	ForeColor(redColor);

	for(a=0;a<3;a++)
	{
		InsetRect(&cursorRect,40,40);
		FrameRect(&cursorRect);
	}

	MoveTo(10,20);
	DrawString("\pArrow cursor region");
	MoveTo(50,60);
	DrawString("\pIBeam cursor region");
	MoveTo(90,100);
	DrawString("\pCross cursor region");
	MoveTo(130,140);
	DrawString("\pPlus cursor region");

	gCursorRegionsActive = true;
	gCursorRegion = NewRgn();
}

//  changeCursor

void  changeCursor(WindowPtr gWindowPtr,RgnHandle cursorRegion)
{
	Rect			cursorRect;
	RgnHandle	arrowCursorRgn;
	RgnHandle	ibeamCursorRgn;
	RgnHandle	crossCursorRgn;
	RgnHandle	plusCursorRgn;
	Point			mousePosition;

	arrowCursorRgn = NewRgn();
	ibeamCursorRgn = NewRgn();	
	crossCursorRgn = NewRgn();
	plusCursorRgn	 = NewRgn();

	SetRectRgn(arrowCursorRgn,-32768,-32768,32766,32766);

	cursorRect = gWindowPtr->portRect;
	LocalToGlobal(&topLeft(cursorRect));
	LocalToGlobal(&botRight(cursorRect));	

	InsetRect(&cursorRect,40,40);
	RectRgn(ibeamCursorRgn,&cursorRect);
	DiffRgn(arrowCursorRgn,ibeamCursorRgn,arrowCursorRgn);

	InsetRect(&cursorRect,40,40);
	RectRgn(crossCursorRgn,&cursorRect);
	DiffRgn(ibeamCursorRgn,crossCursorRgn,ibeamCursorRgn);
	
	InsetRect(&cursorRect,40,40);
	RectRgn(plusCursorRgn,&cursorRect);
	DiffRgn(crossCursorRgn,plusCursorRgn,crossCursorRgn);

	GetMouse(&mousePosition);
	LocalToGlobal(&mousePosition);

	if(PtInRgn(mousePosition,ibeamCursorRgn))
	{
		SetCursor(*(GetCursor(iBeamCursor)));
		CopyRgn(ibeamCursorRgn,cursorRegion);
	}
	else if(PtInRgn(mousePosition,crossCursorRgn))
	{
		SetCursor(*(GetCursor(crossCursor)));
		CopyRgn(crossCursorRgn,cursorRegion);
	}
	else if(PtInRgn(mousePosition,plusCursorRgn))
	{
		SetCursor(*(GetCursor(plusCursor)));
		CopyRgn(plusCursorRgn,cursorRegion);
	}
	else
	{
		SetCursor(&qd.arrow);
		CopyRgn(arrowCursorRgn,cursorRegion);
	}

	DisposeRgn(arrowCursorRgn);
	DisposeRgn(ibeamCursorRgn);
	DisposeRgn(crossCursorRgn);
	DisposeRgn(plusCursorRgn);
}

//  doAnimCursor

void  doAnimCursor(void)
{
	SInt16		animCursResourceID, animCursTickInterval;

	BackColor(whiteColor);
	FillRect(&(gWindowPtr->portRect),&qd.white);

	animCursResourceID	 = rBeachBallCursor;
	animCursTickInterval = kBeachBallTickInterval;

	if(getAnimCursor(animCursResourceID,animCursTickInterval))
	{
		gAnimCursActive = true;
		gSleepTime = animCursTickInterval;
	}	
	else
		SysBeep(10);
}

//  getAnimCursor

Boolean  getAnimCursor(SInt16 resourceID,SInt16 tickInterval)
{
	SInt16	cursorID, a = 0;
	Boolean	noError = false;

	if((gAnimCursHdl = (animCursHandle) GetResource('acur',resourceID)))
	{
		noError = true;
		while((a < (*gAnimCursHdl)->numberOfFrames)  && noError)
		{
			cursorID = (SInt16) HiWord((SInt32) (*gAnimCursHdl)->frame[a]);
			(*gAnimCursHdl)->frame[a] = GetCursor(cursorID);
			if((*gAnimCursHdl)->frame[a])
				a++;
			else
				noError = false;
		}
	}

	if(noError)
	{
		gAnimCursTickInterval = tickInterval;
		gAnimCursLastTick = TickCount();
		(*gAnimCursHdl)->whichFrame = 0;
	}

	return(noError);
}

//  spinAnimCursor

void  spinAnimCursor(void)
{
	register SInt32	newTick;

	newTick = TickCount();
	if(newTick < (gAnimCursLastTick + gAnimCursTickInterval))
		return;

	SetCursor(*((*gAnimCursHdl)->frame[(*gAnimCursHdl)->whichFrame++]));
	if((*gAnimCursHdl)->whichFrame == (*gAnimCursHdl)->numberOfFrames)
		(*gAnimCursHdl)->whichFrame = 0;

	gAnimCursLastTick = newTick;
}

//  releaseAnimCursor

void  releaseAnimCursor(void)
{
	SInt16 a;

	for(a=0;a<(*gAnimCursHdl)->numberOfFrames;a++)
		ReleaseResource((Handle) (*gAnimCursHdl)->frame[a]);

	ReleaseResource((Handle) gAnimCursHdl);
}

//  doIcon

void  doIcon(void)
{
	OSErr				osErr;
	SInt32			response, finalTicks;
	SInt16			a;
	Rect				theRect;
	Handle			iconHdl;
	CIconHandle	cIconHdl;

	BackColor(whiteColor);
	FillRect(&(gWindowPtr->portRect),&qd.white);

	SetRect(&theRect,2,130,34,162);
	
	osErr = Gestalt(gestaltQuickdrawVersion,&response);
	if(response < gestalt8BitQD)
	{
		iconHdl = GetIcon(rIcon);
		for(a=1;a<20;a++)
		{
			PlotIcon(&theRect,iconHdl);	
			InsetRect(&theRect,a*-1,a*-2);
			OffsetRect(&theRect,a*4,0);
			Delay(20,&finalTicks);
		}
	}
	else
	{
		cIconHdl = GetCIcon(rIcon);
			
		for(a=1;a<20;a++)
		{
			PlotCIcon(&theRect,cIconHdl);	
			InsetRect(&theRect,a*-1,a*-2);
			OffsetRect(&theRect,a*4,0);
			Delay(20,&finalTicks);
		}

		DisposeCIcon(cIconHdl);
	}
}

// 
