// 
// Printing.c
// 
// 
// This program:
//
// 	Opens a window in which the contents of the main fields in the TPrint, TPrJob,
//		TPrStl and TPrInfo records are displayed.
//
// 	Allows the user to note changes in these fields after invoking the style dialog
//		and job dialog boxes.
//
// 	Allows the user to print a simulated document.
//
// 	Quits when the user chooses Quit or clicks the window's close box.
//
// The program utilises the following resources:
//
// 	'MBAR' resource and associated 'MENU' resources (preload, non-purgeable).
//
// 	A 'WIND' resource (purgeable).
// 
// 	An 'ALRT' resource and associated 'DITL' resource for an alert which reports
//		printing errors (purgeable).
//
// 	A 'TEXT' resource (non-purgeable) used for printing.
//
// 	A 'PICT' resource (non-purgeable) used for printing.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <Resources.h>
#include <Printing.h>

//  defines

#define mApple			128
#define mFile				129
#define  iQuit			11
#define  iPageSetup	8
#define  iPrint			9
#define rMenubar		128
#define rWindow			128
#define rText				128
#define rPicture		128
#define rPrintAlert	128
#define kMargin			90
#define MAXLONG			0x7FFFFFFF

//  global variables

THPrint		gTPrintHdl;
WindowPtr	gWindowPtr;
Boolean		gDone;
Boolean		gPrintRecordInited = false;
Boolean		gInhibitPrintRecordsInfo = false;
TEHandle	gEditRecHdl;
Handle		gTextHdl;
PicHandle	gPictureHdl;

//  function prototypes

void		main									(void);
void		doInitManagers				(void);
void		doEvents							(EventRecord *);
void		doMouseDown						(EventRecord *);
void		doActivateWindow			(void);
void		doMenuChoice					(long);
void		printLoop							(void);
OSErr		doCreatePrintRecord		(void);
void		doPrStyleDialog				(void);
SInt16	doCalcNumberOfPages		(Rect);
void		doDrawPage						(Rect,SInt16,SInt16);
void		doPrintRecordsInfo		(void);
void		doDrawRectStrings			(Str255,SInt16,SInt16,Str255,SInt16,SInt16,Str255);
void 		doDrawPageOrientation	(void);
SInt16	doGetPageOrientation	(void);
Boolean doIsPrGeneralThere		(void);
void		doPrintError					(SInt16,Boolean);

//  main

void  main(void)
{
	Handle			menubarHdl;
	MenuHandle	menuHdl;
	EventRecord	eventRec;
	Boolean			gotEvent;

	//  initialise managers

	doInitManagers();

	//  set up menu bar and menus

	if(!(menubarHdl = GetNewMBar(rMenubar)))
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	if(!(menuHdl = GetMenuHandle(mApple)))
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');

	//  open window

	if(!(gWindowPtr = GetNewWindow(rWindow,NULL,(WindowPtr)-1)))
		ExitToShell();

	SetPort(gWindowPtr);
	TextSize(10);

	//  load 'TEXT' and 'PICT' resources
	
	gTextHdl = GetResource('TEXT',rText);
	if(gTextHdl == NULL)
		ExitToShell();

	gPictureHdl = GetPicture(rPicture);
	if(gPictureHdl == NULL)
		ExitToShell();

	//  event loop

	gDone = false;
	
	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventRec,MAXLONG,NULL);
		if(gotEvent)
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();	
	FlushEvents(everyEvent,0);
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt8			charCode;

	windowPtr = (WindowPtr) eventRecPtr->message;
	
	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
				doMenuChoice(MenuKey(charCode));
			break;

		case updateEvt:
			BeginUpdate(windowPtr);
			EndUpdate(windowPtr);
			break;

		case activateEvt:
			doActivateWindow();
			break;
	}
}
	
//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);

	switch(partCode)
	{
		case inMenuBar:
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where) == true)
				gDone = true;
			break;
	}
}

//  doActivateWindow

void  doActivateWindow(void)
{
	if(FrontWindow() == gWindowPtr && gPrintRecordInited && !gInhibitPrintRecordsInfo)
		doPrintRecordsInfo();	
}

//  doMenuChoice

void  doMenuChoice(long menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
			daDriverRefNum = OpenDeskAcc(itemName);
			break;
			
		case mFile:
			if(menuItem == iPageSetup)
			{
				gInhibitPrintRecordsInfo = false;
				doPrStyleDialog();
			}
			else if(menuItem == iPrint)
				printLoop();			
			else if(menuItem == iQuit)
				gDone = true;
			break;
	}

	HiliteMenu(0);
}

//  printLoop

void  printLoop(void)
{
	GrafPtr		oldPort;
	SInt16		printError;
	SInt16		numberOfPages, numberOfCopies;
	Boolean		userClickedOK;
	SInt16		firstPage, lastPage, copy, page;
	TPPrPort	printPortPtr;
	TPrStatus	tprStatus;

	GetPort(&oldPort);

	PrOpen();
	if(PrError() == noErr)
	{
		if(!gPrintRecordInited)
			printError = doCreatePrintRecord();
		else
			printError = noErr;
			
		if(printError == noErr)
		{
			numberOfPages = doCalcNumberOfPages((*gTPrintHdl)->prInfo.rPage);

			userClickedOK = PrJobDialog(gTPrintHdl);
			if(userClickedOK)
			{				
				doPrintRecordsInfo();
				doDrawPageOrientation();
				gInhibitPrintRecordsInfo = true;
				
				numberOfCopies = (*gTPrintHdl)->prJob.iCopies;
				firstPage = (*gTPrintHdl)->prJob.iFstPage;
				lastPage = (*gTPrintHdl)->prJob.iLstPage;

				(*gTPrintHdl)->prJob.iFstPage = 1;
				(*gTPrintHdl)->prJob.iLstPage = iPrPgMax;

				if(numberOfPages < lastPage)
					lastPage = numberOfPages;
				
				for(copy=1;copy<numberOfCopies+1;copy++)
				{					
					for(page=firstPage;page<lastPage+1;page++)
					{
						if((page - firstPage) % iPFMaxPgs == 0)
						{	
							if(page != firstPage)
							{
								PrCloseDoc(printPortPtr);
								
								if(((*gTPrintHdl)->prJob.bJDocLoop == bSpoolLoop) && (PrError() == noErr))
									PrPicFile(gTPrintHdl,NULL,NULL,NULL,&tprStatus);
							}
							printPortPtr = PrOpenDoc(gTPrintHdl,NULL,NULL);
						}	
						if(PrError() == noErr)
						{
							PrOpenPage(printPortPtr,NULL);
							if(PrError() == noErr)
								doDrawPage((*gTPrintHdl)->prInfo.rPage,page,numberOfPages);
							PrClosePage(printPortPtr);
						}
					}

					PrCloseDoc(printPortPtr);

					if(((*gTPrintHdl)->prJob.bJDocLoop == bSpoolLoop) && (PrError() == noErr))
						PrPicFile(gTPrintHdl,NULL,NULL,NULL,&tprStatus);
				}
			}
		}
	}
	
	printError = PrError();

	PrClose();

	if(printError != noErr && printError != iPrAbort)
		doPrintError(printError,false);

	SetPort(oldPort);
	doActivateWindow();
}

//  doCreatePrintRecord

OSErr  doCreatePrintRecord(void)
{
	SInt16	printError;

	gTPrintHdl = (THPrint) NewHandleClear(sizeof(TPrint));
	if(gTPrintHdl != NULL )
	{
		PrintDefault(gTPrintHdl);
		printError = PrError();
		if(printError == noErr)
			gPrintRecordInited = true;
		return(printError);
	}
	else
		ExitToShell();
}

//  doPrStyleDialog

void  doPrStyleDialog(void)
{
	SInt16  printError;

	PrOpen();

	printError = PrError();
	if(printError == noErr)
	{
		if(!gPrintRecordInited)
		{
			printError = doCreatePrintRecord();
			if(printError != noErr)
				doPrintError(printError,true);
		}

		PrStlDialog(gTPrintHdl);
	}
	else
		doPrintError(printError,false);

	PrClose();
}

//  doCalcNumberOfPages

SInt16  doCalcNumberOfPages(Rect pageRect)
{
	Rect		destRect, pictureRect;
	SInt16	fontNum, heightDestRect, linesPerPage, numberOfPages;

	EraseRect(&(gWindowPtr->portRect));	

	SetRect(&destRect,pageRect.left + kMargin,pageRect.top + (kMargin * 1.5), 
					pageRect.right - kMargin,pageRect.bottom - (kMargin * 1.5));
	OffsetRect(&destRect,- (kMargin - 5),- ((kMargin * 1.5) - 5));

	GetFNum("\pGeneva",&fontNum);
	TextFont(fontNum);
	TextSize(10);

	gEditRecHdl = TENew(&destRect,&destRect);
	TEInsert(*gTextHdl,GetHandleSize(gTextHdl),gEditRecHdl);

	heightDestRect = destRect.bottom - destRect.top;
	linesPerPage = heightDestRect / (*gEditRecHdl)->lineHeight; 
	numberOfPages = ((*gEditRecHdl)->nLines / linesPerPage) + 1;	

	SetRect(&pictureRect,destRect.left,destRect.top,
				destRect.left + ((*gPictureHdl)->picFrame.right - (*gPictureHdl)->picFrame.left),
				destRect.top + ((*gPictureHdl)->picFrame.bottom - (*gPictureHdl)->picFrame.top));	
	DrawPicture(gPictureHdl,&pictureRect);

	return(numberOfPages);
}

//  doDrawPage

void  doDrawPage(Rect pageRect,SInt16 pageNumber,SInt16 numberOfpages)
{
	Rect			destRect, pictureRect;
	SInt16		heightDestRect, linesPerPage, numberOfLines, fontNum;
	TEHandle	pageEditRecHdl;
	Handle		textHdl;
	SInt32		startOffset, endOffset;
	Str255		theString;
	
	SetRect(&destRect,pageRect.left + kMargin,pageRect.top + (kMargin * 1.5),
					pageRect.right - kMargin,pageRect.bottom - (kMargin * 1.5));

	heightDestRect = destRect.bottom - destRect.top;
	linesPerPage = heightDestRect / (*gEditRecHdl)->lineHeight; 
	numberOfLines = (*gEditRecHdl)->nLines;
	
	GetFNum("\pGeneva",&fontNum);
	TextFont(fontNum);
	TextSize(10);

	pageEditRecHdl = TENew(&destRect,&destRect);
	textHdl = (*gEditRecHdl)->hText;

	startOffset = (*gEditRecHdl)->lineStarts[(pageNumber - 1) * linesPerPage];
	if(pageNumber == numberOfpages)
		endOffset = (*gEditRecHdl)->lineStarts[numberOfLines];
	else
		endOffset = (*gEditRecHdl)->lineStarts[pageNumber * linesPerPage];

	HLock(textHdl);
	TEInsert(*textHdl + startOffset,endOffset - startOffset,pageEditRecHdl);
	HUnlock(textHdl);

	if(pageNumber == 1)
	{
		SetRect(&pictureRect,destRect.left,destRect.top,
				destRect.left + ((*gPictureHdl)->picFrame.right - (*gPictureHdl)->picFrame.left),
				destRect.top + ((*gPictureHdl)->picFrame.bottom - (*gPictureHdl)->picFrame.top));	

		DrawPicture(gPictureHdl,&pictureRect);
	}		

	MoveTo(destRect.left,pageRect.bottom - 25);
	NumToString((SInt32) pageNumber,theString);
	DrawString(theString);
}

//  doPrintRecordsInfo

void  doPrintRecordsInfo(void)
{
	Str255	s2, s3;

	EraseRect(&(gWindowPtr->portRect));	

	MoveTo(20,25);
	DrawString("\pFrom TPrint, TPrInfo and TPrStl records:");

	NumToString((long) (*gTPrintHdl)->rPaper.top,s2);
	NumToString((long) (*gTPrintHdl)->rPaper.left,s3);
	doDrawRectStrings("\pPaper Rectangle (top,left):",20,45,s2,190,45,s3);

	NumToString((long) (*gTPrintHdl)->rPaper.bottom,s2);
	NumToString((long) (*gTPrintHdl)->rPaper.right,s3);
	doDrawRectStrings("\pPaper Rectangle (bottom,right):",20,60,s2,190,60,s3);

	NumToString((long) ((*gTPrintHdl)->prInfo.rPage).top,s2);
	NumToString((long) ((*gTPrintHdl)->prInfo.rPage).left,s3);
	doDrawRectStrings("\pPage Rectangle (top,left):",20,75,s2,190,75,s3);

	NumToString((long) ((*gTPrintHdl)->prInfo.rPage).bottom,s2);
	NumToString((long) ((*gTPrintHdl)->prInfo.rPage).right,s3);
	doDrawRectStrings("\pPage Rectangle (bottom,right):",20,90,s2,190,90,s3);

	MoveTo(20,105);
	DrawString("\pFeed Type:");
	MoveTo(190,105);
	if((*gTPrintHdl)->prStl.feed == 0)
		DrawString("\pCut sheet");
	else if((*gTPrintHdl)->prStl.feed == 1)
		DrawString("\pFanfold");

	MoveTo(20,120);
	DrawString("\pVertical resolution:");
	NumToString((long) (*gTPrintHdl)->prInfo.iVRes,s2);
	MoveTo(190,120);
	DrawString(s2);

	MoveTo(20,135);
	DrawString("\pHorizontal resolution:");
	NumToString((long) (*gTPrintHdl)->prInfo.iHRes,s2);
	MoveTo(190,135);
	DrawString(s2);

	MoveTo(20,155);
	DrawString("\pFrom TPrJob Record:");

	MoveTo(20,175);
	DrawString("\pFirst Page:");
	NumToString((long) (*gTPrintHdl)->prJob.iFstPage,s2);
	MoveTo(190,175);
	DrawString(s2);

	MoveTo(20,190);
	DrawString("\pLast Page:");
	NumToString((long) (*gTPrintHdl)->prJob.iLstPage,s2);
	MoveTo(190,190);
	DrawString(s2);

	MoveTo(20,205);
	DrawString("\pNumber of Copies:");
	NumToString((long) (*gTPrintHdl)->prJob.iCopies,s2);
	MoveTo(190,205);
	DrawString(s2);

	MoveTo(20,225);
	DrawString("\pNote:  Some printer drivers always set the iCopies field of the TPrJob");
	MoveTo(20,240);
	DrawString("\precord to 1 and handle multiple copies internally.");
}

//  doDrawRectStrings

void  doDrawRectStrings(Str255 s1,SInt16 x1,SInt16 y1,Str255 s2,SInt16 x2,SInt16 y2,Str255 s3)
{
	MoveTo(x1,y1);
	DrawString(s1);
	MoveTo(x2,y2);
	DrawString("\p(");
	DrawString(s2);
	DrawString("\p,");
	DrawString(s3);
	DrawString("\p)");
}

//  drawPageOrientation

void  doDrawPageOrientation(void)
{
	SInt16	orientation;

	MoveTo(20,260);
	DrawString("\pOrientation selected:");

	orientation = doGetPageOrientation();

	MoveTo(190,260);		
	if(orientation == 1)
		DrawString("\pLandscape");
	else if(orientation == 2)
		DrawString("\pPortrait");
	else
		DrawString("\p(PrGeneral not supported by driver)");
}

//  doGetPageOrientation

SInt16  doGetPageOrientation(void)
{
	TGetRotnBlk getRotRec;

	if(doIsPrGeneralThere)
	{
		getRotRec.iOpCode = getRotnOp;
		getRotRec.hPrint = gTPrintHdl;
		PrGeneral((Ptr) &getRotRec);
		if((getRotRec.iError == noErr) && (PrError() == noErr) && getRotRec.fLandscape)
			return(1);
		else
			return(2);
	}
	else
		return(3);
}	

//  doIsPrGeneralThere

Boolean  doIsPrGeneralThere(void)
{
	TGetRotnBlk	getRotRec;
	OSErr				printError;

	printError = 0;
	getRotRec.iOpCode = getRotnOp;
	getRotRec.hPrint = gTPrintHdl;

	PrGeneral((Ptr) &getRotRec);

	printError = PrError();
	PrSetError(noErr);

	if(printError == resNotFound)
		return(false);
	else
		return(true);
}

//  doPrintError

void  doPrintError(SInt16 printError,Boolean fatal)
{
	Str255 errorNumberString;

	NumToString((long) printError,errorNumberString);
	ParamText(errorNumberString,NULL,NULL,NULL);
	if(fatal)
	{
		StopAlert(rPrintAlert,NULL);
		ExitToShell();
	}
	else
		CautionAlert(rPrintAlert,NULL);
}

// 
