// 
// HelpDialog.c
// 

//  includes

#include <Dialogs.h>
#include <ToolUtils.h>
#include <Resources.h>

//  defines

#define rHelpModal						129
#define  iOK									1
#define  iTextUserItem				2
#define  iScrollBar						3
#define  iPopupMenu						4
#define rErrorAlert						128
#define  eHelpDialog					9
#define  eHelpDocRecord				10
#define  eHelpText						11
#define  eHelpPicture					12
#define kTextInset						4
#define kReturn								0x0D
#define kEnter								0x03

#define	rTextIntroduction			128
#define rTextCreatingText			129
#define rTextModifyHelp				130
#define	rPictIntroductionBase	128
#define	rPictCreatingTextBase	129

//  typedefs

typedef struct
{
	Rect					bounds;
	PicHandle			pictureHdl;
} pictInfoRec;

typedef struct
{
	TEHandle			editRecHdl;
	ControlHandle	scrollbarHdl;
	SInt16				pictCount;
	pictInfoRec		*pictInfoRecPtr;
}	docRecord, ** docRecordHandle;

//  global variables

SInt16			gTextResourceID;
SInt16			gPictResourceBaseID;
RgnHandle		gSavedClipRgn = NULL;

//  function prototypes

void						doHelp							(void);
void						closeHelp						(DialogPtr,GrafPtr);
pascal void  		drawHelp						(DialogPtr,SInt16);
Boolean					getText							(DialogPtr,SInt16,Rect);
Boolean					getPictureInfo			(DialogPtr,SInt16);
void						handleScrollBar			(DialogPtr,SInt16,Point);
pascal void			actionProcedure			(ControlHandle,SInt16);
void						scrollTextAndPicts	(DialogPtr);
void 						drawPictures				(DialogPtr,Rect *);
pascal Boolean	helpDialogFilter		(DialogPtr,EventRecord *,SInt16 *);

extern void			doUpdate						(EventRecord *);
extern void			doErrorAlert				(SInt16);

//  doHelp

void  doHelp(void)
{
	DialogPtr				modalDlgPtr;
	docRecordHandle	docRecHdl;
	GrafPtr					oldPort;
	SInt16					itemType,itemHit, menuItem;
	Handle					itemHdl;
	Rect						userItemRect, destRect, viewRect, itemRect;

	if(!(modalDlgPtr = GetNewDialog(rHelpModal,NULL,(WindowPtr) -1)))
	{
		doErrorAlert(eHelpDialog);
		return;
	}

	if(!(docRecHdl = (docRecordHandle) NewHandle(sizeof(docRecord))))
	{
		doErrorAlert(eHelpDocRecord);
		DisposeDialog(modalDlgPtr);
		return;
	}
	
	SetWRefCon(modalDlgPtr,(SInt32) docRecHdl);

	GetPort(&oldPort);
	SetPort(modalDlgPtr);

	GetDialogItem(modalDlgPtr,iTextUserItem,&itemType,&itemHdl,&userItemRect);
	SetDialogItem(modalDlgPtr,iTextUserItem,itemType,(Handle) &drawHelp,&userItemRect);

	GetDialogItem(modalDlgPtr,iScrollBar,&itemType,&itemHdl,&itemRect);
	(*docRecHdl)->scrollbarHdl = (ControlHandle) itemHdl;

	InsetRect(&userItemRect,kTextInset,kTextInset / 2);
	destRect = viewRect = userItemRect;
	(*docRecHdl)->editRecHdl = TEStyleNew(&destRect,&viewRect);

	(*docRecHdl)->pictInfoRecPtr = NULL;
	
	gTextResourceID			= rTextIntroduction;
	gPictResourceBaseID	= rPictIntroductionBase;

	if(!(getText(modalDlgPtr,gTextResourceID,viewRect)))
	{
		closeHelp(modalDlgPtr,oldPort);
		return;
	}
	if(!(getPictureInfo(modalDlgPtr,gPictResourceBaseID)))
	{
		closeHelp(modalDlgPtr,oldPort);
		return;
	}
		
	gSavedClipRgn = NewRgn();

	ShowWindow(modalDlgPtr);

	do 
	{
		ModalDialog((ModalFilterUPP) &helpDialogFilter,&itemHit);
	
		if(itemHit == iPopupMenu)
		{
			SetControlValue((*docRecHdl)->scrollbarHdl,0);

			GetDialogItem(modalDlgPtr,iPopupMenu,&itemType,&itemHdl,&itemRect);
			menuItem = GetControlValue((ControlHandle) itemHdl);
			
			switch(menuItem)
			{
				case 1:
					gTextResourceID 		= rTextIntroduction;
					gPictResourceBaseID = rPictIntroductionBase;
					break;
				
				case 2:
					gTextResourceID 		= rTextCreatingText;
					gPictResourceBaseID = rPictCreatingTextBase;
					break;

				case 3:
					gTextResourceID 		= rTextModifyHelp;
					break;
			}
			
			if(!(getText(modalDlgPtr,gTextResourceID,viewRect)))
			{
				closeHelp(modalDlgPtr,oldPort);
				return;
			}
			if(!(getPictureInfo(modalDlgPtr,gPictResourceBaseID)))
			{
				closeHelp(modalDlgPtr,oldPort);
				return;
			}

			drawPictures(modalDlgPtr,&viewRect);
		}
			
	} while(itemHit != iOK);
		
	closeHelp(modalDlgPtr,oldPort);

	return;
}

//  closeHelp

void  closeHelp(DialogPtr modalDlgPtr,GrafPtr oldPort)
{
	docRecordHandle	docRecHdl;
	TEHandle				editRecHdl;
	SInt16					a;
	
	docRecHdl = (docRecordHandle) GetWRefCon(modalDlgPtr);
	editRecHdl = (*docRecHdl)->editRecHdl;

	if(gSavedClipRgn)
		DisposeRgn(gSavedClipRgn);

	if((*docRecHdl)->editRecHdl)
		TEDispose((*docRecHdl)->editRecHdl);

	if((*docRecHdl)->pictInfoRecPtr)
	{
		for(a=0;a<(*docRecHdl)->pictCount;a++)
			ReleaseResource((Handle) (*docRecHdl)->pictInfoRecPtr[a].pictureHdl);
		DisposePtr((Ptr) (*docRecHdl)->pictInfoRecPtr);
	}

	DisposeHandle((Handle) docRecHdl);
	DisposeDialog(modalDlgPtr);

	SetPort(oldPort);
}

//  drawHelp

pascal void  drawHelp(DialogPtr modalDlgPtr,SInt16 theItem)
{
	PenState				oldPenState;
	Handle					itemHdl;
	Rect						itemRect, viewRect;
	SInt16					itemType, buttonOval;
	docRecordHandle	docRecHdl;
	TEHandle				editRecHdl;

	GetPenState(&oldPenState);

	GetDialogItem(modalDlgPtr,iTextUserItem,&itemType,&itemHdl,&itemRect);
	InsetRect(&itemRect,1,1);

	BackColor(whiteColor);
	FillRect(&itemRect,&qd.white);
	InsetRect(&itemRect,-1,-1);
	FrameRect(&itemRect);

	TextFont(0);
	MoveTo(13,309);
	DrawString("\pTopic:");

	GetDialogItem(modalDlgPtr,iOK,&itemType,&itemHdl,&itemRect);
	InsetRect(&itemRect,-4,-4);

	buttonOval = (itemRect.bottom - itemRect.top) / 2 + 2;

	ForeColor(blackColor);
	PenPat(&qd.black);
	PenSize(3,3);
	FrameRoundRect(&itemRect,buttonOval,buttonOval);
	
	docRecHdl = (docRecordHandle) GetWRefCon(modalDlgPtr);
	editRecHdl = (*docRecHdl)->editRecHdl;
	viewRect = (*editRecHdl)->viewRect;

	TEUpdate(&viewRect,editRecHdl);
	drawPictures(modalDlgPtr,&viewRect);

	SetPenState(&oldPenState);
}

//  getText

Boolean  getText(DialogPtr modalDlgPtr,SInt16 textResourceID,Rect viewRect)
{
	docRecordHandle	docRecHdl;
	TEHandle				editRecHdl;
	Handle					helpTextHdl;
	StScrpHandle		stylScrpRecHdl;
	SInt16					numberOfLines, heightOfText, heightToScroll;

	docRecHdl = (docRecordHandle) GetWRefCon(modalDlgPtr);
	editRecHdl = (*docRecHdl)->editRecHdl;

	TESetSelect(0,32767,editRecHdl);
	TEDelete(editRecHdl);
	
	(*editRecHdl)->destRect = (*editRecHdl)->viewRect;
	SetControlValue((*docRecHdl)->scrollbarHdl,0);
			
	helpTextHdl = GetResource('TEXT',textResourceID);
	if(helpTextHdl == NULL)
	{
		doErrorAlert(eHelpText);
		return false;
	}

	stylScrpRecHdl = (StScrpHandle) GetResource('styl',textResourceID);
	if(stylScrpRecHdl == NULL)
	{
		doErrorAlert(eHelpText);
		return false;
	}
		
	TEStyleInsert(*helpTextHdl,GetHandleSize(helpTextHdl),stylScrpRecHdl,editRecHdl);

	ReleaseResource(helpTextHdl);
	ReleaseResource((Handle) stylScrpRecHdl);

	numberOfLines = (*editRecHdl)->nLines;
	heightOfText = TEGetHeight((SInt32) numberOfLines,1,editRecHdl);

	if(heightOfText >  (viewRect.bottom - viewRect.top))
	{
		heightToScroll = TEGetHeight((SInt32) numberOfLines,1,editRecHdl) -
																 (viewRect.bottom - viewRect.top);
		SetControlMaximum((*docRecHdl)->scrollbarHdl,heightToScroll);
		HiliteControl((*docRecHdl)->scrollbarHdl,0);
	}
	else
	{
		HiliteControl((*docRecHdl)->scrollbarHdl,255);
	}

	return true;
}

//  getPictureInfo

Boolean  getPictureInfo(DialogPtr modalDlgPtr,SInt16 firstPictID)
{
	docRecordHandle	docRecHdl;
	TEHandle				editRecHdl;
	Handle					textHdl;
	SInt32					offset, textSize;
	SInt16					numberOfPicts, a, lineHeight, fontAscent;
	SInt8						optionSpace[1] = "\xCA";
	pictInfoRec			*pictInfoPtr;
	Point						picturePoint;
	TextStyle				whatStyle;
	
	docRecHdl = (docRecordHandle) GetWRefCon(modalDlgPtr);

	if((*docRecHdl)->pictInfoRecPtr != NULL)
	{
		for(a=0;a<(*docRecHdl)->pictCount;a++)
			ReleaseResource((Handle) (*docRecHdl)->pictInfoRecPtr[a].pictureHdl);

		DisposePtr((Ptr) (*docRecHdl)->pictInfoRecPtr);
		(*docRecHdl)->pictInfoRecPtr = NULL;
	}

	(*docRecHdl)->pictCount = 0;

	editRecHdl = (*docRecHdl)->editRecHdl;
	textHdl = (*editRecHdl)->hText;

	textSize = GetHandleSize(textHdl);
	offset = 0;
	numberOfPicts = 0;

	HLock(textHdl);

	offset = Munger(textHdl,offset,optionSpace,1,NULL,0);
	while((offset >= 0) && (offset <= textSize))
	{
		numberOfPicts++;
		offset++;
		offset = Munger(textHdl,offset,optionSpace,1,NULL,0);		
	}
	
	if(numberOfPicts == 0)
	{
		HUnlock(textHdl);
		return true;
	}
		
	pictInfoPtr = (pictInfoRec *) NewPtr(sizeof(pictInfoRec) * numberOfPicts);
	(*docRecHdl)->pictInfoRecPtr = pictInfoPtr;

	offset = 0L;

	for(a=0;a<numberOfPicts;a++)
	{
		pictInfoPtr[a].pictureHdl = GetPicture(firstPictID + a);
		if(pictInfoPtr[a].pictureHdl == NULL)
		{
			doErrorAlert(eHelpPicture);
			return false;
		}

		offset = Munger(textHdl,offset,optionSpace,1,NULL,0);
		picturePoint = TEGetPoint((SInt16)offset,editRecHdl);

		TEGetStyle(offset,&whatStyle,&lineHeight,&fontAscent,editRecHdl);
		picturePoint.v -= lineHeight;
		offset++;
		pictInfoPtr[a].bounds = (**pictInfoPtr[a].pictureHdl).picFrame;

		OffsetRect(&pictInfoPtr[a].bounds,
							 (((*editRecHdl)->destRect.right + (*editRecHdl)->destRect.left) -
							 (pictInfoPtr[a].bounds.right + pictInfoPtr[a].bounds.left) ) / 2,
							 - pictInfoPtr[a].bounds.top + picturePoint.v);
	}
	
	(*docRecHdl)->pictCount = a;
	
	HUnlock(textHdl);

	return true;
}

//  handleScrollBar

void  handleScrollBar(DialogPtr modalDlgPtr,SInt16 thePart,Point mouseXY)
{
	docRecordHandle		docRecHdl;

	docRecHdl = (docRecordHandle) GetWRefCon(modalDlgPtr);

	if(thePart == kControlIndicatorPart)
	{
		if(TrackControl((*docRecHdl)->scrollbarHdl,mouseXY,NULL))
			scrollTextAndPicts(modalDlgPtr);
	}
	else
		TrackControl((*docRecHdl)->scrollbarHdl,mouseXY,(ControlActionUPP) &actionProcedure);	
}

//  actionProcedure

pascal void  actionProcedure(ControlHandle scrollbarHdl,SInt16 partCode)
{
	docRecordHandle	docRecHdl;
	DialogPtr				modalDlgPtr;
	TEHandle				editRecHdl;
	SInt16					delta, oldValue, offset, lineHeight, fontAscent;
	Point						thePoint;
	Rect						viewRect;
	TextStyle				style;

	if(partCode)
	{
		modalDlgPtr = (*scrollbarHdl)->contrlOwner;
		docRecHdl = (docRecordHandle) GetWRefCon(modalDlgPtr);
		editRecHdl = (*docRecHdl)->editRecHdl;
		viewRect = (*editRecHdl)->viewRect;
		thePoint.h = viewRect.left + kTextInset;
		
		switch(partCode)
		{
			case kControlUpButtonPart:
				thePoint.v = viewRect.top - 4;
				offset = TEGetOffset(thePoint,editRecHdl);
				thePoint = TEGetPoint(offset,editRecHdl);
				TEGetStyle(offset,&style,&lineHeight,&fontAscent,editRecHdl);
				delta = thePoint.v - lineHeight - viewRect.top;
				break;

			case kControlDownButtonPart:
				thePoint.v = viewRect.bottom + 2;
				offset = TEGetOffset(thePoint,editRecHdl);
				thePoint = TEGetPoint(offset,editRecHdl);
				delta = thePoint.v - viewRect.bottom;
				break;

			case kControlPageUpPart:
				thePoint.v = viewRect.top + 2;
				offset = TEGetOffset(thePoint,editRecHdl);
				thePoint = TEGetPoint(offset,editRecHdl);
				TEGetStyle(offset,&style,&lineHeight,&fontAscent,editRecHdl);
				thePoint.v += lineHeight - fontAscent;
				thePoint.v -= viewRect.bottom - viewRect.top;
				offset = TEGetOffset(thePoint,editRecHdl);
				thePoint = TEGetPoint(offset,editRecHdl);
				TEGetStyle(offset,&style,&lineHeight,&fontAscent,editRecHdl);
				delta = thePoint.v - viewRect.top;
				if(offset == 0)
					delta -= lineHeight;
				break;
				
			case kControlPageDownPart:
				thePoint.v = viewRect.bottom - 2;
				offset = TEGetOffset(thePoint,editRecHdl);
				thePoint = TEGetPoint(offset,editRecHdl);
				TEGetStyle(offset,&style,&lineHeight,&fontAscent,editRecHdl);
				thePoint.v -= fontAscent;
				thePoint.v += viewRect.bottom - viewRect.top;
				offset = TEGetOffset(thePoint,editRecHdl);
				thePoint = TEGetPoint(offset,editRecHdl);
				TEGetStyle(offset,&style,&lineHeight,&fontAscent,editRecHdl);
				delta =  thePoint.v - lineHeight - viewRect.bottom;
				if(offset == (**editRecHdl).teLength)
					delta += lineHeight;
				break;
		}

		oldValue = GetControlValue(scrollbarHdl);

		if(((delta < 0) && (oldValue > 0)) || ((delta > 0) && 
			 (oldValue < GetControlMaximum(scrollbarHdl))))
		{
 			GetClip(gSavedClipRgn);
			ClipRect(&modalDlgPtr->portRect);

			SetControlValue(scrollbarHdl,oldValue + delta);
			SetClip(gSavedClipRgn);
		}

		scrollTextAndPicts(modalDlgPtr);
	}
}

//  scrollTextAndPicts

void  scrollTextAndPicts(DialogPtr modalDlgPtr)
{
	docRecordHandle	docRecHdl;
	TEHandle				editRecHdl;
	SInt16					scrollDistance, oldScroll;
	Rect						updateRect;

	docRecHdl = (docRecordHandle) GetWRefCon(modalDlgPtr);
	editRecHdl = (*docRecHdl)->editRecHdl;

	oldScroll = (*editRecHdl)->viewRect.top -(**editRecHdl).destRect.top;
	scrollDistance = oldScroll - GetControlValue((*docRecHdl)->scrollbarHdl);
	if(scrollDistance == 0)
		return;

	TEScroll(0,scrollDistance,editRecHdl);

	if((*docRecHdl)->pictCount == 0)
		return;
		
	updateRect = (*editRecHdl)->viewRect;

	if(scrollDistance > 0)
	{
		if(scrollDistance < (updateRect.bottom - updateRect.top))
			updateRect.bottom = updateRect.top + scrollDistance;
	}
	else
	{
		if( - scrollDistance < (updateRect.bottom - updateRect.top))
			updateRect.top = updateRect.bottom + scrollDistance;
	}

	drawPictures(modalDlgPtr,&updateRect);
}

//  drawPictures

void  drawPictures(DialogPtr modalDlgPtr,Rect *updateRect)
{
	docRecordHandle	docRecHdl;
	TEHandle				editRecHdl;
	SInt16					pictCount, pictIndex, vOffset;
	PicHandle				thePictHdl;
	Rect						pictLocRect, dummyRect;
	
	docRecHdl = (docRecordHandle) GetWRefCon(modalDlgPtr);
	editRecHdl = (*docRecHdl)->editRecHdl;

	vOffset = (*editRecHdl)->destRect.top - (*editRecHdl)->viewRect.top - kTextInset;
	pictCount = (*docRecHdl)->pictCount;

	for(pictIndex = 0;pictIndex < pictCount;pictIndex++)
	{
		pictLocRect = (*docRecHdl)->pictInfoRecPtr[pictIndex].bounds;
    OffsetRect(&pictLocRect,0,vOffset);

		if(!SectRect(&pictLocRect,updateRect,&dummyRect))
			continue;

		thePictHdl = (*docRecHdl)->pictInfoRecPtr[pictIndex].pictureHdl;

		LoadResource((Handle) thePictHdl);
		HLock((Handle) thePictHdl);

		GetClip(gSavedClipRgn);
		ClipRect(updateRect);
		DrawPicture(thePictHdl,&pictLocRect);

		SetClip(gSavedClipRgn);
		HUnlock((Handle) thePictHdl);
	}
}

//  helpDialogFilter

pascal  Boolean helpDialogFilter(DialogPtr modalDlgPtr, EventRecord	*eventRecPtr,
																 SInt16 *itemHit)
{
	docRecordHandle	docRecHdl;
	SInt16					handledEvent, itemType, thePart;
	SInt8						charCode;
	Rect						itemRect;
	Handle					itemHdl;
	Point						mouseXY;
	SInt32					finalTicks;
	ControlHandle		controlHdl;

	handledEvent = false;

	if((eventRecPtr->what == updateEvt) && ((WindowPtr) eventRecPtr->message != modalDlgPtr))
	{
		doUpdate(eventRecPtr);
	}
	else
	{
		switch(eventRecPtr->what)
		{
			case keyDown:
			case autoKey:
				charCode = eventRecPtr->message & charCodeMask;
				if((charCode == (SInt8) kReturn) || (charCode == (SInt8) kEnter))
				{
					GetDialogItem(modalDlgPtr,iOK,&itemType,&itemHdl,&itemRect);
					HiliteControl((ControlHandle) itemHdl,inButton);
					Delay(10,&finalTicks);
					HiliteControl((ControlHandle) itemHdl,0);
					handledEvent = true;
					*itemHit = iOK;
				}
				break;
			
			case mouseDown:
				mouseXY = eventRecPtr->where;
				GlobalToLocal(&mouseXY);
				thePart = FindControl(mouseXY,modalDlgPtr,&controlHdl);
				docRecHdl = (docRecordHandle) GetWRefCon(modalDlgPtr);
				if(controlHdl == (*docRecHdl)->scrollbarHdl)
				{
					handleScrollBar(modalDlgPtr,thePart,mouseXY);
					*itemHit = iScrollBar;
					handledEvent = true;
				}	
				break;
		}
	}

	return handledEvent;
}

// 


