// 
// Text2.c
// 
//
// This program demonstrates:
//
// 	The use of TextEdit to enter data into the various fields of a database record.
//
// 	The formatting and display of dates, times and numbers.
//
// The program opens a window in which four data entry fields are displayed.  A separate
// TextEdit edit record is associated with each field.  The first field requires text to
// be entered, the second an integer value, the third a floating point value (that is, a
// currency value), and the fourth a date.
//
// The user may select a field for data entry using the mouse or by cycling between
// fields using the Tab key.  When the Return key is pressed, some arithmetic is
// performed using the integer and floating point values, six months are added to the
// date, and the resulting data is formatted and displayed at the bottom of the window.
//
// The current date and time is displayed at the top of the window.
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, and Edit menus (preload,
//		non-purgeable).  
//
// 	A 'WIND' resource (purgeable) (initially visible).  
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch, and
//		is32BitCompatible flags set.	  
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <LowMem.h>

//  defines

#define mApple						128
#define  iAbout						1
#define mFile							129
#define  iQuit						12
#define mEdit							130		
#define  iCut							3
#define  iCopy						4
#define  iPaste						5
#define  iClear						6
#define  iSelectAll				7
#define kTab							0x09
#define kDel							0x7F
#define kReturn						0x0D
#define kMaxCharasItem		20
#define kMaxCharasQuant		10
#define kMaxCharasValue		12
#define kMaxCharasDate		13
#define rWindow						128
#define rMenubar					128

#define topLeft(r)				(((Point *) &(r))[0])
#define botRight(r)				(((Point *) &(r))[1])

//  typedefs

typedef struct
{
	TEHandle	itemEditHdl;
	TEHandle	quantEditHdl;
	TEHandle	valueEditHdl;
	TEHandle	dateEditHdl;
} DocRec, **DocRecHandle;

//  global variables

Boolean		gDone;
Boolean		gInBackground;
RgnHandle	gCursorRegion;
TEHandle	gCurrentEditRecHdl;
SInt16		gMaxCharasThisField;
Rect			gItemRect		= { 45,92, 62, 242 };
Rect			gQuantRect	= { 68,92, 85, 242 };
Rect			gValueRect	= { 91,92, 108,242 };
Rect			gDateRect		= { 114,92,131,242 };

//  function prototypes

void	main										(void);
void	doInitManagers					(void);
void	doSetUpEditRecords			(WindowPtr);
void	eventLoop								(void);
void	doIdle									(void);
void	doEvents								(EventRecord *);
void	doKeyEvent							(SInt8);
void	doHandleTabKey					(void);
void	doHandleDelKey					(void);
void	doMouseDown							(EventRecord *);
void	doInContent							(EventRecord *);
void	doUpdate								(EventRecord *);
void	doActivate							(EventRecord *);
void	doChangeCurrentEditRec	(TEHandle,Boolean,Point,SInt16);
void	doTodaysDate						(void);
void	doAcceptNewRecord				(void);
void	doAcceptItemField				(TEHandle);
void	doAcceptQuantField			(TEHandle);
void	doAcceptValueField			(TEHandle,TEHandle);
void	doAcceptDateField				(TEHandle);
void	doAdjustMenus						(void);
void	doMenuChoice						(SInt32);
void	doEditMenu							(SInt16);
void	doAdjustCursor					(WindowPtr,RgnHandle);
void	doDrawWindow						(void);
void	doEraseRecordDisplay		(void);
	
//  main

void  main(void)
{
	Handle				menubarHdl;
	MenuHandle		menuHdl;
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;

	// initialise managers

	doInitManagers();

	//  set up menu bar and menus

	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');

	//  open window and attach document record

	if(!(windowPtr = GetNewWindow(rWindow,NULL,(WindowPtr) -1)))
		ExitToShell();

	if(!(docRecHdl = (DocRecHandle) NewHandle(sizeof(DocRec))))
		ExitToShell();

	SetWRefCon(windowPtr,(SInt32) docRecHdl);
	SetPort(windowPtr);

	TextSize(10);
	TextFont(geneva);

	//  set up edit records

	doSetUpEditRecords(windowPtr);

	//  enter eventLoop

	eventLoop();
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	InitCursor();

	FlushEvents(everyEvent,0);
}

//  doSetUpEditRecords

void  doSetUpEditRecords(WindowPtr windowPtr)
{
	Rect					aRect;
	DocRecHandle	docRecHdl;

	docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));;

	aRect = gItemRect;
	InsetRect(&aRect,2,2);
	(*docRecHdl)->itemEditHdl = TENew(&aRect,&aRect);

	aRect = gQuantRect;
	InsetRect(&aRect,2,2);
	(*docRecHdl)->quantEditHdl = TENew(&aRect,&aRect);

	aRect = gValueRect;
	InsetRect(&aRect,2,2);
	(*docRecHdl)->valueEditHdl = TENew(&aRect,&aRect);

	aRect = gDateRect;
	InsetRect(&aRect,2,2);
	(*docRecHdl)->dateEditHdl = TENew(&aRect,&aRect);

	gCurrentEditRecHdl = (*docRecHdl)->itemEditHdl;
	gMaxCharasThisField = kMaxCharasItem;
}

//  eventLoop

void  eventLoop(void)
{
	EventRecord	eventRec;
	Boolean			gotEvent;
	SInt32			sleepTime;

	gDone = false;
	gCursorRegion = NewRgn();
	sleepTime = LMGetCaretTime();

	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventRec,sleepTime,gCursorRegion);

		if(!gInBackground)
			doAdjustCursor(FrontWindow(),gCursorRegion);

		if(gotEvent)
			doEvents(&eventRec);
		else
			doIdle();
	}
}

//  doIdle

void  doIdle(void)
{
	static UInt32		oldRawSeconds;
	UInt32					rawSeconds;
	Str255					timeString;
	Rect						eraseRect;

	if(gCurrentEditRecHdl != NULL)
		TEIdle(gCurrentEditRecHdl);

	GetDateTime(&rawSeconds);
	if(rawSeconds > oldRawSeconds) 
	{
		TimeString(rawSeconds,true,timeString,NULL);
		MoveTo(285,22);
		SetRect(&eraseRect,285,12,335,22);
		EraseRect(&eraseRect);
		DrawString(timeString);
		oldRawSeconds = rawSeconds;
	}
}

//  doEvent

void	doEvents(EventRecord *eventRecPtr)
{
	SInt8		charCode;

	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
			{
				doAdjustMenus();
				doMenuChoice(MenuKey(charCode));
			}
			else
				doKeyEvent(charCode);
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case activateEvt:
			doActivate(eventRecPtr);
			break;

		case osEvt:
			if(((eventRecPtr->message >> 24) & 0x000000FF) == suspendResumeMessage)
				gInBackground = (eventRecPtr->message & resumeFlag) == 0;
			break;
	}
}

//  doKeyEvent

void  doKeyEvent(SInt8 charCode)
{
	if(charCode == kTab)
		doHandleTabKey();
	else if(charCode == kDel)
		doHandleDelKey();
	else if(charCode == kReturn)
		doAcceptNewRecord();	
	else
	{
		if(((*gCurrentEditRecHdl)->teLength < gMaxCharasThisField) || (charCode < 0x20))
			TEKey(charCode,gCurrentEditRecHdl);
		else
			SysBeep(10);
	}
}

//  doHandleTabKey

void  doHandleTabKey(void)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	Point					dummy;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) (GetWRefCon(windowPtr));;

	if(gCurrentEditRecHdl == (*docRecHdl)->itemEditHdl)
		doChangeCurrentEditRec((*docRecHdl)->quantEditHdl,false,dummy,kMaxCharasQuant);
	else if(gCurrentEditRecHdl == (*docRecHdl)->quantEditHdl)
		doChangeCurrentEditRec((*docRecHdl)->valueEditHdl,false,dummy,kMaxCharasValue);	
	else if(gCurrentEditRecHdl == (*docRecHdl)->valueEditHdl)
		doChangeCurrentEditRec((*docRecHdl)->dateEditHdl,false,dummy,kMaxCharasDate);	
	else if(gCurrentEditRecHdl == (*docRecHdl)->dateEditHdl)
		doChangeCurrentEditRec((*docRecHdl)->itemEditHdl,false,dummy,kMaxCharasItem);	
}

//  doHandleDelKey

void  doHandleDelKey(void)
{
	SInt16	selectionLength;
		
	selectionLength = (*gCurrentEditRecHdl)->selEnd - (*gCurrentEditRecHdl)->selStart;
	if(selectionLength == 0)
		(*gCurrentEditRecHdl)->selEnd += 1;
	TEDelete(gCurrentEditRecHdl);
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);
	
	switch(partCode)
	{
		case inMenuBar:
			doAdjustMenus();
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			else
				doInContent(eventRecPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where))
				gDone = true;
			break;
	}
}

//  doInContent

void  doInContent(EventRecord *eventRecPtr)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	Point					mouseXY;
	Boolean				shiftKeyPosition;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);

	mouseXY = eventRecPtr->where;
	GlobalToLocal(&mouseXY);

	shiftKeyPosition = (eventRecPtr->modifiers & shiftKey) != 0;

	if(PtInRect(mouseXY,&gItemRect))
	{
		if(gCurrentEditRecHdl == (*docRecHdl)->itemEditHdl)
			TEClick(mouseXY,shiftKeyPosition,(*docRecHdl)->itemEditHdl);
		else
			doChangeCurrentEditRec((*docRecHdl)->itemEditHdl,true,mouseXY,kMaxCharasItem);
	}
	else if(PtInRect(mouseXY,&gQuantRect))
	{
		if(gCurrentEditRecHdl == (*docRecHdl)->quantEditHdl)
			TEClick(mouseXY,shiftKeyPosition,(*docRecHdl)->quantEditHdl);
		else
			doChangeCurrentEditRec((*docRecHdl)->quantEditHdl,true,mouseXY,kMaxCharasQuant);
	}
	else if(PtInRect(mouseXY,&gValueRect))
	{
		if(gCurrentEditRecHdl == (*docRecHdl)->valueEditHdl)
			TEClick(mouseXY,shiftKeyPosition,(*docRecHdl)->valueEditHdl);
		else
			doChangeCurrentEditRec((*docRecHdl)->valueEditHdl,true,mouseXY,kMaxCharasValue);
	}
	else if(PtInRect(mouseXY,&gDateRect))
	{
		if(gCurrentEditRecHdl == (*docRecHdl)->dateEditHdl)
			TEClick(mouseXY,shiftKeyPosition,(*docRecHdl)->dateEditHdl);
		else
			doChangeCurrentEditRec((*docRecHdl)->dateEditHdl,true,mouseXY,kMaxCharasDate);
	}
}

//  doUpdate

void	doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	GrafPtr				oldPort;
		
	windowPtr = (WindowPtr) eventRecPtr->message;
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);

	GetPort(&oldPort);
	SetPort(windowPtr);

	BeginUpdate((WindowPtr) eventRecPtr->message);

	EraseRect(&windowPtr->portRect);
	doDrawWindow();
	TEUpdate(&windowPtr->portRect,(*docRecHdl)->itemEditHdl);
	TEUpdate(&windowPtr->portRect,(*docRecHdl)->quantEditHdl);
	TEUpdate(&windowPtr->portRect,(*docRecHdl)->valueEditHdl);
	TEUpdate(&windowPtr->portRect,(*docRecHdl)->dateEditHdl);
	EndUpdate((WindowPtr) eventRecPtr->message);

	SetPort(oldPort);
}

//  doActivate

void  doActivate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventRecPtr->message;

	becomingActive = ((eventRecPtr->modifiers & activeFlag) == activeFlag);
	
	if(becomingActive)
		TEActivate(gCurrentEditRecHdl);
	else
		TEDeactivate(gCurrentEditRecHdl);
}

//  doChangeCurrentEditRec

void  doChangeCurrentEditRec(TEHandle newEditRecHdl,Boolean teClickFlag,Point mouseXY,
														 SInt16 maxCharas)
{
	TEDeactivate(gCurrentEditRecHdl);
	TEActivate(newEditRecHdl);
	gCurrentEditRecHdl = newEditRecHdl;

	if(teClickFlag)
		TEClick(mouseXY,false,newEditRecHdl);

	TESetSelect(0,32767,gCurrentEditRecHdl);

	gMaxCharasThisField = maxCharas;
}

//  doTodaysDate

void  doTodaysDate(void)
{
	UInt32	rawSeconds;
	Str255	dateString;
		
	GetDateTime(&rawSeconds);
	DateString(rawSeconds,longDate,dateString,NULL);
	MoveTo(110,22);
	DrawString(dateString);
}

//  doAcceptNewRecord

void  doAcceptNewRecord(void)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	
	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);

	if((*(*docRecHdl)->itemEditHdl)->teLength	 == 0 ||
		 (*(*docRecHdl)->quantEditHdl)->teLength == 0 ||
		 (*(*docRecHdl)->valueEditHdl)->teLength == 0	||
		 (*(*docRecHdl)->dateEditHdl)->teLength	 == 0)
	{
		SysBeep(10);
		return;
	}

	doEraseRecordDisplay();

	doAcceptItemField ((*docRecHdl)->itemEditHdl);
	doAcceptQuantField((*docRecHdl)->quantEditHdl);
	doAcceptValueField((*docRecHdl)->valueEditHdl,(*docRecHdl)->quantEditHdl);
	doAcceptDateField ((*docRecHdl)->dateEditHdl);

	TESetSelect(0,32767,(*docRecHdl)->itemEditHdl);
	TEDelete((*docRecHdl)->itemEditHdl);

	TESetSelect(0,32767,(*docRecHdl)->quantEditHdl);
	TEDelete((*docRecHdl)->quantEditHdl);

	TESetSelect(0,32767,(*docRecHdl)->valueEditHdl);
	TEDelete((*docRecHdl)->valueEditHdl);

	TESetSelect(0,32767,(*docRecHdl)->dateEditHdl);
	TEDelete((*docRecHdl)->dateEditHdl);

	TEDeactivate(gCurrentEditRecHdl);
	gCurrentEditRecHdl = (*docRecHdl)->itemEditHdl;
	TEActivate((*docRecHdl)->itemEditHdl);

	gMaxCharasThisField = kMaxCharasItem;
}

//  doAcceptItemField

void  doAcceptItemField(TEHandle itemEditHdl)
{
	Str255	itemString;

	GetDialogItemText((*itemEditHdl)->hText,itemString);
	MoveTo(166,188);
	DrawString(itemString);
}

//  doAcceptQuantField

void  doAcceptQuantField(TEHandle quantEditHdl)
{
	Str255	quantString;

	GetDialogItemText((*quantEditHdl)->hText,quantString);
	MoveTo(166,208);
	DrawString(quantString);
}

//  doAcceptValueField

void  doAcceptValueField(TEHandle valueEditHdl,TEHandle quantEditHdl)
{
	Handle						itl4ResourceHdl;
	SInt32						numpartsOffset;
	SInt32						numpartsLength;
	NumberParts				*numpartsTablePtr;
	Str255						formatString = "\p'$'###,###,###.00;'Valueless';'Valueless'";
	NumFormatString		formatStringRec;
	Str255						inputNumString;
	Str255						formattedNumString;
	extended80				value80Bit;
	Str255						quantityString;
	SInt32						quantity;
	FormatResultType	result;				

	GetIntlResourceTable(smSystemScript,iuNumberPartsTable,&itl4ResourceHdl,&numpartsOffset,
											 &numpartsLength);
	numpartsTablePtr = (NumberPartsPtr) ((SInt32) *itl4ResourceHdl + numpartsOffset);

	StringToFormatRec(formatString,numpartsTablePtr,&formatStringRec);

	GetDialogItemText((*valueEditHdl)->hText,inputNumString);

	StringToExtended(inputNumString,&formatStringRec,numpartsTablePtr,&value80Bit);
	ExtendedToString(&value80Bit,&formatStringRec,numpartsTablePtr,formattedNumString);

	MoveTo(166,228);
	DrawString(formattedNumString);

	GetDialogItemText((*quantEditHdl)->hText,quantityString);
	StringToNum(quantityString,&quantity);
	value80Bit = value80Bit * quantity;	

	result = ExtendedToString(&value80Bit,&formatStringRec,numpartsTablePtr,
														formattedNumString);
	MoveTo(166,248);
	if(result == fFormatOverflow)
		DrawString("\pToo large to display");
	else
		DrawString(formattedNumString);
}

//  doAcceptDateField

void  doAcceptDateField(TEHandle dateEditHdl)
{
	DateCacheRecord	dateCacheRec;
	Ptr							textPtr;
	SInt32					textLength, lengthUsed;
	LongDateRec			longDateTimeRec;
	LongDateTime		longDateTimeValue;
	Str255					dateString;

	InitDateCache(&dateCacheRec);
	textPtr = (*(*dateEditHdl)->hText);
	textLength = (*dateEditHdl)->teLength;

	StringToDate(textPtr,textLength,&dateCacheRec,&lengthUsed,&longDateTimeRec);

	LongDateToSeconds(&longDateTimeRec,&longDateTimeValue);
	longDateTimeValue.lo += 15724800;
	LongDateString(&longDateTimeValue,longDate,dateString,NULL);

	MoveTo(166,268);
	DrawString(dateString);
}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	MenuHandle	fileMenuHdl, editMenuHdl;
	
	fileMenuHdl = GetMenuHandle(mFile);
	editMenuHdl = GetMenuHandle(mEdit);
		
	if((*gCurrentEditRecHdl)->selStart < (*gCurrentEditRecHdl)->selEnd)
	{
		EnableItem(editMenuHdl,iCut);
		EnableItem(editMenuHdl,iCopy);
		EnableItem(editMenuHdl,iClear);
	}
	else
	{
		DisableItem(editMenuHdl,iCut);
		DisableItem(editMenuHdl,iCopy);
		DisableItem(editMenuHdl,iClear);
	}

	if(TEGetScrapLength() > 0)
		EnableItem(editMenuHdl,iPaste);
	else
		DisableItem(editMenuHdl,iPaste);

	if((*gCurrentEditRecHdl)->teLength > 0)
		EnableItem(editMenuHdl,iSelectAll);
	else
		DisableItem(editMenuHdl,iSelectAll);

	DrawMenuBar();
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16  menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID	 = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
				SysBeep(10);
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;
			
		case mFile:
			if(menuItem == iQuit)
				gDone = true;
			break;
			
		case mEdit:
			doEditMenu(menuItem);
			break;
	}

	HiliteMenu(0);
}

//  doEditMenu

void	doEditMenu(SInt16 menuItem)
{
	switch(menuItem)
	{
		case iCut:
			TECut(gCurrentEditRecHdl);
			break;

		case iCopy:
			TECopy(gCurrentEditRecHdl);
			break;

		case iPaste:
			TEPaste(gCurrentEditRecHdl);
			break;

		case iClear:
			TEDelete(gCurrentEditRecHdl);
			break;

		case iSelectAll:
			TESetSelect(0,(*gCurrentEditRecHdl)->teLength,gCurrentEditRecHdl);
			break;
	}
}

//  doAdjustCursor

void  doAdjustCursor(WindowPtr windowPtr,RgnHandle mouseRegion)
{
	GrafPtr		oldPort;
	RgnHandle	arrowRegion, itemRegion, quantRegion;
	RgnHandle valueRegion, dateRegion, iBeamRegions;
	Rect			itemRect, quantRect, valueRect, dateRect;
	Point			mouseXY;

	if(gInBackground)
	{
		SetCursor(&qd.arrow);
		return;
	}

	GetPort(&oldPort);
	SetPort(windowPtr);

	arrowRegion	 = NewRgn();
	itemRegion   = NewRgn();	
	quantRegion  = NewRgn();	
	valueRegion  = NewRgn();	
	dateRegion   = NewRgn();	
	iBeamRegions = NewRgn();

	SetRectRgn(arrowRegion,-32768,-32768,32766,32766);

	itemRect  = gItemRect;
	quantRect = gQuantRect;
	valueRect = gValueRect;
	dateRect  = gDateRect;

	LocalToGlobal(&topLeft(itemRect));
	LocalToGlobal(&botRight(itemRect));	
	RectRgn(itemRegion,&itemRect);
	LocalToGlobal(&topLeft(quantRect));
	LocalToGlobal(&botRight(quantRect));	
	RectRgn(quantRegion,&quantRect);
	LocalToGlobal(&topLeft(valueRect));
	LocalToGlobal(&botRight(valueRect));	
	RectRgn(valueRegion,&valueRect);
	LocalToGlobal(&topLeft(dateRect));
	LocalToGlobal(&botRight(dateRect));	
	RectRgn(dateRegion,&dateRect);

	UnionRgn(itemRegion,quantRegion,iBeamRegions);
	UnionRgn(valueRegion,iBeamRegions,iBeamRegions);
	UnionRgn(dateRegion,iBeamRegions,iBeamRegions);

	DiffRgn(arrowRegion,iBeamRegions,arrowRegion);
	
	GetMouse(&mouseXY);
	LocalToGlobal(&mouseXY);

	if(PtInRgn(mouseXY,iBeamRegions))
	{
		SetCursor(*(GetCursor(iBeamCursor)));
		CopyRgn(iBeamRegions,mouseRegion);
	}
	else
	{
		SetCursor(&qd.arrow);
		CopyRgn(arrowRegion,mouseRegion);
	}

	DisposeRgn(arrowRegion);
	DisposeRgn(itemRegion);
	DisposeRgn(quantRegion);	
	DisposeRgn(valueRegion);	
	DisposeRgn(dateRegion);	
	DisposeRgn(iBeamRegions);

	SetPort(oldPort);
}

//  doDrawWindow

void  doDrawWindow(void)
{
	Rect	lastItemRect;

	doTodaysDate();

	MoveTo(31,34);
	LineTo(331,34);
	MoveTo(31,142);
	LineTo(331,142);

	FrameRect(&gItemRect);
	FrameRect(&gQuantRect);
	FrameRect(&gValueRect);
	FrameRect(&gDateRect);

	SetRect(&lastItemRect,31,171,331,279);
	FrameRect(&lastItemRect);

	MoveTo(31,22);
	DrawString("\pToday's date is:");
	MoveTo(38,57);
	DrawString("\pItem Title:");
	MoveTo(45,80);
	DrawString("\pQuantity:");
	MoveTo(36,103);
	DrawString("\pUnit Value:");
	MoveTo(65,126);
	DrawString("\pDate:");
	MoveTo(248,57);
	MoveTo(118,162);
	DrawString("\pLast Record Entered:");
	MoveTo(248,57);
	DrawString("\pEg: Barometers");
	MoveTo(248,80);
	DrawString("\pEg: 34");
	MoveTo(248,103);
	DrawString("\pEg: 135.58");
	MoveTo(248,126);
	DrawString("\pEg: 24 Jul 95");
	MoveTo(110,188);
	DrawString("\pItem Title:");
	MoveTo(116,208);
	DrawString("\pQuantity:");
	MoveTo(107,228);
	DrawString("\pUnit Value:");
	MoveTo(104,248);
	DrawString("\pTotal Value:");
	MoveTo(98,268);
	DrawString("\pReview Date:");
}

//  doEraseRecordDisplay

void  doEraseRecordDisplay(void)
{
	Rect	rectToErase;

	SetRect(&rectToErase,166,172,330,278);
	EraseRect(&rectToErase);
}

// 