// 
// CDEF1.c										Custom control definition function for radio button control
// 
//
// This CDEF displays:
//
// 	3D coloured radio buttons on colour displays set to pixel depths greater than 2.
//
// 	Black-and-white buttons on colour displays set to pixel depths less than 4.
//
// 	Black-and-white radio buttons if Color QuickDraw is not present.
//
// This CDEF utilises six 'cicn' resources (purgeable).  The bitmap component, as opposed
// to the pixel map component, of each of these resources is automatically utilised if 
// the icon is being displayed on a colour device for which the pixel depth has been set
// to 1 or 2.  The appearance of the coloured radio buttons conforms to the specification
// for radio button controls contained in the document Apple Greyscale Appearance for
// System 7.5 published by Apple Computer Inc.
//
//  

//  includes

#include <Gestalt.h>
#include <ToolUtils.h>
#include <A4Stuff.h>
#include <Fonts.h>
#include <Palettes.h>
#include <LowMem.h>

//  defines

#define rActiveDeselect		128
#define rActiveSelect			129
#define rInActiveDeselect	130
#define rInactiveSelect		131
#define rFeedbackDeselect	132
#define rFeedbackSelect		133
#define partCode  				1

//  global variables

QDGlobals		*gQDGlobalsPtr;
Boolean			gColorQuickDrawPresent = false;
Boolean			gColourDisplay				 = false;
CIconHandle	gActiveDeselectHdl		 = NULL;
CIconHandle	gActiveSelectHdl			 = NULL;
CIconHandle	gInactiveDeselectHdl	 = NULL;
CIconHandle	gInactiveSelectHdl		 = NULL;
CIconHandle	gFeedbackDeselectHdl	 = NULL;
CIconHandle	gFeedbackSelectHdl		 = NULL;

//  function prototypes

pascal SInt32  main (short varCode,ControlHandle controlHdl,short message,SInt32 param);

void		doInitMessage		(void);
void		doDrawMessage		(ControlHandle);
void		drawColour			(ControlHandle,SInt16,Rect);
void		drawMono				(ControlHandle,SInt16,Rect);
SInt32	doTestMessage		(ControlHandle,SInt32);

//  main

pascal SInt32  main(short varCode,ControlHandle controlHdl,short message,SInt32 param)
{
	SInt32	returnValue;

	EnterCodeResource();				 
													 			 
	switch(message)
	{
		case initCntl:
			doInitMessage();
			break;

		case drawCntl:
			if((*controlHdl)->contrlVis)
			{
				doDrawMessage(controlHdl);
			}
			returnValue = 0;
			break;

		case testCntl:
			returnValue = doTestMessage(controlHdl,param);
			break;
			
		default:
			returnValue = 0;
	}

	ExitCodeResource();														  
	return returnValue;
}

//  doInitMessage

void	doInitMessage(void)
{
	OSErr			osErr;
	SInt32		response;
	
	gQDGlobalsPtr = (QDGlobals*) (*((SInt32*) SetCurrentA5()) - (sizeof(QDGlobals) - 
																															 sizeof(GrafPtr)));
	osErr = Gestalt(gestaltQuickdrawVersion,&response);
	if(response >= gestalt8BitQD)
		gColorQuickDrawPresent = true;
}

//  doDrawMessage

void  doDrawMessage(ControlHandle controlHdl)
{
	WindowPtr	windowPtr;
	GrafPtr		oldPort;
	SInt16		oldFont, oldSize, oldTextMode, controlValue, fontNum;
	PenState	oldPenState;
	Rect			controlRect;
	GDHandle	mainDeviceHdl;
	SInt16		bitsPerPixel;
		
	windowPtr = (WindowPtr) (*controlHdl)->contrlOwner;

	GetPort(&oldPort);
	oldFont = windowPtr->txFont;
	oldSize = windowPtr->txSize;
	oldTextMode = windowPtr->txMode;
	GetPenState(&oldPenState);
	
	SetPort(windowPtr);

	controlValue = GetControlValue(controlHdl);

	controlRect = (*controlHdl)->contrlRect;
	controlRect.right = controlRect.left + 12;
	controlRect.bottom = controlRect.top + 12;

	GetFNum("\pChicago",&fontNum);
	TextFont(fontNum);
	TextSize(12);

	if(gColorQuickDrawPresent)
	{
		if(gActiveDeselectHdl == NULL)
			gActiveDeselectHdl = GetCIcon(rActiveDeselect);
		if(gActiveSelectHdl == NULL)
			gActiveSelectHdl = GetCIcon(rActiveSelect);
		if(gInactiveDeselectHdl == NULL)
			gInactiveDeselectHdl = GetCIcon(rInActiveDeselect);
		if(gInactiveSelectHdl == NULL)
			gInactiveSelectHdl = GetCIcon(rInactiveSelect);
		if(gFeedbackDeselectHdl == NULL)
			gFeedbackDeselectHdl = GetCIcon(rFeedbackDeselect);
		if(gFeedbackSelectHdl == NULL)
			gFeedbackSelectHdl = GetCIcon(rFeedbackSelect);
	}
	
	mainDeviceHdl = LMGetMainDevice();
	bitsPerPixel = (*(*mainDeviceHdl)->gdPMap)->pixelSize;
	if(bitsPerPixel > 1)
		gColourDisplay = true;
	else
		gColourDisplay = false;
	
	if(gColorQuickDrawPresent && gColourDisplay)
		drawColour(controlHdl,controlValue,controlRect);
	else
		drawMono(controlHdl,controlValue,controlRect);

	SetPenState(&oldPenState);
	TextMode(oldTextMode);
	TextFont(oldFont);
	TextSize(oldSize);
	SetPort(oldPort);
}

//  drawColour

void  drawColour(ControlHandle controlHdl,SInt16 controlValue,Rect controlRect)
{
	RGBColor	oldForeColour;
	RGBColor	blackColour		= { 0x0000, 0x0000, 0x0000 };
	RGBColor	gray8					= { 0x7777, 0x7777, 0x7777 };

	GetForeColor(&oldForeColour);
				
	if((*controlHdl)->contrlHilite == 255)
	{
		if(gColourDisplay)
			RGBForeColor(&gray8);
		else
			TextMode(grayishTextOr);

		if(controlValue == 1)
			PlotCIcon(&controlRect,gInactiveSelectHdl);
		else
			PlotCIcon(&controlRect,gInactiveDeselectHdl);
	}		
	else if((*controlHdl)->contrlHilite == 0)
	{
		RGBForeColor(&blackColour);
		TextMode(srcOr);
		
		if(controlValue == 1)
			PlotCIcon(&controlRect,gActiveSelectHdl);
		else
			PlotCIcon(&controlRect,gActiveDeselectHdl);
	}
	else if((*controlHdl)->contrlHilite == partCode)
	{
		if(controlValue == 1)
			PlotCIcon(&controlRect,gFeedbackSelectHdl);
		else
			PlotCIcon(&controlRect,gFeedbackDeselectHdl);
	}

	MoveTo(controlRect.left + 17,controlRect.top + 10);
	DrawString((*controlHdl)->contrlTitle);

	RGBForeColor(&oldForeColour);
}

//  drawMono

void  drawMono(ControlHandle controlHdl,SInt16 controlValue,Rect controlRect)
{
	ForeColor(blackColor);
	BackColor(whiteColor);
	
	PenNormal();
				
	if((*controlHdl)->contrlHilite == 255 || (*controlHdl)->contrlHilite == 0)
	{
		if((*controlHdl)->contrlHilite == 255)
		{
			PenPat(&gQDGlobalsPtr->gray);
			TextMode(grayishTextOr);
		}
		else
		{
			PenPat(&gQDGlobalsPtr->black);
			TextMode(srcOr);
		}

		FrameOval(&controlRect);

		InsetRect(&controlRect,1,1);
		FillOval(&controlRect,&gQDGlobalsPtr->white);
		InsetRect(&controlRect,-1,-1);

		if(controlValue == 1)
		{
			InsetRect(&controlRect,3,3);
			if((*controlHdl)->contrlHilite == 255)
				FillOval(&controlRect,&gQDGlobalsPtr->gray);
			else
				FillOval(&controlRect,&gQDGlobalsPtr->black);
			InsetRect(&controlRect,-3,-3);
		}
	}		
	else if((*controlHdl)->contrlHilite == partCode)
	{
		InsetRect(&controlRect,1,1);
		FrameOval(&controlRect);
		InsetRect(&controlRect,-1,-1);
	}

	MoveTo(controlRect.left + 17,controlRect.top + 10);
	DrawString((*controlHdl)->contrlTitle);
}

//  doTestMessage

SInt32  doTestMessage(ControlHandle controlHdl,SInt32 param)
{
	Rect	controlRect;
	Point	mouseXY;

	controlRect =	(*controlHdl)->contrlRect;

	mouseXY.v = HiWord(param);
	mouseXY.h = LoWord(param);
	
	if(PtInRect(mouseXY,&controlRect))
		return partCode;
	else
		return (SInt32) 0;
}

// 



