// 
// FloatRoutines.c							 			 							 Routines to support floating windows
// 

//  includes

#include "Floaters.h"

//  newGetNewWindow

WindowPtr  newGetNewWindow(SInt16 windResourceID,WindowPtr behind,
													 ActivateProcPtr activateHandler,SInt16 windowType)
{
	docRecordHandle	docRecordHdl;
	OSErr						osErr;
	SInt32					response;
	WindowPtr				newWindowPtr;
	WindowPtr				lastFloatingWindowPtr;

	docRecordHdl = (docRecordHandle) NewHandle(sizeof(docRecord));

	osErr = Gestalt(gestaltQuickdrawVersion,&response);
	if(response < gestalt32BitQD)
		newWindowPtr = GetNewWindow(windResourceID,NULL,(WindowPtr) behind);
	else
		newWindowPtr = GetNewCWindow(windResourceID,NULL,(WindowPtr) behind);

	if(newWindowPtr != NULL)
	{
		SetWRefCon(newWindowPtr,(SInt32) docRecordHdl);
		setActivateHandler(newWindowPtr,activateHandler);

		if(windowType == kFloatingKind)
		{
			setWindowKind(newWindowPtr,kFloatingKind);
			HiliteWindow(newWindowPtr,true);
		}
		else
		{
			setWindowKind(newWindowPtr,kDocumentKind);
			if(behind == (WindowPtr) -1)
			{
				lastFloatingWindowPtr = findLastFloatingWindow();
					
				if(lastFloatingWindowPtr != NULL)
					SendBehind(newWindowPtr,lastFloatingWindowPtr);
				else
					BringToFront(newWindowPtr);
			}
		}
	}
	else
		DisposeHandle((Handle) docRecordHdl);

	return newWindowPtr;
}

//  newDisposeWindow

void  newDisposeWindow(WindowPtr windowPtr)
{
	if(getWindowVisible(windowPtr))
		newHideWindow(windowPtr);
	CloseWindow(windowPtr);
	DisposeHandle((Handle) GetWRefCon(windowPtr));
	DisposePtr((Ptr) windowPtr);
}

//  newSelectWindow

void  newSelectWindow(WindowPtr windowToSelectPtr)
{
	Boolean		isFloatingWindow;
	WindowPtr	currentFrontWindowPtr;
	WindowPtr	lastFloatingWindowPtr;
	
	if(getWindowKind(windowToSelectPtr) == kFloatingKind)
	{
		isFloatingWindow = true;
		currentFrontWindowPtr = FrontWindow();
	}
	else
	{
		isFloatingWindow = false;
		currentFrontWindowPtr = findFrontNonFloatingWindow();
		lastFloatingWindowPtr = findLastFloatingWindow();
	}

	if(currentFrontWindowPtr != windowToSelectPtr)
	{
		if(isFloatingWindow)
			BringToFront(windowToSelectPtr);
		else
		{
			if(lastFloatingWindowPtr == NULL)
				SelectWindow(windowToSelectPtr);
			else
			{
				deactivateWindow(currentFrontWindowPtr);
				SendBehind(windowToSelectPtr,lastFloatingWindowPtr);
				activateWindow(windowToSelectPtr);
			}
		}
	}
}

//  newHideWindow

void  newHideWindow(WindowPtr windowToHidePtr)
{
	WindowPtr	frontFloaterPtr;
	WindowPtr	frontNonFloaterPtr;
	WindowPtr	windowBehindPtr;
	WindowPtr	lastFloaterPtr;

	if(getWindowVisible(windowToHidePtr) == false)
		return;

	frontFloaterPtr = FrontWindow();
	if(getWindowKind(frontFloaterPtr) != kFloatingKind)
		frontFloaterPtr = NULL;

	frontNonFloaterPtr = findFrontNonFloatingWindow();

	ShowHide(windowToHidePtr,false);

	if(windowToHidePtr == frontFloaterPtr)
	{
		windowBehindPtr = getNextWindow(windowToHidePtr);

		if((windowBehindPtr != NULL) && (getWindowKind(windowBehindPtr) == kFloatingKind))
		{
			setNextWindow(windowToHidePtr,getNextWindow(windowBehindPtr));
			setNextWindow(windowBehindPtr,windowToHidePtr);
			setWindowList(windowBehindPtr);
		}
	}
	else
	{
		if(windowToHidePtr == frontNonFloaterPtr)
		{
			windowBehindPtr = getNextWindow(windowToHidePtr);

			if(windowBehindPtr != NULL)
			{
				setNextWindow(windowToHidePtr,getNextWindow(windowBehindPtr));
				setNextWindow(windowBehindPtr,windowToHidePtr);

				lastFloaterPtr = findLastFloatingWindow();
				if(lastFloaterPtr != NULL)
					setNextWindow(lastFloaterPtr,windowBehindPtr);
				else
					setWindowList(windowBehindPtr);

				activateWindow(windowBehindPtr);
			}
		}
	}
}

//  newShowWindow

void  newShowWindow(WindowPtr windowToShowPtr)
{
	SInt16					windowType;
	WindowPtr				windowBehindPtr;
	WindowPtr				frontNonFloatingWindowPtr;
	Boolean					windowIsInFront = false;
	ActivateProcPtr	activateHandler;

	if(getWindowVisible(windowToShowPtr) != false)
		return;

	windowType = getWindowKind(windowToShowPtr);
	
	if(windowType != kFloatingKind)
	{
		windowBehindPtr = getNextWindow(windowToShowPtr);
		if(windowBehindPtr == findFrontNonFloatingWindow())
		{
			if(windowBehindPtr != NULL)
				deactivateWindow(windowBehindPtr);

			setWindowHilite(windowToShowPtr,true);
			windowIsInFront = true;
		}
	}
	else
	{
		frontNonFloatingWindowPtr = findFrontNonFloatingWindow();

		if((frontNonFloatingWindowPtr != NULL) && 
			 (frontNonFloatingWindowPtr == FrontWindow()) &&
			 (isWindowModal(frontNonFloatingWindowPtr)))
		{
			setWindowHilite(windowToShowPtr,false);
		}
		else
		{
			setWindowHilite(windowToShowPtr,true);
			windowIsInFront = true;
		}
	}

	ShowHide(windowToShowPtr,true);

	if(windowIsInFront)
	{
		activateHandler = getActivateHandler(windowToShowPtr);
		(*activateHandler) (windowToShowPtr,true);
	}
}

//  newDragWindow

void  newDragWindow(WindowPtr windowPtr,Point startPoint,const Rect *dragBounds)
{
	SInt16		topLimit;
	Rect			slopRect;
	GrafPtr		oldPort;
	GrafPtr		windowManagerPort;
	KeyMap		keyMap;
	Boolean		commandKeyDown = false;
	RgnHandle	dragRegion;
	SInt32		dragResult;
	SInt16		horizOffset;
	SInt16		vertOffset;
	RgnHandle	windowContentRegion;
	SInt16		newHorizWindowPosition;
	SInt16		newVertWindowPosition;

	if(WaitMouseUp())
	{
		topLimit = GetMBarHeight() + 4;
		slopRect = *dragBounds;

		if(slopRect.top < topLimit)
			slopRect.top = topLimit;

		GetPort(&oldPort);
		GetWMgrPort(&windowManagerPort);
		SetPort(windowManagerPort);

		SetClip(GetGrayRgn());

		GetKeys(keyMap);
		if(keyMap[1] & 0x8000)
			commandKeyDown = true;

		if((commandKeyDown == true) || (getWindowKind(windowPtr) != kFloatingKind))
		{
			if (commandKeyDown == false)
				ClipAbove((WindowRef) ((WindowPeek) findFrontNonFloatingWindow()));
			else
				ClipAbove((WindowRef) ((WindowPeek) windowPtr));
		}

		dragRegion = NewRgn();
		CopyRgn(getStructureRegion(windowPtr), dragRegion);

		dragResult = DragGrayRgn(dragRegion,startPoint,&slopRect,&slopRect,noConstraint,nil);

		SetPort(oldPort);

		if(dragResult != 0)
		{
			horizOffset = dragResult & 0xFFFF;
			vertOffset = dragResult >> 16;

			if(vertOffset != -32768)
			{
				windowContentRegion = getContentRegion(windowPtr);
	
				newHorizWindowPosition = (**windowContentRegion).rgnBBox.left +  horizOffset;
				newVertWindowPosition = (**windowContentRegion).rgnBBox.top + vertOffset;
				
				MoveWindow(windowPtr,newHorizWindowPosition,newVertWindowPosition,false);
			}
		}

		if(commandKeyDown == false)
			newSelectWindow(windowPtr);
	
		DisposeRgn(dragRegion);
	}
}

//  handleSuspendEvent

void  handleSuspendEvent(void)
{
	WindowPtr	currentWindowPtr;
	Boolean		windowIsVisible;
	
	currentWindowPtr = getWindowList();

	if(getWindowKind(currentWindowPtr) != kFloatingKind)
		return;

	do
	{
		windowIsVisible = getWindowVisible(currentWindowPtr);
		setWasVisible(currentWindowPtr,windowIsVisible);
		if(windowIsVisible)
			ShowHide(currentWindowPtr,false);
		currentWindowPtr = getNextWindow(currentWindowPtr);
	} while((currentWindowPtr != NULL) &&
					(getWindowKind(currentWindowPtr) == kFloatingKind));

	currentWindowPtr = findFrontNonFloatingWindow();
	if(currentWindowPtr != NULL)
	{
		DrawGrowIcon(currentWindowPtr);
		deactivateWindow(currentWindowPtr);
	}
}

//  handleResumeEvent

void  handleResumeEvent(void)
{
	WindowPtr	currentWindowPtr;
	Boolean		windowWasVisible;

	currentWindowPtr = getWindowList();

	if(getWindowKind(currentWindowPtr) != kFloatingKind)
		return;

	do
	{
		windowWasVisible = getWasVisible(currentWindowPtr);
		if(windowWasVisible)
		{
			ShowHide(currentWindowPtr,true);
			activateWindow(currentWindowPtr);
		}
		currentWindowPtr = getNextWindow(currentWindowPtr);
	} while((currentWindowPtr != NULL) &&
					(getWindowKind(currentWindowPtr) == kFloatingKind));

	currentWindowPtr = findFrontNonFloatingWindow();
	if(currentWindowPtr != NULL)
	{
		DrawGrowIcon(currentWindowPtr);
		activateWindow(currentWindowPtr);
	}
}

//  deactivateFloatsAndFirstDocWin

void  deactivateFloatsAndFirstDocWin(void)
{
	WindowPtr	firstWindowPtr;
	WindowPtr	secondDocumentWindowPtr;
	WindowPtr	currentWindowPtr;

	firstWindowPtr = FrontWindow();
	secondDocumentWindowPtr = findFrontNonFloatingWindow();
	if(secondDocumentWindowPtr != NULL)
		secondDocumentWindowPtr = getNextWindow(secondDocumentWindowPtr);

	currentWindowPtr = firstWindowPtr;
	while(currentWindowPtr != secondDocumentWindowPtr)
	{
		if(getWindowVisible(currentWindowPtr))
			deactivateWindow(currentWindowPtr);
		currentWindowPtr = getNextWindow(currentWindowPtr);
	}
}

//  activateFloatsAndFirstDocWin

void  activateFloatsAndFirstDocWin(void)
{
	OSErr								getFrontProcessResult;
	OSErr								getCurrentProcessResult;
	ProcessSerialNumber	frontPSN;
	ProcessSerialNumber	currentPSN;
	OSErr								sameProcessResult;
	Boolean							isSameProcess;
	WindowPtr						firstWindowPtr;
	WindowPtr						secondDocumentWindowPtr;
	WindowPtr						currentWindowPtr;

	getFrontProcessResult = GetFrontProcess(&frontPSN);
	getCurrentProcessResult = GetCurrentProcess(&currentPSN);

	if((getFrontProcessResult == noErr) && (getCurrentProcessResult == noErr))
		sameProcessResult = SameProcess(&frontPSN,&currentPSN,&isSameProcess);

	if((sameProcessResult == noErr) && (isSameProcess == false))
		handleSuspendEvent();
	else
	{
		firstWindowPtr = FrontWindow();
		secondDocumentWindowPtr = findFrontNonFloatingWindow();
		if(secondDocumentWindowPtr != NULL)
			secondDocumentWindowPtr = getNextWindow(secondDocumentWindowPtr);
		currentWindowPtr = firstWindowPtr;

		while(currentWindowPtr != secondDocumentWindowPtr)
		{
			if(getWindowVisible(currentWindowPtr))
				activateWindow(currentWindowPtr);
			currentWindowPtr = getNextWindow(currentWindowPtr);
		}
	}
}

//  deactivateWindow

void  deactivateWindow(WindowPtr windowPtr)
{
	highlightAndActivateWindow(windowPtr,false);	
}

//  activateWindow

void  activateWindow(WindowPtr windowPtr)
{
	highlightAndActivateWindow(windowPtr,true);
}

//  highlightAndActivateWindow

void  highlightAndActivateWindow(WindowPtr windowPtr,Boolean activate)
{	
	ActivateProcPtr activateHandler;

	HiliteWindow(windowPtr,activate);
	activateHandler = getActivateHandler(windowPtr);
	(*activateHandler) (windowPtr,activate);
}

//  findFrontNonFloatingWindow

WindowPtr  findFrontNonFloatingWindow(void)
{
	WindowPtr	frontWindowPtr;

	frontWindowPtr = FrontWindow();

	while((frontWindowPtr != NULL) && (getWindowKind(frontWindowPtr) == kFloatingKind))
	{
		do
		{
			frontWindowPtr = getNextWindow(frontWindowPtr);
		}	while((frontWindowPtr != NULL) && (getWindowVisible(frontWindowPtr) == false));
	}

	return(frontWindowPtr);
}

//  findLastFloatingWindow

WindowPtr  findLastFloatingWindow(void)
{
	WindowPtr 	windowPtr;
	WindowPtr		lastFloatingWindowPtr;

	windowPtr = getWindowList();
	lastFloatingWindowPtr = NULL;

	while(windowPtr != NULL)
	{
		if(getWindowKind(windowPtr) == kFloatingKind)
			lastFloatingWindowPtr = windowPtr;
		windowPtr = getNextWindow(windowPtr);
	}

	return lastFloatingWindowPtr;
}

//  isWindowModal

Boolean  isWindowModal(WindowPtr windowPtr)
{
	SInt16	variant;

	variant = GetWVariant(windowPtr);

	if((((WindowPeek) windowPtr)->windowKind == dialogKind) &&
		 ((variant == dBoxProc) || (variant == movableDBoxProc)))
		return true;
	else
		return false;
}

//  getWindowList

WindowPtr  getWindowList(void)
{
	return(LMGetWindowList());
}

//  setWindowList

void  setWindowList(WindowPtr windowPtr)
{
	LMSetWindowList(windowPtr);
}

//  getNextWindow

WindowPtr  getNextWindow(WindowPtr windowPtr)
{
	return((WindowPtr) ((WindowPeek) windowPtr)->nextWindow);
}

//  setNextWindow

void  setNextWindow(WindowPtr windowPtr,WindowPtr nextWindowPtr)
{
	((WindowPeek) windowPtr)->nextWindow = (WindowPeek) nextWindowPtr;
}

//  getWasVisible

Boolean  getWasVisible(WindowPtr windowPtr)
{
	docRecordHandle	docRecordHdl;

	docRecordHdl = (docRecordHandle) GetWRefCon(windowPtr);
	return((*docRecordHdl)->wasVisible);
}

//  setWasVisible

void  setWasVisible(WindowPtr windowPtr,Boolean wasVisible)
{
	docRecordHandle	docRecordHdl;

	docRecordHdl = (docRecordHandle) GetWRefCon(windowPtr);
	(*docRecordHdl)->wasVisible = wasVisible;
}

//  getWindowKind

SInt16  getWindowKind(WindowPtr windowPtr)
{
	docRecordHandle	docRecordHdl;

	if(((WindowPeek) windowPtr)->windowKind != dialogKind)
	{
		docRecordHdl = (docRecordHandle) GetWRefCon(windowPtr);
		return((*docRecordHdl)->windowType);
	}
 	else
		return(0);
}

//  setWindowKind

void  setWindowKind(WindowPtr windowPtr,SInt16 windowKind)
{
	docRecordHandle	docRecordHdl;

	docRecordHdl = (docRecordHandle) GetWRefCon(windowPtr);
	(*docRecordHdl)->windowType = windowKind;
}

//  getWindowVisible

Boolean  getWindowVisible(WindowPtr windowPtr)
{
	return(((WindowPeek) windowPtr)->visible);
}

//  getStructureRegion

RgnHandle  getStructureRegion(WindowPtr windowPtr)
{
	return(((WindowPeek) windowPtr)->strucRgn);
}

//  getContentRegion

RgnHandle  getContentRegion(WindowPtr windowPtr)
{
	return(((WindowPeek) windowPtr)->contRgn);
}

//  setWindowHilite

void  setWindowHilite(WindowPtr windowPtr,Boolean windowHilite)
{
	((WindowPeek) windowPtr)->hilited = windowHilite;
}

//  getActivateHandler

ActivateProcPtr  getActivateHandler(WindowPtr windowPtr)
{
	docRecordHandle	docRecordHdl;

	docRecordHdl = (docRecordHandle) GetWRefCon(windowPtr);
	return((*docRecordHdl)->activateHandler);
}

//  setActivateHandler

void  setActivateHandler(WindowPtr windowPtr,ActivateProcPtr activateHandler)
{
	docRecordHandle	docRecordHdl;

	docRecordHdl = (docRecordHandle) GetWRefCon(windowPtr);
	(*docRecordHdl)->activateHandler = activateHandler;
}

// 