// 
// FloaterDemo.c
// 

//  includes

#include "Floaters.h"

//  global variables

Boolean		gColorQuickDrawPresent = false;
Boolean		gColourDisplay				 = false;
Boolean		gDone;
Boolean		gInBackground;
SInt16		gNumberDocWindowsOpen;
WindowPtr	gToolsWindowPtr;
WindowPtr	gColoursWindowPtr;

//  main

void  main(void)
{
	OSErr				osErr;
	SInt32			response;
	GDHandle		mainDeviceHdl;
	SInt16			bitsPerPixel;
	Handle			menubarHdl;
	MenuHandle	menuHdl;

	//  initialise managers

	doInitManagers();

	//  check for Color QuickDraw

	osErr = Gestalt(gestaltQuickdrawVersion,&response);
	if(response >= gestalt8BitQD)
	{
		gColorQuickDrawPresent = true;

		mainDeviceHdl = LMGetMainDevice();
		bitsPerPixel = (*(*mainDeviceHdl)->gdPMap)->pixelSize;
		if(bitsPerPixel > 1)
			gColourDisplay = true;
	}

	//  set up menu bar and menus

	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		ExitToShell();
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		ExitToShell();
	else
		AppendResMenu(menuHdl,'DRVR');
		
	//  open floating windows

	doOpenFloatingWindows();
	
	CheckItem(GetMenuHandle(mFloaters),iTools,true);
	CheckItem(GetMenuHandle(mFloaters),iColours,true);

	//  enter eventLoop

	eventLoop();
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();	
	FlushEvents(everyEvent,0);
}

//  doOpenFloatingWindows

void 	doOpenFloatingWindows(void)
{
	SInt16					resourceOffset = 0;
	PicHandle				pictureHdl;
	ActivateProcPtr procPtr;
	
	if(gColorQuickDrawPresent == false || gColourDisplay == false)
		resourceOffset = 4;
	
	procPtr = (ActivateProcPtr) &toolsActivateHandler;
	gToolsWindowPtr = newGetNewWindow(rToolsWindow,(WindowPtr) -1,procPtr,kFloatingKind);
	if(gToolsWindowPtr == NULL)
	{
		SysBeep(10);
		ExitToShell();
	}

	pictureHdl = GetPicture(rToolsPict + resourceOffset);
	SetWindowPic(gToolsWindowPtr,pictureHdl);

	procPtr = (ActivateProcPtr) &coloursActivateHandler;
	gColoursWindowPtr = newGetNewWindow(rColoursWindow,(WindowPtr) -1,procPtr,kFloatingKind);
	if(gColoursWindowPtr == NULL)
	{
		SysBeep(10);
		ExitToShell();
	}

	pictureHdl = GetPicture(rColoursPict + resourceOffset);
	SetWindowPic(gColoursWindowPtr,pictureHdl);

	HiliteWindow(gToolsWindowPtr,true);
}

//  eventLoop

void  eventLoop(void)
{
	EventRecord	eventRec;
	DialogPtr		theDialogPtr;
	SInt16			itemHit;

	gDone = false;
	
	while(!gDone)
	{
		if(WaitNextEvent(everyEvent,&eventRec,MAXLONG,NULL))
		{
			if(!IsDialogEvent(&eventRec))
				doEvents(&eventRec);
			else
			{
				DialogSelect(&eventRec,&theDialogPtr,&itemHit);
				if((itemHit == ok) || (itemHit == cancel))
					doDisposeFindDialog();
			}
		}
	}
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	SInt8	charCode;

	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
			{
				doAdjustMenus();
				doMenuChoice(MenuKey(charCode));
			}
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case osEvt:
			doOSEvent(eventRecPtr);
			HiliteMenu(0);
			break;
	}
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	SInt16					partCode;
	WindowPtr				windowPtr;
	WindowPtr				frontWindowPtr;
	docRecordHandle	docRecordHdl;
	SInt16					windowType;
	UInt32					newSize;
	Rect						growRect;
	
	partCode = FindWindow(eventRecPtr->where,&windowPtr);

	switch(partCode)
	{
		case inMenuBar:
			doAdjustMenus();
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			frontWindowPtr = FrontWindow();
			if(((WindowPeek) frontWindowPtr)->windowKind != dialogKind)
				doProvingBeeps(windowPtr);
			if((isWindowModal(frontWindowPtr)) && (windowPtr != frontWindowPtr))
				SysBeep(10);
			else
				if(windowPtr != frontWindowPtr)
					newSelectWindow(windowPtr);
			break;

		case inDrag:
			frontWindowPtr = FrontWindow();
			if((isWindowModal(frontWindowPtr)) && (windowPtr != frontWindowPtr))
				SysBeep(10);
			else
				newDragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where))
			{
				docRecordHdl = (docRecordHandle) GetWRefCon(windowPtr);
				windowType = (*docRecordHdl)->windowType;
				if(windowType == kFloatingKind)
				{
					newHideWindow(windowPtr);
					doAdjustMenus();					
				}
				else if(windowType == kDocumentKind)
					doCloseDocWindow();
			}	
			break;

		case inGrow:
			growRect = qd.screenBits.bounds;
			growRect.top = 145;
			growRect.left = 335;
			newSize = GrowWindow(windowPtr,eventRecPtr->where,&growRect);
			if(newSize != 0)
			{
				invalidateScrollBarArea(windowPtr);
				SizeWindow(windowPtr,LoWord(newSize),HiWord(newSize),true);
				invalidateScrollBarArea(windowPtr);
			}
			break;

		case inZoomIn:
		case inZoomOut:
			if(TrackBox(windowPtr,eventRecPtr->where,partCode))
			{
				SetPort(windowPtr);
				EraseRect(&windowPtr->portRect);
				ZoomWindow(windowPtr,partCode,false);
				InvalRect(&windowPtr->portRect);
			}
			break;
	}
}

//  doUpdate

void  doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;

	windowPtr = (WindowPtr)eventRecPtr->message;
	SetPort(windowPtr);

	BeginUpdate(windowPtr);
	EraseRect(&windowPtr->portRect);
	doDrawDocWindowContent();
	DrawGrowIcon(windowPtr);
	EndUpdate(windowPtr);
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
	switch((eventRecPtr->message >> 24) & 0x000000FF)
	{
		case suspendResumeMessage:
			if(eventRecPtr->message & resumeFlag)
			{
				gInBackground = false;
				handleResumeEvent();
				SetCursor(&qd.arrow);
			}
			else
			{
				gInBackground = true;
				handleSuspendEvent();
			}
			break;

		case mouseMovedMessage:
			break;
	}
}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	MenuHandle	appleMenuHdl, fileMenuHdl, floatMenuHdl;

	appleMenuHdl = GetMenuHandle(mApple);
	fileMenuHdl = GetMenuHandle(mFile);
	floatMenuHdl = GetMenuHandle(mFloaters);

	if(isWindowModal(FrontWindow()))
	{
		DisableItem(appleMenuHdl,1);
		DisableItem(fileMenuHdl,0);
		DisableItem(floatMenuHdl,0);
	}
	else
	{
		EnableItem(appleMenuHdl,1);
		EnableItem(fileMenuHdl,0);
		EnableItem(floatMenuHdl,0);

		if(gNumberDocWindowsOpen)
			EnableItem(fileMenuHdl,iClose);
		else
			DisableItem(fileMenuHdl,iClose);
	}

	if(getWindowVisible(gToolsWindowPtr))
		CheckItem(floatMenuHdl,iTools,true);
	else
		CheckItem(floatMenuHdl,iTools,false);

	if(getWindowVisible(gColoursWindowPtr))
		CheckItem(floatMenuHdl,iColours,true);
	else
		CheckItem(floatMenuHdl,iColours,false);

	DrawMenuBar();
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16		menuID, menuItem;
	Str255	itemName;
	SInt16		daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
			{
				deactivateFloatsAndFirstDocWin();
				NoteAlert(rAboutAlert,NULL);
				activateFloatsAndFirstDocWin();
			}
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;

		case mFile:
			doFileMenu(menuItem);
			break;

		case mFloaters:
			doFloatersMenu(menuItem);
			break;
	}

	HiliteMenu(0);
}

//  doFileMenu

void	doFileMenu(SInt16 menuItem)
{
	switch(menuItem)
	{
		case iNew:
			doOpenDocWindow();
			break;

		case iClose:
			doCloseDocWindow();
			break;

		case iFind:
			doOpenFindDialog();
			break;

		case iQuit:
			gDone = true;
			break;
	}
}

//  doFloatersMenu

void	doFloatersMenu(SInt16 menuItem)
{
	Boolean		floaterVisible;
	WindowPtr	floaterPicked;

	if(menuItem == iTools)
		floaterPicked = gToolsWindowPtr;
	else if(menuItem == iColours)
		floaterPicked = gColoursWindowPtr;

	floaterVisible = getWindowVisible(floaterPicked);

	if(floaterVisible == false)
		newShowWindow(floaterPicked);
	else
		newHideWindow(floaterPicked);

	doAdjustMenus();
}

//  doOpenDocWindow

void  doOpenDocWindow(void)
{
	ActivateProcPtr	procPtr;
	WindowPtr				windowPtr;

	if(gNumberDocWindowsOpen > 3)
		SysBeep(10);
	else
	{
		procPtr = (ActivateProcPtr) &docActivateHandler;
		windowPtr = newGetNewWindow(rDocWindow,(WindowPtr) -1,procPtr,kDocumentKind);
		newShowWindow(windowPtr);
		gNumberDocWindowsOpen ++;
	}
}

//  doCloseDocWindow

void  doCloseDocWindow(void)
{
	WindowPtr		windowPtr;

	windowPtr = findFrontNonFloatingWindow();

	if(windowPtr != NULL)
		newDisposeWindow(windowPtr);

	gNumberDocWindowsOpen --;
}

//  doDrawDocWindowContent

void  doDrawDocWindowContent(void)
{
	Str255	theString;
	SInt16	a;

	for(a=1;a<4;a++)
	{
		GetIndString(theString,128,a);
		MoveTo(20,a*30);
		DrawString(theString);
	}
}

//  doProvingBeeps

void  doProvingBeeps(WindowPtr windowPtr)
{
	Handle	soundHdl;
	SInt16	repeat, a;

	soundHdl = GetResource('snd ',rSound);

	if(windowPtr == gToolsWindowPtr)
		repeat = 2;
	else if(windowPtr == gColoursWindowPtr)
		repeat = 3;
	else
		repeat = 1;

	for(a=0;a<repeat;a++)
		SndPlay(NULL,(SndListHandle) soundHdl,false);

	ReleaseResource(soundHdl); 
}

//  doOpenFindDialog

void  doOpenFindDialog(void)
{
	DialogPtr findDialogPtr;

	deactivateFloatsAndFirstDocWin();
	findDialogPtr = GetNewDialog(rFindDialog,NULL,(WindowPtr) - 1);

	doAdjustMenus();
}

//  doDisposeFindDialog

void  doDisposeFindDialog(void)
{
	DialogPtr	findDialogPtr;
	
	findDialogPtr = FrontWindow();
	DisposeDialog(findDialogPtr);

	activateFloatsAndFirstDocWin();

	doAdjustMenus();
}

//  docActivateHandler

pascal void  docActivateHandler(WindowPtr windowPtr,Boolean activate)
{
	GrafPtr	oldPort;
	SInt32	finalTicks;
	Rect		theRect;

	GetPort(&oldPort);
	SetPort(windowPtr);

	DrawGrowIcon(windowPtr);
	
	MoveTo(20,120);
	if(activate)
		DrawString("\pActivating");
	else
		DrawString("\pDeactivating");

	Delay(30,&finalTicks);

	SetRect(&theRect,20,105,100,125);
	EraseRect(&theRect);

	SetPort(oldPort);
}

//  toolsActivateHandler

pascal void  toolsActivateHandler(WindowPtr windowPtr,Boolean activate)
{
	GrafPtr		oldPort;
	SInt16		resourceOffset = 0;
	PicHandle	pictureHdl;	
	SInt16		whichPicture;
	Rect			theRect;

	GetPort(&oldPort);
	SetPort(windowPtr);

	if(activate == true)
		whichPicture = rToolsPict;
	else
		whichPicture = rToolsPictDim;

	if(gColorQuickDrawPresent == false || gColourDisplay == false)
		resourceOffset = 4;

	pictureHdl = GetPicture(whichPicture + resourceOffset);
	theRect = (*pictureHdl)->picFrame;
	OffsetRect(&theRect,- theRect.left, - theRect.top);
	if(gColorQuickDrawPresent == false || gColourDisplay == false)
		EraseRect(&windowPtr->portRect);
	DrawPicture(pictureHdl,&theRect);
	SetWindowPic(windowPtr,pictureHdl);

	SetPort(oldPort);
}

//  coloursActivateHandler

pascal void  coloursActivateHandler(WindowPtr windowPtr,Boolean activate)
{
	GrafPtr		oldPort;
	SInt16		resourceOffset = 0;
	PicHandle	pictureHdl;	
	SInt16		whichPicture;
	Rect			theRect;

	GetPort(&oldPort);
	SetPort(windowPtr);

	if(activate == true)
		whichPicture = rColoursPict;
	else
		whichPicture = rColoursPictDim;

	if(gColorQuickDrawPresent == false || gColourDisplay == false)
		resourceOffset = 4;

	pictureHdl = GetPicture(whichPicture + resourceOffset);
	theRect = (*pictureHdl)->picFrame;
	OffsetRect(&theRect,- theRect.left, - theRect.top);
	if(gColorQuickDrawPresent == false || gColourDisplay == false)
		EraseRect(&windowPtr->portRect);
	DrawPicture(pictureHdl,&theRect);
	SetWindowPic(windowPtr,pictureHdl);

	SetPort(oldPort);
}

//  invalidateScrollBarArea

void  invalidateScrollBarArea(WindowPtr windowPtr)
{
	Rect  tempRect;
  
	SetPort(windowPtr);

	tempRect = windowPtr->portRect;
	tempRect.left = tempRect.right - 15;
	InvalRect(&tempRect);

	tempRect = windowPtr->portRect;
	tempRect.top = tempRect.bottom - 15;
	InvalRect(&tempRect);
}

// 