// 
// Floaters.h
// 
//
// This program opens two floating windows and allows the user to open up to three
// document windows.  The floating windows and the document windows may be closed and
// opened via their close boxes, by choosing the relevant items in the File and Floaters
// menus, and by pressing the relevant Command key equivalents.
//
// The program is supported by specialised floating windows routines in FloatRoutines.c
// and by a custom window definition function which defines the appearance and behaviour
// of the floating windows.
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit and Floaters menus
//		(preload, non-purgeable).
//
// 	'WIND' resources (purgeable) (initially visible) for the document and floating
//		windows.  
//
// 	An 'ALRT' resource (purgeable), and associated 'DITL' resource (purgeable), for
//		an alert invoked by the user choosing the About item in the Apple menu.
//
// 	A 'DLOG' resource (purgeable), and associated 'DITL' resource (purgeable), for a
//		dialog box invoked when the user chooses the Find item in the File menu.
//
// 	'PICT' resources (non-purgeable, colour versions preload) containing pictures to
//		be drawn in the floating windows.
//
// 	A 'snd ' resource (purgeable) containing a sound invoked when the user clicks
//		in the content region of the document and floating windows.
//
// 	A 'STR# ' resource (purgeable) containing text to be drawn in the document windows.
//
// 	A custom 'WDEF' resource (non-purgeable) containing the window definition function
//		utilised by the floating windows.
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch, and
//		is32BitCompatible flags set.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <Resources.h>
#include <Sound.h>
#include <Gestalt.h>
#include <Processes.h>
#include <LowMem.h>

//  defines

#define mApple								128
#define  iAbout								1
#define mFile									129
#define  iNew									1
#define  iClose								4
#define  iFind								11
#define  iQuit								13
#define mEdit									130
#define mFloaters							131
#define  iTools								1
#define  iColours							2
#define rMenubar							128
#define rDocWindow						128
#define rToolsWindow					129
#define rColoursWindow				130
#define rAboutAlert						128
#define rFindDialog						129
#define rToolsPict						128
#define rColoursPict					129
#define rToolsPictDim					130
#define rColoursPictDim				131
#define rSound								8192
#define kDocumentKind					1
#define kFloatingKind					2
#define MAXLONG								0x7FFFFFFF

//  typedefs

typedef pascal void (*ActivateProcPtr) (WindowPtr theWindow,Boolean activate);

typedef struct
{
	SInt16					windowType;
	ActivateProcPtr	activateHandler;
	Boolean					wasVisible;
}	docRecord, **docRecordHandle;

//  function prototypes

// FloaterDemo.c

void						main														(void);
void						doInitManagers									(void);
void						doOpenFloatingWindows						(void);
void						eventLoop												(void);
void						doEvents												(EventRecord *);
void						doMouseDown											(EventRecord *);
void						doUpdate												(EventRecord *);
void						doActivate											(EventRecord *);
void						doOSEvent												(EventRecord *);
void						doAdjustMenus										(void);
void						doMenuChoice										(SInt32);
void						doFileMenu											(SInt16);
void						doFloatersMenu									(SInt16);
void						doOpenDocWindow									(void);
void						doCloseDocWindow								(void);
void						doDrawDocWindowContent					(void);
void						doProvingBeeps									(WindowPtr);
void						doOpenFindDialog								(void);
void						doDisposeFindDialog							(void);
pascal void			docActivateHandler							(WindowPtr,Boolean);
pascal void			toolsActivateHandler						(WindowPtr,Boolean);
pascal void			coloursActivateHandler					(WindowPtr,Boolean);
void						invalidateScrollBarArea					(WindowPtr);

// FloatRoutines.c

WindowPtr				newGetNewWindow									(SInt16,WindowPtr,ActivateProcPtr,SInt16);
void						newDisposeWindow								(WindowPtr);
void						newSelectWindow									(WindowPtr);
void						newHideWindow										(WindowPtr);
void						newShowWindow										(WindowPtr);
void						newDragWindow										(WindowPtr,Point,const Rect *);
void						handleSuspendEvent							(void);
void						handleResumeEvent								(void);
void						deactivateFloatsAndFirstDocWin	(void);
void						activateFloatsAndFirstDocWin		(void);
void						deactivateWindow								(WindowPtr);
void						activateWindow									(WindowPtr);
void						highlightAndActivateWindow			(WindowPtr,Boolean);
WindowPtr				findFrontNonFloatingWindow			(void);
WindowPtr				findLastFloatingWindow					(void);
Boolean					isWindowModal										(WindowPtr);
WindowPtr				getWindowList										(void);
void						setWindowList										(WindowPtr);
WindowPtr				getNextWindow										(WindowPtr);
void						setNextWindow										(WindowPtr,WindowPtr);
Boolean					getWasVisible										(WindowPtr);
void						setWasVisible										(WindowPtr,Boolean);
SInt16					getWindowKind										(WindowPtr);
void						setWindowKind										(WindowPtr,SInt16);
Boolean					getWindowVisible								(WindowPtr);
RgnHandle				getStructureRegion							(WindowPtr);
RgnHandle				getContentRegion								(WindowPtr);
void						setWindowHilite									(WindowPtr,Boolean);
ActivateProcPtr	getActivateHandler							(WindowPtr);
void						setActivateHandler							(WindowPtr,ActivateProcPtr);

// 


