// 
// Miscellany.h
// 
// 
// Miscellany source code is contained in two files, namely, Main.c and Demos.c.  Within
// the CodeWarrior project, Main.c is in Segment 2 and Demos.c is in Segment 3.  (Note
// that this small program does not really require such segmentation; the code is 
// segmented only to facilitate demonstration of the Segment Loader aspects.
//
// This program demonstrates:
//
// 	The use of stubs in code segments, together with a function which uses those stubs
//		to unlock code segments and make them purgeable. 
//
// 	The use of a status bar to graphically indicate the current status of a time-
//		consuming operation.
//
// 	The use of the Command-period key combination to terminate a time-consuming
//		operation before it concludes.
//
// 	The use of the Notification Manager to allow an application running in the 
//		background to communicate with the foreground application.
//
// 	The determination of whether a particular application is currently the foreground
//		application.
//
// 	The use of the Color Picker to solicit a choice of colour from the user.
//
// 	The determination of whether a particular trap is available.
//
// 	Image drawing optimisation and window zooming in a multi-monitors environment. 
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit and Demonstration
//		menus (preload, non-purgeable).
//
// 	A 'WIND' resource (purgeable) (initially visible) for a window in which graphics
//		and information relevant to the demonstrations is displayed.
//
// 	An 'ALRT' resource (purgeable), and associated 'DITL' resource (purgeable), for
//		displaying a message to the user from within the Notification Manager demonstration.
//
// 	A 'DLOG' resource (purgeable), and associated 'DITL' and 'dctb' resources
//		(purgeable), for a dialog box in which the status bar is displayed.
//
// 	'icn#', 'ics4', and 'ics8' resources (non-purgeable) which contain the application
//		icon shown in the Application menu during the Notification Manager demonstration.
//
// 	A 'snd ' resource (non-purgeable) used in the Notification Manager demonstration.
//
// 	A 'STR ' resource (non-purgeable) containing the text displayed in the alert box
//		invoked by the Notification Manager.
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch,
//		canBackgound, and is32BitCompatible flags set.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <Processes.h>
#include <Gestalt.h>
#include <Resources.h>
#include <Sound.h>
#include <Notification.h>
#include <Icons.h>
#include <Processes.h>
#include <ColorPicker.h>
#include <Traps.h>
#include <LowMem.h>

//  defines

#define mApple					128
#define  iAbout					1
#define mFile						129
#define  iQuit					11
#define mDemonstration	131
#define  iCommandPeriod	1
#define  iNotification	2
#define  iColourPicker	3
#define  iTrapAvailable	4
#define  iMultiMonitors	5
#define rMenubar				128
#define rWindow					128
#define rAlert					128
#define rDialog					129
#define  iUserItem			1
#define rIconFamily			128
#define rBarkSound			8192
#define rString					128
#define topLeft(r)			(((Point *) &(r))[0])
#define botRight(r)			(((Point *) &(r))[1])

//  function prototypes

// Main.c

void				main													(void);
void				doInitManagers								(void);
void				doEvents											(EventRecord *);
void				doMouseDown										(EventRecord *);
void				doKeyDown											(UInt8);
void				doMenuChoice									(SInt32 menuChoice);
void				unloadSegments								(void);

// Miscellany.c

void				demosSegment									(void);
void  			doCommandPeriodAndStatusBar		(void);
Boolean			doCheckForCommandPeriod				(void);
void				doDrawStatusBar								(DialogPtr,Rect,SInt16,SInt16);
void				doSetUpNotification						(void);
void				doPrepareNotificationRecord		(void);
void				doNullEvent										(void);
void				doOSEvent											(EventRecord *);
void				doDisplayMessageToUser				(void);
void				doColourPicker								(void);
char 			*doDecimalToHexadecimal					(UInt16 n);
Boolean			doCheckSlotVInstallAvailable	(void);
Boolean			trapAvailable									(SInt16 theTrap);
pascal void	doDeviceLoopDraw							(SInt16,SInt16,GDHandle,SInt32);
void				doZoomWindowMultiMonitors			(WindowPtr,SInt16);
void				doRedoWindowContent						(WindowPtr);

// 

