// 
// FilesPPC.c
// 
//
// This program demonstrates application-defined file handling functions associated with:
//
// 	The user invoking the File menu Open..., Close, Save..., Save As, Revert to Saved,
//		and Quit commands of a typical application.
//
// 	Receipt of the required Apple events Open Application, Open Documents and Quit
//		Application.
//
// The uncompiled program may be run from within CodeWarrior to demonstrate responses to
// the File menu commands.  The built application, together with supplied TEXT and a PICT
// files, may be used to demonstrate the additional aspect of integrating the receipt
// of required Apple events with the overall file handling mechanism.
//
// To keep the code not specifically related to files and file-handling to a minimim, a 
// Demonstration menu is included to allow the user to simulate making a window
// "dirty", that is, modifying the contents of the associated document.  Choosing the
// single menu item in this menu sets the window-dirty flag in the window's document
// record to true and draws a diagonal line across the window, this latter so that the
// user can keep track of which windows are "dirty".
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit and Demonstration
//		menus (preload, non-purgeable).  
//
// 	A 'WIND' resource (purgeable) (initially not visible).  
//
// 	Three 'ALRT' resources (purgeable) and associated 'DITL' resources (purgeable).
//		The first alert is used to display error messages.  The second is used to support
//		the Revert to Saved menu item.  The third is used when an attempt is made to close
//		a modified document before that document has been saved.     
//
// 	A 'STR ' resource (purgeable) containing the "missing application name" string 
//		resource, which is copied to all document files created by the program.  
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, isHighLevelEventAware, and
//		is32BitCompatible flags set (non-purgeable).	  
//
// 	The 'BNDL' resource (non-purgeable), 'FREF' resources (non-purgeable), signature
//		resource (non-purgeable), and icon family resources (purgeable), required to 
//		support the built application.
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <AppleEvents.h>
#include <StandardFile.h>
#include <Folders.h>
#include <Resources.h>
#include <Script.h>

//  typedefs

typedef struct
{
	TEHandle	editRec;
	PicHandle	pictureHdl;
	SInt16		fileRefNum;
	FSSpec		fileFSSpec;
	Boolean		windowDirty;
}	docRecord, *docRecordPointer, **docRecordHandle;

//  defines

#define mApple						128
#define  iAbout						1
#define mFile							129
#define  iNew							1
#define  iOpen						2
#define  iClose						4
#define  iSave						5
#define  iSaveAs					6
#define  iRevert					7
#define  iQuit						12
#define mDemonstration		131
#define  iDirty						1
#define rNewWindow				128
#define rMenubar					128
#define rErrorAlert				128
#define  eInstallHandler	1000
#define  eMaxWindows			1001
#define  eFileIsOpen		 	-49
#define rRevertAlert			129
#define rCloseFileAlert		130	
#define kMaxWindows				10
#define kUserCancelled		1002
#define MAXLONG						0x7FFFFFFF

//  global variables

Boolean		gDone;
Boolean		gInBackground;
WindowPtr	gWindowPtr;
SInt16		gCurrentNumberOfWindows = 0;
Rect			gDestRect,gViewRect;
SInt16		gAppResFileRefNum;

AEEventHandlerUPP	doOpenAppEventRD;																						 // PowerPC
AEEventHandlerUPP	doOpenDocsEventRD;																					 // PowerPC
AEEventHandlerUPP	doQuitAppEventRD;																						 // PowerPC

//  function prototypes

void	main								(void);
void	eventLoop						(void);
void	doInitManagers			(void);
void	doInstallAEHandlers	(void);
void	doEvents						(EventRecord *);
void	doMouseDown					(EventRecord *);
void	doUpdate						(EventRecord *);
void	doMenuChoice				(SInt32);
void	doFileMenu					(SInt16);
void	doAdjustMenus				(void);
void	doError							(SInt16);
void	doMakeWindowDirty		(void);

pascal OSErr	doOpenAppEvent				(AppleEvent *,AppleEvent *,SInt32);
pascal OSErr	doOpenDocsEvent				(AppleEvent *,AppleEvent *,SInt32);
pascal OSErr	doQuitAppEvent				(AppleEvent *,AppleEvent *,SInt32);
OSErr					hasGotRequiredParams	(AppleEvent *);

OSErr	doNewCommand					(void);
OSErr	doOpenCommand					(void);
OSErr	doCloseCommand				(void);
OSErr	doSaveCommand					(void);
OSErr	doSaveAsCommand				(void);
OSErr	doRevertCommand				(void);
OSErr	doQuitCommand					(void);
OSErr	doNewDocWindow				(Boolean,OSType);
OSErr	doOpenFile						(FSSpec,OSType);
OSErr	doReadTextFile				(WindowPtr);
OSErr	doReadPictFile				(WindowPtr);
OSErr	doCloseFile						(WindowPtr,docRecordHandle);
OSErr	doWriteFile						(WindowPtr,Boolean);
OSErr	doWriteTextData				(WindowPtr,SInt16);
OSErr	doWritePictData				(WindowPtr,SInt16);
OSErr	doCopyAppNameResource	(WindowPtr);
OSErr	doCopyResource				(ResType,SInt16,SInt16,SInt16);

//  main

void  main(void)
{
	Handle			menubarHdl;
	MenuHandle	menuHdl;

	// initialise managers

	doInitManagers();

	//  create routine decriptors

	doOpenAppEventRD		= NewAEEventHandlerProc((ProcPtr) doOpenAppEvent);			 // PowerPC
	doOpenDocsEventRD		= NewAEEventHandlerProc((ProcPtr) doOpenDocsEvent);			 // PowerPC
	doQuitAppEventRD		= NewAEEventHandlerProc((ProcPtr) doQuitAppEvent);			 // PowerPC

	//  set application's resource fork as current resource file

	gAppResFileRefNum = CurResFile();

	//  set up menu bar and menus
	
	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		doError(MemError());
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		doError(MemError());
	else
		AppendResMenu(menuHdl,'DRVR');

	//  install required Apple event handlers

	doInstallAEHandlers();
	
	//  enter event loop

	eventLoop();
}

//  eventLoop

void  eventLoop(void)
{
	EventRecord eventRec;

	gDone = false;

	while(!gDone)
	{
		if(WaitNextEvent(everyEvent,&eventRec,MAXLONG,NULL))
			doEvents(&eventRec);
	}
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);

	InitCursor();
	FlushEvents(everyEvent,0);
}

//  doInstallAEHandlers

void  doInstallAEHandlers(void)
{
	OSErr	err;

	err=AEInstallEventHandler(kCoreEventClass,kAEOpenApplication,
													  doOpenAppEventRD,0L,false);												 // PowerPC
	if(err != noErr)	doError(eInstallHandler);

	err=AEInstallEventHandler(kCoreEventClass,kAEOpenDocuments,
													  doOpenDocsEventRD,0L,false);											 // PowerPC
	if(err != noErr)	doError(eInstallHandler);

	err=AEInstallEventHandler(kCoreEventClass,kAEQuitApplication,
													  doQuitAppEventRD,0L,false);												 // PowerPC
	if(err != noErr)	doError(eInstallHandler);
}

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	UInt8	charCode;

	switch(eventRecPtr->what)
	{
		case kHighLevelEvent:
			AEProcessAppleEvent(eventRecPtr);
			break;

		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
			{
				doAdjustMenus();
				doMenuChoice(MenuKey(charCode));
			}
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case osEvt:
			switch((eventRecPtr->message >> 24) & 0x000000FF)
			{
				case suspendResumeMessage:
					gInBackground = (eventRecPtr->message & resumeFlag) == 0;
					break;
			}
			HiliteMenu(0);
			break;
	}
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);
	
	switch(partCode)
	{
		case inMenuBar:
			doAdjustMenus();
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where) == true)
				doCloseCommand();
			break;
	}
}

//  doUpdate

void  doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr				windowPtr;
	docRecordHandle	docRecHdl;
	GrafPtr					oldPort;
	Rect						destRect;

	windowPtr = (WindowPtr) eventRecPtr->message;
	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);

	GetPort(&oldPort);
	SetPort(windowPtr);

	BeginUpdate(windowPtr);

	if((*docRecHdl)->pictureHdl)
	{
		destRect=(*(*docRecHdl)->pictureHdl)->picFrame;
		HLock((Handle) (*docRecHdl)->pictureHdl);
		DrawPicture((*docRecHdl)->pictureHdl,&destRect);
		HUnlock((Handle) (*docRecHdl)->pictureHdl);
	}
	else if((*docRecHdl)->editRec)
	{
		HLock((Handle) (*docRecHdl)->editRec);
		TEUpdate(&gDestRect,(*docRecHdl)->editRec);
		HUnlock((Handle) (*docRecHdl)->editRec);
	}

	if((*docRecHdl)->windowDirty)
	{
		MoveTo(windowPtr->portRect.left,windowPtr->portRect.top);
		LineTo(windowPtr->portRect.right,windowPtr->portRect.bottom);
	}

	EndUpdate((WindowPtr)eventRecPtr->message);

	SetPort(oldPort);
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16	menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
			daDriverRefNum = OpenDeskAcc(itemName);
			break;

		case mFile:
			doFileMenu(menuItem);
			break;

		case mDemonstration:
			doMakeWindowDirty();
			break;
	}

	HiliteMenu(0);
}

//  doFileMenu

void	doFileMenu(SInt16 menuItem)
{
	OSErr	osError;

	switch(menuItem)
	{
		case iNew:
			if(osError = doNewCommand())
				doError(osError);
			break;

		case iOpen:
			if(osError = doOpenCommand())
				doError(osError);
			break;

		case iClose:
			if((osError = doCloseCommand()) && osError != kUserCancelled)
				doError(osError);
			break;

		case iSave:
			if(osError = doSaveCommand())
				doError(osError);
			break;

		case iSaveAs:
			if(osError = doSaveAsCommand())
				doError(osError);
			break;

		case iRevert:
			if(osError = doRevertCommand())
				doError(osError);
			break;

		case iQuit:
			if((osError = doQuitCommand()) && osError != kUserCancelled)
				doError(osError);
			if(osError != kUserCancelled)
				gDone = true;
			break;
	}
}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	MenuHandle			menuHdl;
	WindowPtr				windowPtr;
	docRecordHandle	docRecHdl;

	windowPtr = FrontWindow();
	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);

	menuHdl = GetMenuHandle(mFile);

	if(gCurrentNumberOfWindows > 0)
	{
		menuHdl = GetMenuHandle(mFile);
		EnableItem(menuHdl,iClose);
		EnableItem(menuHdl,iSave);
		EnableItem(menuHdl,iSaveAs);
		EnableItem(menuHdl,iRevert);

		if(((*docRecHdl)->pictureHdl == NULL) && ((*(*docRecHdl)->editRec)->teLength == 0))
			DisableItem(menuHdl,iRevert);

		menuHdl = GetMenuHandle(mDemonstration);
		if((*docRecHdl)->windowDirty == false)
		
			EnableItem(menuHdl,iDirty);
		else
			DisableItem(menuHdl,iDirty);
	}
	else
	{
		menuHdl = GetMenuHandle(mFile);
		DisableItem(menuHdl,iClose);
		DisableItem(menuHdl,iSave);
		DisableItem(menuHdl,iSaveAs);
		DisableItem(menuHdl,iRevert);
		menuHdl = GetMenuHandle(mDemonstration);
		DisableItem(menuHdl,iDirty);
	}	

	DrawMenuBar();
}

//  doError

void  doError(SInt16 errorCode)
{
	Str255	errorString;

	if(errorCode == eInstallHandler)
		ParamText("\pError installing Required Apple event handlers",NULL,NULL,NULL);
	else if(errorCode == eMaxWindows)
		ParamText("\pSorry.  No more windows",NULL,NULL,NULL);
	else if(errorCode == eFileIsOpen)
		ParamText("\pFile is currently open",NULL,NULL,NULL);		
	else
	{
		NumToString((SInt32) errorCode,errorString);
		ParamText("\pAn error occurred.  The error number is ",errorString,NULL,NULL);
	}
	
	if(errorCode == memFullErr)
	{
		StopAlert(rErrorAlert,NULL);
			ExitToShell();
	}
	else	
		CautionAlert(rErrorAlert,NULL);
}

//  doMakeWindowDirty

void	doMakeWindowDirty(void)
{
	WindowPtr				windowPtr;
	docRecordHandle	docRecHdl;

	windowPtr = FrontWindow();
	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);
	
	SetPort(windowPtr);
	MoveTo(windowPtr->portRect.left,windowPtr->portRect.top);
	LineTo(windowPtr->portRect.right,windowPtr->portRect.bottom);

	(*docRecHdl)->windowDirty = true;
}

//  doOpenAppEvent

pascal OSErr  doOpenAppEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefCon)
{
	OSErr			osError;

	osError = hasGotRequiredParams(appEvent);
	if(osError == noErr)
		osError = doNewCommand();

	return(osError);
}

//  doOpenDocsEvent

pascal OSErr  doOpenDocsEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefcon)
{
	FSSpec			fileSpec;
	AEDescList	docList;
	OSErr				osError, ignoreErr;
	SInt32			index, numberOfItems;
	Size				actualSize;
	AEKeyword		keyWord;
	DescType		returnedType;
	FInfo				fileInfo;

	osError = AEGetParamDesc(appEvent,keyDirectObject,typeAEList,&docList);

	if(osError == noErr)
	{
		osError = hasGotRequiredParams(appEvent);
		if(osError == noErr)
		{
			AECountItems(&docList,&numberOfItems);
			if(osError == noErr)
			{
				for(index=1;index<=numberOfItems;index++)
				{
					osError = AEGetNthPtr(&docList,index,typeFSS,&keyWord,&returnedType,
															(Ptr) &fileSpec,sizeof(fileSpec),&actualSize);
					if(osError == noErr)
					{
						osError = FSpGetFInfo(&fileSpec,&fileInfo);
						if(osError == noErr)
						{
							if(osError = doOpenFile(fileSpec,fileInfo.fdType))
								doError(osError);
						}
					}
					else
						doError(osError);
				}
			}
		}
		else
			doError(osError);

		ignoreErr = AEDisposeDesc(&docList);
	}
	else
		doError(osError);

	return(osError);
}

//  doQuitAppEvent

pascal OSErr  doQuitAppEvent(AppleEvent *appEvent,AppleEvent *reply,SInt32 handlerRefcon)
{
	OSErr		osError;

	osError = hasGotRequiredParams(appEvent);
	if(osError == noErr)
	{
		while(FrontWindow())
		{
			osError = doCloseCommand();
			if(osError != noErr && osError != kUserCancelled)
				doError(osError);
			if(osError == kUserCancelled)
				eventLoop();
		}
	}

	gDone = true;

	return(osError);
}

//  hasGotRequiredParams

OSErr  hasGotRequiredParams(AppleEvent *appEvent)
{
	DescType	returnedType;
	Size			actualSize;
	OSErr			osError;

	osError = AEGetAttributePtr(appEvent,keyMissedKeywordAttr,typeWildCard,&returnedType,
														NULL,0,&actualSize);
	if(osError == errAEDescNotFound)
		return(noErr);
	else if(osError == noErr)
		return(errAEParamMissed);
}

//  doNewCommand

OSErr  doNewCommand(void)
{
	OSErr 		osError;
	OSType		documentType = 'TEXT';

	osError = doNewDocWindow(true,documentType);
	return(osError);
}

//  doOpenCommand

OSErr  doOpenCommand(void)
{
	SFTypeList				fileTypes;
	StandardFileReply	fileReply;
	OSType						documentType;
	OSErr							osError = 0;		

	fileTypes[0] = 'TEXT';
	fileTypes[1] = 'PICT';

	StandardGetFile(NULL,2,fileTypes,&fileReply);

	documentType = fileReply.sfType;

	if(fileReply.sfGood)
		osError = doOpenFile(fileReply.sfFile,documentType);

	return(osError);
}

//  doCloseCommand

OSErr  doCloseCommand(void)
{
	WindowPtr				windowPtr;
	SInt16					windowKind;
	docRecordHandle	docRecHdl;
	OSErr						osError = 0;

	windowPtr = FrontWindow();
	windowKind = ((WindowPeek) windowPtr)->windowKind;

	switch(windowKind)
	{
		case userKind:
			docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);
			osError = doCloseFile(windowPtr,docRecHdl);
			if(osError == kUserCancelled)
				return(kUserCancelled);
			else if(osError == noErr)
			{
				DisposeWindow(windowPtr);
				gCurrentNumberOfWindows --;
			}
			break;
			
		case dialogKind:
			// Hide or close modeless dialog, as required.
			break;
	}

	return(osError);
}

//  doSaveCommand

OSErr  doSaveCommand(void)
{
	WindowPtr				windowPtr;
	docRecordHandle	docRecHdl;
	OSErr						osError = 0;
	
	windowPtr = FrontWindow();
	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);

	if((*docRecHdl)->fileRefNum)
		osError = doWriteFile(windowPtr,false);
	else
		osError = doSaveAsCommand();

	return(osError);
}

//  doSaveAsCommand

OSErr  doSaveAsCommand(void)
{
	WindowPtr					windowPtr;
	docRecordHandle		docRecHdl;
	StandardFileReply	fileReply;
	OSType						fileType;
	SInt16						fileRefNum;
	OSErr							osError = 0;

	windowPtr = FrontWindow();
	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);

	StandardPutFile("\pSave as:","\pUntitled",&fileReply);

	if(fileReply.sfGood)
	{
		if(!(fileReply.sfReplacing))
		{
			if((*docRecHdl)->editRec)
				fileType = 'TEXT';
			else if((*docRecHdl)->pictureHdl)
				fileType = 'PICT';
			osError = FSpCreate(&fileReply.sfFile,'BJKK',fileType,smSystemScript);
			if(osError != noErr)
				return(osError);
		}

		(*docRecHdl)->fileFSSpec = fileReply.sfFile;

		if((*docRecHdl)->fileRefNum != 0)
		{
			osError = FSClose((*docRecHdl)->fileRefNum);
			(*docRecHdl)->fileRefNum = 0;
		}

		if(osError == noErr)
			osError = FSpOpenDF(&(*docRecHdl)->fileFSSpec,fsRdWrPerm,&fileRefNum);

		if(osError == noErr)
		{
			(*docRecHdl)->fileRefNum = fileRefNum;
			SetWTitle(windowPtr,fileReply.sfFile.name);
			osError = doWriteFile(windowPtr,true);
		}
	}

	return(osError);
}

//  doRevertCommand

OSErr  doRevertCommand(void)
{
	WindowPtr				windowPtr;
	docRecordHandle	docRecHdl;
	SInt16					fileRefNum;
	Str255					fileName;
	SInt16					itemHit;
	OSErr						osError = 0;
	
	windowPtr = FrontWindow();
	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);
	fileRefNum = (*docRecHdl)->fileRefNum;

	SetPort(windowPtr);

	GetWTitle(windowPtr,fileName);
	ParamText(fileName,NULL,NULL,NULL);

	SetPort(windowPtr);

	itemHit = CautionAlert(rRevertAlert,NULL);

	if(itemHit == 1)
	{
		EraseRect(&windowPtr->portRect);
		if((*docRecHdl)->editRec)
			osError = doReadTextFile(windowPtr);
		else if((*docRecHdl)->pictureHdl)
		{
			KillPicture((*docRecHdl)->pictureHdl);
			(*docRecHdl)->pictureHdl = NULL;
			osError = doReadPictFile(windowPtr);
		}

		(*docRecHdl)->windowDirty = false;

		InvalRect(&windowPtr->portRect);
	}

	return(osError);
}

//  doQuitCommand

OSErr  doQuitCommand(void)
{
	OSErr	osError = 0;

	while(FrontWindow())
	{
		osError = doCloseCommand();
		if(osError != noErr)
			return(osError);
	}

	return(osError);
}

//  doNewDocWindow

OSErr  doNewDocWindow(Boolean showWindow,OSType documentType)
{
	docRecordHandle	docRecHdl;

	if(gCurrentNumberOfWindows == kMaxWindows)
		return(eMaxWindows);

	if(!(gWindowPtr = GetNewWindow(rNewWindow,NULL,(WindowPtr)-1)))
		return(MemError());

	SetPort(gWindowPtr);

	if(!(docRecHdl = (docRecordHandle) NewHandle(sizeof(docRecord))))
	{
		DisposeWindow(gWindowPtr);
		gCurrentNumberOfWindows --;
		return(MemError());
	}

	SetWRefCon(gWindowPtr,(SInt32) docRecHdl);

	(*docRecHdl)->editRec = NULL;
	(*docRecHdl)->pictureHdl = NULL;
	(*docRecHdl)->fileRefNum = 0;
	(*docRecHdl)->windowDirty = false;

	if(documentType == 'TEXT')
	{
		gDestRect = gWindowPtr->portRect;
		InsetRect(&gDestRect,6,6);
		gViewRect = gDestRect;

		MoveHHi((Handle) docRecHdl);
		HLock((Handle) docRecHdl);

		if(!((*docRecHdl)->editRec = TENew(&gDestRect,&gViewRect)))
		{
			DisposeWindow(gWindowPtr);
			gCurrentNumberOfWindows --;
			DisposeHandle((Handle) docRecHdl);
			return(MemError());
		}

		HUnlock((Handle) docRecHdl);
	}

	if(showWindow)
		ShowWindow(gWindowPtr);

	gCurrentNumberOfWindows ++;

	return(noErr);
}

//  doOpenFile

OSErr  doOpenFile(FSSpec fileSpec,OSType documentType)
{
	OSErr						osError;
	SInt16					fileRefNum;
	docRecordHandle	docRecHdl;

	if(osError = doNewDocWindow(false,documentType))
		return(osError);

	SetWTitle(gWindowPtr,fileSpec.name);	

	if(osError = FSpOpenDF(&fileSpec,fsRdWrPerm,&fileRefNum))
	{
		DisposeWindow(gWindowPtr);
		gCurrentNumberOfWindows --;
		return(osError);
	}
	
	docRecHdl = (docRecordHandle) GetWRefCon(gWindowPtr);
	(*docRecHdl)->fileRefNum = fileRefNum;
	(*docRecHdl)->fileFSSpec = fileSpec;

	if(documentType == 'TEXT')
	{
		if(osError = doReadTextFile(gWindowPtr))
			return(osError);
	}
	else if(documentType == 'PICT')
	{
		if(osError = doReadPictFile(gWindowPtr))
			return(osError);
	}

	ShowWindow(gWindowPtr);

	return(noErr);
}

//  doCloseFile

OSErr  doCloseFile(WindowPtr windowPtr,docRecordHandle docRecHdl)
{
	Str255	fileName;
	SInt16	itemHit;
	OSErr		osError;

	if((*docRecHdl)->windowDirty)
	{
		GetWTitle(windowPtr,fileName);
		ParamText(fileName,NULL,NULL,NULL);

		itemHit = CautionAlert(rCloseFileAlert,NULL);
		if(itemHit == 2)
			return(kUserCancelled);
		else if(itemHit == 1)
		{
			if(osError = doSaveCommand())
				return(osError);
		}
	}

	if((*docRecHdl)->fileRefNum != 0)
	{
		if(!(osError = FSClose((*docRecHdl)->fileRefNum)))	
		{
			osError = FlushVol(NULL,(*docRecHdl)->fileFSSpec.vRefNum);
			(*docRecHdl)->fileRefNum = 0;
		}
	}

	if((*docRecHdl)->editRec)
		TEDispose((*docRecHdl)->editRec);
	if((*docRecHdl)->pictureHdl)
		KillPicture((*docRecHdl)->pictureHdl);

	DisposeHandle((Handle) docRecHdl);
	
	return(osError);
}

//  doWriteFile

OSErr  doWriteFile(WindowPtr windowPtr,Boolean newFile)
{
	docRecordHandle	docRecHdl;
	FSSpec					fileSpecActual, fileSpecTemp;
	UInt32					currentTime;
	Str255					tempFileName;
	SInt16					tempFileVolNum, tempFileRefNum;
	SInt32					tempFileDirID;
	OSErr						osError;

	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);
	fileSpecActual = (*docRecHdl)->fileFSSpec;

	GetDateTime(&currentTime);
	NumToString((SInt32) currentTime,tempFileName);
	
	osError = FindFolder(fileSpecActual.vRefNum,kTemporaryFolderType,kCreateFolder,
												&tempFileVolNum,&tempFileDirID);
	if(osError == noErr)
		osError = FSMakeFSSpec(tempFileVolNum,tempFileDirID,tempFileName,&fileSpecTemp);
	if(osError == noErr || osError == fnfErr)
		osError = FSpCreate(&fileSpecTemp,'trsh','trsh',smSystemScript);
	if(osError == noErr)
		osError = FSpOpenDF(&fileSpecTemp,fsRdWrPerm,&tempFileRefNum);
	if(osError == noErr)
	{
		if((*docRecHdl)->editRec)
			osError = doWriteTextData(windowPtr,tempFileRefNum);
		else if((*docRecHdl)->pictureHdl)
			osError = doWritePictData(windowPtr,tempFileRefNum);
	}	
	if(osError == noErr)
		osError = FSClose(tempFileRefNum);
	if(osError == noErr)
		osError = FSClose((*docRecHdl)->fileRefNum);
	if(osError == noErr)
		osError = FSpExchangeFiles(&fileSpecTemp,&fileSpecActual);
	if(osError == noErr)
		osError = FSpDelete(&fileSpecTemp);
	if(osError == noErr)
		osError = FSpOpenDF(&fileSpecActual,fsRdWrPerm,&(*docRecHdl)->fileRefNum);

	if(osError == noErr)
	{
		if(newFile)
			osError = doCopyAppNameResource(windowPtr);
	}

	return(osError);
}

//  doReadTextFile

OSErr  doReadTextFile(WindowPtr windowPtr)
{
	docRecordHandle	docRecHdl;
	SInt16					fileRefNum;
	TEHandle				textEditHdl;
	SInt32					numberOfBytes;
	Handle					textBuffer;
	OSErr						osError;

	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);
	fileRefNum = (*docRecHdl)->fileRefNum;

	textEditHdl = (*docRecHdl)->editRec;
	(*textEditHdl)->txSize = 10;
	(*textEditHdl)->lineHeight = 15;

	SetFPos(fileRefNum,fsFromStart,0);
	GetEOF(fileRefNum,&numberOfBytes);

	if(numberOfBytes > 32767)
		numberOfBytes = 32767;

	if(!(textBuffer = NewHandle((Size) numberOfBytes)))
		return(MemError());

	osError = FSRead(fileRefNum,&numberOfBytes,*textBuffer);
	if(osError == noErr || osError == eofErr)
	{
		MoveHHi(textBuffer);
		HLockHi(textBuffer);
		TESetText(*textBuffer,numberOfBytes,(*docRecHdl)->editRec);
		HUnlock(textBuffer);
		DisposeHandle(textBuffer);
	}
	else
		return(osError);

	return(noErr);
}

//  doReadPictFile

OSErr  doReadPictFile(WindowPtr windowPtr)
{
	docRecordHandle	docRecHdl;
	SInt16					fileRefNum;
	SInt32					numberOfBytes;
	OSErr						osError;

	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);
	fileRefNum = (*docRecHdl)->fileRefNum;

	GetEOF(fileRefNum,&numberOfBytes);
	SetFPos(fileRefNum,fsFromStart,512);
	numberOfBytes -= 512;

	if(!((*docRecHdl)->pictureHdl = (PicHandle) NewHandle(numberOfBytes)))
		return(MemError());

	osError = FSRead(fileRefNum,&numberOfBytes,*(*docRecHdl)->pictureHdl);
	if(osError == noErr || osError == eofErr)
		return(noErr);
	else
		return(osError);
}

//  doWriteTextData

OSErr  doWriteTextData(WindowPtr windowPtr,SInt16 tempFileRefNum)
{
	docRecordHandle	docRecHdl;
	TEHandle				textEditHdl;
	Handle					editText;
	SInt32					numberOfBytes;
	SInt16					volRefNum;
	OSErr						osError;

	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);
	textEditHdl = (*docRecHdl)->editRec;
	editText = (*textEditHdl)->hText;
	numberOfBytes = (*textEditHdl)->teLength;

	osError = SetFPos(tempFileRefNum,fsFromStart,0);
	if(osError == noErr)
		osError = FSWrite(tempFileRefNum,&numberOfBytes,*editText);
	if(osError == noErr)
		osError = SetEOF(tempFileRefNum,numberOfBytes);
	if(osError == noErr)
		osError = GetVRefNum(tempFileRefNum,&volRefNum);
	if(osError == noErr)
		osError = FlushVol(NULL,volRefNum);

	if(osError == noErr)
		(*docRecHdl)->windowDirty = false;

	return(osError);
}

//  doWritePictData

OSErr  doWritePictData(WindowPtr windowPtr,SInt16 tempFileRefNum)
{
	docRecordHandle	docRecHdl;
	PicHandle				pictureHdl;
	SInt32					numberOfBytes, dummyData;
	SInt16					volRefNum;
	OSErr						osError;

	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);
	pictureHdl = (*docRecHdl)->pictureHdl;

	numberOfBytes = 512;
	dummyData = 0;

	osError = SetFPos(tempFileRefNum,fsFromStart,0);

	if(osError == noErr)
		osError = FSWrite(tempFileRefNum,&numberOfBytes,&dummyData);

	numberOfBytes = GetHandleSize((Handle) (*docRecHdl)->pictureHdl);

	if(osError == noErr)
	{
		HLock((Handle) (*docRecHdl)->pictureHdl);
		osError = FSWrite(tempFileRefNum,&numberOfBytes,*(*docRecHdl)->pictureHdl);
		HUnlock((Handle) (*docRecHdl)->pictureHdl);
	}

	if(osError == noErr)
		osError = SetEOF(tempFileRefNum,512 + numberOfBytes);
	if(osError == noErr)
		osError = GetVRefNum(tempFileRefNum,&volRefNum);
	if(osError == noErr)
		osError = FlushVol(NULL,volRefNum);

	if(osError == noErr)
		(*docRecHdl)->windowDirty = false;

	return(osError);
}

//  doCopyAppNameResource

OSErr  doCopyAppNameResource(WindowPtr windowPtr)
{
	docRecordHandle	docRecHdl;
	OSType					fileType;
	OSErr						osError;
	SInt16					fileRefNum;

	docRecHdl = (docRecordHandle) GetWRefCon(windowPtr);

	if((*docRecHdl)->editRec)
		fileType = 'TEXT';
	else if((*docRecHdl)->pictureHdl)
		fileType = 'PICT';

	FSpCreateResFile(&(*docRecHdl)->fileFSSpec,'BJKK',fileType,smSystemScript);

	osError = ResError();
	if(osError == noErr)
		fileRefNum = FSpOpenResFile(&(*docRecHdl)->fileFSSpec,fsRdWrPerm);

	if(fileRefNum > 0)
		osError = doCopyResource('STR ',-16396,gAppResFileRefNum,fileRefNum);
	else
		osError = ResError();

	if(osError == noErr)
		CloseResFile(fileRefNum); 

	osError = ResError();
	return(osError);
}

//  doCopyResource

OSErr  doCopyResource(ResType resourceType,SInt16 resourceID,SInt16 sourceFileRefNum,
											SInt16 destFileRefNum)
{
	Handle	sourceResourceHdl;
	Str255	sourceResourceName;
	ResType	ignoredType;
	SInt16	ignoredID;

	UseResFile(sourceFileRefNum);

	sourceResourceHdl = GetResource(resourceType,resourceID);

	if(sourceResourceHdl != NULL)
	{
		GetResInfo(sourceResourceHdl,&ignoredID,&ignoredType,sourceResourceName);
		DetachResource(sourceResourceHdl);
		UseResFile(destFileRefNum);
		AddResource(sourceResourceHdl,resourceType,resourceID,sourceResourceName);
		if(ResError() == noErr)
			UpdateResFile(destFileRefNum);
	}

	ReleaseResource(sourceResourceHdl);

	return(ResError());
}

// 