// 
// Text1PPC.c
// 
//
// This program demonstrates:
//
// 	A "bare-bones" monostyled text editor.
//
// 	A Help dialog which features the integrated scrolling of multistyled text and 
//		pictures.
//
// In the monostyled text editor demonstration, a panel is displayed at the bottom of all
// opened windows.  This panel displays the edit record length, number of lines, line 
// height, destination rectangle (top), scroll bar value, and scroll bar maximum value. 
//
// The bulk of the source code for the Help dialog is contained in the file HelpDialog.c.
// The dialog itself displays information intended to assist the user in adapting the
// Help dialog source code and resources to the requirements of his/her own application.
//
// The program utilises the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit, and Help dialog
//		pop-up menus (preload, non-purgeable).  
//
// 	A 'WIND' resource (purgeable) (initially visible).  
//
// 	'CNTL' resources (purgeable) for the vertical scroll bars in the text editor window
//		and Help dialog, and for the pop-up menu in the Help Dialog.
//
// 	An 'ALRT' resource (purgeable), and associated 'DITL' resource (purgeable), for the
//		display of error messages.
//     
// 	A 'DLOG' resource (purgeable, initially invisible) and associated 'dctb' resource 
//		(purgeable) for the Help dialog.
//
// 	'DITL' resources (purgeable) for the 'ALRT' and 'DLOG' resources.
//
// 	'TEXT' and associated 'styl' resources (all purgeable) for the Help dialog.
//
// 	'PICT' resources (purgeable) for the Help dialog.
//
// 	A 'STR ' resource  (purgeable) containing error text strings.  
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch, and
//		is32BitCompatible	flags set.	  
//
// 

//  includes

#include <Fonts.h>
#include <Menus.h>
#include <TextEdit.h>
#include <Dialogs.h>
#include <SegLoad.h>
#include <ToolUtils.h>
#include <Devices.h>
#include <Scrap.h>
#include <StandardFile.h>
#include <Balloons.h>	
#include <LowMem.h>

//  defines

#define mApple					128
#define  iAbout					1
#define mFile						129
#define  iNew						1
#define  iOpen					2
#define  iClose					4
#define  iSaveAs				6
#define  iQuit					12
#define mEdit						130
#define  iUndo					1
#define  iCut						3
#define  iCopy					4
#define  iPaste					5
#define  iClear					6
#define  iSelectAll			7
#define kMaxTELength		32767
#define kTab						0x09
#define kDel						0x7F
#define kReturn					0x0D
#define rWindow					128
#define rMenubar				128
#define rVScrollbar			128
#define rErrorAlert			128
#define rErrorStrings		128
#define  eMenuBar				1
#define  eMenu					2
#define  eWindow				3
#define  eDocRecord			4
#define  eEditRecord		5
#define  eExceedChara		6
#define  eNoSpaceCut		7
#define  eNoSpacePaste	8

#define topLeft(r)			(((Point *) &(r))[0])
#define botRight(r)			(((Point *) &(r))[1])

//  typedefs

typedef struct
{
	TEHandle			editRecHdl;
	ControlHandle vScrollbarHdl;
} DocRec, **DocRecHandle;

//  global variables

Boolean		gDone;
Boolean		gInBackground;
RgnHandle	gCursorRegion;
SInt16		gNumberOfWindows = 0;

ControlActionUPP	scrollActionProcRD;																					 // PowerPC
TEClickLoopUPP		customClickLoopRD;																					 // PowerPC

//  function prototypes

void						main								(void);
void						doInitManagers			(void);
void						eventLoop						(void);
void						doIdle							(void);
void						doEvents						(EventRecord *);
void						doKeyEvent					(SInt8);
void						doMouseDown					(EventRecord *);
pascal void  		scrollActionProc		(ControlHandle,SInt16);
void						doInContent					(EventRecord *);
void						doUpdate						(EventRecord *);
void						doActivate					(EventRecord *);
void						doActivateDocWindow	(WindowPtr,Boolean);
void						doOSEvent						(EventRecord *);
WindowPtr				doNewDocWindow			(void);
pascal Boolean	customClikLoop			(void);
void						setScrollBarValue		(ControlHandle,SInt16 *);
void						doAdjustMenus				(void);
void						doMenuChoice				(SInt32);
void						doFileMenu					(SInt16);
void						doEditMenu					(SInt16);
SInt16					doGetSelectLength		(TEHandle);
void						doAdjustScrollbar		(WindowPtr);
void						doAdjustCursor			(WindowPtr,RgnHandle);
void						doCloseWindow				(WindowPtr);
void						doSaveAsFile				(TEHandle);
void						doOpenCommand				(void);
void						doOpenFile					(FSSpec);
void						doDrawDataPanel			(WindowPtr);
void						doErrorAlert				(SInt16);
void						doHelpMenu					(SInt16);

extern void			doHelp							(void);

//  main

void  main(void)
{
	Handle			menubarHdl;
	MenuHandle	menuHdl;
	OSErr				osErr;	
	
	// initialise managers

	doInitManagers();

	//  create routine descriptors

	scrollActionProcRD = NewControlActionProc((ProcPtr) scrollActionProc);			 // PowerPC
	customClickLoopRD = NewTEClickLoopProc((ProcPtr) customClikLoop);						 // PowerPC

	//  set up menu bar and menus
	
	menubarHdl = GetNewMBar(rMenubar);
	if(menubarHdl == NULL)
		doErrorAlert(eMenuBar);
	SetMenuBar(menubarHdl);
	DrawMenuBar();

	menuHdl = GetMenuHandle(mApple);
	if(menuHdl == NULL)
		doErrorAlert(eMenu);
	else
		AppendResMenu(menuHdl,'DRVR');

	osErr = HMGetHelpMenuHandle(&menuHdl);
	if(osErr == noErr)
		AppendMenu(menuHdl,"\pText1 Help");
	else
		doErrorAlert(eMenu);

	//  open an untitled window

	doNewDocWindow();

	//  enter eventLoop

	eventLoop();
}

//  doInitManagers

void  doInitManagers(void)
{
	MaxApplZone();
	MoreMasters();

	InitGraf(&qd.thePort);
	InitFonts();
	InitWindows();
	InitMenus();
	TEInit();
	InitDialogs(NULL);
	InitCursor();

	FlushEvents(everyEvent,0);
}

//  eventLoop

void  eventLoop(void)
{
	EventRecord	eventRec;
	Boolean			gotEvent;
	SInt32			sleepTime;

	gDone = false;
	gCursorRegion = NewRgn();
	sleepTime = LMGetCaretTime();

	while(!gDone)
	{
		gotEvent = WaitNextEvent(everyEvent,&eventRec,sleepTime,gCursorRegion);

		if(!gInBackground && gNumberOfWindows > 0)
			doAdjustCursor(FrontWindow(),gCursorRegion);

		if(gotEvent)
			doEvents(&eventRec);
		else
		{
			if(gNumberOfWindows > 0)
				doIdle();
		}
	}
}

//  doIdle

void  doIdle(void)
{
	DocRecHandle	docRecHdl;
	WindowPtr			windowPtr;
	
	windowPtr = FrontWindow();

	docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
	if(docRecHdl != NULL)
		TEIdle((*docRecHdl)->editRecHdl);
}	

//  doEvents

void	doEvents(EventRecord *eventRecPtr)
{
	SInt8		charCode;

	switch(eventRecPtr->what)
	{
		case mouseDown:
			doMouseDown(eventRecPtr);
			break;

		case keyDown:
		case autoKey:
			charCode = eventRecPtr->message & charCodeMask;
			if((eventRecPtr->modifiers & cmdKey) != 0)
			{
				doAdjustMenus();
				doMenuChoice(MenuKey(charCode));
			}
			else
				doKeyEvent(charCode);
			break;

		case updateEvt:
			doUpdate(eventRecPtr);
			break;

		case activateEvt:
			doActivate(eventRecPtr);
			break;

		case osEvt:
			doOSEvent(eventRecPtr);
			break;
	}
}

//  doKeyEvent

void  doKeyEvent(SInt8 charCode)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;
	SInt16				selectionLength;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
	editRecHdl = (*docRecHdl)->editRecHdl;

	if(charCode == kTab)
	{
		// Do tab key handling here if required.
	}
	else if(charCode == kDel)
	{
		selectionLength = doGetSelectLength(editRecHdl);
		if(selectionLength == 0)
			(*editRecHdl)->selEnd += 1;
		TEDelete(editRecHdl);
		doAdjustScrollbar(windowPtr);
	}
	else
	{
		selectionLength = doGetSelectLength(editRecHdl);
		if(((*editRecHdl)->teLength - selectionLength + 1) < kMaxTELength)
		{
			TEKey(charCode,editRecHdl);
			doAdjustScrollbar(windowPtr);
		}
		else
			doErrorAlert(eExceedChara);
	}

	doDrawDataPanel(windowPtr);
}

//  doMouseDown

void	doMouseDown(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	SInt16		partCode;

	partCode = FindWindow(eventRecPtr->where,&windowPtr);
	
	switch(partCode)
	{
		case inMenuBar:
			doAdjustMenus();
			doMenuChoice(MenuSelect(eventRecPtr->where));
			break;

		case inSysWindow:
			SystemClick(eventRecPtr,windowPtr);
			break;

		case inContent:
			if(windowPtr != FrontWindow())
				SelectWindow(windowPtr);
			else
				doInContent(eventRecPtr);
			break;

		case inDrag:
			DragWindow(windowPtr,eventRecPtr->where,&qd.screenBits.bounds);
			break;

		case inGoAway:
			if(TrackGoAway(windowPtr,eventRecPtr->where))
				doCloseWindow(FrontWindow());
			break;
	}
}

//  scrollActionProc

pascal void  scrollActionProc(ControlHandle controlHdl,SInt16 partCode)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;
	SInt16				linesToScroll;
	SInt16				controlValue, controlMax;

	if(partCode != 0)
	{
		windowPtr = (*controlHdl)->contrlOwner;
		docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
		editRecHdl = (*docRecHdl)->editRecHdl;

		switch(partCode)
		{
			case kControlUpButtonPart:
			case kControlDownButtonPart:
				linesToScroll = 1;
				break;
				
			case kControlPageUpPart:
			case kControlPageDownPart:
				linesToScroll = (((*editRecHdl)->viewRect.bottom - (*editRecHdl)->viewRect.top) /
												 (*editRecHdl)->lineHeight) -1;
				break;
		}

		if((partCode == kControlDownButtonPart) || (partCode == kControlPageDownPart))
			linesToScroll = -linesToScroll;

		controlValue = GetControlValue(controlHdl);
		controlMax = GetControlMaximum(controlHdl);

		linesToScroll = controlValue - linesToScroll;
		if(linesToScroll < 0)
			linesToScroll = 0;
		else if(linesToScroll > controlMax)
			linesToScroll = controlMax;

		SetControlValue(controlHdl,linesToScroll);

		linesToScroll = controlValue - linesToScroll;

		if(linesToScroll != 0)
			TEScroll(0,linesToScroll * (*editRecHdl)->lineHeight,editRecHdl);

		doDrawDataPanel(windowPtr);
	}
}

//  doInContent

void  doInContent(EventRecord *eventRecPtr)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;
	Point					mouseXY;
	ControlHandle	controlHdl;
	SInt16				partCode, controlValue;
	Boolean				shiftKeyPosition = false;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) GetWRefCon(windowPtr);
	editRecHdl = (*docRecHdl)->editRecHdl;

	mouseXY = eventRecPtr->where;
	GlobalToLocal(&mouseXY);

	if((partCode = FindControl(mouseXY,windowPtr,&controlHdl)) != 0)
	{
		switch(partCode)
		{
			case kControlUpButtonPart:
			case kControlDownButtonPart:
			case kControlPageUpPart:
			case kControlPageDownPart:
				TrackControl(controlHdl,mouseXY,scrollActionProcRD);									 // PowerPC
				break;

			case kControlIndicatorPart:
				controlValue = GetControlValue(controlHdl);
				partCode = TrackControl(controlHdl,mouseXY,NULL);
				if(partCode != 0)
				{
					controlValue -= GetControlValue(controlHdl);
					if(controlValue != 0)
						TEScroll(0,controlValue * (*editRecHdl)->lineHeight,editRecHdl);
				}
				doDrawDataPanel(windowPtr);
				break;
		}
	}
	else if(PtInRect(mouseXY,&(*editRecHdl)->viewRect))
	{
		if((eventRecPtr->modifiers & shiftKey) != 0)
			shiftKeyPosition = true;
		TEClick(mouseXY,shiftKeyPosition,editRecHdl);
	}
}


//  doUpdate

void	doUpdate(EventRecord *eventRecPtr)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;
	GrafPtr				oldPort;

	windowPtr = (WindowPtr) eventRecPtr->message;
	docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
	editRecHdl = (*docRecHdl)->editRecHdl;
	
	GetPort(&oldPort);
	SetPort(windowPtr);

	BeginUpdate((WindowPtr)eventRecPtr->message);

	EraseRect(&windowPtr->portRect);
	TEUpdate(&windowPtr->portRect,editRecHdl);
	UpdateControls(windowPtr,windowPtr->visRgn);

	doDrawDataPanel(windowPtr);

	EndUpdate((WindowPtr)eventRecPtr->message);

	SetPort(oldPort);
}

//  doActivate

void  doActivate(EventRecord *eventRecPtr)
{
	WindowPtr	windowPtr;
	Boolean		becomingActive;

	windowPtr = (WindowPtr) eventRecPtr->message;
	becomingActive = ((eventRecPtr->modifiers & activeFlag) == activeFlag);
	doActivateDocWindow(windowPtr,becomingActive);
}

//  doActivateDocWindow

void  doActivateDocWindow(WindowPtr windowPtr,Boolean becomingActive)
{
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;

	docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
	editRecHdl = (*docRecHdl)->editRecHdl;

	if(becomingActive)
	{
		SetPort(windowPtr);

		(*editRecHdl)->viewRect.bottom = ((((*editRecHdl)->viewRect.bottom -
																		 		(*editRecHdl)->viewRect.top) / 
																		 		(*editRecHdl)->lineHeight) * 
																		 		(*editRecHdl)->lineHeight) + 
																		 		(*editRecHdl)->viewRect.top;
		(*editRecHdl)->destRect.bottom = (*editRecHdl)->viewRect.bottom;

		TEActivate(editRecHdl);
		HiliteControl((*docRecHdl)->vScrollbarHdl,0);
		doAdjustScrollbar(windowPtr);
	}
	else
	{
		TEDeactivate(editRecHdl);
		HiliteControl((*docRecHdl)->vScrollbarHdl,255);
	}
}

//  doOSEvent

void	doOSEvent(EventRecord *eventRecPtr)
{
	if(((eventRecPtr->message >> 24) & 0x000000FF) == suspendResumeMessage)
	{
		gInBackground = (eventRecPtr->message & resumeFlag) == 0;
		if(gNumberOfWindows > 0)
			doActivateDocWindow(FrontWindow(),!gInBackground);
		HiliteMenu(0);
	}
}

//  doNewDocWindow

WindowPtr  doNewDocWindow(void)
{
	WindowPtr 		windowPtr;
	DocRecHandle	docRecHdl;
	Rect					destAndViewRect;

	if(!(windowPtr = GetNewWindow(rWindow,NULL,(WindowPtr) -1)))
	{
		doErrorAlert(eWindow);
		return(NULL);
	}

	SetPort(windowPtr);

	TextSize(10);
	TextFont(geneva);

	if(!(docRecHdl = (DocRecHandle) NewHandle(sizeof(DocRec))))
	{
		doErrorAlert(eDocRecord);
		return(NULL);
	}
	SetWRefCon(windowPtr,(SInt32) docRecHdl);

	gNumberOfWindows ++;

	(*docRecHdl)->vScrollbarHdl = GetNewControl(rVScrollbar,windowPtr);
	
	destAndViewRect = windowPtr->portRect;
	destAndViewRect.right -= 15;
	destAndViewRect.bottom -= 15;
	InsetRect(&destAndViewRect,2,2);

	MoveHHi((Handle) docRecHdl);
	HLock((Handle) docRecHdl);

	if(!((*docRecHdl)->editRecHdl = TENew(&destAndViewRect,&destAndViewRect)))
	{
		DisposeWindow(windowPtr);
		gNumberOfWindows --;
		DisposeHandle((Handle) docRecHdl);
		doErrorAlert(eEditRecord);
		return(NULL);
	}
	
	HUnlock((Handle) docRecHdl);
	
	TESetClickLoop(customClickLoopRD,(*docRecHdl)->editRecHdl);							 		 // PowerPC
	TEAutoView(true,(*docRecHdl)->editRecHdl);
	TEFeatureFlag(teFOutlineHilite,1,(*docRecHdl)->editRecHdl);

	return(windowPtr);
}

//  customClikLoop

pascal Boolean  customClikLoop(void)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;
	GrafPtr				oldPort;
	RgnHandle			oldClip;
	Rect					tempRect;
	Point					mouseXY;
	SInt16				linesToScroll;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
	editRecHdl = (*docRecHdl)->editRecHdl;

	GetPort(&oldPort);
	SetPort(windowPtr);
	oldClip = NewRgn();
	GetClip(oldClip);
	SetRect(&tempRect,-32767,-32767,32767,32767);
	ClipRect(&tempRect);

	GetMouse(&mouseXY);
	if(mouseXY.v < windowPtr->portRect.top)
	{
		linesToScroll = 1;
		setScrollBarValue((*docRecHdl)->vScrollbarHdl,&linesToScroll);
		if(linesToScroll != 0)
			TEScroll(0,linesToScroll * ((*editRecHdl)->lineHeight),editRecHdl);
	}
	else if(mouseXY.v > windowPtr->portRect.bottom)
	{
		linesToScroll = -1;
		setScrollBarValue((*docRecHdl)->vScrollbarHdl,&linesToScroll);
		if(linesToScroll != 0)
			TEScroll(0,linesToScroll * ((*editRecHdl)->lineHeight),editRecHdl);
	}

	doDrawDataPanel(windowPtr);

	SetClip(oldClip);
	DisposeRgn(oldClip);
	SetPort(oldPort);

	return(true);
}

//  setScrollBarValue

void  setScrollBarValue(ControlHandle controlHdl,SInt16 *linesToScroll)
{
	SInt16	controlValue, controlMax;

	controlValue = GetControlValue(controlHdl);
	controlMax = GetControlMaximum(controlHdl);

	*linesToScroll = controlValue - *linesToScroll;
	if(*linesToScroll < 0)
		*linesToScroll = 0;
	else if(*linesToScroll > controlMax)
		*linesToScroll = controlMax;

	SetControlValue(controlHdl,*linesToScroll);
	*linesToScroll = controlValue - *linesToScroll;
}

//  doAdjustMenus

void  doAdjustMenus(void)
{
	MenuHandle		fileMenuHdl, editMenuHdl;
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;
	SInt32				scrapOffset;

	fileMenuHdl = GetMenuHandle(mFile);
	editMenuHdl = GetMenuHandle(mEdit);

	if(gNumberOfWindows > 0)
	{
		windowPtr = FrontWindow();
		docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
		editRecHdl = (*docRecHdl)->editRecHdl;

		EnableItem(fileMenuHdl,iClose);

		if((*editRecHdl)->selStart < (*editRecHdl)->selEnd)
		{
			EnableItem(editMenuHdl,iCut);
			EnableItem(editMenuHdl,iCopy);
			EnableItem(editMenuHdl,iClear);
		}
		else
		{
			DisableItem(editMenuHdl,iCut);
			DisableItem(editMenuHdl,iCopy);
			DisableItem(editMenuHdl,iClear);
		}

		if(GetScrap(NULL,'TEXT',&scrapOffset) > 0)
			EnableItem(editMenuHdl,iPaste);
		else
			DisableItem(editMenuHdl,iPaste);
			
		if((*editRecHdl)->teLength > 0)
		{
			EnableItem(fileMenuHdl,iSaveAs);
			EnableItem(editMenuHdl,iSelectAll);
		}
		else
		{
			DisableItem(fileMenuHdl,iSaveAs);
			DisableItem(editMenuHdl,iSelectAll);
		}
	}
	else
	{
		DisableItem(fileMenuHdl,iClose);
		DisableItem(fileMenuHdl,iSaveAs);
		DisableItem(editMenuHdl,iClear);
		DisableItem(editMenuHdl,iSelectAll);
	}		

	DrawMenuBar();
}

//  doMenuChoice

void  doMenuChoice(SInt32 menuChoice)
{
	SInt16  menuID, menuItem;
	Str255	itemName;
	SInt16	daDriverRefNum;

	menuID	 = HiWord(menuChoice);
	menuItem = LoWord(menuChoice);

	if(menuID == 0)
		return;

	switch(menuID)
	{
		case mApple:
			if(menuItem == iAbout)
				SysBeep(10);
			else
			{
				GetMenuItemText(GetMenuHandle(mApple),menuItem,itemName);
				daDriverRefNum = OpenDeskAcc(itemName);
			}
			break;

		case mFile:
			doFileMenu(menuItem);
			break;

		case mEdit:
			doEditMenu(menuItem);
			break;

		case kHMHelpMenuID:
			if(FrontWindow())
				doActivateDocWindow(FrontWindow(),false);			
			doHelpMenu(menuItem);
			break;
	}

	HiliteMenu(0);
}

//  doFileMenu

void	doFileMenu(SInt16 menuItem)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;

	switch(menuItem)
	{
		case iNew:
			if(windowPtr = doNewDocWindow())
				ShowWindow(windowPtr);
			break;

		case iOpen:
			doOpenCommand();
			break;

		case iClose:
			doCloseWindow(FrontWindow());
			break;

		case iSaveAs:
			docRecHdl = (DocRecHandle) ((WindowPeek) FrontWindow())->refCon;
			editRecHdl = (*docRecHdl)->editRecHdl;
			doSaveAsFile(editRecHdl);
			break;

		case iQuit:
			gDone = true;
			break;
	}
}

//  doEditMenu

void	doEditMenu(SInt16 menuItem)
{
	WindowPtr			windowPtr;
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;
	SInt32				totalSize, contigSize, newSize, scrapOffset;
	SInt16				selectionLength;

	windowPtr = FrontWindow();
	docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
	editRecHdl = (*docRecHdl)->editRecHdl;

	switch(menuItem)
	{
		case iUndo:
			break;

		case iCut:
			if(ZeroScrap() == noErr)
			{
				PurgeSpace(&totalSize,&contigSize);
				selectionLength = doGetSelectLength(editRecHdl);
				if(selectionLength > contigSize)
					doErrorAlert(eNoSpaceCut);
				else
				{
					TECut(editRecHdl);
					doAdjustScrollbar(windowPtr);
					if(TEToScrap() != noErr)
						ZeroScrap();
				}
			}
			break;

		case iCopy:
			if(ZeroScrap() == noErr)
			{
				TECopy(editRecHdl);
				if(TEToScrap() != noErr)
					ZeroScrap();
			}
			break;

		case iPaste:
			newSize = (*editRecHdl)->teLength + GetScrap(NULL,'TEXT',&scrapOffset);
			if(newSize > kMaxTELength)
				doErrorAlert(eNoSpacePaste);
			else
			{
				if(TEFromScrap() == noErr)
				{
					TEPaste(editRecHdl);
					doAdjustScrollbar(windowPtr);
				}
			}
			break;

		case iClear:
			TEDelete(editRecHdl);
			doAdjustScrollbar(windowPtr);
			break;

		case iSelectAll:
			TESetSelect(0,(*editRecHdl)->teLength,editRecHdl);
			break;
	}

	doDrawDataPanel(windowPtr);
}

//  doGetSelectLength

SInt16  doGetSelectLength(TEHandle editRecHdl)
{
	SInt16	selectionLength;
		
	selectionLength = (*editRecHdl)->selEnd - (*editRecHdl)->selStart;
	return(selectionLength);
}

//  doAdjustScrollbar

void  doAdjustScrollbar(WindowPtr windowPtr)
{
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;
	SInt16				numberOfLines, controlMax, controlValue;

	docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
	editRecHdl = (*docRecHdl)->editRecHdl;

	numberOfLines = (*editRecHdl)->nLines;
	if(*(*(*editRecHdl)->hText + (*editRecHdl)->teLength - 1) == kReturn)
		numberOfLines += 1;

	controlMax = numberOfLines - (((*editRecHdl)->viewRect.bottom - 
							 (*editRecHdl)->viewRect.top) /
							 (*editRecHdl)->lineHeight);
	if(controlMax < 0)
		controlMax = 0;
	SetControlMaximum((*docRecHdl)->vScrollbarHdl,controlMax);

	controlValue = ((*editRecHdl)->viewRect.top - (*editRecHdl)->destRect.top) / 
									(*editRecHdl)->lineHeight;
	if(controlValue < 0)
		controlValue = 0;
	else if(controlValue > controlMax)
		controlValue = controlMax;

	SetControlValue((*docRecHdl)->vScrollbarHdl,controlValue);
	
	TEScroll(0,((*editRecHdl)->viewRect.top - (*editRecHdl)->destRect.top) - 
					 		(GetControlValue((*docRecHdl)->vScrollbarHdl) *
							(*editRecHdl)->lineHeight),editRecHdl);
}

//  doAdjustCursor

void  doAdjustCursor(WindowPtr windowPtr,RgnHandle mouseRegion)
{
	GrafPtr		oldPort;
	RgnHandle	arrowRegion, iBeamRegion;
	Rect			cursorRect;
	Point			mouseXY;

	if(gInBackground)
	{
		SetCursor(&qd.arrow);
		return;
	}

	GetPort(&oldPort);
	SetPort(windowPtr);

	arrowRegion = NewRgn();
	iBeamRegion = NewRgn();	
	SetRectRgn(arrowRegion,-32768,-32768,32766,32766);

	cursorRect = windowPtr->portRect;
	cursorRect.bottom -= 15;
	cursorRect.right	-= 15;
	LocalToGlobal(&topLeft(cursorRect));
	LocalToGlobal(&botRight(cursorRect));	

	RectRgn(iBeamRegion,&cursorRect);
	DiffRgn(arrowRegion,iBeamRegion,arrowRegion);
	
	GetMouse(&mouseXY);
	LocalToGlobal(&mouseXY);

	if(PtInRgn(mouseXY,iBeamRegion))
	{
		SetCursor(*(GetCursor(iBeamCursor)));
		CopyRgn(iBeamRegion,mouseRegion);
	}
	else
	{
		SetCursor(&qd.arrow);
		CopyRgn(arrowRegion,mouseRegion);
	}

	DisposeRgn(arrowRegion);
	DisposeRgn(iBeamRegion);

	SetPort(oldPort);
}

//  doCloseWindow

void  doCloseWindow(WindowPtr windowPtr)
{
	DocRecHandle	docRecHdl;

	docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;

	DisposeControl((*docRecHdl)->vScrollbarHdl);
	TEDispose((*docRecHdl)->editRecHdl);
	DisposeHandle((Handle) docRecHdl);
	DisposeWindow(windowPtr);

	gNumberOfWindows --;
}

//  doSaveAsFile

void  doSaveAsFile(TEHandle editRecHdl)
{
	StandardFileReply	fileReply;
	WindowPtr					windowPtr;
	SInt16						fileRefNum;
	SInt32						dataLength;
	Handle						editTextHdl;

	StandardPutFile("\pSave as:","\pUntitled",&fileReply);
	if(fileReply.sfGood)
	{
		windowPtr = FrontWindow();
		SetWTitle(windowPtr,fileReply.sfFile.name);

		if(!(fileReply.sfReplacing))
			FSpCreate(&fileReply.sfFile,' KJB','TEXT',fileReply.sfScript);

		FSpOpenDF(&fileReply.sfFile,fsCurPerm,&fileRefNum);

		dataLength = (*editRecHdl)->teLength;
		editTextHdl = (*editRecHdl)->hText;
		FSWrite(fileRefNum,&dataLength,*editTextHdl);

		FSClose(fileRefNum);
	}
}

//  doOpenCommand

void  doOpenCommand(void)
{
	StandardFileReply	fileReply;
	SFTypeList				fileTypes;	

	fileTypes[0] = 'TEXT';

	StandardGetFile(NULL,1,fileTypes,&fileReply);
	if(fileReply.sfGood)
		doOpenFile(fileReply.sfFile);
}

//  doOpenFile

void  doOpenFile(FSSpec fileSpec)
{
	WindowPtr 		windowPtr;
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;
	SInt16				fileRefNum;
	SInt32				textLength;
	Handle				textBuffer;

	if((windowPtr = doNewDocWindow()) == NULL)
		return;

	docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
	editRecHdl = (*docRecHdl)->editRecHdl;

	SetWTitle(windowPtr,fileSpec.name);

	FSpOpenDF(&fileSpec,fsCurPerm,&fileRefNum);

	SetFPos(fileRefNum,fsFromStart,0);
	GetEOF(fileRefNum,&textLength);

	if(textLength > 32767)
		textLength = 32767;

	textBuffer = NewHandle((Size) textLength);

	FSRead(fileRefNum,&textLength,*textBuffer);

	MoveHHi(textBuffer);
	HLock(textBuffer);

	TESetText(*textBuffer,textLength,editRecHdl);

	HUnlock(textBuffer);
	DisposeHandle(textBuffer);

	FSClose(fileRefNum);

	(*editRecHdl)->selStart = 0;
	(*editRecHdl)->selEnd = 0;

	ShowWindow(windowPtr);
}

//  doDrawDataPanel

void  doDrawDataPanel(WindowPtr windowPtr)
{
	DocRecHandle	docRecHdl;
	TEHandle			editRecHdl;
	ControlHandle	controlHdl;
	Rect					panelRect;
	Str255				textString;
				
	docRecHdl = (DocRecHandle) ((WindowPeek) windowPtr)->refCon;
	editRecHdl = (*docRecHdl)->editRecHdl;
	controlHdl = (*docRecHdl)->vScrollbarHdl;

	TextFont(helvetica);
	TextSize(9);

	MoveTo(3,249);
	DrawString("\pteLength:                nLines:              lineHeight:        ");
	MoveTo(193,249);
	DrawString("\pdestRect.top:                     contrlValue:              contrlMax:");

	MoveTo(0,238);
	LineTo(432,238);
	MoveTo(190,239);
	LineTo(190,252);
	MoveTo(287,239);
	LineTo(287,252);
	MoveTo(364,239);
	LineTo(364,252);

	SetRect(&panelRect,43,239,71,252);
	EraseRect(&panelRect);	
	SetRect(&panelRect,105,239,129,252);
	EraseRect(&panelRect);	
	SetRect(&panelRect,176,239,189,252);
	EraseRect(&panelRect);	
	SetRect(&panelRect,248,239,286,252);
	EraseRect(&panelRect);	
	SetRect(&panelRect,339,239,363,252);
	EraseRect(&panelRect);	
	SetRect(&panelRect,410,239,432,252);
	EraseRect(&panelRect);	

	NumToString((SInt32) (*editRecHdl)->teLength,textString);
	MoveTo(45,249);
	DrawString(textString);

	NumToString((SInt32) (*editRecHdl)->nLines,textString);
	MoveTo(107,249);
	DrawString(textString);

	NumToString((SInt32) (*editRecHdl)->lineHeight,textString);
	MoveTo(177,249);
	DrawString(textString);

	NumToString((SInt32) (*editRecHdl)->destRect.top,textString);
	MoveTo(250,249);
	DrawString(textString);

	NumToString((SInt32) GetControlValue(controlHdl),textString);
	MoveTo(341,249);
	DrawString(textString);

	NumToString((SInt32) GetControlMaximum(controlHdl),textString);
	MoveTo(412,249);
	DrawString(textString);

	TextFont(geneva);
	TextSize(10);
}

//  doErrorAlert

void  doErrorAlert(SInt16 errorCode)
{
	Str255	errorString;

	GetIndString(errorString,rErrorStrings,errorCode);
	ParamText(errorString,NULL,NULL,NULL);
	
	if(errorCode < eWindow)
	{
		StopAlert(rErrorAlert,NULL);
		ExitToShell();
	}
	else
		CautionAlert(rErrorAlert,NULL);
}

//  doHelpMenu

void	doHelpMenu(SInt16 menuItem)
{
	MenuHandle	helpMenuHdl;
	SInt16			origHelpItems, numItems;

	HMGetHelpMenuHandle(&helpMenuHdl);

	numItems = CountMItems(helpMenuHdl);
	origHelpItems = numItems - 1;

	if(menuItem > origHelpItems)
		doHelp();
}

// 
