{ 
// SysMemResPascal.p
// 
//
// This program:
//
// 	Initializes the system software managers.
//
// 	Creates a nonrelocatable block of memory for a window record.
//
// 	Loads a window template ('WIND') resource and creates a window.
//
// 	Loads a purgeable 'PICT' resource and a non-purgeable 'STR ' resource and draws
// 		them in the window.
//
// 	Checks if any error codes were generated as a result to Memory Manager and 
// 		Resource Manager routines.
//
// 	Terminates when the mouse button is clicked.
//
//  }


program SysMemRes(input, output);

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, TextEdit, Dialogs, TextUtils, Resources, Memory,
	QuickDraw, Types, Processes, OSUtils, QuickDrawText, Events, SegLoad;

{  define the following constants }

const

rWindowResourceID = 128;
rStringResourceID = 128;
rPictureResourceID = 128;

{  initialise various Managers }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;
	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);
	InitCursor;
	end;
		{of procedure DoInitManagers}
		
{  DoNewWindow }

procedure DoNewWindow;

	var
	myWindowPtr : WindowPtr;
	windowRecPtr : Ptr;
	
	begin
	windowRecPtr := NewPtr(sizeof(WindowRecord));
	if (windowRecPtr = nil) then
		begin
		SysBeep(10);
		ExitToShell;
		end;

	myWindowPtr := GetNewWindow(rWindowResourceID, windowRecPtr, WindowPtr(-1));
	
	if (myWindowPtr = nil) then
		begin
		SysBeep(10);
		ExitToShell;
		end;

	SetPort(myWindowPtr);
	TextFont(systemFont);
	
	end;
		{of procedure DoNewWindow}
		
{  DoDrawPictAndString }
		
procedure DoDrawPictAndString;

	var
	pictureHdl : PicHandle;
	stringHdl : StringHandle;
	pictureRect : Rect;
	resourceError : integer;
	memoryError : OSErr;			

	begin
	pictureHdl := GetPicture(rPictureResourceID);
	if (pictureHdl = nil) then
		begin
		SysBeep(10);
		ExitToShell;
		end;

	SetRect(pictureRect, 148, 25, 353, 170);

	HNoPurge(Handle(pictureHdl));
	DrawPicture(pictureHdl, pictureRect);
	HPurge(Handle(pictureHdl));

	stringHdl := GetString(rStringResourceID);
	if (stringHdl = nil) then
		begin
		SysBeep(10);
		ExitToShell;
		end;

	MoveTo(105, 210);
	DrawString(stringHdl^^);

	ReleaseResource(Handle(pictureHdl));
	ReleaseResource(Handle(stringHdl));

	resourceError := ResError;
	if (resourceError = noErr) then
		begin
		MoveTo(162, 240);
		DrawString('No Resource Manager errors');
		end;	

	memoryError := MemError;
	if (memoryError = noErr) then
		begin
		MoveTo(165, 255);
		DrawString('No Memory Manager errors');
		end;
		
	end;
		{of procedure DoDrawPictAndString}
		
{  start of main program }

begin 

DoInitManagers;
DoNewWindow;
DoDrawPictAndString;
while not Button do {nothing};

end.

{  }
