{ 
// LowEventsPascal.p
// 
//
// This program:
//
// 	Contains a main event loop function, together with subsidiary functions which
// 	perform nominal handling only of low-level and Operating System events.
//
// 	Opens a window in which the types of all received low-level and Operating System
// 	events are displayed.
//
// 	Terminates when the user clicks the window's close box.
//
// Event handling is only nominal in this program because its main purpose is to
// demonstrate the basics of an application's main event loop.
//
// Programs in later chapters demonstrate the full gamut of individual event handling. 
//
// The program utilizes the following resources:
//
// 	A 'WIND' resource (purgeable).
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateonFGSwitch, and
// 	is32BitCompatible flags set.
// 
//  }

program LowEventsPascal(input, output);

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, TextEdit, Dialogs, Memory, Types, Events, Quickdraw, 
	QuickdrawText, Processes, OSUtils, ToolUtils, DiskInit, Segload; 

{  define the following constants }

const

rWindowResource	= 128;

{  global variables }

var

gDone : Boolean;
gInBackground : Boolean;
gMouseMoved : Boolean;

{  DoInitManagers }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;

	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);

	InitCursor;
	FlushEvents(everyEvent, 0);
	end;
		{of procedure DoInitManagers}
		
{  DoNewWindow }

procedure DoNewWindow;

	var
	myWindowPtr : WindowPtr;

	begin
	myWindowPtr := GetNewWindow (rWindowResource, nil, WindowPtr(-1));
	if (myWindowPtr = nil) then
		begin
		SysBeep(10);
		ExitToShell;
		end;

	SetPort(myWindowPtr);
	TextSize(10);
	end;
		{of procedure DoNewWindow}

{  DoMouseDown }

procedure DoMouseDown(var eventRec : EventRecord);

	var
	partCode : integer;
	myWindowPtr : WindowPtr;

	begin
	partCode := FindWindow(eventRec.where, myWindowPtr);
	
	case partCode of
	
		inSysWindow:
			begin
			SystemClick(eventRec, myWindowPtr);
			end;

		inContent:
			begin
			if (myWindowPtr <> FrontWindow) then
				SelectWindow(myWindowPtr);
			end;

		inDrag:
			begin
			DragWindow(myWindowPtr, eventRec.where, qd.screenBits.bounds);
			end;

		inGoAway:
			begin
			if (TrackGoAway(myWindowPtr, eventRec.where)) then
				gDone := true;
			end;
			
		end;
			{of case statement}
	end;
		{of procedure DoMouseDown}

{  DoUpdate }

procedure DoUpdate (var eventRec : EventRecord);

	begin
	BeginUpdate(WindowPtr(eventRec.message));
	EndUpdate(WindowPtr(eventRec.message));
	end;
		{of procedure DoUpdate}

{  DoDisk }

procedure DoDisk (var eventRec : EventRecord);

	var
	thePoint : Point;
	theErr : OSErr;

	begin
	if (HiWord(eventRec.message) <> noErr) 
		then	begin
			SetPt(thePoint, 120, 120);
			theErr := DIBadMount(thePoint, eventRec.message);
			end
				
		else	begin
			{Attempt to mount was successful.  Record drive number for accessing 
			the disk, etc.}
			end;	
	end;
		{of procedure DoDisk}

{  DoOSEvent }

procedure DoOSEvent (var eventRec : EventRecord);

	begin
	case BAnd(BSR(eventRec.message, 24), $000000FF) of
	 	suspendResumeMessage:
	 		begin
			if (BAnd(eventRec.message, resumeFlag) = 1)
				then	begin	
					gInBackground := false;
					DrawString('Resume event');
					end
							
				else	begin
					gInBackground := true;
					DrawString('Suspend event');
					end;
			end;
				
		mouseMovedMessage:
	 			begin
	 			gMouseMoved := true;
				DrawString('Mouse-moved event');
				end;
		end;
			{of case statement}
	end;
		{of procedure DoOSEvent}

{  DrawEventString }

procedure DrawEventString(eventString : string);

	var
	tempRegion : RgnHandle;
	myWindowPtr : WindowPtr;
	
	begin
	myWindowPtr := FrontWindow;
	tempRegion := NewRgn;

	ScrollRect(myWindowPtr^.portRect, 0, -15, tempRegion);
	DisposeRgn(tempRegion);
	
	MoveTo(8, 291);
	DrawString(eventString);
	end;
		{of procedure DrawEventString}

{  DoAdjustCursor }

procedure DoAdjustCursor(frontWindow : WindowPtr; mouseRegion : RgnHandle);

	var
	myArrowRegion : RgnHandle;
	myIBeamRegion : RgnHandle;
	cursorRect : Rect;
	mousePt : Point;
	
	begin
	myArrowRegion := NewRgn;
	myIBeamRegion := NewRgn;	
	SetRectRgn(myArrowRegion, -32768, -32768, 32766, 32766);
	
	cursorRect := frontWindow^.portRect;
	LocalToGlobal(cursorRect.topLeft);
	LocalToGlobal(cursorRect.botRight);	

	RectRgn(myIBeamRegion, cursorRect);
	DiffRgn(myArrowRegion, myIBeamRegion, myArrowRegion);
	
	GetMouse(mousePt);
	LocalToGlobal(mousePt);
	
	if (PtInRgn(mousePt, myIBeamRegion))
		then	begin
			SetCursor(GetCursor(iBeamCursor)^^);
			CopyRgn(myIBeamRegion, mouseRegion);
			end
				
		else	begin
			SetCursor(qd.arrow);
			CopyRgn(myArrowRegion, mouseRegion);
			end;
	
	DisposeRgn(myArrowRegion);
	DisposeRgn(myIBeamRegion);
	end;
		{of procedure DoAdjustCursor}

{  DoEvents }

procedure DoEvents(var eventRec : EventRecord);

	begin
	case eventRec.what of
	
		mouseDown:
			begin
			DrawEventString('    mouseDown');
			DoMouseDown(eventRec);
			end;

		mouseUp:
			begin
			DrawEventString('    mouseUp');
			end;

		keyDown:
			begin
			DrawEventString('    keyDown');
			end;

		autoKey:
			begin
			DrawEventString('    autoKey');
			end;

		updateEvt:
			begin
			DrawEventString('    updateEvt');
			DoUpdate(eventRec);
			end;

		diskEvt:
			begin
			DrawEventString('    diskEvt');
			DoDisk(eventRec);
			end;

		activateEvt:
			begin
			DrawEventString('    activateEvt');
			end;

		osEvt:
			begin
			DrawEventString('    osEvt - ');
			DoOSEvent(eventRec);
			end;
			
		otherwise
			begin
			end
		end;
			{of case statement}
	end;
		{of procedure DoEvents}
	
{  EventLoop }

procedure EventLoop;

	var
	cursorRegion : RgnHandle;
	eventRec : EventRecord;
	gotEvent : Boolean;
	
	begin
	gDone := false;
	cursorRegion := NewRgn;
	
	while (not gDone) do
		begin
		if (not gInBackground & gMouseMoved) then
			begin
			DoAdjustCursor(FrontWindow, cursorRegion);
			gMouseMoved := false;
			end;

		gotEvent := WaitNextEvent(everyEvent, eventRec, 180, cursorRegion);
		if (gotEvent) then
			DoEvents(eventRec);
		end;
	end;
		{of procedure EventLoop}

{  start of main program }

begin

	DoInitManagers;
	DoNewWindow;
	EventLoop;

end.
	
{  }