{ 
// MenusPascal.p
// 
// 
// This program:
//
// 	Opens a window.
//
// 	Creates these pull-down-down menus: Apple, File, Edit, Font, Size and Special.
//
//		The Apple menu includes an 'About' menu item for the program.
//
//		The second menu item in the Special menu contains a submenu.
//
//		A 'Help' menu item for the program is appended to the Help menu.
//
// 	Creates a pop-up menu in the window. 
//
// 	Displays text in the window indicating the menu selection made by the user.
//
// The implementation of the Size menu is nominal only.  The current size is indicated
// with a checkmark; however, the number of sizes shown is not font-dependent and there
// is no 'Other' item.
//
// Because the primary purpose of the program is to demonstrate menu creation and
// handling, no code is included to update and activate/deactivate the window or to
// respond to events which are not relevant to the demonstration. 
//
// The program is terminated by selecting Quit from the File menu, by pressing the
// keyboard equivalent for that item (Command-Q), or by clicking in the window's go-away
// box. 
//  
// The program utilizes the following resources:
//
// 	A 'WIND' resource (purgeable) (initially not visible).
//
// 	An 'MBAR' resource (preload, non-purgeable).
//
// 	'MENU' resources for the drop-down, hierarchical and pop-up menus (all preload, 
//		all non-purgeable).
//
// 	A 'CNTL' resource for the pop-up menu (purgeable).
//
//  }

program MenusPascal(input, output);

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, Controls, TextEdit, Dialogs, Quickdraw, Devices, Events, Types,
	Processes, Memory, Balloons, ToolUtils, QuickdrawText, OSUtils, Segload;

{  define the following constants }

const

mApple = 128;
	iAbout = 1;
	
mFile = 129;
	iQuit = 11;
	
mEdit = 130;	
	iUndo = 1;
	iCut = 3;
	iCopy = 4;
	iPaste = 5;
	iClear = 6;
	
mFont = 131;

mStyle = 132;
	iPlain = 1;
	iBold = 3;
	iItalic = 4;
	iUnderline = 5;
	iOutline = 6;
	iShadow = 7;
	
mSize = 133;
	iTen = 1;
	iTwelve = 2;
	iEighteen = 3;
	iTwentyFour = 4;
	
mSpecial = 134;		
	iFirstItem = 1;
	hmSecondItem = 100;
		siFirstSub = 1;
		siSecondSub = 2;
		
pControlResID = 128;
	pSydney = 1;
	pNewYork = 2;
	pLondon = 3;
	pRome = 4;
	
rWindowResource = 128;

{  global variables }

var

gDone : Boolean;
gCurrentFont : integer;
gCurrentStyle : Style;
gCurrentSize : integer;
eventRec : EventRecord;
myWindowPtr : WindowPtr;

{  ProcedureName }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;

	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);

	InitCursor;	
	FlushEvents(everyEvent, 0);
	end;
		{of procedure DoInitManagers}

{  DrawItemString }

procedure DrawItemString(eventString : string);

	var
	tempRegion : RgnHandle;
	myWindowPtr : WindowPtr;
	scrollBox : Rect;

	begin
	myWindowPtr := FrontWindow;
	tempRegion := NewRgn;

	scrollBox := myWindowPtr^.portRect;
	scrollBox.top := scrollBox.top + 50;
	
	ScrollRect(scrollBox, 0, -24, tempRegion);
	DisposeRgn(tempRegion);
	
	MoveTo(8, 286);
	DrawString(eventString);
	end;
		{of procedure DrawItemString}

{  DoCheckForControlAndValue }

function DoCheckForControlAndValue(eventRec : EventRecord;
										myWindowPtr: WindowPtr) : integer;

	var
	controlHdl : ControlHandle;
	startControlValue, finishControlValue, ignored : integer;
	
	begin
	startControlValue := 0;
	finishControlValue := 0;
	SetPort(myWindowPtr);
	GlobalToLocal(eventRec.where);
	
	if (FindControl(eventRec.where, myWindowPtr, controlHdl) <> 0) then
		begin
		startControlValue := GetControlValue(controlHdl);
		ignored := TrackControl(controlHdl, eventRec.where, ControlActionUPP(-1));
		finishControlValue := GetControlValue(controlHdl);
		end;

	if (finishControlValue <> startControlValue) 
		then DoCheckForControlAndValue := finishControlValue
		else DoCheckForControlAndValue := 0;
		
	end;
		{of function DoCheckForControlAndValue}

{  DoAdjustMenus }

procedure DoAdjustMenus;

	begin
	
		{Adjust menus here}
		
	end;
		{of procedure DoAdjustMenus}

{  DoPopupMenuChoice }

procedure DoPopupMenuChoice(popupItem : integer);

	begin
	case (popupItem) of

		pSydney:
			begin
			DrawItemString('Sydney');
			end;

		pNewYork:
			begin
			DrawItemString('New York');
			end;

		pLondon:
			begin
			DrawItemString('London');
			end;

		pRome:
			begin
			DrawItemString('Rome');
			end;
		end;
			{of case statement}

	end;
		{of procedure DoPopupMenuChoice}

{  DoAppleMenu }

procedure DoAppleMenu(menuItem : integer);

	var
	itemName : string;
	daDriverRefNum : integer;
	
	begin
	if (menuItem = iAbout) 
		then 	DrawItemString('About Menus')
		
		else	begin
				GetMenuItemText(GetMenuHandle(mApple), menuItem, itemName);
				daDriverRefNum := OpenDeskAcc(itemName);
				end;
	
	end;
		{of procedure DoAppleMenu}
		
{  DoFileMenu }

procedure DoFileMenu(menuItem : integer);

	begin
	if (menuItem  = iQuit) then
		gDone := true;
	end;
		{of procedure DoFileMenu}

{  DoEditMenu }

procedure DoEditMenu(menuItem : integer);

	begin
	case (menuItem) of

		iUndo:
			begin
			DrawItemString('Undo');
			end;

		iCut:
			begin
			DrawItemString('Cut');
			end;

		iCopy:
			begin
			DrawItemString('Copy');
			end;

		iPaste:
			begin
			DrawItemString('Paste');
			end;

		iClear:
			begin
			DrawItemString('Clear');
			end;
			
		end;
			{of case statement}
			
	end;
		{of procedure DoEditMenu}
		
{  DoFontMenu }

procedure DoFontMenu(menuItem : integer);

	var
	fontMenuHdl : MenuHandle;
	fontName : string;
	fontNumber : integer;
	
	begin
	fontMenuHdl := GetMenuHandle(mFont);
	
	CheckItem(fontMenuHdl, gCurrentFont, false);
	CheckItem(fontMenuHdl, menuItem, true);

	gCurrentFont := menuItem;	
			
	GetMenuItemText(fontMenuHdl, menuItem, fontName);
	GetFNum(fontName, fontNumber);
	TextFont(fontNumber);

	DrawItemString(fontName);

	end;
		{of procedure DoFontMenu}
		
{  DoStyleMenu }

procedure DoStyleMenu(menuItem : integer);

	var
	styleMenuHdl : MenuHandle;
	
	begin
	case (menuItem) of 

		iPlain:
			begin
			gCurrentStyle := [];
			end;

		iBold:
			begin
			if (bold in gCurrentStyle)
				then gCurrentStyle := gCurrentStyle - [bold]
				else gCurrentStyle := gCurrentStyle + [bold];
			end;

		iItalic:
			begin
			if (italic in gCurrentStyle)
				then gCurrentStyle := gCurrentStyle - [italic]
				else gCurrentStyle := gCurrentStyle + [italic];
			end;

		iUnderline:
			begin
			if (underline in gCurrentStyle)
				then gCurrentStyle := gCurrentStyle - [underline]
				else gCurrentStyle := gCurrentStyle + [underline];
			end;

		iOutline:
			begin
			if (outline in gCurrentStyle)
				then gCurrentStyle := gCurrentStyle - [outline]
				else gCurrentStyle := gCurrentStyle + [outline];
			end;

		iShadow:
			begin
			if (shadow in gCurrentStyle)
				then gCurrentStyle := gCurrentStyle - [shadow]
				else gCurrentStyle := gCurrentStyle + [shadow];
			end;
		end;
			{of case statement}

	styleMenuHdl := GetMenuHandle(mStyle);

	CheckItem(styleMenuHdl, iPlain, (gCurrentStyle = []));
	CheckItem(styleMenuHdl, iBold, (bold in gCurrentStyle));
	CheckItem(styleMenuHdl, iItalic, (italic in gCurrentStyle));
	CheckItem(styleMenuHdl, iUnderline, (underline in gCurrentStyle));
	CheckItem(styleMenuHdl, iOutline, (outline in gCurrentStyle));
	CheckItem(styleMenuHdl, iShadow, (shadow in gCurrentStyle));
	
	TextFace(gCurrentStyle);

	DrawItemString('Style change');

	end;
		{of procedure DoStyleMenu}
		
{  DoSizeMenu }

procedure DoSizeMenu(menuItem : integer);

	var
	sizeMenuHdl : MenuHandle;	

	begin
	case (menuItem) of

		iTen:
			begin
			TextSize(10);
			end;

		iTwelve:
			begin
			TextSize(12);
			end;

		iEighteen:
			begin
			TextSize(18);
			end;

		iTwentyFour:
			begin
			TextSize(24);
			end;
		end;
			{of case statement}

	sizeMenuHdl := GetMenuHandle(mSize);

	CheckItem(sizeMenuHdl, gCurrentSize, false);
	CheckItem(sizeMenuHdl, menuItem, true);
	
	gCurrentSize := menuItem;
	
	DrawItemString('Size change');

	end;
		{of procedure DoSizeMenu}
		
{  DoSpecialMenu }

procedure DoSpecialMenu(menuItem : integer);

	begin
	if(menuItem = iFirstItem) then
		DrawItemString('First Item');
	end;
		{of procedure DoSpecial}
		
{  DoSubMenus }

procedure DoSubMenus(menuItem : integer);

	begin
	case (menuItem) of

		siFirstSub:
			begin
			DrawItemString('Subitem 1');
			end;

		siSecondSub:
			begin
			DrawItemString('Subitem 2');
			end;
		
		end;
			{of case statement}
	end;
		{of procedure DoSubMenus}

{  DoHelpMenu }

procedure DoHelpMenu(menuItem : integer);

	var
	helpMenuHdl : MenuHandle;
	origHelpItems, numItems : integer;
	theErr : OSErr;
	
	begin
	theErr := HMGetHelpMenuHandle(helpMenuHdl);
	
	if (theErr <> noErr) then
		begin
		SysBeep(10);
		ExitToShell;
		end;
		
	numItems := CountMItems(helpMenuHdl);
	origHelpItems := numItems - 1;

	if (menuItem > origHelpItems) then
		DrawItemString('Menus Help');
	end;
		{of procedure DoHelpMenu}

{  DoMenuChoice }

procedure DoMenuChoice(menuChoice : longint);

	var
	menuID, menuItem : longint;
		
	begin
	menuID := HiWord(menuChoice);
	menuItem := LoWord(menuChoice);
	
	if (menuID = 0) then
		exit(DoMenuChoice);

	case (menuID) of

		mApple:
			begin
			DoAppleMenu(menuItem);
			end;

		mFile:
			begin
			DoFileMenu(menuItem);
			end;
			
		mEdit:
			begin
			DoEditMenu(menuItem);
			end;

		mFont:
			begin
			DoFontMenu(menuItem);
			end;

		mStyle:
			begin
			DoStyleMenu(menuItem);
			end;

		mSize:
			begin
			DoSizeMenu(menuItem);
			end;

		mSpecial:
			begin
			DoSpecialMenu(menuItem);
			end;

		hmSecondItem:
			begin
			DoSubMenus(menuItem);
			end;

		kHMHelpMenuID:
			begin
			DoHelpMenu(menuItem);
			end;
		
		end;
			{of case statement}

	HiliteMenu(0);
	end;
		{of procedure DoMenuChoice}
		
{  DoMouseDown }

procedure DoMouseDown(eventRec : EventRecord);

	var
	myWindowPtr : WindowPtr;
	partCode, popupItem : integer;
	menuChoice : longint;
	
	begin
	partCode := FindWindow(eventRec.where, myWindowPtr);
	
	case (partCode) of 

		inSysWindow:
			begin
			SystemClick(eventRec, myWindowPtr);
			end;

		inMenuBar:
			begin
			DoAdjustMenus;
			menuChoice := MenuSelect(eventRec.where);
			DoMenuChoice(menuChoice);
			end;

		inContent:
			begin
			if (myWindowPtr <> FrontWindow)
				then 	SelectWindow(myWindowPtr)
				
				else	begin
						popupItem := DoCheckForControlAndValue(eventRec, myWindowPtr);
						if (popupItem <> 0) then
							DoPopupMenuChoice(popupItem);
						end;
			end;

		inDrag:
			begin
			DragWindow(myWindowPtr, eventRec.where, qd.screenBits.bounds);
			end;

		inGoAway:
			begin
			if (TrackGoAway(myWindowPtr, eventRec.where)) then
				gDone := true;
			end;
			
		end;
			{of case statement}
			
	end;
		{of procedure DoMouseDown}

{  DoGetMenus }

procedure  DoGetMenus(myWindowPtr : WindowPtr);

	var
	menubarHdl : Handle;
	menuHdl : MenuHandle;
	theErr : OSErr;
	popupControlHdl : ControlHandle;
	
	begin
	menubarHdl := GetNewMBar(128);
	if (menubarHdl = nil) then
		ExitToShell;
	SetMenuBar(menubarHdl);
	DrawMenuBar;

	menuHdl := GetMenuHandle(mApple);
	if (menuHdl <>  nil)
		then AppendResMenu(menuHdl, 'DRVR')
		else ExitToShell;

	menuHdl := GetMenuHandle(mFont);
	if (menuHdl <>  nil)
		then AppendResMenu(menuHdl, 'FONT')
		else ExitToShell;

	menuHdl := GetMenu(hmSecondItem);
	if (menuHdl <>  nil)
		then InsertMenu(menuHdl, hierMenu)
		else ExitToShell;

	theErr := HMGetHelpMenuHandle(menuHdl);
	if (theErr = noErr)
		then AppendMenu(menuHdl, 'Menus Help')
		else ExitToShell;
	
	popupControlHdl := GetNewControl(pControlResID, myWindowPtr);
	if (popupControlHdl =  nil) then
		ExitToShell;

	DoFontMenu(gCurrentFont);
	DoStyleMenu(0);
	DoSizeMenu(gCurrentSize);
	
	end;
		{of procedure DoGetMenus}

{  DoEvents }

procedure	DoEvents(eventRec : EventRecord);

	var
	theChar: char;
	
	begin
	case (eventRec.what) of

		mouseDown:
			begin
			DoMouseDown(eventRec);
			end;

		keyDown, autoKey:
			begin
			theChar := CHR(BAnd(eventRec.message, charCodeMask));
			if (BAnd(eventRec.modifiers, cmdKey) <> 0) then
				begin
				DoAdjustMenus;
				DoMenuChoice(MenuKey(theChar));
				end;
			end;

		updateEvt:
			begin
			BeginUpdate(WindowPtr(eventRec.message));
			EndUpdate(WindowPtr(eventRec.message));
			end;

		osEvt:
			begin
			HiliteMenu(0);
			end;

		end;
			{of case statement}

	end;
		{of DoEvents}
		
{  start of main program }

begin
	gCurrentFont := 1;
	gCurrentStyle := [];
	gCurrentSize := 2;
	
	{  initialize manager }

	DoInitManagers;			
	
	{  open a window }
	
	myWindowPtr := GetNewWindow(rWindowResource, nil, WindowPtr(-1));
	if (myWindowPtr = nil) then
		begin
		SysBeep(10);
		ExitToShell;
		end;

	SetPort(myWindowPtr);

	{  set up menu bar and menus, then show window and pop-up menu }
	
	DoGetMenus(myWindowPtr);
	ShowWindow(myWindowPtr);
	DrawControls(myWindowPtr);

	{  event loop }			

	gDone := false;

	while (not gDone) do
		begin
		if (WaitNextEvent(everyEvent, eventRec, 180, nil)) then
			DoEvents(eventRec);
		end;

end.
	
{  }