{ 
// BasicQuickdrawPascal.p
// 
// 
// This program:
//
//   Opens a window in which the results of various basic QuickDraw drawing operations
//	  are displayed.
//
//	  Individual drawing operations (eg, draw lines, draw rectangles, draw polygons, etc)
//	  are selected from a pull-down menu titled 'Demonstration'.
//
//   Quits when the user selects Quit from the File menu or clicks the window's close
//	  box.
//
// The program utilizes the following resources:
//
//   'WIND' resources for the main window, and a small window used for the CopyBits
//	  demonstration (purgeable) (initially visible).
//
//   An 'MBAR' resource and associated 'MENU' resources (preload, non-purgeable).
//
//   Two 'ICON' resources (purgeable) used for the transfer modes demonstration. 
//
//   A 'PICT' resource (purgeable) used for the CopyBits demonstration.
//
//   'STR#' resources (purgeable) containing strings used by the CopyBits and text
//	  demonstrations.
//
//  }

program BasicQuickdrawPascal(input, output);

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, TextEdit, Quickdraw, Dialogs, QuickdrawText, Processes, Types, 
	Memory, Events, TextUtils, ToolUtils, OSUtils, Devices, Icons, Segload;

{  define the following constants }

const

mApple = 128;
mFile = 129;
	iQuit = 11;
mDemonstration = 131;
	iLine = 1;
	iRectAndOval = 2;
	iArcAndWedge = 3;
	iPolygon = 4;
	iRegion = 5;
	iTransferMode = 6;
	iCopyBits = 7;
	iText = 8;
	iBasicColour = 9;
	iDrawWithMouse = 10;
rMenubar = 128;
rWindow = 128;
rSmallWindow = 129;
rCrossIcon = 128;
rSquareIcon = 129;
rModeStringList = 128;
rTextStringList = 129;
rPicture = 128;
kMaxLong = $7FFFFFFF;

{  global variables }

var

gDone : boolean;
gWindowPtr : WindowPtr;
gDrawWithMouseActivated: boolean;
menubarHdl : Handle;
menuHdl : MenuHandle;
eventRec : EventRecord;
gotEvent : boolean;		

{  DoInitManagers }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;

	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);

	InitCursor;	
	FlushEvents(everyEvent, 0);
	end;
		{of procedure DoInitManagers}

{  DoRandomNumber }

function DoRandomNumber(range : integer) : integer;

	begin
	DoRandomNumber := (Abs(Random) mod (range + 1));
	end;
		{of function DoRandomNumber}

{  DoRectOval }

procedure DoRectOval;

	var
	theRect : Rect;
	finalTicks : longint;
	systemPattern : Pattern;

	begin
	FillRect(gWindowPtr^.portRect, qd.white);

	PenPat(qd.black);
	PenSize(10, 20);
	PenMode(patCopy);

	SetRect(theRect, 10, 20, 245, 130);

	MoveTo(10, 15);
	DrawString('FrameRect');
	FrameRect(theRect);
	Delay(30, finalTicks);

	MoveTo(255, 15);
	DrawString('PaintRect');
	OffsetRect(theRect, 245, 0);
	PenPat(qd.ltGray);
	PaintRect(theRect);
	Delay(30, finalTicks);

	MoveTo(10, 154);
	DrawString('FillRoundRect');
	OffsetRect(theRect, -245, 140);
	GetIndPattern(systemPattern, sysPatListID, 12);
	FillRoundRect(theRect, 120, 60, systemPattern);
	Delay(30, finalTicks);

	MoveTo(255, 154);
	DrawString('FrameOval');
	OffsetRect(theRect, 245, 0);
	PenSize(40, 20);
	PenPat(qd.dkGray);
	FrameOval(theRect);

	SetWTitle(gWindowPtr, 'Click Mouse For Invert and Erase');
	while not (Button) do ;
	SetWTitle(gWindowPtr, 'Basic QuickDraw');
	SetRect(theRect, 10, 145, 490, 154);
	EraseRect(theRect);
	SetRect(theRect, 255, 160, 490, 270);
	Delay(30, finalTicks);

	MoveTo(10, 154);
	DrawString('InvertRoundRect');
	OffsetRect(theRect, -245, 0);
	InvertRoundRect(theRect, 120, 60);
	Delay(30, finalTicks);

	MoveTo(255, 154);
	DrawString('EraseOval');
	OffsetRect(theRect, 245, 0);
	EraseOval(theRect);
	Delay(30, finalTicks);
	end;

{  DoArcWedge }

procedure DoArcWedge;

	var
	theRect : Rect;
	a : integer;
	finalTicks : longint;
	systemPattern : Pattern;

	begin
	FillRect(gWindowPtr^.portRect, qd.white);

	PenSize(60, 10);
	PenPat(qd.dkGray);
	PenMode(patCopy);

	SetRect(theRect, 10, 20, 245, 278);

	MoveTo(10, 15);
	DrawString('FrameArc');
	for a := 0 to 269 do
		FrameArc(theRect, 135, a);
	Delay(30, finalTicks);

	MoveTo(255, 15);
	DrawString('FillArc');
	OffsetRect(theRect, 245, 0);
	GetIndPattern(systemPattern, sysPatListID, 16);
	FillArc(theRect, 315, 270, systemPattern);
	Delay(30, finalTicks);
	OffsetRect(theRect, -30, 0);
	FillArc(theRect, 225, 90, systemPattern);
	end;
		{of procedure DoArcWedge}

{  DoPolygon }

procedure DoPolygon;

	var
	polygonHdl : PolyHandle;
	finalTicks : longint;
	systemPattern : Pattern;

	begin
	FillRect(gWindowPtr^.portRect, qd.white);

	PenSize(10, 30);
	PenPat(qd.gray);
	PenMode(patCopy);

	polygonHdl := OpenPoly;
	MoveTo(10, 20);
	LineTo(225, 40);
	LineTo(100, 120);
	LineTo(215, 248);	
	LineTo(10, 248);
	LineTo(50, 200);
	ClosePoly;

	MoveTo(10, 15);
	DrawString('FramePoly');
	FramePoly(polygonHdl);
	Delay(30, finalTicks);

	MoveTo(265, 15);
	DrawString('FillPoly');
	OffsetPoly(polygonHdl, 255, 0);
	GetIndPattern(systemPattern, sysPatListID, 9);
	FillPoly(polygonHdl, systemPattern);

	KillPoly(polygonHdl);
	end;
		{of procedure DoPoly}

{  DoRegion }

procedure  DoRegion;

	var
	regionHdl : RgnHandle;
	theRect : Rect;
	finalTicks : longint;

	begin
	FillRect(gWindowPtr^.portRect, qd.white);
	PenPat(qd.gray);
	PenMode(patCopy);

	regionHdl := NewRgn;

	OpenRgn;
	SetRect(theRect, 10, 20, 100, 130);
	FrameRect(theRect);
	SetRect(theRect, 155, 20, 245, 130);
	FrameRect(theRect);
	SetRect(theRect, 55, 30, 200, 120);
	FrameOval(theRect);
	CloseRgn(regionHdl);

	MoveTo(10, 15);
	DrawString('FrameRgn');
	PenPat(qd.black);
	PenSize(10, 20);
	FrameRgn(regionHdl);
	Delay(30, finalTicks);

	MoveTo(255, 15);
	DrawString('1. FillRgn');
	OffsetRgn(regionHdl, 245, 0);
	FillRgn(regionHdl, qd.dkGray);
	Delay(30, finalTicks);

	MoveTo(10, 154);
	DrawString('2. InsetRgn (10 horizontal, 10 vertical)');
	OffsetRgn(regionHdl, -245, 140);
	InsetRgn(regionHdl, 10, 10);
	PenPat(qd.dkGray);
	PaintRgn(regionHdl);
	Delay(30, finalTicks);

	MoveTo(255, 154);
	DrawString('3. InsetRgn (-10 horizontal, -10 vertical)');
	OffsetRgn(regionHdl, 245, 0);
	InsetRgn(regionHdl, -10, -10);
	PenPat(qd.dkGray);
	PaintRgn(regionHdl);

	DisposeRgn(regionHdl);		
	end;
		{of procedure DoRegion

{  DoTransferMode }

procedure  DoTransferMode;

	var
	crossIconHdl, squareIconHdl : Handle;
	destRect : Rect;
	a, b, i, j : integer;
	squareIconMap : BitMap;
	finalTicks : longint;
	sourceMode : integer;
	sourceString : string;

	begin
	sourceMode := 0; 
	FillRect(gWindowPtr^.portRect, qd.white);

	PenSize(1, 1);
	PenPat(qd.gray);
	PenMode(patOr);

	crossIconHdl := GetIcon(rCrossIcon);
	if (crossIconHdl = nil) then
		begin
		SysBeep(10);
		Exit(DoTransferMode);
		end;
		
	squareIconHdl := GetIcon(rSquareIcon);
	if (squareIconHdl = nil) then
		begin
		SysBeep(10);
		Exit(DoTransferMode);
		end;

	SetRect(destRect, 120, 8, 190, 78);
	PlotIcon(destRect, crossIconHdl);
	FrameRect(destRect);
	MoveTo(200, 48);
	DrawString('Destination');

	SetRect(destRect, 270, 8, 340, 78);
	PlotIcon(destRect, squareIconHdl);
	FrameRect(destRect);
	MoveTo(350, 48);
	DrawString('Source');

	for i := 0 to 1 do
		begin
		a := i * 100 + 91;
		for j := 0 to 3 do
			begin
			b := j * 120 + 30;
			SetRect(destRect, b, a, b+70, a+70);
			PlotIcon(destRect, crossIconHdl);
			end;
		end;

	HLock(squareIconHdl);

	squareIconMap.baseAddr := squareIconHdl^;
	squareIconMap.rowBytes := 4;
	SetRect(squareIconMap.bounds, 0, 0, 31, 31);

	for i := 0 to 1 do
		begin
		a := i * 100 + 91;
		for j := 0 to 3 do
			begin
			b := j * 120 + 30;
			Delay(30, finalTicks);
			SetRect(destRect, b, a, b+70, a+70);
			CopyBits(squareIconMap, qd.thePort^.portBits, squareIconMap.bounds, 
						destRect, sourceMode, nil);
			sourceMode := sourceMode + 1;
			GetIndString(sourceString, rModeStringList, sourceMode);
			MoveTo(b, a + 82);
			DrawString(sourceString);
			end;
		end;

	HUnlock(squareIconHdl);
	end;
		{of procedure DoTransferMode}

{  DoCopyBits }

procedure DoCopyBits;

	var
	myWindowPtr : WindowPtr;
	oldPort : GrafPtr;
	pictureHdl : PicHandle;
	sourceRect, destRect : Rect;
	finalTicks : longint;

	begin
	FillRect(gWindowPtr^.portRect, qd.white);
	
	myWindowPtr := GetNewWindow(rSmallWindow, nil, WindowPtr(-1));
	if (myWindowPtr = nil) then
		ExitToShell;

	GetPort(oldPort);
	SetPort(myWindowPtr);
	
	pictureHdl := GetPicture(rPicture);
	if (pictureHdl = nil) then
		begin
		DisposeWindow(myWindowPtr);
		SysBeep(10);
		Exit(DoCopyBits);
		end;

	HNoPurge(Handle(pictureHdl));	
	SetRect(sourceRect, 65, 40, 165, 182);
	DrawPicture(pictureHdl, sourceRect);
	HPurge(Handle(pictureHdl));	

	SetWTitle(myWindowPtr, 'Click Mouse for CopyBits');
	while not (Button) do ;
	
	SetRect(destRect, 20, 21, 210, 272);

	CopyBits(myWindowPtr^.portBits, oldPort^.portBits, sourceRect, destRect, 
					 srcCopy, nil);

	SetWTitle(myWindowPtr, 'Click Mouse to Close');
	Delay(60, finalTicks);
	while not (Button) do ;
	
	DisposeWindow(myWindowPtr);
	SetPort(oldPort);
	end;
		{of procedure DoCopyBits}

{  DoText }

procedure DoText;

	var
	windowCentre, a, fontNum, widthOfString : integer;
	textString : string;
	theWindow : WindowRef;

	begin
	FillRect(gWindowPtr^.portRect, qd.white);

	theWindow := FrontWindow;
	windowCentre := trunc((theWindow^.portRect.right - theWindow^.portRect.left) / 2);

	for a := 1 to 8 do
		begin
		if (a = 1) then 	
			begin
			GetFNum('Geneva', fontNum);
			TextFont(fontNum);
			TextFace([]);
			end
				
		else if (a = 2) then
				TextFace([bold])
				
		else if (a = 3) then
				begin
				GetFNum('Times', fontNum);
				TextFont(fontNum);
				TextFace([italic]);
				end
				
		else if (a = 4) then
			TextFace([underline])
			
		else if (a = 5) then
			begin
			GetFNum('Helvetica', fontNum);
			TextFont(fontNum);
			TextFace([outline]);
			end
			
		else if (a = 6) then
			TextFace([shadow])
			
		else if (a = 7) then
			begin
			GetFNum('Chicago', fontNum);
			TextFont(fontNum);
			TextFace([condense]);
			end
			
		else if (a = 8) then
			begin
			TextFace([extend]);
			TextMode(grayishTextOr);
			end;

		if (a < 7) 
			then	TextSize(a * 2 + 10)
			else	TextSize(12);

		GetIndString(textString, rTextStringList, a);
		widthOfString := StringWidth(textString);
		MoveTo(trunc(windowCentre - (widthOfString / 2)), a * 35 - 10);
		DrawString(textString);
		end;
			{of for loop}

	GetFNum('Geneva', fontNum);
	TextFont(fontNum);
	TextSize(10);
	TextMode(srcOr);
	TextFace(Style(0));
	end;
		{of procedure DoText}
		
{  DoBasicColours }

procedure DoBasicColours;

	var
	a : integer;
	theRect : Rect;
	finalTicks : longint;

	begin
	FillRect(gWindowPtr^.portRect, qd.dkGray);
	PenPat(qd.black);
	PenMode(patCopy);

	for a := 1 to 8 do
		begin
		Delay(30, finalTicks);
		if (a = 1) then ForeColor(blackColor);
		if (a = 2) then ForeColor(whiteColor);
		if (a = 3) then ForeColor(redColor);
		if (a = 4) then ForeColor(greenColor);
		if (a = 5) then ForeColor(blueColor);
		if (a = 6) then ForeColor(cyanColor);
		if (a = 7) then ForeColor(magentaColor);
		if (a = 8) then ForeColor(yellowColor);

		SetRect(theRect, 35, a*28, 465, a*28+23);
		PaintRect(theRect);			
		end;

	ForeColor(blackColor);
	end;
		{of procedure DoBasicColours}

{  DoLines }

procedure DoLines;

	var
	top, left, bottom, right, a, b, c : integer;
	oldClipRgn : RgnHandle;
	newClipRect : Rect;
	systemPattern : Pattern;
	finalTicks : longint;

	begin
	FillRect(gWindowPtr^.portRect, qd.white);

	PenMode(patCopy);
		
	left := gWindowPtr^.portRect.left + 10;
	top	:= gWindowPtr^.portRect.top + 10;
	right := gWindowPtr^.portRect.right - 10;
	bottom := gWindowPtr^.portRect.bottom - 10;

	oldClipRgn := NewRgn;
	GetClip(oldClipRgn);
	SetRect(newClipRect, left, top, right, bottom);
	ClipRect(newClipRect);

	for a := 1 to 38 do
		begin
		b := DoRandomNumber(gWindowPtr^.portRect.right - gWindowPtr^.portRect.left);
		c := DoRandomNumber(gWindowPtr^.portRect.right - gWindowPtr^.portRect.left);

		GetIndPattern(systemPattern, sysPatListID, a);
		PenPat(systemPattern);
		PenSize(a * 2, 1);		

		MoveTo(b, gWindowPtr^.portRect.top);
		LineTo(c, gWindowPtr^.portRect.bottom);

		Delay(15, finalTicks);
		end;
		
	SetClip(oldClipRgn);
	DisposeRgn(oldClipRgn);

	SetWTitle(gWindowPtr, 'Click Mouse for More Lines');
	while not (Button) do ;	
	SetWTitle(gWindowPtr, 'Basic QuickDraw');

	FillRect(gWindowPtr^.portRect, qd.white);
	PenSize(1, 1);		
	PenPat(qd.black);
	PenMode(patXor);

	b := right;
	for a := left to (right + 1) do
		begin
		MoveTo(a, top);
		LineTo(b, bottom);
		b := b - 1;
		end;
		
	a := bottom;
	for b := top to (bottom + 1) do
		begin
		MoveTo(left, a);
		LineTo(right, b);
		a := a - 1;
		end;
	end;
		{of procedure DoLines}

{  DoDrawWithMouse }

procedure DoDrawWithMouse;

	var
	mouseDownMouse, previousMouse, currentMouse : Point;
	drawRect : Rect;
	thePattern : Pattern;
        
	begin
	PenSize(1, 1);
	PenPat(qd.gray);
	PenMode(patXor);

	GetMouse(mouseDownMouse);                     

	drawRect.left := mouseDownMouse.h;
	drawRect.right := mouseDownMouse.h;
	drawRect.top := mouseDownMouse.v;
	drawRect.bottom := mouseDownMouse.v;

	GetMouse(previousMouse);

	while StillDown do
		begin
		GetMouse(currentMouse);

		if ((currentMouse.v <> previousMouse.v) or (currentMouse.h <> previousMouse.h))
			then begin
			FrameRect(drawRect);

			drawRect.right := currentMouse.h;
			drawRect.bottom := currentMouse.v;

			FrameRect(drawRect);
			end;
			
		previousMouse.v := currentMouse.v;
		previousMouse.h := currentMouse.h;
		end;

	FrameRect(drawRect);

	PenMode(patCopy);

	PenSize(2, 2);
	PenPat(qd.black);
	ForeColor(redColor);
	FrameRect(drawRect);

	InsetRect(drawRect, 10, 10);
	PenSize(8, 8);
	GetIndPattern(thePattern ,0, 5);
	PenPat(thePattern);
	ForeColor(blueColor);
	FrameRoundRect(drawRect, 40, 40);

	InsetRect(drawRect, 16, 16);
	PenSize(14, 14);
	GetIndPattern(thePattern,0, 6);
	PenPat(thePattern);
	ForeColor(greenColor);
	PaintOval(drawRect);

	PenMode(patCopy);
	ForeColor(blackColor);
	
	end;
		{of procedure DoDrawWithMouse}
		
{  DoDemonstrationMenu }

procedure DoDemonstrationMenu(menuItem : integer);

	begin

	gDrawWithMouseActivated := false;

	case (menuItem) of

		iLine:
			begin
			DoLines;
			end;

		iRectAndOval:
			begin
			DoRectOval;
			end;			

		iArcAndWedge:
			begin
			DoArcWedge;
			end;

		iPolygon:
			begin
			DoPolygon;
			end;

		iRegion:
			begin
			DoRegion;
			end;

		iTransferMode:
			begin
			DoTransferMode;
			end;

		iCopyBits:
			begin
			DoCopyBits;
			end;

		iText:
			begin
			DoText;
			end;

		iBasicColour:
			begin
			DoBasicColours;
			end;
		
		iDrawWithMouse:
			begin
			FillRect(gWindowPtr^.portRect, qd.white);
			MoveTo(10, 25);
			DrawString('Click in the window and drag the mouse to the right and down');  
			gDrawWithMouseActivated := true;
			end;

		end;
			{of case statement}
	end;
		{of procedure DoDemonstrationMenu}

{  DoMenuChoice }

procedure DoMenuChoice(menuChoice : longint);

	var
	menuID, menuItem : integer;
	itemName : string;
	daDriverRefNum : integer;

	begin
	menuID := HiWord(menuChoice);
	menuItem := LoWord(menuChoice);

	if (menuID = 0) then
		Exit(DoMenuChoice);

	case (menuID) of

		mApple:
			begin
			GetMenuItemText(GetMenuHandle(mApple), menuItem, itemName);
			daDriverRefNum := OpenDeskAcc(itemName);
			end;

		mFile:
			begin
			if (menuItem = iQuit) then
				gDone := true;
			end;

		mDemonstration:
			begin
			DoDemonstrationMenu(menuItem);
			end;
			
		end;
			{of case statement}
	
	HiliteMenu(0);
	end;
		{of procedure DoMenuChoice}

{  DoMouseDown }

procedure DoMouseDown(var theEvent : EventRecord);

	var
	myWindowPtr : WindowPtr;
	partCode : integer;

	begin
	partCode := FindWindow(theEvent.where, myWindowPtr);
	
	case (partCode) of

		inMenuBar:
			begin
			DoMenuChoice(MenuSelect(theEvent.where));
			end;

		inSysWindow:
			begin
			SystemClick(theEvent, myWindowPtr);
			end;

		inContent:
			begin
			if (myWindowPtr <> FrontWindow)
				then SelectWindow(myWindowPtr)
				else if gDrawWithMouseActivated = true then 
					DoDrawWithMouse;
			end;

		inDrag:
			begin
			DragWindow(myWindowPtr, theEvent.where, qd.screenBits.bounds);
			end;

		inGoAway:
			begin
			if (TrackGoAway(myWindowPtr, theEvent.where)) then
				gDone := true;
			end;
			
		end;
			{of case statement}
	end;
		{of procedure DoMouseDown}

{  DoEvents }

procedure DoEvents(var theEvent : EventRecord);

	var
	myWindowPtr : WindowPtr;
	charCode : char;

	begin
	myWindowPtr := WindowPtr(theEvent.message);
	
	case (theEvent.what) of

		mouseDown:
			begin
			DoMouseDown(theEvent);
			end;

		keyDown, autoKey:
			begin
			charCode := chr(BAnd(theEvent.message, charCodeMask));
			if (BAnd(theEvent.modifiers, cmdKey) <> 0) then
				DoMenuChoice(MenuKey(charCode));
			end;

		updateEvt:
			begin
			BeginUpdate(myWindowPtr);
			EndUpdate(myWindowPtr);
			end;
		end;
			{of case statement}
	end;
		{of procedure DoEvents}

{  start of main program }

begin

	{  initialize managers }

	DoInitManagers;
	
	{  set random number generator }
	
	GetDateTime(qd.randSeed);
	
	{  set up menu bar and menus }

	menubarHdl := GetNewMBar(rMenubar);
	if (menubarHdl = nil) then
		ExitToShell;
	SetMenuBar(menubarHdl);
	DrawMenuBar;
	
	menuHdl := GetMenuHandle(mApple);
	if (menuHdl = nil) 
		then	ExitToShell
		else	AppendResMenu(menuHdl, 'DRVR');

	{  open window }

	gWindowPtr := GetNewWindow(rWindow, nil, WindowPtr(-1));
	if (gWindowPtr = nil) then
		ExitToShell;

	SetPort(gWindowPtr);
	TextSize(10);

	{  eventLoop }

	gDone := false;

	while not (gDone) do
		begin
		gotEvent := WaitNextEvent(everyEvent, eventRec, kMaxLong, nil);
		if (gotEvent) then
			DoEvents(eventRec);
		end;

end.
	{of program}
	
{  }
