{ 
// ColorQuickDrawPascal.p
// 
// 
// This program:
//
//   Opens a window in which the results of various basic Color QuickDraw drawing
//	  operations are displayed.
//
//	  Individual drawing operations are selected from a pull-down menu titled
//	  'Demonstration'.)
//
//   Quits when the user selects Quit from the File menu or clicks the window's close
//	  box.
//
// The program utilizes the following resources:
//
//   An 'MBAR' resource and associated 'MENU' resources (preload, non-purgeable).
//
//   'WIND' resources (purgeable) (initially visible) for the main window, and for small
//	  windows used for the CopyDeepMask and Transfer Modes demonstrations.
//
//   An 'ALRT' resource and associated 'DITL' resource (purgeable).
//
//   Three 'PICT' resources (purgeable).
//
//   Two 'pltt' resources (purgeable).
//
//   Two 'ppat' resources (purgeable).
//
//   A 'STR#' resource (purgeable).
//
//  }

program ColorQuickDrawPascal(input, output);

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, TextEdit, Quickdraw, Dialogs, QuickdrawText, Processes, Types, 
	Memory, Events, TextUtils, ToolUtils, OSUtils, Devices, Palettes, QDOffscreen,
	Resources, LowMem, GestaltEqu, Segload;

{  define the following constants }

const

mApple = 128;
mFile = 129;
	iQuit = 11;
mDemonstration = 131;
	iBitPattern = 1;
	iPixelPattern = 2;
	iCopyDeepMask = 3;
	iTransferModes = 4;
	iHighlighting = 5;
	iColorTable = 6;
rWindow = 128;
rImageWindow = 129;
rMenubar = 128;
rAlert = 128;
rIndexedStrings = 128;
rPaletteBaseID = 128;
rPixelPattern1 = 128;
rPixelPattern2 = 129;
rPicture = 128;
sColorQuickdraw = 1;
sSettingMonitor = 2;
sNeedMonitor = 3;
sRestoringMonitor = 4;

kMaxLong = $7FFFFFFF;

{  global variables }

var

gDone : boolean;
gWindowPtr : WindowPtr;
gWhiteColour : RGBColor;
gBlackColour : RGBColor;
gOchreColour : RGBColor;
gGreenColour : RGBColor;
theErr, ignored : OSErr;
response : longint;
alertString : string;
menubarHdl : Handle;
menuHdl : MenuHandle;
eventRec : EventRecord;
gotEvent : boolean;

{  DoInitManagers }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;

	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);

	InitCursor;	
	FlushEvents(everyEvent, 0);
	end;
		{of procedure DoInitManagers}

{  DoBitPattern }

procedure DoBitPattern;

	var
	a : integer;
	paletteHdl : PaletteHandle;
	theRect : Rect;
	sysPattern : Pattern;
	theString : string;

	begin
	for a := 0 to 1 do
		begin
		paletteHdl := GetNewPalette(rPaletteBaseID + a);
		SetPalette(gWindowPtr, paletteHdl, true);

		PmBackColor(2);
		FillRect(gWindowPtr^.portRect, qd.white);

		SetRect(theRect, 10, 30, 245, 150);
		PenSize(10, 20);
		GetIndPattern(sysPattern, sysPatListID, 16);
		PenPat(sysPattern);
		PmForeColor(35);
		PmBackColor(229);
		FrameRect(theRect);	
	
		OffsetRect(theRect, 245, 0);
		GetIndPattern(sysPattern, sysPatListID, 37);
		PenPat(sysPattern);
		PmForeColor(229);
		PmBackColor(210);
		PaintRect(theRect);	

		OffsetRect(theRect, -245, 130);
		GetIndPattern(sysPattern, sysPatListID, 18);
		PmForeColor(210);
		PmBackColor(11);
		FillRoundRect(theRect, 50, 50, sysPattern);	

		OffsetRect(theRect, 245, 0);
		GetIndPattern(sysPattern, sysPatListID, 19);
		PmForeColor(1);
		PmBackColor(0);
		FillOval(theRect, sysPattern);	

		MoveTo(10, 20);
		PmForeColor(1);
		DrawString('Foreground  background colours set with PmForeColor  PmBackColor');
		NumToString(longint(a+1), theString);
		DrawString('       Palette No ');
		DrawString(theString);   

		if (a = 0) then
			begin
			SetWTitle(gWindowPtr, 'Click mouse for another palette');
			while not (Button) do ;
			DisposePalette(paletteHdl);			
			end;
		end;
			{of for loop}

	SetWTitle(gWindowPtr, 'Color QuickDraw');
	DisposePalette(paletteHdl);
	PenPat(qd.black);
	end;
		{of procedure DoBitPattern}

{  DoPixelPattern }

procedure DoPixelPattern;

	var
	pixpat1Hdl, pixpat2Hdl : PixPatHandle;
	theRect : Rect;
	oldClipHdl, regionAHdl, regionBHdl, regionCHdl, scrollRegionHdl : RgnHandle;
	a : integer;

	begin
	RGBBackColor(gWhiteColour);
	FillRect(gWindowPtr^.portRect, qd.white);
	
	pixpat1Hdl := GetPixPat(rPixelPattern1);
	if (pixpat1Hdl = nil) then
		ExitToShell;
	PenPixPat(pixpat1Hdl);
	PenSize(50, 0);
	SetRect(theRect, 15, 15, 240, 280);
	FrameRect(theRect);
	SetRect(theRect, 260, 15, 485, 280);
	FillCRect(theRect, pixpat1Hdl);

	pixpat2Hdl := GetPixPat(rPixelPattern2);
	if (pixpat2Hdl = nil) then
		ExitToShell;
	BackPixPat(pixpat2Hdl);

	regionAHdl := NewRgn;
	regionBHdl := NewRgn;
	regionCHdl := NewRgn;
	SetRect(theRect, 65, 15, 190, 280);
	RectRgn(regionAHdl, theRect);
	SetRect(theRect, 260, 15, 485, 280);
	RectRgn(regionBHdl, theRect);
	UnionRgn(regionAHdl, regionBHdl, regionCHdl);

	oldClipHdl := NewRgn;
	GetClip(oldClipHdl);
	SetClip(regionCHdl);

	SetRect(theRect, 65, 15, 485, 280);

	scrollRegionHdl := NewRgn;

	for a := 0 to 279 do
		begin
		ScrollRect(theRect, 0, 1, scrollRegionHdl);
		theRect.top := theRect.top + 1;
		end;

	SetRect(theRect, 65, 15, 485, 280);
	BackPixPat(pixpat1Hdl);

	for a := 0 to 279 do
		begin
		ScrollRect(theRect, 0, -1, scrollRegionHdl);
		theRect.bottom := theRect.bottom - 1;
		end;

	SetClip(oldClipHdl);

	DisposePixPat(pixpat1Hdl);
	DisposePixPat(pixpat2Hdl);
	DisposeRgn(oldClipHdl);
	DisposeRgn(regionAHdl);
	DisposeRgn(regionBHdl);
	DisposeRgn(regionCHdl);
	DisposeRgn(scrollRegionHdl);
	
	PenPat(qd.black);
	end;
		{of procedure DoPixelPattern}

{  DoCopyDeepMask }

procedure DoCopyDeepMask;

	var		
	sourceWindowPtr : WindowPtr;
	picture1Hdl, picture2Hdl : PicHandle;
	sourceRect, maskRect, destRect, maskDisplayRect : Rect;
	windowPortPtr : CGrafPtr;	
	deviceHdl : GDHandle;
	gworldPortPtr : GWorldPtr;
	gworldPixMapHdl : PixMapHandle;
	regionHdl : RgnHandle;
	finalTicks : longint;
	ignored : OSErr;
	alsoIgnored : boolean;

	begin
	RGBForeColor(gBlackColour);
	RGBBackColor(gWhiteColour);
	FillRect(gWindowPtr^.portRect, qd.white);

	sourceWindowPtr := GetNewCWindow(rImageWindow, nil, WindowPtr(-1));
	if (sourceWindowPtr = nil) then
		ExitToShell;
	SetPort(sourceWindowPtr);

	picture1Hdl := GetPicture(rPicture);
	if (picture1Hdl = nil) then
		ExitToShell;
	HNoPurge(Handle(picture1Hdl));
	SetRect(sourceRect, 10, 10, 167, 122);
	DrawPicture(picture1Hdl, sourceRect);
	HPurge(Handle(picture1Hdl));

	SetRect(maskRect, 0, 0, 157, 112);
	GetGWorld(windowPortPtr, deviceHdl);	
	ignored := NewGWorld(gworldPortPtr, 0, maskRect, nil, nil, 0);	
	SetGWorld(gworldPortPtr, nil);
	gworldPixMapHdl := GetGWorldPixMap(gworldPortPtr);
	alsoIgnored := LockPixels(gworldPixMapHdl);
	EraseRect(gworldPortPtr^.portRect);
	picture2Hdl := GetPicture(rPicture+1);
	if (picture2Hdl = nil) then
		ExitToShell;
	HNoPurge(Handle(picture2Hdl));
	DrawPicture(picture2Hdl, maskRect);
	SetGWorld(windowPortPtr, deviceHdl);

	SetPort(gWindowPtr);
	SetRect(maskDisplayRect, 19, 165, 176, 277);
	DrawPicture(picture2Hdl, maskDisplayRect);
	HPurge(Handle(picture2Hdl));
	MoveTo(43, 160);
	DrawString('Copy of offscreen mask');
	
	SetRect(destRect, 220, 20, 470, 275);
	regionHdl := NewRgn;
	OpenRgn;
	FrameOval(destRect);
	CloseRgn(regionHdl);

	PenSize(1, 1);
	PenPat(qd.ltGray);
	FrameRgn(regionHdl);
	MoveTo(315, 150);
	DrawString('The region');

	SetWTitle(sourceWindowPtr, 'Click Mouse to Copy');
	while not (Button) do ;
	FillRect(destRect, qd.white);

	CopyDeepMask(GrafPtr(sourceWindowPtr)^.portBits, GrafPtr(gworldPortPtr)^.portBits, 
							GrafPtr(gWindowPtr)^.portBits, sourceRect, maskRect, 
							destRect, srcCopy+ditherCopy, regionHdl);

	SetWTitle(sourceWindowPtr, 'Click Mouse to Close');
	Delay(60, finalTicks);

	while not (Button) do ;
	FillRect(gWindowPtr^.portRect, qd.white);
	
	UnlockPixels(gworldPixMapHdl);
	DisposeGWorld(gworldPortPtr);	

	ReleaseResource(Handle(picture1Hdl));
	ReleaseResource(Handle(picture2Hdl));
	DisposeRgn(regionHdl);
	DisposeWindow(sourceWindowPtr);

	PenPat(qd.black);
	end;
		{of procedure DoCopyDeepMask}

{  DoCheckMonitor }

function DoCheckMonitor : integer;

	var
	mainDeviceHdl : GDHandle;
	result : integer;
	alertString : string;	
	pixMapHdl : PixMapHandle;
	pixelDepth : integer;
	ignored : OSErr;

	begin
	mainDeviceHdl := LMGetMainDevice;
	result := HasDepth(mainDeviceHdl, 16, 0, 0);

	if (result = 0)
		then	begin
				GetIndString(alertString, rIndexedStrings, sNeedMonitor);
				ParamText(alertString, '', '', '');
				ignored := NoteAlert(rAlert, nil);
				DoCheckMonitor := 0;
				Exit(DoCheckMonitor);
				end
			
		else	begin
				pixMapHdl := mainDeviceHdl^^.gdPMap;
				pixelDepth := pixMapHdl^^.pixelSize;
				if (pixelDepth < 16) then
					begin
					GetIndString(alertString, rIndexedStrings, sSettingMonitor);
					ParamText(alertString, '', '', '');
					ignored := NoteAlert(rAlert, nil);
					ignored := SetDepth(mainDeviceHdl, 16, 0, 0);
					DoCheckMonitor := pixelDepth;
					Exit(DoCheckMonitor);
					end;
				DoCheckMonitor := 2;
				end;
	end;
		{of function DoCheckMonitor}

{  DoRestoreMonitor }

procedure DoRestoreMonitor(monitorCheckResult : integer);

	var
	alertString : string;	
	mainDeviceHdl : GDHandle;
	ignored : OSErr;

	begin
	GetIndString(alertString, rIndexedStrings, sRestoringMonitor);
	ParamText(alertString, '', '', '');
	ignored := NoteAlert(rAlert, nil);

	mainDeviceHdl := LMGetMainDevice;
	ignored := SetDepth(mainDeviceHdl, monitorCheckResult, 0, 0);
	end;
		{of procedure DoRestoreMonitor}

{  DoTransferModes }

procedure DoTransferModes;

	var
	monitorCheckResult, transferMode, stringIndex : integer;
	sourceWindowPtr : WindowPtr;
	sourceHdl, destinationHdl : PicHandle;
	sourceRect, destRect, blankRect : Rect;
	modeString : string;
	finalTicks : longint;

	begin
	monitorCheckResult := DoCheckMonitor;
	if (monitorCheckResult = 0) then
		Exit(DoTransferModes);

	RGBForeColor(gBlackColour);
	RGBBackColor(gWhiteColour);
	FillRect(gWindowPtr^.portRect, qd.white);

	sourceWindowPtr := GetNewCWindow(rImageWindow, nil, WindowPtr(-1));
	if (sourceWindowPtr = nil) then
		ExitToShell;
	SetWTitle(sourceWindowPtr, 'Source Image');

	SetPort(sourceWindowPtr);
	sourceHdl := GetPicture(rPicture);
	if (sourceHdl = nil) then
		ExitToShell;
	HNoPurge(Handle(sourceHdl));
	SetRect(sourceRect, 10, 10, 167, 122);
	DrawPicture(sourceHdl, sourceRect);
	HPurge(Handle(sourceHdl));

	SetPort(gWindowPtr);
	destinationHdl := GetPicture(rPicture+2);
	if (destinationHdl = nil) then
		ExitToShell;
	HNoPurge(Handle(destinationHdl));
	SetRect(destRect, 19, 165, 176, 277);
	DrawPicture(destinationHdl, destRect);
	MoveTo(55, 160);
	DrawString('Destination Image');
	
	SetRect(destRect, 270, 95, 427, 207);
	DrawPicture(destinationHdl, destRect);
	SetRect(blankRect, 270, 50, 427, 207);

	stringIndex := 5;
	for transferMode := 0 to 39 do
		begin
		if (transferMode = 8) then
			transferMode := 32;

		GetIndString(modeString, rIndexedStrings, stringIndex);
		MoveTo(270, 70);
		DrawString('Click Mouse for ');
		DrawString(modeString);

		while not (Button) do ;

		FillRect(blankRect, qd.white);
		DrawPicture(destinationHdl, destRect);
		Delay(30, finalTicks);

		CopyBits(GrafPtr(sourceWindowPtr)^.portBits, GrafPtr(gWindowPtr)^.portBits, 
						 sourceRect, destRect, transferMode + ditherCopy, nil);

		MoveTo(270, 92);
		if (transferMode < 8) 
			then	DrawString('Boolean mode: ')
			else	DrawString('Arithmetic mode: ');
		DrawString(modeString);
		Delay(60, finalTicks);
		stringIndex := stringIndex + 1;
		end;
			{of for loop}

	MoveTo(270, 70);
	DrawString('Click Mouse to exit');
	while not (Button) do ;
	
	FillRect(gWindowPtr^.portRect, qd.white);

	ReleaseResource(Handle(sourceHdl));
	ReleaseResource(Handle(destinationHdl));	
	DisposeWindow(sourceWindowPtr);

	if (monitorCheckResult <> 2) then
		DoRestoreMonitor(monitorCheckResult);
	end;
		{of procedure DoTransferModes}

{  DoHighlighting }

procedure DoHighlighting;

	var
	grafVarsHdl : GVarHandle;
	oldHighlightColour : RGBColor;
	a : integer;
	theRect : Rect;
	hiliteVal : ByteParameter;
	finalTicks : longint;

	begin
	RGBBackColor(gWhiteColour);
	FillRect(gWindowPtr^.portRect, qd.white);

	grafVarsHdl := GVarHandle (CGrafPtr(gWindowPtr)^.grafVars);
	oldHighlightColour := grafVarsHdl^^.rgbHiliteColor;

	for a := 0 to 2 do
		begin
		MoveTo(20, a*80+40);
		DrawString('Clearing the highlight bit and calling InvertRect.');
		Delay(60, finalTicks);
		SetRect(theRect, 10, a * 80 + 20, 490, a * 80 + 80);

		hiliteVal := LMGetHiliteMode;
		BitClr(Ptr(@hiliteVal), pHiliteBit);
		LMSetHiliteMode(hiliteVal);

		if (a = 1)
			then	HiliteColor(gOchreColour)
			else 	if (a = 2) then
						HiliteColor(gGreenColour);
		InvertRect(theRect);

		MoveTo(20, a*80+55);
		Delay(60, finalTicks);
		DrawString('Click mouse to unhighlight.  ');
		DrawString('(Note:  The call to InvertRect reset the highlight bit ...');

		while not (Button) do ;

		MoveTo(20, a*80+70);
		DrawString('... so we clear the highlight bit again before calling InvertRect.)');
		Delay(60, finalTicks);

		LMSetHiliteMode(hiliteVal);
		InvertRect(theRect);	
		end;
			{of for loop}

	HiliteColor(oldHighlightColour);
	Delay(60, finalTicks);
	MoveTo(20, 260);
	DrawString('Original highlight colour has been reset.');
	end;
		{of procedure DoHighlighting}

{  DoColourTable }

procedure DoColourTable;

	var
	pixMapHdl : PixMapHandle;
	colorTableHdl : CTabHandle;
	entries, a, b, c, i, j : integer;
	theRect : Rect;
	theColour : RGBColor;

	begin
	entries := 0;
	a := 0;
	b := 0;
	c := 0;
	RGBForeColor(gBlackColour);
	FillRect(gWindowPtr^.portRect, qd.black);

	pixMapHdl := CGrafPtr(gWindowPtr)^.portPixMap;
	colorTableHdl := pixMapHdl^^.pmTable;
	entries := colorTableHdl^^.ctSize;	

	if (entries = 0)
		then	begin
			RGBForeColor(gWhiteColour);
			MoveTo(90,135);
			DrawString('You need to set the monitor to 256 colours or less to get some');
			MoveTo(90,150);
			DrawString('entries in the colour table.  At present, we have zero entries.');
			end;

	for i := 0 to 15 do
		begin
		a := i * 30 + 12;
		for j := 0 to 15 do
			begin
			b := j * 18 + 5;
			if (c > entries) then Exit(DoColourTable);
			SetRect(theRect, a, b, a+28, b+17);
			theColour := colorTableHdl^^.ctTable[c].rgb;
			c := c + 1;
			RGBForeColor(theColour);
			PaintRect(theRect);
			end;
		end;
	end;
		{of procedure DoColourTable}

{  DoRGBColours }

procedure DoRGBColours;

	begin
	gWhiteColour.red := $FFFF;
	gWhiteColour.green := $FFFF;
	gWhiteColour.blue := $FFFF;

	gBlackColour.red := $0000;
	gBlackColour.green := $0000;
	gBlackColour.blue := $0000;

	gOchreColour.red := $CCCC;
	gOchreColour.green := $71FC;
	gOchreColour.blue := $6A28;

	gGreenColour.red := $460D;
	gGreenColour.green := $CCCC;
	gGreenColour.blue := $6BE2;
	end;
		{of procedure DoRGBColours}

{  DoDemonstrationMenu }

procedure DoDemonstrationMenu(menuItem : integer);

	begin
	case (menuItem) of

		iBitPattern:
			begin
			DoBitPattern;
			end;

		iPixelPattern:
			begin
			DoPixelPattern;
			end;

		iCopyDeepMask:
			begin
			DoCopyDeepMask;
			end;

		iTransferModes:
			begin
			DoTransferModes;
			end;

		iHighlighting:
			begin
			DoHighlighting;
			end;

		iColorTable:
			begin
			DoColourTable;
			end;
		
		end;
			{of case statement}
	end;
		{of procedure DoDemonstrationMenu}

{  DoMenuChoice }

procedure DoMenuChoice(menuChoice : longint);

	var
	menuID, menuItem : integer;
	itemName : string;
	daDriverRefNum : integer;

	begin
	menuID := HiWord(menuChoice);
	menuItem := LoWord(menuChoice);

	if (menuID = 0) then
		Exit(DoMenuChoice);

	case (menuID) of

		mApple:
			begin
			GetMenuItemText(GetMenuHandle(mApple), menuItem, itemName);
			daDriverRefNum := OpenDeskAcc(itemName);
			end;

		mFile:
			begin
			if (menuItem = iQuit) then
				gDone := true;
			end;

		mDemonstration:
			begin
			DoDemonstrationMenu(menuItem);
			end;
			
		end;
			{of case statement}
	
	HiliteMenu(0);
	end;
		{of procedure DoMenuChoice}

{  DoMouseDown }

procedure DoMouseDown(var eventRec : EventRecord);

	var
	myWindowPtr : WindowPtr;
	partCode : integer;

	begin
	partCode := FindWindow(eventRec.where, myWindowPtr);

	case (partCode) of

		inMenuBar:
			begin
			DoMenuChoice(MenuSelect(eventRec.where));
			end;

		inSysWindow:
			begin
			SystemClick(eventRec, myWindowPtr);
			end;

		inContent:
			begin
			if (myWindowPtr <> FrontWindow) then
				SelectWindow(myWindowPtr);
			end;

		inDrag:
			begin
			DragWindow(myWindowPtr, eventRec.where, qd.screenBits.bounds);
			end;

		inGoAway:
			begin
			if (TrackGoAway(myWindowPtr, eventRec.where)) then
				gDone := true;
			end;
		
		end;
			{of case statement}
	end;
		{of procedure DoMouseDown}

{  DoEvents }

procedure DoEvents(var eventRec : EventRecord);

	var		
	myWindowPtr : WindowPtr;
	charCode : char;

	begin
	myWindowPtr := WindowPtr(eventRec.message);

	case (eventRec.what) of

		mouseDown:
			begin
			DoMouseDown(eventRec);
			end;

		keyDown, autoKey:
			begin
			charCode := chr(BAnd(eventRec.message, charCodeMask));
			if (BAnd(eventRec.modifiers, cmdKey) <> 0) then
				DoMenuChoice(MenuKey(charCode));
			end;

		updateEvt:
			begin
			BeginUpdate(myWindowPtr);
			EndUpdate(myWindowPtr);
			end;
			
		end;
			{of case statement}
	end;
		{of procedure DoEvents}

{  start of main program }

begin

	{  initialize managers }

	DoInitManagers;

	{  check for Color QuickDraw }

	theErr := Gestalt(gestaltQuickdrawVersion, response);
	if (response < gestalt8BitQD) then
		begin
		GetIndString(alertString, rIndexedStrings, sColorQuickdraw);
		ParamText(alertString, '', '', '');
		ignored := StopAlert(rAlert, nil);
		ExitToShell;
		end;

	{  set up menu bar and menus }
	
	menubarHdl := GetNewMBar(rMenubar);
	if (menubarHdl = nil) then
		ExitToShell;
	SetMenuBar(menubarHdl);
	DrawMenuBar;

	menuHdl := GetMenuHandle(mApple);
	if (menuHdl = nil) 
		then	ExitToShell
		else	AppendResMenu(menuHdl, 'DRVR');

	{  open window }

	gWindowPtr := GetNewCWindow(rWindow, nil, WindowPtr(-1));
	if (gWindowPtr = nil) then
		ExitToShell;

	SetPort(gWindowPtr);
	TextSize(10);

	{  create some RGB colours }

	DoRGBColours;

	{  eventLoop }

	gDone := false;

	while not (gDone) do
		begin
		gotEvent := WaitNextEvent(everyEvent, eventRec, kMaxLong, nil);
		if (gotEvent) then
			DoEvents(eventRec);
		end;

end.
	
{  }
