{ 
// PrintingPascal.p
// 
// 
// This program:
//
// 	Opens a window in which the contents of the main fields in the TPrint, TPrJob,
//		TPrStl and TPrInfo records are displayed.
//
// 	Allows the user to note changes in these fields after invoking the style dialog
//		and job dialog boxes.
//
// 	Allows the user to print a simulated document.
//
// 	Quits when the user chooses Quit or clicks the window's close box.
//
// The program utilises the following resources:
//
// 	'MBAR' resource and associated 'MENU' resources (preload, non-purgeable).
//
// 	A 'WIND' resource (purgeable).
// 
// 	An 'ALRT' resource and associated 'DITL' resource for an alert which reports
//		printing errors (purgeable).
//
// 	A 'TEXT' resource (non-purgeable) used for printing.
//
// 	A 'PICT' resource (non-purgeable) used for printing.
//
//  }

program PrintingPascal(input, output);

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, TextEdit, Quickdraw, Dialogs, QuickdrawText, Processes, Types, 
	Memory, Events, TextUtils, ToolUtils, OSUtils, Devices, Printing, Resources, 
	SegLoad, Errors;

{  define the following constants }

const

mApple = 128;
mFile = 129;
	iQuit = 11;
	iPageSetup = 8;
	iPrint = 9;
rMenubar = 128;
rWindow = 128;
rText = 128;
rPicture = 128;
rPrintAlert = 128;
kMargin = 90;
kMaxLong = $7FFFFFFF;

{  global variables }

var

gTPrintHdl : THPrint;
gWindowPtr : WindowPtr;
gDone : boolean;
gPrintRecordInited : boolean;
gInhibitPrintRecordsInfo : boolean;
gEditRecHdl : TEHandle;
gTextHdl : Handle;
gPictureHdl : PicHandle;
menubarHdl : Handle;
menuHdl : MenuHandle;
eventRec : EventRecord;
gotEvent : boolean;

{  DoInitManagers }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;

	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);

	InitCursor;	
	FlushEvents(everyEvent, 0);
	end;
		{of procedure DoInitManagers}

{  DoPrintError }

procedure DoPrintError(printError : longint; fatal : boolean);

	var
	errorNumberString : string;
	ignored : OSErr;

	begin
	NumToString(printError, errorNumberString);
	ParamText(errorNumberString, '', '', '');
	if (fatal) then
		begin
		ignored := StopAlert(rPrintAlert, nil);
		ExitToShell;
		end
	else
		ignored := CautionAlert(rPrintAlert, nil);
	end;
		{of procedure DoPrintError}

{  DoIsPrGeneralThere }

function DoIsPrGeneralThere : boolean;

	var
	getRotRec : TGetRotnBlk;
	printError : OSErr;

	begin
	printError := 0;
	getRotRec.iOpCode := getRotnOp;
	getRotRec.hPrint := gTPrintHdl;

	PrGeneral(Ptr(@getRotRec));

	printError := PrError;
	PrSetError(noErr);

	if (printError = resNotFound) then
		DoIsPrGeneralThere := false
	else
		DoIsPrGeneralThere := true;
	end;
		{of function DoIsPrGeneralThere}

{  DoGetPageOrientation }

function DoGetPageOrientation : longint;

	var
	getRotRec : TGetRotnBlk;

	begin
	if (DoIsPrGeneralThere) then
		begin
		getRotRec.iOpCode := getRotnOp;
		getRotRec.hPrint := gTPrintHdl;
		PrGeneral(Ptr(@getRotRec));
		if ((getRotRec.iError = noErr) and (PrError = noErr) and (getRotRec.fLandscape))
			then 	DoGetPageOrientation := 1
			else	DoGetPageOrientation := 2;
		end
	else
		DoGetPageOrientation := 3;
	end;
		{of function DoGetPageOrientation}

{  DoDrawPageOrientation }

procedure DoDrawPageOrientation;
	
	var
	orientation : longint;

	begin
	MoveTo(20, 260);
	DrawString('Orientation selected:');

	orientation := DoGetPageOrientation;

	MoveTo(190, 260);		
	if (orientation = 1) then
		DrawString('Landscape')
	else if (orientation = 2) then
		DrawString('Portrait')
	else
		DrawString('(PrGeneral not supported by driver)');
	end;
		{of procedure DoDrawPageOrientation}

{  DoDrawRectStrings }

procedure DoDrawRectStrings(s1: string; x1, y1: integer; s2: string; x2, y2: integer;
									 s3: string);
	
	begin
	MoveTo(x1, y1);
	DrawString(s1);
	MoveTo(x2, y2);
	DrawString('(');
	DrawString(s2);
	DrawString(', ');
	DrawString(s3);
	DrawString(')');
	end;
		{of procedure DoDrawRectStrings}

{  DoPrintRecordsInfo }

procedure DoPrintRecordsInfo;

	var
	s2, s3 : string;

	begin
	EraseRect(gWindowPtr^.portRect);	

	MoveTo(20, 25);
	DrawString('From TPrint, TPrInfo and TPrStl records:');

	NumToString(integer(gTPrintHdl^^.rPaper.top), s2);
	NumToString(integer(gTPrintHdl^^.rPaper.left), s3);
	DoDrawRectStrings('Paper Rectangle (top, left):', 20, 45, s2, 190, 45, s3);

	NumToString(integer(gTPrintHdl^^.rPaper.bottom), s2);
	NumToString(integer(gTPrintHdl^^.rPaper.right), s3);
	DoDrawRectStrings('Paper Rectangle (bottom, right):', 20, 60, s2, 190, 60, s3);

	NumToString(integer(gTPrintHdl^^.prInfo.rPage.top), s2);
	NumToString(integer(gTPrintHdl^^.prInfo.rPage.left), s3);
	DoDrawRectStrings('Page Rectangle (top, left):', 20, 75, s2, 190, 75, s3);

	NumToString(integer(gTPrintHdl^^.prInfo.rPage.bottom), s2);
	NumToString(integer(gTPrintHdl^^.prInfo.rPage.right), s3);
	DoDrawRectStrings('Page Rectangle (bottom, right):', 20, 90, s2, 190, 90, s3);

	MoveTo(20, 105);
	DrawString('Feed Type:');
	MoveTo(190, 105);
	if (gTPrintHdl^^.prStl.feed = 0) then
		DrawString('Cut sheet')
	else if (gTPrintHdl^^.prStl.feed = 1) then
		DrawString('Fanfold');

	MoveTo(20, 120);
	DrawString('Vertical resolution:');
	NumToString(integer(gTPrintHdl^^.prInfo.iVRes), s2);
	MoveTo(190, 120);
	DrawString(s2);

	MoveTo(20, 135);
	DrawString('Horizontal resolution:');
	NumToString(integer(gTPrintHdl^^.prInfo.iHRes), s2);
	MoveTo(190, 135);
	DrawString(s2);

	MoveTo(20, 155);
	DrawString('From TPrJob Record:');

	MoveTo(20, 175);
	DrawString('First Page:');
	NumToString(integer(gTPrintHdl^^.prJob.iFstPage), s2);
	MoveTo(190, 175);
	DrawString(s2);

	MoveTo(20, 190);
	DrawString('Last Page:');
	NumToString(integer(gTPrintHdl^^.prJob.iLstPage), s2);
	MoveTo(190, 190);
	DrawString(s2);

	MoveTo(20, 205);
	DrawString('Number of Copies:');
	NumToString(integer(gTPrintHdl^^.prJob.iCopies), s2);
	MoveTo(190, 205);
	DrawString(s2);

	MoveTo(20, 225);
	DrawString('Note:  Some printer drivers always set the iCopies field of the TPrJob');
	MoveTo(20, 240);
	DrawString('record to 1 and handle multiple copies internally.');
	end;
		{of procedure DoPrintRecordsInfo}

{  DoActivateWindow }

procedure DoActivateWindow;

	begin
	if ((FrontWindow = gWindowPtr) and gPrintRecordInited 
			and not gInhibitPrintRecordsInfo) then
		DoPrintRecordsInfo;	
	end;
		{of procedure DoActivateWindow}

{  DoDrawPage }

procedure DoDrawPage(pageRect : Rect; pageNumber, numberOfpages : integer);

	var

	destRect, pictureRect : Rect;
	heightDestRect, linesPerPage, numberOfLines, fontNum : SInt16;
	pageEditRecHdl : TEHandle;
	textHdl : Handle;
	startOffset, endOffset : SInt32;
	theString : Str255;
	
	begin
	SetRect(destRect, pageRect.left + kMargin, pageRect.top + (trunc(kMargin * 1.5)),
					pageRect.right - kMargin, pageRect.bottom - (trunc(kMargin * 1.5)));

	heightDestRect := destRect.bottom - destRect.top;
	linesPerPage := trunc(heightDestRect / gEditRecHdl^^.lineHeight); 
	numberOfLines := gEditRecHdl^^.nLines;
	
	GetFNum('Geneva',fontNum);
	TextFont(fontNum);
	TextSize(10);

	pageEditRecHdl := TENew(destRect,destRect);
	textHdl := gEditRecHdl^^.hText;

	startOffset := gEditRecHdl^^.lineStarts[(pageNumber - 1) * linesPerPage];
	if (pageNumber = numberOfpages) then
		endOffset := gEditRecHdl^^.lineStarts[numberOfLines]
	else
		endOffset := gEditRecHdl^^.lineStarts[pageNumber * linesPerPage];

	HLock(textHdl);
	TEInsert(Ptr(SInt32(textHdl^) + startOffset),endOffset - startOffset,pageEditRecHdl);
	HUnlock(textHdl);

	if (pageNumber = 1) then
		begin
		SetRect(pictureRect, destRect.left, destRect.top,
				destRect.left + (gPictureHdl^^.picFrame.right - gPictureHdl^^.picFrame.left),
				destRect.top + (gPictureHdl^^.picFrame.bottom - gPictureHdl^^.picFrame.top));	
		DrawPicture(gPictureHdl,pictureRect);
		end;

	MoveTo(destRect.left, pageRect.bottom - 25);
	NumToString(SInt32 (pageNumber), theString);
	DrawString(theString);
	end;
		{of procedure DoDrawPrintPage}

{  DoCalcNumberOfPages }

function DoCalcNumberOfPages(pageRect : Rect) : integer;

	var
	destRect, pictureRect : Rect;
	fontNum, heightDestRect, linesPerPage, numberOfPages : SInt16;

	begin
	EraseRect(gWindowPtr^.portRect);	

	SetRect(destRect, pageRect.left + kMargin, pageRect.top + (trunc(kMargin * 1.5)), 
					pageRect.right - kMargin, pageRect.bottom - (trunc(kMargin * 1.5)));
	OffsetRect(destRect, - (kMargin - 5), - ((trunc(kMargin * 1.5)) - 5));

	GetFNum('Geneva',fontNum);
	TextFont(fontNum);
	TextSize(10);

	gEditRecHdl := TENew(destRect, destRect);
	TEInsert(gTextHdl^, GetHandleSize(gTextHdl), gEditRecHdl);

	heightDestRect := destRect.bottom - destRect.top;
	linesPerPage := trunc(heightDestRect / gEditRecHdl^^.lineHeight); 
	numberOfPages := trunc((gEditRecHdl^^.nLines / linesPerPage) + 1);	

	SetRect(pictureRect,destRect.left, destRect.top,
				destRect.left + (gPictureHdl^^.picFrame.right - gPictureHdl^^.picFrame.left),
				destRect.top + (gPictureHdl^^.picFrame.bottom - gPictureHdl^^.picFrame.top));	
	DrawPicture(gPictureHdl,pictureRect);

	DoCalcNumberOfPages := numberOfPages;
	end;
		{of procedure DoCalcNumberOfPages}

{  DoCreatePrintRecord }

 function DoCreatePrintRecord : OSErr;

	var
	printError : integer;
	
	begin
	gTPrintHdl := THPrint(NewHandleClear(sizeof(TPrint)));
	if (gTPrintHdl <> nil) then
		begin
		PrintDefault(gTPrintHdl);
		printError := PrError;
		if (printError = noErr) then
			gPrintRecordInited := true;
		DoCreatePrintRecord := printError;
		end
	else
		ExitToShell;
	end;
		{of procedure DoCreatePrintRecord}

{  DoPrStyleDialog }

procedure DoPrStyleDialog;

	var
	printError : OSErr;
	ignored : boolean;

	begin
	PrOpen;
	printError := PrError;

	if (printError = noErr) then
		begin
		if not (gPrintRecordInited) then
			begin
			printError := DoCreatePrintRecord;
			if (printError <> noErr) then
				DoPrintError(printError, true);
			end;
		ignored := PrStlDialog(gTPrintHdl);
		end
	else
		DoPrintError(printError, false);

	PrClose;
	end;
		{of procedure DoPrStyleDialog}	

{  PrintLoop }

procedure PrintLoop;

	var
	oldPort : GrafPtr;
	printError : integer;
	numberOfPages, numberOfCopies : integer;
	userClickedOK : Boolean;
	firstPage, lastPage, copy, page : integer;
	printPortPtr : TPPrPort;
	printStatus : TPrStatus;

	begin
	GetPort(oldPort);

	PrOpen;
	if (PrError = noErr) then
		begin
		if not (gPrintRecordInited) then 
			printError := DoCreatePrintRecord
		else
			printError := noErr;
			
		if (printError = noErr) then
			begin
			numberOfPages := DoCalcNumberOfPages(gTPrintHdl^^.prInfo.rPage);

			userClickedOK := PrJobDialog(gTPrintHdl);
			if (userClickedOK) then
				begin				
				DoPrintRecordsInfo;
				DoDrawPageOrientation;
				gInhibitPrintRecordsInfo := true;
				
				numberOfCopies := gTPrintHdl^^.prJob.iCopies;
				firstPage := gTPrintHdl^^.prJob.iFstPage;
				lastPage := gTPrintHdl^^.prJob.iLstPage;

				gTPrintHdl^^.prJob.iFstPage := 1;
				gTPrintHdl^^.prJob.iLstPage := iPrPgMax;

				if (numberOfPages < lastPage) then
					lastPage := numberOfPages;
				
				for copy := 1 to numberOfCopies do
					begin					
					for page := firstPage to lastPage do
						begin
						if ((page - firstPage) mod iPFMaxPgs = 0) then
							begin	
							if (page <> firstPage) then
								begin
								PrCloseDoc(printPortPtr);
								
								if ((gTPrintHdl^^.prJob.bJDocLoop = bSpoolLoop) and (PrError = noErr)) then
									PrPicFile(gTPrintHdl, nil, nil, nil, printStatus);
								end;
							printPortPtr := PrOpenDoc(gTPrintHdl, nil, nil);
							end;	
						if (PrError = noErr) then
							begin
							PrOpenPage(printPortPtr, nil);
							if (PrError = noErr) then
								DoDrawPage(gTPrintHdl^^.prInfo.rPage, page,numberOfPages);
							PrClosePage(printPortPtr);
							end;
						end;

					PrCloseDoc(printPortPtr);

					if ((gTPrintHdl^^.prJob.bJDocLoop = bSpoolLoop) and (PrError = noErr)) then
						PrPicFile(gTPrintHdl, nil, nil, nil, printStatus);
					end;
				end;
			end;
		end;
	
	printError := PrError;

	PrClose;

	if ((printError <> noErr) and (printError <> iPrAbort)) then
		DoPrintError(printError, false);

	SetPort(oldPort);
	DoActivateWindow;
	
	end;
		{of procedure PrintLoop}

{  DoMenuChoice }

procedure DoMenuChoice(menuChoice : longint);

	var
	menuID, menuItem : integer;
	itemName : string;
	daDriverRefNum : integer;

	begin
	menuID := HiWord(menuChoice);
	menuItem := LoWord(menuChoice);

	if (menuID = 0) then
		Exit(DoMenuChoice);

	case (menuID) of

		mApple:
			begin
			GetMenuItemText(GetMenuHandle(mApple), menuItem, itemName);
			daDriverRefNum := OpenDeskAcc(itemName);
			end;
			
		mFile:
			begin
			if (menuItem = iPageSetup) then
				begin
				gInhibitPrintRecordsInfo := false;
				DoPrStyleDialog;
				end
			else if (menuItem = iPrint) then
				PrintLoop			
			else if (menuItem = iQuit) then
				gDone := true;
			end;
		end;
			{of case statement}

	HiliteMenu(0);
	end;
		{of procedure DoMenuChoice}

{  DoMouseDown }

procedure DoMouseDown(eventRec : EventRecord);

	var
	myWindowPtr : WindowPtr;
	partCode : integer;

	begin
	partCode := FindWindow(eventRec.where, myWindowPtr);

	case (partCode) of

		inMenuBar:
			begin
			DoMenuChoice(MenuSelect(eventRec.where));
			end;

		inSysWindow:
			begin
			SystemClick(eventRec, myWindowPtr);
			end;

		inContent:
			begin
			if (myWindowPtr <> FrontWindow) then
				SelectWindow(myWindowPtr);
			end;

		inDrag:
			begin
			DragWindow(myWindowPtr, eventRec.where, qd.screenBits.bounds);
			end;

		inGoAway:
			begin
			if (TrackGoAway(myWindowPtr, eventRec.where)) then
				gDone := true;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoMouseDown}
	
{  DoEvents }

procedure DoEvents(eventRec : EventRecord);

 	var
	myWindowPtr : WindowPtr;
	charCode : char;

	begin
	myWindowPtr := WindowPtr(eventRec.message);
	
	case (eventRec.what) of

		mouseDown:
			begin
			DoMouseDown(eventRec);
			end;

		keyDown, autoKey:
			begin
			charCode := chr(BAnd(eventRec.message, charCodeMask));
			if (BAnd(eventRec.modifiers, cmdKey) <> 0) then
				DoMenuChoice(MenuKey(charCode));
			end;

		updateEvt:
			begin
			BeginUpdate(myWindowPtr);
			EndUpdate(myWindowPtr);
			end;

		activateEvt:
			begin
			DoActivateWindow;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoEvents}
	
{  start of main program }

begin

	{  initialize managers }

	DoInitManagers;
	gPrintRecordInited := false;
	gInhibitPrintRecordsInfo := false;
	
	{  set up menu bar and menus }

	menubarHdl := GetNewMBar(rMenubar);
	if (menubarHdl = nil) then
		ExitToShell;
	SetMenuBar(menubarHdl);
	DrawMenuBar;
	menuHdl := GetMenuHandle(mApple);
	if (menuHdl = nil)
		then	ExitToShell
		else	AppendResMenu(menuHdl, 'DRVR');

	{  open window }

	gWindowPtr := GetNewWindow(rWindow, nil, WindowPtr(-1));
	if (gWindowPtr = nil) then
		ExitToShell;

	SetPort(gWindowPtr);
	TextSize(10);

	{  load 'TEXT' and 'PICT' resources }
	
	gTextHdl := GetResource('TEXT',rText);
	if (gTextHdl = nil) then
		ExitToShell;

	gPictureHdl := GetPicture(rPicture);
	if (gPictureHdl = nil) then
		ExitToShell;

	{  event loop }

	gDone := false;
	
	while not (gDone) do 
		begin
		gotEvent := WaitNextEvent(everyEvent, eventRec, kMaxLong, nil);
		if (gotEvent) then
			DoEvents(eventRec);
		end;
end.
	
{  }
