{ 
// ScrapPascal.p
// 
// 
// This program utilizes the desk scrap and Scrap Manager routines to allow the user to:
//
//    Cut, copy and clear pictures from, and paste pictures to, two windows opened by the
//	   program.
//
//    Paste pictures cut or copied from another application to the two windows opened
//	   by the program.
//
//    Open and close a Clipboard window, in which the current contents of the desk scrap
//	   are displayed.
//
// In addition to the pictures cut and copied from either the program's windows or from
// another application's windows, the Clipboard window will display text copied to the
// desk scrap as a result of text cut and copy operations in another application.  The
// program, however, does not support the pasting of this text to documents displayed in
// the program's windows.  (The demonstration program at Chapter 17  Text and TextEdit
// shows how to cut, copy and paste text from and to a TextEdit edit record using the
// desk scrap.)   
//
// The program utilizes the following resources:
//
//    An 'MBAR' resource, and 'MENU' resources for Apple, File, and Edit menus (preload, 
//	   non-purgeable).  
//
//    Three 'WIND' resources (purgeable) (initially visible), two for the program's main
//	   windows and one for the Clipboard window.  
//
//    A 'PICT' resource (non-purgeable) containing a picture which may be cut, copied, 
//	   and pasted between the windows.
//
//    An 'ALRT' resource (purgeable) and associated 'DITL' resource (purgeable) for use
//	   by an error Alert.  
//
//    A 'STR#' resource (purgeable) containing strings to be displayed in the error
//	   Alert.  
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents, doesActivateOnFGSwitch, and
//		is32BitCompatible flags set.
//
//  }

program ScrapPascal(input, output);

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, TextEdit, Quickdraw, Dialogs, QuickdrawText, Processes, Types, 
	Memory, Events, TextUtils, ToolUtils, OSUtils, Devices, Scrap, SegLoad;

{  define the following constants }

const

mApple = 128;
	iAbout = 1;
mFile = 129;
	iClose = 4;
	iQuit = 11;
mEdit = 130;
	iCut = 3;
	iCopy = 4;
	iPaste = 5;
	iClear = 6;
	iClipboard = 9;

rMenubar = 128;
rWindow = 128;
rClipboardWindow = 130;
rPicture = 128;
rAlertBox = 128;
rErrorStrings = 128;
	eFailMenu = 1;
	eFailWindow = 2;
	eFailDocRec = 3;
	eZeroScrap = 4;
	ePutScrap = 5;
	eNoPictInScrap = 6;

kDocumentType = 1;
kClipboardType = 2;

{  user defined types }

type

DocRec = record
	pictureHdl : PicHandle;
	selectFlag : boolean;
	windowType : integer;
	end;
	
DocRecPtr = ^DocRec;
DocRecHandle = ^DocRecPtr;

{  global variables }

var

gDone : boolean;
gInBackground : boolean;
gWindowPtrs : array[0..1] of WindowPtr;
gClipboardWindowPtr : WindowPtr;
gClipboardShowing : boolean;
gMarqueePattern : Pattern;
menubarHdl : Handle;
menuHdl : MenuHandle;
gotEvent : boolean ;
theEvent : EventRecord;

{  DoInitManagers }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;

	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);

	InitCursor;	
	FlushEvents(everyEvent, 0);
	end;
		{of procedure DoInitManagers}

{  DoErrorAlert }

procedure DoErrorAlert(errorCode : integer);

	var
	errorString : string;
	ignored : OSErr;

	begin
	GetIndString(errorString, rErrorStrings, errorCode);
	ParamText(errorString, '', '', '');

	if (errorCode < ePutScrap) then
		begin
		ignored := StopAlert(rAlertBox, nil);
		ExitToShell;
		end
	else
		ignored := CautionAlert(rAlertBox, nil);
	end;
		{of procedure DoErrorAlert}

{  DoOpenWindows }

procedure DoOpenWindows;

	var
	a : integer;
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;

	begin
	for a := 0 to 1 do
		begin
		myWindowPtr := GetNewWindow(rWindow + a, nil, WindowPtr(-1));
		if (myWindowPtr = nil) then
			DoErrorAlert(eFailWindow);
		gWindowPtrs[a] := myWindowPtr;

		docRecHdl := DocRecHandle(NewHandle(sizeof(DocRec)));
		if (docRecHdl = nil) then
			DoErrorAlert(eFailDocRec);
		SetWRefCon(myWindowPtr, longint(docRecHdl));

		docRecHdl^^.pictureHdl := nil;
		docRecHdl^^.windowType := kDocumentType;
		docRecHdl^^.selectFlag := false;
		end;

	SetPort(myWindowPtr);

	docRecHdl^^.pictureHdl := GetPicture(rPicture);
	end;
		{of procedure DoOpenWindows}

{  DoSetDestRect }

function DoSetDestRect(var picFrame : Rect; myWindowPtr : WindowPtr) : Rect;

	var
	destRect : Rect;
	diffX, diffY : integer;

	begin
	destRect := picFrame;

	OffsetRect(destRect, -(picFrame.left), -(picFrame.top));

	diffX := (myWindowPtr^.portRect.right - myWindowPtr^.portRect.left) - 
					(picFrame.right - picFrame.left);
	diffY := (myWindowPtr^.portRect.bottom - myWindowPtr^.portRect.top) - 
					(picFrame.bottom - picFrame.top);
						
	OffsetRect(destRect, diffX div 2, diffY div 2);
	
	DoSetDestRect := destRect;
	end;
		{of procedure DoSetDestRect}

{  DoDrawPictureWindow }

procedure DoDrawPictureWindow(myWindowPtr : WindowPtr);
	var
	oldPort : GrafPtr;
	destRect : Rect;
	docRecHdl : DocRecHandle;

	begin
	GetPort(oldPort);
	SetPort(myWindowPtr);

	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));
	destRect := DoSetDestRect(docRecHdl^^.pictureHdl^^.picFrame, myWindowPtr);

	DrawPicture(docRecHdl^^.pictureHdl, destRect);

	if (docRecHdl^^.selectFlag) then
		begin
		InsetRect(destRect, -2, -2);
		PenPat(gMarqueePattern);
		FrameRect(destRect);	
		end;

	SetPort(oldPort);
	end;
		{of procedure DoDrawPictureWindow}

{  DoDrawClipboardWindow }

procedure DoDrawClipboardWindow;

	var
	oldPort : GrafPtr;
	sizeOfPictData, sizeOfTextData, scrapOffset : longint;
	tempHdl : Handle;
	destRect : Rect;	

	begin
	GetPort(oldPort);
	SetPort(gClipboardWindowPtr);	

	EraseRect(gClipboardWindowPtr^.portRect);

	MoveTo(0, 18);
	LineTo(505, 18);
	MoveTo(0, 20);
	LineTo(505, 20);

	TextFont(applFont);
	TextSize(9);
	MoveTo(4, 13);
	DrawString('Clipboard contents:');

	sizeOfPictData := GetScrap(nil, 'PICT', scrapOffset);
	if (sizeOfPictData > 0) then
		begin
		MoveTo(95, 13);
		DrawString('picture');

		tempHdl := NewHandle(Size(sizeOfPictData));	
		HLock(tempHdl);

		sizeOfPictData := GetScrap(tempHdl, 'PICT', scrapOffset);

		destRect := PicHandle(tempHdl)^^.picFrame;
		OffsetRect(destRect, -(PicHandle(tempHdl)^^.picFrame.left - 2), 
										-(PicHandle(tempHdl)^^.picFrame.top - 22));
		DrawPicture(PicHandle(tempHdl), destRect);

		HUnlock(tempHdl);
		DisposeHandle(tempHdl);
		end;

	sizeOfTextData := GetScrap(nil, 'TEXT', scrapOffset);
	if (sizeOfTextData > 0) then
		begin
		MoveTo(95, 13);
		DrawString('text');
	
		tempHdl := NewHandle(Size(sizeOfTextData));	
		HLock(tempHdl);
	
		sizeOfTextData := GetScrap(tempHdl, 'TEXT', scrapOffset);

		destRect := gClipboardWindowPtr^.portRect;
		destRect.top := destRect.top + 20;
		InsetRect(destRect, 2, 2);

		TETextBox(tempHdl^, sizeOfTextData, destRect, 0);

		HUnlock(tempHdl);
		DisposeHandle(tempHdl);
		end;

	SetPort(oldPort);
	end;
		{of procedure DoIdle}

{  DoClipboardCommand }

procedure DoClipboardCommand;

	var
	editMenuHdl : MenuHandle;
	docRecHdl : DocRecHandle;	

	begin
	editMenuHdl := GetMenu(mEdit);

	if (gClipboardWindowPtr = nil) then
		begin
		gClipboardWindowPtr := GetNewWindow(rClipboardWindow, nil, WindowPtr(-1));
		if (gClipboardWindowPtr = nil) then
			DoErrorAlert(eFailWindow);
			
		docRecHdl := DocRecHandle(NewHandle(sizeof(DocRec)));
		if (docRecHdl = nil) then
			DoErrorAlert(eFailDocRec);
		SetWRefCon(gClipboardWindowPtr, longint(docRecHdl));
		docRecHdl^^.windowType := kClipboardType;

		gClipboardShowing := true;

		SetMenuItemText(editMenuHdl, iClipboard, 'Hide Clipboard');
		end
	else
		begin
		if (gClipboardShowing) then
			begin
			HideWindow(gClipboardWindowPtr);
			gClipboardShowing := false;
			SetMenuItemText(editMenuHdl, iClipboard, 'Show Clipboard');
			end
		else
			begin
			ShowWindow(gClipboardWindowPtr);
			gClipboardShowing := true;
			SetMenuItemText(editMenuHdl, iClipboard, 'Hide Clipboard');				
			end;
		end;
	end;
		{of procedure DoClipboardCommand}

{  DoClearCommand }

procedure DoClearCommand;

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;	
	oldPort : GrafPtr;

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	GetPort(oldPort);
	SetPort(myWindowPtr);

	DisposeHandle(Handle(docRecHdl^^.pictureHdl));
	docRecHdl^^.pictureHdl := nil;
	docRecHdl^^.selectFlag := false;
	EraseRect(myWindowPtr^.portRect);

	SetPort(oldPort);
	end;
		{of procedure DoClearCommand}

{  DoPasteCommand }

procedure DoPasteCommand;

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;	
	oldPort : GrafPtr;
	sizeOfPictData, scrapOffset : longint;
	tempHdl : Handle;
	destRect : Rect;	

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	GetPort(oldPort);
	SetPort(myWindowPtr);	

	sizeOfPictData := GetScrap(nil, 'PICT', scrapOffset);
	if (sizeOfPictData > 0) then
		begin
		tempHdl := NewHandle(Size(sizeOfPictData));	
		HLock(tempHdl);

		sizeOfPictData := GetScrap(tempHdl, 'PICT', scrapOffset);

		EraseRect(myWindowPtr^.portRect);
		docRecHdl^^.selectFlag := false;
		destRect := DoSetDestRect(PicHandle(tempHdl)^^.picFrame, myWindowPtr);

		DrawPicture(PicHandle(tempHdl), destRect);

		if (docRecHdl^^.pictureHdl <> nil) then
			DisposeHandle(Handle(docRecHdl^^.pictureHdl));

		docRecHdl^^.pictureHdl := PicHandle(NewHandle(Size(sizeOfPictData)));
		BlockMoveData(tempHdl^, docRecHdl^^.pictureHdl^, Size(sizeOfPictData));

		HUnlock(tempHdl);
		DisposeHandle(tempHdl);
		end;

	SetPort(oldPort);
	end;
		{of procedure DoPasteCommand}

{  DoCutCopyCommand }

procedure DoCutCopyCommand(cutFlag : boolean);

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;	
	dataLength : Size;
	errorCode : longint;
	oldPort : GrafPtr;

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	if (docRecHdl^^.selectFlag = false) then
		Exit(DoCutCopyCommand);

	if (ZeroScrap = noErr) then
		begin
		dataLength := GetHandleSize(Handle(docRecHdl^^.pictureHdl));
		HLock(Handle(docRecHdl^^.pictureHdl));

		errorCode := PutScrap(longint(dataLength), 'PICT', Handle(docRecHdl^^.pictureHdl)^);
		if (errorCode <> noErr) then
			DoErrorAlert(ePutScrap);

		HUnlock(Handle(docRecHdl^^.pictureHdl));
		end
	else
		DoErrorAlert(eZeroScrap);

	if (cutFlag) then
		begin
		GetPort(oldPort);
		SetPort(myWindowPtr);

		DisposeHandle(Handle(docRecHdl^^.pictureHdl));
		docRecHdl^^.pictureHdl := nil;
		docRecHdl^^.selectFlag := false;
		EraseRect(myWindowPtr^.portRect);

		SetPort(oldPort);
		end;

	if (gClipboardWindowPtr <> nil) then
		DoDrawClipboardWindow;
	end;
		{of procedure DoIdle}

{  DoInContent }

procedure DoInContent(mouseXY : Point);

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;
	oldPort : GrafPtr;
	pictRect : Rect;

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	if (docRecHdl^^.windowType = kClipboardType) then
		Exit(DoInContent);

	GetPort(oldPort);
	SetPort(myWindowPtr);

	if (docRecHdl^^.pictureHdl <> nil) then
		begin
		pictRect := DoSetDestRect(docRecHdl^^.pictureHdl^^.picFrame, myWindowPtr);
		InsetRect(pictRect, -2, -2);

		GlobalToLocal(mouseXY);

		if (PtInRect(mouseXY, pictRect)) then
			docRecHdl^^.selectFlag := true
		else
			begin
			docRecHdl^^.selectFlag := false;

			PenPat(qd.black);
			ForeColor(whiteColor);
			FrameRect(pictRect);	
			ForeColor(blackColor);
			end;
		end;

	SetPort(oldPort);
	end;
		{of procedure DoInContent}

{  DoCloseWindow }

procedure DoCloseWindow;

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;
	editMenuHdl : MenuHandle;

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	if (docRecHdl^^.windowType = kClipboardType) then
		begin
		DisposeWindow(myWindowPtr);
		gClipboardWindowPtr := nil;
		gClipboardShowing := false;
		editMenuHdl := GetMenu(mEdit);
		SetMenuItemText(editMenuHdl, iClipboard, 'Show Clipboard');
		end;
	end;
		{of procedure DoCloseWindow}

{  DoEditMenu }

procedure DoEditMenu(menuItem : integer);

	begin
	case (menuItem) of

		iCut:
			begin
			DoCutCopyCommand(true);
			end;

		iCopy:
			begin
			DoCutCopyCommand(false);
			end;

		iPaste:
			begin
			DoPasteCommand;
			end;

		iClear:
			begin
			DoClearCommand;
			end;

		iClipboard:
			begin
			DoClipboardCommand;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoEditMenu}

{  DoMenuChoice }

procedure DoMenuChoice(menuChoice : longint);

	var
	menuID, menuItem : integer;
	itemName : string;
	daDriverRefNum : integer;

	begin
	menuID := HiWord(menuChoice);
	menuItem := LoWord(menuChoice);

	if (menuID = 0) then
		Exit(DoMenuChoice);

	case (menuID) of

		mApple:
			begin
			if (menuItem = iAbout) then
				SysBeep(10)
			else
				begin
				GetMenuItemText(GetMenuHandle(mApple), menuItem, itemName);
				daDriverRefNum := OpenDeskAcc(itemName);
				end;
			end;

		mFile:
			begin
			if (menuItem = iClose) then
				DoCloseWindow
			else if (menuItem = iQuit) then
				gDone := true;
			end;

		mEdit:
			begin
			DoEditMenu(menuItem);
			end;		
		end;
			{of case statement}

	HiliteMenu(0);
	end;
		{of procedure DoMenuChoice}

{  DoAdjustMenus }

procedure DoAdjustMenus;

	var
	fileMenuHdl, editMenuHdl : MenuHandle;
	docRecHdl : DocRecHandle;
	scrapOffset : longint;

	begin
	fileMenuHdl := GetMenuHandle(mFile);
	editMenuHdl := GetMenuHandle(mEdit);

	docRecHdl := DocRecHandle(GetWRefCon(FrontWindow));

	if (docRecHdl^^.windowType = kClipboardType) then
		EnableItem(fileMenuHdl, iClose)
	else
		DisableItem(fileMenuHdl, iClose);

	if ((docRecHdl^^.pictureHdl <> nil) & (docRecHdl^^.selectFlag)) then
		begin
		EnableItem(editMenuHdl, iCut);
		EnableItem(editMenuHdl, iCopy);
		EnableItem(editMenuHdl, iClear);
		end
	else
		begin
		DisableItem(editMenuHdl, iCut);
		DisableItem(editMenuHdl, iCopy);
		DisableItem(editMenuHdl, iClear);
		end;

	if ((GetScrap(nil, 'PICT', scrapOffset) <> 0) & (docRecHdl^^.windowType <> kClipboardType))
		then EnableItem(editMenuHdl, iPaste)
		else DisableItem(editMenuHdl, iPaste);

	DrawMenuBar;
	end;
		{of procedure DoAdjustMenus}

{  DoOSEvent }

procedure DoOSEvent(var theEvent : EventRecord);

	var
	myWindowPtr : WindowPtr;

	begin
	myWindowPtr := FrontWindow;

	case BAnd(BSR(theEvent.message, 24), $000000FF) of

		suspendResumeMessage:
			begin
			gInBackground := (BAnd(theEvent.message, resumeFlag) = 0);
			if ((gClipboardWindowPtr <> nil) & gClipboardShowing) then
				begin
				if (gInBackground) then
					HideWindow(gClipboardWindowPtr)
				else
					ShowWindow(gClipboardWindowPtr);
				end;
			end;

		mouseMovedMessage:
			begin
			end;
		end;
			{of case statement}
	end;
		{of procedure DoOSEvent}

{  DoUpdate }

procedure DoUpdate(var theEvent : EventRecord);

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;
	windowType : longint;

	begin
	myWindowPtr := WindowPtr(theEvent.message);
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));
	windowType := docRecHdl^^.windowType;

	BeginUpdate(myWindowPtr);

	if (windowType = kDocumentType) then
		begin
		if (docRecHdl^^.pictureHdl <> nil) then
			DoDrawPictureWindow(myWindowPtr);
		end
	else if (windowType = kClipboardType) then
		DoDrawClipboardWindow;

	EndUpdate(myWindowPtr);
	end;
		{of procedure DoUpdate}

{  DoMouseDown }

procedure DoMouseDown(var theEvent : EventRecord);

	var
	partCode : integer;
	myWindowPtr : WindowPtr;

	begin
	partCode := FindWindow(theEvent.where, myWindowPtr);

	case (partCode) of

		inMenuBar:
			begin
			DoAdjustMenus;
			DoMenuChoice(MenuSelect(theEvent.where));
			end;

		inSysWindow:
			begin
			SystemClick(theEvent, myWindowPtr);
			end;

		inContent:
			begin
			if (myWindowPtr <> FrontWindow) then
				SelectWindow(myWindowPtr)
			else
				DoInContent(theEvent.where);
			end;

		inDrag:
			begin
			DragWindow(myWindowPtr, theEvent.where, qd.screenBits.bounds);
			end;

		inGoAway:
			begin
			if (TrackGoAway(myWindowPtr, theEvent.where) = true) then
				DoCloseWindow;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoMouseDown}

{  DoEvents }

procedure DoEvents(var theEvent : EventRecord);

	var
	charCode : char;

	begin
	case (theEvent.what) of

		mouseDown:
			begin
			DoMouseDown(theEvent);
			end;

		keyDown, autoKey:
			begin
			charCode := chr(BAnd(theEvent.message, charCodeMask));
			if (BAnd(theEvent.modifiers, cmdKey) <> 0) then
				begin
				DoAdjustMenus;
				DoMenuChoice(MenuKey(charCode));
				end;
			end;

		updateEvt:
			begin
			DoUpdate(theEvent);
			end;

		osEvt:
			begin
			DoOSEvent(theEvent);
			HiliteMenu(0);
			end;
		end;
			{of case statement}
	end;
		{of procedure DoEvents}
			
{  DoIdle }

procedure DoIdle;
	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;
	oldPort : GrafPtr;
	marqueeRect : Rect;
	lastByte : SInt8;
	a : integer;

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	if ((docRecHdl^^.windowType = kClipboardType) or (docRecHdl^^.selectFlag = false)) then
		begin
		Exit(DoIdle);
		end
	else
		begin
		GetPort(oldPort);
		SetPort(myWindowPtr);

		marqueeRect := DoSetDestRect(docRecHdl^^.pictureHdl^^.picFrame, myWindowPtr);
		InsetRect(marqueeRect, -2, -2);

		lastByte := gMarqueePattern.pat[7];
		for a := 7 downto 1 do
			gMarqueePattern.pat[a] := gMarqueePattern.pat[a - 1];
		gMarqueePattern.pat[0] := lastByte;
		
		PenPat(gMarqueePattern);
		FrameRect(marqueeRect);
		
		SetPort(oldPort);
		end;
	end;
		{of procedure DoIdle}

{  start of main program }

begin
gClipboardWindowPtr	:= nil;
gClipboardShowing := false;
{$R-}
gMarqueePattern.pat[0] := SInt8($1F);
gMarqueePattern.pat[1] := SInt8($3E);
gMarqueePattern.pat[2] := SInt8($7C);
gMarqueePattern.pat[3] := SInt8($F8);
gMarqueePattern.pat[4] := SInt8($F1);
gMarqueePattern.pat[5] := SInt8($E3);
gMarqueePattern.pat[6] := SInt8($C7);
gMarqueePattern.pat[7] := SInt8($8F);
{$R+}
	{  initialize managers }

	DoInitManagers;
	
	{  set up menu bar and menus }
	
	menubarHdl := GetNewMBar(rMenubar);
	if (menubarHdl = nil) then
		DoErrorAlert(eFailMenu);
	SetMenuBar(menubarHdl);
	DrawMenuBar;

	menuHdl := GetMenuHandle(mApple);
	if (menuHdl = nil) then
		DoErrorAlert(eFailMenu)
	else
		AppendResMenu(menuHdl, 'DRVR');

	{  open windows } 

	DoOpenWindows;

	{  enter eventLoop }
	
	gDone := false;

	while not (gDone) do
		begin
		gotEvent := WaitNextEvent(everyEvent, theEvent, 2, nil);

		if (gotEvent) then
			DoEvents(theEvent)
		else
			DoIdle;
		end;

end.
	{of main program block}
	
{  }
