{ 
// UFloaters.p
//  }

unit UFloaters;



interface

{  include the following Universal Interfaces }

uses

	Types, Events, Quickdraw;

{  define and export the following constants }

const

mApple = 128;
 iAbout = 1;
mFile = 129;
 iNew = 1;
 iClose = 4;
 iFind = 11;
 iQuit = 13;
mEdit = 130;
mFloaters = 131;
 iTools = 1;
 iColours = 2;

rMenubar = 128;
rDocWindow = 128;
rToolsWindow = 129;
rColoursWindow = 130;
rAboutAlert = 128;
rFindDialog = 129;
rToolsPict = 128;
rColoursPict = 129;
rToolsPictDim = 130;
rColoursPictDim = 131;
rSound = 8192;

kDocumentKind = 1;
kFloatingKind = 2;

kMaxLong = $7FFFFFFF;

{  user-defined types }

type

{$IFC PROCTYPE }
	ActivateProcPtr = PROCEDURE (theWindowPtr : WindowPtr; activate : boolean);
{$ELSEC}
	ActivateProcPtr = ProcPtr;{ PROCEDURE(theWindowPtr : WindowPtr; activate : boolean); }
{$ENDC}

DocRecord = record
	windowType : integer;
	activateHandler : ActivateProcPtr;
	wasVisible : boolean;
	end;
	
DocRecordPtr = ^DocRecord;
DocRecordHandle = ^DocRecordPtr;
 
{  exported global variables }

var

gColorQuickDrawPresent : boolean;
gColourDisplay : boolean;
gDone : boolean;
gInBackground : boolean;
gNumberDocWindowsOpen : integer;
gToolsWindowPtr : WindowPtr;
gColoursWindowPtr : WindowPtr;

{  exported functions and procedures }

	procedure DoInitManagers;
	procedure DoOpenFloatingWindows;
	procedure EventLoop;



implementation

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, TextEdit, Dialogs, QuickdrawText, Processes, Resources,
	Memory, TextUtils, ToolUtils, OSUtils, Devices, SegLoad, Sound,
	
{  include the following user-defined units }

	UFloatRoutines;

{  procedure and function definitions }

	procedure DoEvents(eventRec : EventRecord); forward;
	procedure DoMouseDown(eventRec : EventRecord); forward;
	procedure DoUpdate(eventRec : EventRecord); forward;
	procedure DoOSEvent(eventRec : EventRecord); forward;
	procedure DoAdjustMenus; forward;
	procedure DoMenuChoice(menuChoice : longint); forward;
	procedure DoFileMenu(menuItem : integer); forward;
	procedure DoFloatersMenu(menuItem : integer); forward;
	procedure DoOpenDocWindow; forward;
	procedure DoCloseDocWindow; forward;
	procedure DoDrawDocWindowContent; forward;
	procedure DoProvingBeeps(theWindowPtr : WindowPtr); forward;
	procedure DoOpenFindDialog; forward;
	procedure DoDisposeFindDialog; forward;
	procedure DocActivateHandler(theWindowPtr : WindowPtr; activate : boolean); forward;
	procedure ToolsActivateHandler(theWindowPtr : WindowPtr; activate : boolean); forward;
	procedure ColoursActivateHandler(theWindowPtr : WindowPtr; 
				activate : boolean); forward;
	procedure InvalidateScrollBarArea(theWindowPtr : WindowPtr); forward;

{  procedure and function implementations }

{  DoInitManagers }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;

	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);

	InitCursor;	
	FlushEvents(everyEvent, 0);
	end;
		{of procedure DoInitManagers}

{  DoOpenFloatingWindows }

procedure DoOpenFloatingWindows;

	var
	resourceOffset : integer;
	pictureHdl : PicHandle;
	theProcPtr : ActivateProcPtr;
	
	begin
	resourceOffset := 0;
	
	if ((gColorQuickDrawPresent = false) or (gColourDisplay = false)) then
		resourceOffset := 4;
	
	theProcPtr := ActivateProcPtr(@ToolsActivateHandler);
	gToolsWindowPtr := NewGetNewWindow(rToolsWindow, WindowPtr(-1), theProcPtr, 
				kFloatingKind);
	if (gToolsWindowPtr = nil) then
		begin
		SysBeep(10);
		ExitToShell;
		end;

	pictureHdl := GetPicture(rToolsPict + resourceOffset);
	SetWindowPic(gToolsWindowPtr, pictureHdl);

	theProcPtr := ActivateProcPtr(@ColoursActivateHandler);
	gColoursWindowPtr := NewGetNewWindow(rColoursWindow, WindowPtr(-1), theProcPtr, 
				kFloatingKind);
	if (gColoursWindowPtr = nil) then
		begin
		SysBeep(10);
		ExitToShell;
		end;

	pictureHdl := GetPicture(rColoursPict + resourceOffset);
	SetWindowPic(gColoursWindowPtr, pictureHdl);

	HiliteWindow(gToolsWindowPtr, true);
	end;
		{of procedure DoOpenFloatingWindows}

{  EventLoop }

procedure EventLoop;

	var
	eventRec : EventRecord;
		theDialogPtr : DialogPtr;
	itemHit : integer;
	ignoredBool : boolean;

	begin
	gDone := false;
	
	while not (gDone) do
		begin
		if (WaitNextEvent(everyEvent, eventRec, kMaxLong, nil)) then
			begin
			if not (IsDialogEvent(eventRec)) then
				DoEvents(eventRec)
			else begin
				ignoredBool := DialogSelect(eventRec, theDialogPtr, itemHit);
				if ((itemHit = ok) or (itemHit = cancel)) then
					DoDisposeFindDialog;
				end;
			end;
		end;
	end;
		{of procedure EventLoop}

{  DoEvents }

procedure DoEvents(eventRec : EventRecord);

	var
	charCode : char;

	begin
	case  (eventRec.what) of

		mouseDown: begin
			DoMouseDown(eventRec);
			end;

		keyDown, autoKey: begin
			charCode := chr(BAnd(eventRec.message, charCodeMask));
			if (BAnd(eventRec.modifiers, cmdKey) <> 0) then
				begin
				DoAdjustMenus;
				DoMenuChoice(MenuKey(charCode));
				end;
			end;

		updateEvt: begin
			DoUpdate(eventRec);
			end;

		osEvt: begin
			DoOSEvent(eventRec);
			HiliteMenu(0);
			end;
		end;
	end;
		{of procedure DoEvents}

{  DoMouseDown }

procedure DoMouseDown(eventRec : EventRecord);

	var
	partCode : integer;
	theWindowPtr : WindowPtr;
	frontWindowPtr : WindowPtr;
	docRecordHdl : DocRecordHandle;
	windowType : integer;
	newSize : UInt32;
	growRect : Rect;
	
	begin
	partCode := FindWindow(eventRec.where, theWindowPtr);

	case  (partCode) of

		inMenuBar: begin
			DoAdjustMenus;
			DoMenuChoice(MenuSelect(eventRec.where));
			end;

		inSysWindow: begin
			SystemClick(eventRec, theWindowPtr);
			end;

		inContent: begin
			frontWindowPtr := FrontWindow;
			if (WindowPeek(frontWindowPtr)^.windowKind <> dialogKind) then
				DoProvingBeeps(theWindowPtr);
			if ((IsWindowModal(frontWindowPtr)) and (theWindowPtr <> frontWindowPtr)) then
				SysBeep(10)
			else if (theWindowPtr <> frontWindowPtr) then
				NewSelectWindow(theWindowPtr);
			end;

		inDrag: begin
			frontWindowPtr := FrontWindow;
			if ((IsWindowModal(frontWindowPtr)) and (theWindowPtr <> frontWindowPtr)) then
				SysBeep(10)
			else 
				NewDragWindow(theWindowPtr, eventRec.where, qd.screenBits.bounds);
			end;

		inGoAway: begin
			if (TrackGoAway(theWindowPtr, eventRec.where)) then
				begin
				docRecordHdl := DocRecordHandle(GetWRefCon(theWindowPtr));
				windowType := docRecordHdl^^.windowType;
				if (windowType = kFloatingKind) then
					begin
					NewHideWindow(theWindowPtr);
					DoAdjustMenus;					
					end
				else if (windowType = kDocumentKind) then
					DoCloseDocWindow;
				end;	
			end;

		inGrow: begin
			growRect := qd.screenBits.bounds;
			growRect.top := 145;
			growRect.left := 335;
			newSize := GrowWindow(theWindowPtr, eventRec.where, growRect);
			if (newSize <> 0) then
				begin
				InvalidateScrollBarArea(theWindowPtr);
				SizeWindow(theWindowPtr, LoWord(newSize), HiWord(newSize), true);
				InvalidateScrollBarArea(theWindowPtr);
				end;
			end;

		inZoomIn, inZoomOut: begin
			if (TrackBox(theWindowPtr, eventRec.where, partCode)) then
				begin
				SetPort(theWindowPtr);
				EraseRect(theWindowPtr^.portRect);
				ZoomWindow(theWindowPtr, partCode, false);
				InvalRect(theWindowPtr^.portRect);
				end;
			end;
		end;
	end;
		{of procedure DoMouseDown}

{  DoUpdate }

procedure DoUpdate(eventRec : EventRecord);

	var
	theWindowPtr : WindowPtr;

	begin
	theWindowPtr := WindowPtr(eventRec.message);
	SetPort(theWindowPtr);

	BeginUpdate(theWindowPtr);
	EraseRect(theWindowPtr^.portRect);
	DoDrawDocWindowContent;
	DrawGrowIcon(theWindowPtr);
	EndUpdate(theWindowPtr);
	end;
		{of procedure DoUpdate}

{  DoOSEvent }

procedure DoOSEvent(eventRec : EventRecord);

	begin
	case BAnd(BSR(eventRec.message, 24), $000000FF) of

		suspendResumeMessage: begin
			if (BAnd(eventRec.message, resumeFlag) <> 0) then
				begin
				gInBackground := false;
				HandleResumeEvent;
				SetCursor(qd.arrow);
				end
			else begin
				gInBackground := true;
				HandleSuspendEvent;
				end;
			end;

		mouseMovedMessage: begin
			end;
		end;
			{of case statement}
	end;
		{of procedure DoOSEvent}
{  DoAdjustMenus }

procedure DoAdjustMenus;

	var
	appleMenuHdl, fileMenuHdl, floatMenuHdl : MenuHandle;

	begin
	appleMenuHdl := GetMenuHandle(mApple);
	fileMenuHdl := GetMenuHandle(mFile);
	floatMenuHdl := GetMenuHandle(mFloaters);

	if (IsWindowModal(FrontWindow)) then
		begin
		DisableItem(appleMenuHdl, 1);
		DisableItem(fileMenuHdl, 0);
		DisableItem(floatMenuHdl, 0);
		end
	else begin
		EnableItem(appleMenuHdl, 1);
		EnableItem(fileMenuHdl, 0);
		EnableItem(floatMenuHdl, 0);

		if (gNumberDocWindowsOpen > 0) then
			EnableItem(fileMenuHdl, iClose)
		else
			DisableItem(fileMenuHdl, iClose);
		end;

	if (GetWindowVisible(gToolsWindowPtr)) then
		CheckItem(floatMenuHdl, iTools, true)
	else
		CheckItem(floatMenuHdl, iTools, false);

	if (GetWindowVisible(gColoursWindowPtr)) then
		CheckItem(floatMenuHdl, iColours, true)
	else
		CheckItem(floatMenuHdl, iColours, false);

	DrawMenuBar;
	end;
		{of procedure DoAdjustMenus}

{  DoMenuChoice }

procedure DoMenuChoice(menuChoice : longint);

	var
	menuID, menuItem : integer;
	itemName : string;
	daDriverRefNum : integer;
	ignoredInt : integer;

	begin
	menuID := HiWord(menuChoice);
	menuItem := LoWord(menuChoice);

	if (menuID = 0) then
		Exit(DoMenuChoice);

	case (menuID) of

		mApple: begin
			if (menuItem = iAbout) then
				begin
				DeactivateFloatsAndFirstDocWin;
				ignoredInt := NoteAlert(rAboutAlert, nil);
				ActivateFloatsAndFirstDocWin;
				end
			else begin
				GetMenuItemText(GetMenuHandle(mApple), menuItem, itemName);
				daDriverRefNum := OpenDeskAcc(itemName);
				end;
			end;

		mFile: begin
			DoFileMenu(menuItem);
			end;

		mFloaters: begin
			DoFloatersMenu(menuItem);
			end;
		end;
			{of case statement}

	HiliteMenu(0);
	end;
		{of procedure DoMenuChoice}

{  DoFileMenu }

procedure DoFileMenu(menuItem : integer);

	begin
	case (menuItem) of

		iNew: begin
			DoOpenDocWindow;
			end;

		iClose: begin
			DoCloseDocWindow;
			end;

		iFind: begin
			DoOpenFindDialog;
			end;

		iQuit: begin
			gDone := true;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoFileMenu}
		
{  DoFloatersMenu }

procedure DoFloatersMenu(menuItem : integer);

	var
	floaterVisible : boolean;
	floaterPicked : WindowPtr;

	begin
	if (menuItem = iTools) then
		floaterPicked := gToolsWindowPtr
	else if (menuItem = iColours) then
		floaterPicked := gColoursWindowPtr;

	floaterVisible := GetWindowVisible(floaterPicked);

	if (floaterVisible = false) then
		NewShowWindow(floaterPicked)
	else
		NewHideWindow(floaterPicked);

	DoAdjustMenus;
	end;
		{of procedure DoFloatersMenu}

{  DoOpenDocWindow }

procedure DoOpenDocWindow;

	var
	theProcPtr : ActivateProcPtr;
	theWindowPtr : WindowPtr;

	begin
	if (gNumberDocWindowsOpen > 3) then
		SysBeep(10)
	else begin
		theProcPtr := ActivateProcPtr(@DocActivateHandler);
		theWindowPtr := NewGetNewWindow(rDocWindow, WindowPtr(-1), theProcPtr, 
					kDocumentKind);
		NewShowWindow(theWindowPtr);
		gNumberDocWindowsOpen := gNumberDocWindowsOpen + 1;
		end;
	end;
		{of procedure DoOpenDocWindow}

{  DoCloseDocWindow }

procedure DoCloseDocWindow;

	var
	theWindowPtr : WindowPtr;

	begin
	theWindowPtr := FindFrontNonFloatingWindow;

	if (theWindowPtr <> nil) then
		begin
		NewDisposeWindow(theWindowPtr);
		gNumberDocWindowsOpen := gNumberDocWindowsOpen - 1;
		end;
	end;
		{of procedure DoCloseDocWindow}

{  DoDrawDocWindowContent }

procedure DoDrawDocWindowContent;

	var
	theString : string;
	a : integer;

	begin
	for a := 1 to 3 do
		begin
		GetIndString(theString, 128, a);
		MoveTo(20, a*30);
		DrawString(theString);
		end;
	end;
		{of procedure DoDrawDocWindowContent}

{  DoProvingBeeps }

procedure DoProvingBeeps(theWindowPtr : WindowPtr);

	var
	soundHdl : Handle;
	repeats, a : integer;
	ignoredErr : OSErr;

	begin
	soundHdl := GetResource('snd ', rSound);

	if (theWindowPtr = gToolsWindowPtr) then
		repeats := 2
	else if (theWindowPtr = gColoursWindowPtr) then
		repeats := 3
	else
		repeats := 1;

	for a := 0 to (repeats - 1) do
		ignoredErr := SndPlay(nil, SndListHandle(soundHdl), false);

	ReleaseResource(soundHdl); 
	end;
		{of procedure DoProvingBeeps}

{  DoOpenFindDialog }

procedure DoOpenFindDialog;

	var
	findDialogPtr : DialogPtr;

	begin
	DeactivateFloatsAndFirstDocWin;
	findDialogPtr := GetNewDialog(rFindDialog, nil, WindowPtr(-1));

	DoAdjustMenus;
	end;
		{of procedure DoOpenFindDialog}

{  DoDisposeFindDialog }

procedure DoDisposeFindDialog;

	var
	findDialogPtr : DialogPtr;
	
	begin
	findDialogPtr := FrontWindow;
	DisposeDialog(findDialogPtr);
	ActivateFloatsAndFirstDocWin;

	DoAdjustMenus;
	end;
		{of procedure DoDisposeFindDialog}

{  DocActivateHandler }

procedure DocActivateHandler(theWindowPtr : WindowPtr; activate : boolean);

	var
	oldPort : GrafPtr;
	finalTicks : longint;
	theRect : Rect;

	begin
	GetPort(oldPort);
	SetPort(theWindowPtr);

	DrawGrowIcon(theWindowPtr);
	
	MoveTo(20, 120);
	if (activate) then
		DrawString('Activating')
	else
		DrawString('Deactivating');

	Delay(30, finalTicks);

	SetRect(theRect, 20, 105, 100, 125);
	EraseRect(theRect);

	SetPort(oldPort);
	end;
		{of procedure DocActivateHandler}

{  ToolsActivateHandler }

procedure ToolsActivateHandler(theWindowPtr : WindowPtr; activate : boolean);

	var
	oldPort : GrafPtr;
	resourceOffset : integer;
	pictureHdl : PicHandle;	
	whichPicture : integer;
	theRect : Rect;

	begin
	resourceOffset := 0;
	
	GetPort(oldPort);
	SetPort(theWindowPtr);

	if (activate = true) then
		whichPicture := rToolsPict
	else
		whichPicture := rToolsPictDim;

	if ((gColorQuickDrawPresent = false) or (gColourDisplay = false)) then
		resourceOffset := 4;

	pictureHdl := GetPicture(whichPicture + resourceOffset);
	theRect := pictureHdl^^.picFrame;
	OffsetRect(theRect, - theRect.left, - theRect.top);
	if ((gColorQuickDrawPresent = false) or (gColourDisplay = false)) then
		EraseRect(theWindowPtr^.portRect);
	DrawPicture(pictureHdl, theRect);
	SetWindowPic(theWindowPtr, pictureHdl);

	SetPort(oldPort);
	end;
		{of procedure ToolsActivateHandler}

{  ColoursActivateHandler }

procedure ColoursActivateHandler(theWindowPtr : WindowPtr; activate : boolean);

	var
	oldPort : GrafPtr;
	resourceOffset : integer;
	pictureHdl : PicHandle;	
	whichPicture : integer;
	theRect : Rect;

	begin
	resourceOffset := 0;
	
	GetPort(oldPort);
	SetPort(theWindowPtr);

	if (activate = true) then
		whichPicture := rColoursPict
	else
		whichPicture := rColoursPictDim;

	if ((gColorQuickDrawPresent = false) or (gColourDisplay = false)) then
		resourceOffset := 4;

	pictureHdl := GetPicture(whichPicture + resourceOffset);
	theRect := pictureHdl^^.picFrame;
	OffsetRect(theRect, - theRect.left, - theRect.top);
	if ((gColorQuickDrawPresent = false) or (gColourDisplay = false)) then
		EraseRect(theWindowPtr^.portRect);
	DrawPicture(pictureHdl, theRect);
	SetWindowPic(theWindowPtr, pictureHdl);

	SetPort(oldPort);
	end;
		{of procedure ColoursActivateHandler}

{  InvalidateScrollBarArea }

procedure InvalidateScrollBarArea(theWindowPtr : WindowPtr);

	var
	tempRect : Rect;
  
  	begin
	SetPort(theWindowPtr);

	tempRect := theWindowPtr^.portRect;
	tempRect.left := tempRect.right - 15;
	InvalRect(tempRect);

	tempRect := theWindowPtr^.portRect;
	tempRect.top := tempRect.bottom - 15;
	InvalRect(tempRect);
	end;
		{of procedure ColoursActivateHandler}

end.
	{of unit UFloaters.p}
	
{  }