{ 
// FilesPascalPPC.p
// 
//
// This program demonstrates application-defined file handling functions associated with:
//
//    The user invoking the File menu Open, Close, Save, Save As, Revert to Saved, and
//	   Quit commands of a typical application.
//
//    Receipt of the required Apple events Open Application, Open Documents and Quit
//	   Application.
//
// The uncompiled program may be run from within CodeWarrior to demonstrate responses to
// the File menu commands.  The built application, together with supplied TEXT and a PICT
// files, may be used to demonstrate the additional aspect of integrating the receipt
// of required Apple events with the overall file handling mechanism.
//
// To keep the code not specifically related to files and file-handling to a minimim, a 
// 'Demonstration' menu is included to allow the user to simulate making a window
// 'dirty', that is, modifying the contents of the associated document.  Choosing the
// single menu item in this menu sets the window-dirty flag in the window's document
// record to true and draws a diagonal line across the window, this latter so that the
// user can keep track of which windows are 'dirty'.
//
// The program utilizes the following resources:
//
//    An 'MBAR' resource, and 'MENU' resources for Apple, File, Edit and Demonstration
//	   menus (preload, non-purgeable).  
//
//    A 'WIND' resource (purgeable) (initially not visible).  
//
//    Three 'ALRT' resources (purgeable) and associated 'DITL' resources (purgeable).
//	   The first alert is used to display error messages.  The second is used to support
//	   the Revert to Saved menu item.  The third is used when an attempt is made to close
//	   a modified document before that document has been saved.     
//
//    A 'STR ' resource (purgeable) containing the 'missing application name' string 
//	   resource, which is copied to all document files created by the program.  
//
//    A 'SIZE' resource with the acceptSuspendResumeEvents, isHighLevelEventAware, and
//	   is32BitCompatible flags set (non-purgeable).	  
//
//    The 'BNDL' resource (non-purgeable), 'FREF' resources (non-purgeable), signature
//	   resource (non-purgeable), and icon family resources (purgeable), required to 
//	   support the built application.
//
//  }

program FilesPascal(input, output);

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, TextEdit, Quickdraw, Dialogs, QuickdrawText, Processes, Types, 
	Memory, Events, TextUtils, ToolUtils, OSUtils, Devices, Resources, Files, Errors,
	Folders, Script, StandardFile, AppleEvents, Finder, EPPC, SegLoad;

{  define the following constants }

const

mApple = 128;
	iAbout = 1;
mFile = 129;
	iNew = 1;
	iOpen = 2;
	iClose = 4;
	iSave = 5;
	iSaveAs = 6;
	iRevert = 7;
	iQuit = 12;
mDemonstration = 131;
	iDirty = 1;

rNewWindow = 128;
rMenubar = 128;
rErrorAlert = 128;
	eInstallHandler = 1000;
	eMaxWindows = 1001;
	eFileIsOpen = -49;
rRevertAlert = 129;
rCloseFileAlert = 130;	

kMaxWindows = 10;
kUserCancelled = 1002;
kMaxLong = $7FFFFFFF;

{  user defined types }

type

DocRecord = record
	editRec : TEHandle;
	pictureHdl : PicHandle;
	fileRefNum : integer;
	fileFSSpec : FSSpec;
	windowDirty : boolean;
	end;
DocRecordPointer = ^DocRecord;
DocRecordHandle = ^DocRecordPointer;

{  global variables }

var

gDone : boolean;
gInBackground : boolean;
gWindowPtr : WindowPtr;
gCurrentNumberOfWindows : longint;
gDestRect, gViewRect : Rect;
gAppResFileRefNum : longint;
menubarHdl : Handle;
menuHdl : MenuHandle;

doOpenAppEventRD : AEEventHandlerUPP;                                      { For PowerPC }
doOpenDocsEventRD : AEEventHandlerUPP;                                     { For PowerPC }
doQuitAppEventRD : AEEventHandlerUPP;                                      { For PowerPC }

{  DoInitManagers }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;

	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);

	InitCursor;	
	FlushEvents(everyEvent, 0);
	end;
		{of procedure DoInitManagers}

{  DoCopyResource }

function DoCopyResource(resourceType : ResType; resourceID, sourceFileRefNum, 
													destFileRefNum : integer) : OSErr;

	var
	sourceResourceHdl : Handle;
	sourceResourceName : string;
	ignoredType : ResType;
	ignoredID : integer;

	begin
	UseResFile(sourceFileRefNum);

	sourceResourceHdl := GetResource(resourceType, resourceID);

	if (sourceResourceHdl <> nil) then
		begin
		GetResInfo(sourceResourceHdl, ignoredID, ignoredType, sourceResourceName);
		DetachResource(sourceResourceHdl);
		UseResFile(destFileRefNum);
		AddResource(sourceResourceHdl, resourceType, resourceID, sourceResourceName);
		if (ResError = noErr) then
			UpdateResFile(destFileRefNum);
		end;

	ReleaseResource(sourceResourceHdl);

	DoCopyResource := ResError;
	end;
		{of function DoCopyResource}

{  DoError }

procedure DoError(errorCode : integer);

	var
	errorString : string;
	ignored : OSErr;

	begin
	if (errorCode = eInstallHandler) then
		ParamText('Error installing Required Apple event handlers', '', '', '')
	else if (errorCode = eMaxWindows) then
		ParamText('Sorry.  No more windows', '', '', '')
	else if (errorCode = eFileIsOpen) then
		ParamText('File is currently open', '', '', '')		
	else
		begin
		NumToString(errorCode, errorString);
		ParamText('An error occurred.  The error number is ', errorString, '', '');
		end;
	
	if (errorCode = memFullErr) then
		begin
		ignored := StopAlert(rErrorAlert, nil);
		ExitToShell;
		end
	else	
		ignored := CautionAlert(rErrorAlert, nil);
	end;
		{of procedure DoInitManagers}

{  DoCopyAppNameResource }

function DoCopyAppNameResource(myWindowPtr : WindowPtr) : OSErr;

	var
	docRecHdl : DocRecordHandle;
	fileType : OSType;
	osError : OSErr;
	fileRefNum : integer;

	begin
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));

	if (docRecHdl^^.editRec <> nil) then
		fileType := 'TEXT'
	else if (docRecHdl^^.pictureHdl <> nil) then
		fileType := 'PICT';

	FSpCreateResFile(docRecHdl^^.fileFSSpec, 'KKJB', fileType, smSystemScript);

	osError := ResError;
	if (osError = noErr) then
		fileRefNum := FSpOpenResFile(docRecHdl^^.fileFSSpec, fsRdWrPerm);

	if (fileRefNum > 0) then
		osError := DoCopyResource('STR ', -16396, gAppResFileRefNum, fileRefNum)
	else
		osError := ResError;

	if (osError = noErr) then
		CloseResFile(fileRefNum); 

	osError := ResError;
	DoCopyAppNameResource := osError;
	end;
		{of function DoCopyAppNameResource}

{  DoWritePictData }

function  DoWritePictData(myWindowPtr : WindowPtr; tempFileRefNum : integer) : OSErr;

	var
	docRecHdl : DocRecordHandle;
	pictureHdl : PicHandle;
	numberOfBytes, dummyData : longint;
	volRefNum : integer;
	osError : OSErr;

	begin
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));
	pictureHdl := docRecHdl^^.pictureHdl;

	numberOfBytes := 512;
	dummyData := 0;

	osError := SetFPos(tempFileRefNum, fsFromStart, 0);

	if (osError = noErr) then
		osError := FSWrite(tempFileRefNum, numberOfBytes, dummyData);

	numberOfBytes := GetHandleSize(Handle(docRecHdl^^.pictureHdl));

	if (osError = noErr) then
		begin
		HLock(Handle(docRecHdl^^.pictureHdl));
		osError := FSWrite(tempFileRefNum, numberOfBytes, docRecHdl^^.pictureHdl^);
		HUnlock(Handle(docRecHdl^^.pictureHdl));
		end;

	if (osError = noErr) then
		osError := SetEOF(tempFileRefNum, 512 + numberOfBytes);
	if (osError = noErr) then
		osError := GetVRefNum(tempFileRefNum, volRefNum);
	if (osError = noErr) then
		osError := FlushVol(nil, volRefNum);

	if (osError = noErr) then
		docRecHdl^^.windowDirty := false;

	DoWritePictData := osError;
	end;
		{of function DoWritePictData}

{  DoWriteTextData }

function DoWriteTextData(myWindowPtr : WindowPtr; tempFileRefNum : integer) : OSErr;

	var
	docRecHdl : DocRecordHandle;
	textEditHdl : TEHandle;
	editText : Handle;
	numberOfBytes : longint;
	volRefNum : integer;
	osError : OSErr;

	begin
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));
	textEditHdl := docRecHdl^^.editRec;
	editText := textEditHdl^^.hText;
	numberOfBytes := textEditHdl^^.teLength;

	osError := SetFPos(tempFileRefNum, fsFromStart, 0);
	if (osError = noErr) then
		osError := FSWrite(tempFileRefNum, numberOfBytes, editText^);
	if (osError = noErr) then
		osError := SetEOF(tempFileRefNum, numberOfBytes);
	if (osError = noErr) then
		osError := GetVRefNum(tempFileRefNum, volRefNum);
	if (osError = noErr) then
		osError := FlushVol(nil, volRefNum);

	if (osError = noErr) then
		docRecHdl^^.windowDirty := false;

	DoWriteTextData := osError;
	end;
		{of function DoWriteTextData}

{  DoReadPictFile }

function DoReadPictFile(myWindowPtr : WindowPtr) : OSErr;
	
	var
	docRecHdl : DocRecordHandle;
	fileRefNum : integer;
	numberOfBytes : longint;
	osError, ignored : OSErr;

	begin
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));
	fileRefNum := docRecHdl^^.fileRefNum;

	ignored := GetEOF(fileRefNum, numberOfBytes);
	ignored := SetFPos(fileRefNum, fsFromStart, 512);
	numberOfBytes := numberOfBytes - 512;
	
	docRecHdl^^.pictureHdl := PicHandle(NewHandle(numberOfBytes));
	if (docRecHdl^^.pictureHdl = nil) then
		begin
		DoReadPictFile := MemError;
		Exit(DoReadPictFile);
		end;

	osError := FSRead(fileRefNum, numberOfBytes, docRecHdl^^.pictureHdl^);
	if ((osError = noErr) or (osError = eofErr)) then
		begin
		DoReadPictFile := noErr;
		Exit(DoReadPictFile);
		end
	else
		DoReadPictFile := osError;
	end;
		{of function DoReadPictFile}

{  DoReadTextFile }

function DoReadTextFile(myWindowPtr : WindowPtr) : OSErr;

	var
	docRecHdl : DocRecordHandle;
	fileRefNum : integer;
	textEditHdl : TEHandle;
	numberOfBytes : longint;
	textBuffer : Handle;
	osError, ignored : OSErr;

	begin
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));
	fileRefNum := docRecHdl^^.fileRefNum;

	textEditHdl := docRecHdl^^.editRec;
	textEditHdl^^.txSize := 10;
	textEditHdl^^.lineHeight := 15;

	ignored := SetFPos(fileRefNum, fsFromStart, 0);
	ignored := GetEOF(fileRefNum, numberOfBytes);

	if (numberOfBytes > 32767) then
		numberOfBytes := 32767;
		
	textBuffer := NewHandle(Size(numberOfBytes));
	if (textBuffer = nil) then
		begin
		DoReadTextFile := MemError;
		Exit(DoReadTextFile);
		end;

	osError := FSRead(fileRefNum, numberOfBytes, textBuffer^);
	if ((osError = noErr) or (osError = eofErr)) then
		begin
		MoveHHi(textBuffer);
		HLockHi(textBuffer);
		TESetText(textBuffer^, numberOfBytes, docRecHdl^^.editRec);
		HUnlock(textBuffer);
		DisposeHandle(textBuffer);
		end
	else
		begin
		DoReadTextFile := osError;
		Exit(DoReadTextFile);
		end;

	DoReadTextFile := noErr;
	end;
		{of function DoReadTextFile}

{  DoWriteFile }

function DoWriteFile(myWindowPtr : WindowPtr; newFile : boolean) : OSErr;

	var
	docRecHdl : DocRecordHandle;
	fileSpecActual, fileSpecTemp : FSSpec;
	currentTime : longint;
	tempFileName : string;
	tempFileVolNum, tempFileRefNum : integer;
	tempFileDirID : longint;
	osError : OSErr;

	begin
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));
	fileSpecActual := docRecHdl^^.fileFSSpec;

	GetDateTime(currentTime);
	NumToString(currentTime, tempFileName);
	
	osError := FindFolder(fileSpecActual.vRefNum, kTemporaryFolderType, kCreateFolder, 
												tempFileVolNum, tempFileDirID);
	if (osError = noErr) then
		osError := FSMakeFSSpec(tempFileVolNum, tempFileDirID, tempFileName, fileSpecTemp);
	if ((osError = noErr) or (osError = fnfErr)) then
		osError := FSpCreate(fileSpecTemp, 'trsh', 'trsh', smSystemScript);
	if (osError = noErr) then
		osError := FSpOpenDF(fileSpecTemp, fsRdWrPerm, tempFileRefNum);
	if (osError = noErr) then
		begin
		if (docRecHdl^^.editRec <> nil) then
			osError := DoWriteTextData(myWindowPtr, tempFileRefNum)
		else if (docRecHdl^^.pictureHdl <> nil) then
			osError := DoWritePictData(myWindowPtr, tempFileRefNum);
		end;	
	if (osError = noErr) then
		osError := FSClose(tempFileRefNum);
	if (osError = noErr) then
		osError := FSClose(docRecHdl^^.fileRefNum);
	if (osError = noErr) then
		osError := FSpExchangeFiles(fileSpecTemp, fileSpecActual);
	if (osError = noErr) then
		osError := FSpDelete(fileSpecTemp);
	if (osError = noErr) then
		osError := FSpOpenDF(fileSpecActual, fsRdWrPerm, docRecHdl^^.fileRefNum);

	if (osError = noErr) then
		begin
		if (newFile) then
			osError := DoCopyAppNameResource(myWindowPtr);
		end;

	DoWriteFile := osError;
	end;
		{of function DoWriteFile}

{  DoSaveAsCommand }

function DoSaveAsCommand : OSErr;

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecordHandle;
	fileReply : StandardFileReply;
	fileType : OSType;
	fileRefNum : integer;
	osError : OSErr;

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));

	StandardPutFile('Save as:', 'Untitled', fileReply);

	if (fileReply.sfGood) then
		begin
		if not (fileReply.sfReplacing) then
			begin
			if (docRecHdl^^.editRec <> nil) then
				fileType := 'TEXT'
			else if (docRecHdl^^.pictureHdl <> nil) then
				fileType := 'PICT';
			osError := FSpCreate(fileReply.sfFile, 'KKJB', fileType, smSystemScript);
			if (osError <> noErr) then
				begin
				DoSaveAsCommand := osError;
				Exit(DoSaveAsCommand);
				end;
			end;

		docRecHdl^^.fileFSSpec := fileReply.sfFile;

		if (docRecHdl^^.fileRefNum <> 0) then
			begin
			osError := FSClose(docRecHdl^^.fileRefNum);
			docRecHdl^^.fileRefNum := 0;
			end;

		if (osError = noErr) then
			osError := FSpOpenDF(docRecHdl^^.fileFSSpec, fsRdWrPerm, fileRefNum);

		if (osError = noErr) then
			begin
			docRecHdl^^.fileRefNum := fileRefNum;
			SetWTitle(myWindowPtr, fileReply.sfFile.name);
			osError := DoWriteFile(myWindowPtr, true);
			end;
		end;

	DoSaveAsCommand := osError;
	end;
		{of function DoSaveAsCommand}

{  DoSaveCommand }

function DoSaveCommand : OSErr;

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecordHandle;
	osError : OSErr;
	
	begin
	osError := 0;
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));

	if (docRecHdl^^.fileRefNum <> 0) then
		osError := DoWriteFile(myWindowPtr, false)
	else
		osError := DoSaveAsCommand;

	DoSaveCommand := osError;
	end;
		{of function DoSaveCommand}

{  DoCloseFile }

function DoCloseFile(myWindowPtr : WindowPtr; docRecHdl : DocRecordHandle) : OSErr;

	var
	fileName : string;
	itemHit : integer;
	osError : OSErr;
	
	begin
	if (docRecHdl^^.windowDirty) then
		begin
		GetWTitle(myWindowPtr, fileName);
		ParamText(fileName, '', '', '');

		itemHit := CautionAlert(rCloseFileAlert, nil);
		if (itemHit = 2) then
			begin
			DoCloseFile := kUserCancelled;
			Exit(DoCloseFile);
			end
		else if (itemHit = 1) then
			begin
			osError := DoSaveCommand;
			if (osError <> noErr) then
				begin
				DoCloseFile := osError;
				Exit(DoCloseFile);
				end;
			end;
		end;

	if (docRecHdl^^.fileRefNum <> 0) then
		begin
		osError := FSClose(docRecHdl^^.fileRefNum);
		if (osError <> noErr) then	
			begin
			osError := FlushVol(nil, docRecHdl^^.fileFSSpec.vRefNum);
			docRecHdl^^.fileRefNum := 0;
			end;
		end;

	if (docRecHdl^^.editRec <> nil) then
		TEDispose(docRecHdl^^.editRec);
	if (docRecHdl^^.pictureHdl <> nil) then
		KillPicture(docRecHdl^^.pictureHdl);

	DisposeHandle(Handle(docRecHdl));
	
	DoCloseFile := osError;
	end;
		{of function DoCloseFile}

{  DoNewDocWindow }

function DoNewDocWindow(showWindow : boolean; documentType : OSType) : OSErr;

	var
	docRecHdl : DocRecordHandle;

	begin
	if (gCurrentNumberOfWindows = kMaxWindows) then
		begin
		DoNewDocWindow := eMaxWindows;
		Exit(DoNewDocWindow);
		end;

	gWindowPtr := GetNewWindow(rNewWindow, nil, WindowPtr(-1));
	if (gWindowPtr = nil) then
		begin
		DoNewDocWindow := MemError;
		Exit(DoNewDocWindow);
		end;

	SetPort(gWindowPtr);

	docRecHdl := DocRecordHandle(NewHandle(sizeof(DocRecord)));
	if (docRecHdl = nil) then
		begin
		DisposeWindow(gWindowPtr);
		gCurrentNumberOfWindows := gCurrentNumberOfWindows -1;
		DoNewDocWindow := MemError;
		Exit(DoNewDocWindow);
		end;

	SetWRefCon(gWindowPtr, longint(docRecHdl));

	docRecHdl^^.editRec := nil;
	docRecHdl^^.pictureHdl := nil;
	docRecHdl^^.fileRefNum := 0;
	docRecHdl^^.windowDirty := false;

	if (documentType = 'TEXT') then
		begin
		gDestRect := gWindowPtr^.portRect;
		InsetRect(gDestRect, 6, 6);
		gViewRect := gDestRect;

		MoveHHi(Handle(docRecHdl));
		HLock(Handle(docRecHdl));
		
		docRecHdl^^.editRec := TENew(gDestRect, gViewRect);
		if (docRecHdl^^.editRec = nil) then
			begin
			DisposeWindow(gWindowPtr);
			gCurrentNumberOfWindows := gCurrentNumberOfWindows -1;
			DisposeHandle(Handle(docRecHdl));
			DoNewDocWindow := MemError;
			Exit(DoNewDocWindow);
			end;

		HUnlock(Handle(docRecHdl));
		end;

	if (showWindow) then
		ShowWindow(gWindowPtr);

	gCurrentNumberOfWindows := gCurrentNumberOfWindows + 1;

	DoNewDocWindow := noErr;
	end;
		{of function DoNewDocWindow}

{  DoOpenFile }

function DoOpenFile(fileSpec : FSSpec; documentType : OSType) : OSErr;

	var
	osError : OSErr;
	fileRefNum : integer;
	docRecHdl : DocRecordHandle;

	begin
	osError := DoNewDocWindow(false, documentType);
	if (osError <> noErr) then
		begin
		DoOpenFile := osError;
		Exit(DoOpenFile);
		end;

	SetWTitle(gWindowPtr, fileSpec.name);	

	osError := FSpOpenDF(fileSpec, fsRdWrPerm, fileRefNum);
	if (osError <> noErr) then
		begin
		DisposeWindow(gWindowPtr);
		gCurrentNumberOfWindows := gCurrentNumberOfWindows - 1;
		DoOpenFile := osError;
		Exit(DoOpenFile);
		end;
	
	docRecHdl := DocRecordHandle(GetWRefCon(gWindowPtr));
	docRecHdl^^.fileRefNum := fileRefNum;
	docRecHdl^^.fileFSSpec := fileSpec;

	if (documentType = 'TEXT') then
		begin
		osError := DoReadTextFile(gWindowPtr);
		if (osError <> noErr) then
			begin
			DoOpenFile := osError;
			Exit(DoOpenFile);
			end;
		end
	else if (documentType = 'PICT') then
		begin
		osError := DoReadPictFile(gWindowPtr);
		if (osError <> noErr) then
			begin
			DoOpenFile := osError;
			Exit(DoOpenFile);
			end;
		end;

	ShowWindow(gWindowPtr);

	DoOpenFile := noErr;
	end;
		{of function DoOpenFile}

{  DoCloseCommand }

function DoCloseCommand : OSErr;

	var
	myWindowPtr : WindowPtr;
	windowKind : integer;
	docRecHdl : DocRecordHandle;
	osError : OSErr;

	begin
	osError := 0;
	myWindowPtr := FrontWindow;
	windowKind := WindowPeek(myWindowPtr)^.windowKind;

	case (windowKind) of

		userKind:
			begin
			docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));
			osError := DoCloseFile(myWindowPtr, docRecHdl);
			if (osError = kUserCancelled) then
				begin
				DoCloseCommand := kUserCancelled;
				Exit(DoCloseCommand);
				end
			else if (osError = noErr) then
				begin
				DisposeWindow(myWindowPtr);
				gCurrentNumberOfWindows := gCurrentNumberOfWindows - 1;
				end;
			end;
			
		dialogKind:
			begin
			{ Hide or close modeless dialog, as required.}
			end;
		end;
			{of case statement}

	DoCloseCommand := osError;
	end;
		{of function DoCloseCommand}

{  DoQuitCommand }

function DoQuitCommand : OSErr;

	var
	osError : OSErr;

	begin
	osError := 0;
	while (FrontWindow <> nil) do
		begin
		osError := DoCloseCommand;
		if (osError <> noErr) then
			begin
			DoQuitCommand := osError;
			Exit(DoQuitCommand);
			end;
		end;

	DoQuitCommand := osError;
	end;
		{of function DoQuitCommand}

{  DoRevertCommand }

function  DoRevertCommand : OSErr;

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecordHandle;
	fileRefNum : integer;
	fileName : string;
	itemHit : integer;
	osError : OSErr;
	
	begin
	osError := 0;
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));
	fileRefNum := docRecHdl^^.fileRefNum;

	SetPort(myWindowPtr);

	GetWTitle(myWindowPtr, fileName);
	ParamText(fileName, '', '', '');

	SetPort(myWindowPtr);

	itemHit := CautionAlert(rRevertAlert, nil);

	if (itemHit = 1) then
		begin
		EraseRect(myWindowPtr^.portRect);
		if (docRecHdl^^.editRec <> nil) then
			osError := DoReadTextFile(myWindowPtr)
		else if (docRecHdl^^.pictureHdl <> nil) then
			begin
			KillPicture(docRecHdl^^.pictureHdl);
			docRecHdl^^.pictureHdl := nil;
			osError := DoReadPictFile(myWindowPtr);
			end;

		docRecHdl^^.windowDirty := false;

		InvalRect(myWindowPtr^.portRect);
		end;

	DoRevertCommand := osError;
	end;
		{of function DoRevertCommand}
		
{  DoOpenCommand }

function DoOpenCommand : OSErr;

	var
	fileTypes : SFTypeList;
	fileReply : StandardFileReply;
	documentType : OSType;
	 osError : OSErr;		

	begin
	osError := 0;		
	fileTypes[0] := 'TEXT';
	fileTypes[1] := 'PICT';

	StandardGetFile(nil, 2, ConstSFTypeListPtr(@fileTypes), fileReply);

	documentType := fileReply.sfType;

	if (fileReply.sfGood) then
		osError := DoOpenFile(fileReply.sfFile, documentType);

	DoOpenCommand := osError;
	end;
		{of function DoOpenCommand}

{  DoNewCommand }

function DoNewCommand : OSErr;

	var
	osError : OSErr;
	documentType : OSType;

	begin
	documentType := 'TEXT';
	osError := DoNewDocWindow(true, documentType);
	DoNewCommand := osError;
	end;
		{of function DoNewCommand}

{  HasGotRequiredParams }

function HasGotRequiredParams(appEvent : AppleEvent) : OSErr;

	var
	returnedType : DescType;
	actualSize : Size;
	osError : OSErr;

	begin
	osError := AEGetAttributePtr(appEvent, keyMissedKeywordAttr, typeWildCard, returnedType, 
												nil, 0, actualSize);
	if (osError = errAEDescNotFound) then
		HasGotRequiredParams := noErr
	else if (osError = noErr) then
		HasGotRequiredParams := errAEParamMissed;
	end;
		{of function HasGotRequiredParams}

{  DoOpenDocsEvent }

function DoOpenDocsEvent(var appEvent, reply : AppleEvent; handlerRefcon : longint) : OSErr;

	var
	fileSpec : FSSpec;
	docList : AEDescList;
	osError, ignoreErr : OSErr;
	index, numberOfItems : longint;
	actualSize : Size;
	keyWord : AEKeyword;
	returnedType : DescType;
	fileInfo : FInfo;

	begin
	osError := AEGetParamDesc(appEvent, keyDirectObject, typeAEList, docList);

	if (osError = noErr) then
		begin
		osError := HasGotRequiredParams(appEvent);
		if (osError = noErr) then
			begin
			ignoreErr := AECountItems(docList, numberOfItems);
			if (osError = noErr) then
				begin
				for index := 1 to numberOfItems do
					begin
					osError := AEGetNthPtr(docList, index, typeFSS, keyWord, returnedType, 
											@fileSpec, sizeof(fileSpec), actualSize);
					if (osError = noErr) then
						begin
						osError := FSpGetFInfo(fileSpec, fileInfo);
						if (osError = noErr) then
							begin
							osError := DoOpenFile(fileSpec, fileInfo.fdType);
							if (osError <> noErr) then
								DoError(osError);
							end;
						end
					else
						DoError(osError);
					end;
				end;
			end
		else
			DoError(osError);

		ignoreErr := AEDisposeDesc(docList);
		end
		
	else
		DoError(osError);

	DoOpenDocsEvent := osError;
	end;
		{of function DoOpenDocsEvent}

{  DoOpenAppEvent }

function DoOpenAppEvent(var appEvent, reply : AppleEvent; handlerRefCon : longint)
									 : OSErr;

	var
	osError : OSErr;

	begin
	osError := HasGotRequiredParams(appEvent);
	if (osError = noErr) then
		osError := DoNewCommand;

	DoOpenAppEvent := osError;
	end;
		{of function DoOpenAppEvent}

{  DoMakeWindowDirty }

procedure DoMakeWindowDirty;

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecordHandle;

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));
	
	SetPort(myWindowPtr);
	MoveTo(myWindowPtr^.portRect.left, myWindowPtr^.portRect.top);
	LineTo(myWindowPtr^.portRect.right, myWindowPtr^.portRect.bottom);

	docRecHdl^^.windowDirty := true;
	end;
		{of procedure DoMakeWindowDirty}

{  DoAdjustMenus }

procedure DoAdjustMenus;

	var
	menuHdl : MenuHandle;
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecordHandle;

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));

	menuHdl := GetMenuHandle(mFile);

	if (gCurrentNumberOfWindows > 0) then
		begin
		menuHdl := GetMenuHandle(mFile);
		EnableItem(menuHdl, iClose);
		EnableItem(menuHdl, iSave);
		EnableItem(menuHdl, iSaveAs);
		EnableItem(menuHdl, iRevert);

		{Note the use of the short-circuit boolean 'and' operator. Something to be aware
		of when porting C to Pascal :)  /kdg}
		if ((docRecHdl^^.pictureHdl = nil) & (docRecHdl^^.editRec^^.teLength = 0))
			then DisableItem(menuHdl, iRevert);

		menuHdl := GetMenuHandle(mDemonstration);
		if not (docRecHdl^^.windowDirty) then
			EnableItem(menuHdl, iDirty)
		else
			DisableItem(menuHdl, iDirty);
		end
	else
		begin
		menuHdl := GetMenuHandle(mFile);
		DisableItem(menuHdl, iClose);
		DisableItem(menuHdl, iSave);
		DisableItem(menuHdl, iSaveAs);
		DisableItem(menuHdl, iRevert);
		menuHdl := GetMenuHandle(mDemonstration);
		DisableItem(menuHdl, iDirty);
		end;	

	DrawMenuBar;
	end;
		{of procedure DoAdjustMenus}

{  DoFileMenu }

procedure DoFileMenu(menuItem : longint);

	var
	osError : OSErr;

	begin
	case (menuItem) of

		iNew:
			begin
			osError := DoNewCommand;
			if (osError <> noErr) then
				DoError(osError);
			end;

		iOpen:
			begin
			osError := DoOpenCommand;
			if (osError <> noErr) then
				DoError(osError);
			end;

		iClose:
			begin
			osError := DoCloseCommand;
			if ((osError <> noErr) and not (osError = kUserCancelled)) then
				DoError(osError);
			end;

		iSave:
			begin
			osError := DoSaveCommand;
			if (osError <> noErr) then
				DoError(osError);
			end;

		iSaveAs:
			begin
			osError := DoSaveAsCommand;
			if (osError <> noErr) then
				DoError(osError);
			end;

		iRevert:
			begin
			osError := DoRevertCommand;
			if (osError <> noErr) then
				DoError(osError);
			end;

		iQuit:
			begin
			osError := DoQuitCommand;
			if ((osError <> noErr) and not (osError = kUserCancelled)) then
				DoError(osError);
			if (osError <> kUserCancelled) then 
				gDone := true;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoInitManagers}

{  DoMenuChoice }

procedure DoMenuChoice(menuChoice : longint);

	var
	menuID, menuItem : longint;
	itemName : string;
	daDriverRefNum : longint;

	begin
	menuID := HiWord(menuChoice);
	menuItem := LoWord(menuChoice);

	if (menuID = 0) then
		Exit(DoMenuChoice);

	case (menuID) of 

		 mApple:
			begin
			GetMenuItemText(GetMenuHandle(mApple), menuItem, itemName);
			daDriverRefNum := OpenDeskAcc(itemName);
			end;

		mFile:
			begin
			DoFileMenu(menuItem);
			end;

		mDemonstration:
			begin
			DoMakeWindowDirty;
			end;
		end;
			{of case statement}

	HiliteMenu(0);
	end;
		{of procedure DoMenuChoice}

{  DoUpdate }

procedure DoUpdate(eventRec : EventRecord);

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecordHandle;
	destRect : Rect;

	begin
	myWindowPtr := WindowPtr(eventRec.message);
	docRecHdl := DocRecordHandle(GetWRefCon(myWindowPtr));

	SetPort(myWindowPtr);

	BeginUpdate(myWindowPtr);

	if (docRecHdl^^.pictureHdl <> nil) then
		begin
		destRect := docRecHdl^^.pictureHdl^^.picFrame;
		HLock(Handle(docRecHdl^^.pictureHdl));
		DrawPicture(docRecHdl^^.pictureHdl, destRect);
		HUnlock(Handle(docRecHdl^^.pictureHdl));
		end
	else if (docRecHdl^^.editRec <> nil) then
		begin
		HLock(Handle(docRecHdl^^.editRec));
		TEUpdate(gDestRect, docRecHdl^^.editRec);
		HUnlock(Handle(docRecHdl^^.editRec));
		end;

	if (docRecHdl^^.windowDirty) then
		begin
		MoveTo(myWindowPtr^.portRect.left, myWindowPtr^.portRect.top);
		LineTo(myWindowPtr^.portRect.right, myWindowPtr^.portRect.bottom);
		end;

	EndUpdate(myWindowPtr);
	end;
		{of procedure DoUpdate}

{  DoMouseDown }

procedure DoMouseDown(eventRec : EventRecord);

	var
	myWindowPtr : WindowPtr;
	partCode : longint;
	ignored : OSErr;

	begin
	partCode := FindWindow(eventRec.where, myWindowPtr);
	
	case (partCode) of

		inMenuBar:
			begin
			DoAdjustMenus;
			DoMenuChoice(MenuSelect(eventRec.where));
			end;

		inSysWindow:
			begin
			SystemClick(eventRec, myWindowPtr);
			end;

		inContent:
			begin
			if (myWindowPtr <> FrontWindow) then
				SelectWindow(myWindowPtr);
			end;

		inDrag:
			begin
			DragWindow(myWindowPtr, eventRec.where, qd.screenBits.bounds);
			end;

		inGoAway:
			begin
			if (TrackGoAway(myWindowPtr, eventRec.where)) then
				ignored := DoCloseCommand;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoMouseDown}

{  DoEvents }

procedure DoEvents(eventRec : EventRecord);

	var
	charCode : char;
	ignored : OSErr;

	begin
	case (eventRec.what) of

		kHighLevelEvent:
			begin
			ignored := AEProcessAppleEvent(eventRec);
			end;

		mouseDown:
			begin
			DoMouseDown(eventRec);
			end;

		keyDown, autoKey:
			begin
			charCode := chr(BAnd(eventRec.message, charCodeMask));
			if (BAnd(eventRec.modifiers, cmdKey) <> 0) then
				begin
				DoAdjustMenus;
				DoMenuChoice(MenuKey(charCode));
				end;
			end;

		updateEvt:
			begin
			DoUpdate(eventRec);
			end;

		osEvt:
			begin
			case (BAnd(BSR(eventRec.message, 24), $000000FF)) of

				suspendResumeMessage:
					begin
					gInBackground := (BAnd(eventRec.message, resumeFlag) = 0);
					end;
				end;
					{of inner case statement}
					
			HiliteMenu(0);
			end;
		end;
			{of outer case statement}
	end;
		{of procedure DoEvents}
		
{  EventLoop }

procedure EventLoop;

	var
	eventRec : EventRecord;

	begin
	gDone := false;

	while not (gDone) do
		begin
		if (WaitNextEvent(everyEvent, eventRec, kMaxLong, nil)) then
			DoEvents(eventRec);
		end;
	
	end;
		{of procedure EventLoop}

{  DoQuitAppEvent }

function DoQuitAppEvent(var appEvent, reply : AppleEvent; handlerRefcon : longint)
								 : OSErr;

	var
	osError : OSErr;
	begin
	osError := HasGotRequiredParams(appEvent);
	if (osError = noErr) then 
		begin
		while (FrontWindow <> nil) do 
			begin
			osError := DoCloseCommand;
			if ((osError <> noErr) and (osError <> kUserCancelled)) then 
				DoError(osError);
           if (osError = kUserCancelled) then 
				EventLoop;
        	gDone := true;
        	end;
		end;
	DoQuitAppEvent := osError;
	end;
		{of function DoQuitAppEvent}

{  DoInstallAEHandlers }

procedure DoInstallAEHandlers;
	
	var
	err : OSErr;
	
	begin
	err := AEInstallEventHandler(kCoreEventClass, kAEOpenApplication,
			NewAEEventHandlerProc(AEEventHandlerProcPtr(@DoOpenAppEvent)), 0, false);
                                                                           { For PowerPC }
	if (err <> noErr) then
		 DoError(eInstallHandler);

	err := AEInstallEventHandler(kCoreEventClass, kAEOpenDocuments,
			NewAEEventHandlerProc(AEEventHandlerProcPtr(@DoOpenDocsEvent)), 0, false);
                                                                           { For PowerPC }
	if (err <> noErr) then	
		 DoError(eInstallHandler);

	err := AEInstallEventHandler(kCoreEventClass, kAEQuitApplication,
			NewAEEventHandlerProc(AEEventHandlerProcPtr(@DoQuitAppEvent)), 0, false);
                                                                           { For PowerPC }
	if (err <> noErr) then
		 DoError(eInstallHandler);

	end;
		{of procedure DoInstallAEHandlers}

{  start of main program }

begin

	{  initialize managers }

	DoInitManagers;
	gCurrentNumberOfWindows := 0;

	{  create routine decriptors }

	doOpenAppEventRD := NewAEEventHandlerProc(ProcPtr(@DoOpenAppEvent));   { For PowerPC }
	doOpenDocsEventRD := NewAEEventHandlerProc(ProcPtr(@DoOpenDocsEvent)); { For PowerPC }
	doQuitAppEventRD := NewAEEventHandlerProc(ProcPtr(@DoQuitAppEvent));   { For PowerPC }

	{  set application's resource fork as current resource file }

	gAppResFileRefNum := CurResFile;

	{  set up menu bar and menus }
	
	menubarHdl := GetNewMBar(rMenubar);
	if (menubarHdl = nil) then
		DoError(MemError);
	SetMenuBar(menubarHdl);
	DrawMenuBar;

	menuHdl := GetMenuHandle(mApple);
	if (menuHdl = nil) then
		DoError(MemError)
	else
		AppendResMenu(menuHdl, 'DRVR');

	{  install required Apple event handlers }

	DoInstallAEHandlers;

	{  enter EventLoop }

	EventLoop;

end.
	
{  }
