{ 
// Text2PascalPPC.p
// 
//
// This program demonstrates:
//
// 	The use of TextEdit to enter data into the various fields of a database record.
//
// 	The formatting and display of dates, times and numbers.
//
// The program opens a window in which four data entry fields are displayed.  A separate
// TextEdit edit record is associated with each field.  The first field requires text to
// be entered, the second an integer value, the third a floating point value (that is, a
// currency value), and the fourth a date.
//
// The user may select a field for data entry using the mouse or by cycling between
// fields using the Tab key.  When the Return key is pressed, some arithmetic is
// performed using the integer and floating point values, six months are added to the
// date, and the resulting data is formatted and displayed at the bottom of the window.
//
// The current date and time is displayed at the top of the window.
//
// The program utilizes the following resources:
//
// 	An 'MBAR' resource, and 'MENU' resources for Apple, File, and Edit menus (preload, 
//		non-purgeable).  
//
// 	A 'WIND' resource (purgeable) (initially visible).  
//
// 	A 'SIZE' resource with the acceptSuspendResumeEvents and doesActivateOnFGSwitch
//		flags set.	  
//
//  }

program Text2Pascal(input, output);

{  include the following Universal Interfaces }

uses

	Windows, Fonts, Menus, TextEdit, Quickdraw, Dialogs, QuickdrawText, Processes, Types, 
	Memory, Events, TextUtils, ToolUtils, OSUtils, Devices, SegLoad, IntlResources,
	Script, LowMem, fp;

{  define the following constants }

const

mApple = 128;
 iAbout = 1;
mFile = 129;
 iQuit = 12;
mEdit = 130;	
 iCut = 3;
 iCopy = 4;
 iPaste = 5;
 iClear = 6;
 iSelectAll = 7;

kTab = $09;
kDel = $7F;
kReturn = $0D;
kMaxCharasItem = 20;
kMaxCharasQuant = 10;
kMaxCharasValue = 12;
kMaxCharasDate = 13;

rWindow = 128;
rMenubar = 128;

{  user-defined types }

type

DocRec = record
	itemEditHdl : TEHandle;
	quantEditHdl : TEHandle;
	valueEditHdl : TEHandle;
	dateEditHdl : TEHandle;
	end;
DocRecPointer = ^DocRec;
DocRecHandle = ^DocRecPointer;

{  global variables }

var

gDone : Boolean;
gInBackground : Boolean;
gCursorRegion : RgnHandle;
gCurrentEditRecHdl : TEHandle;
gMaxCharasThisField : integer;
gItemRect : Rect;
gQuantRect : Rect;
gValueRect : Rect;
gDateRect : Rect;
gOldRawSeconds : UInt32;

menubarHdl : Handle;
menuHdl : MenuHandle;
myWindowPtr : WindowPtr;
docRecHdl : DocRecHandle;

{  DoInitManagers }

procedure DoInitManagers;

	begin
	MaxApplZone;
	MoreMasters;

	InitGraf(@qd.thePort);
	InitFonts;
	InitWindows;
	InitMenus;
	TEInit;
	InitDialogs(nil);

	InitCursor;	
	FlushEvents(everyEvent, 0);
	end;
		{of procedure DoInitManagers}

{  DoSetUpEditRecords }

procedure DoSetUpEditRecords(myWindowPtr : WindowPtr);

	var
	aRect : Rect;
	docRecHdl : DocRecHandle;

	begin
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	aRect := gItemRect;
	InsetRect(aRect, 2, 2);
	docRecHdl^^.itemEditHdl := TENew(aRect, aRect);

	aRect := gQuantRect;
	InsetRect(aRect, 2, 2);
	docRecHdl^^.quantEditHdl := TENew(aRect, aRect);

	aRect := gValueRect;
	InsetRect(aRect, 2, 2);
	docRecHdl^^.valueEditHdl := TENew(aRect, aRect);

	aRect := gDateRect;
	InsetRect(aRect, 2, 2);
	docRecHdl^^.dateEditHdl := TENew(aRect, aRect);

	gCurrentEditRecHdl := docRecHdl^^.itemEditHdl;
	gMaxCharasThisField := kMaxCharasItem;
	end;
		{of procedure DoSetUpEditRecords}
		
{  DoTodaysDate }

procedure DoTodaysDate;

	var
	rawSeconds : UInt32;
	theDateString : string;
		
	begin
	GetDateTime(rawSeconds);
	DateString(rawSeconds, longDate, theDateString, nil);
	MoveTo(110, 22);
	DrawString(theDateString);
	end;
		{of procedure DoTodaysDate}

{  DoEraseRecordDisplay }

procedure DoEraseRecordDisplay;

	var
	rectToErase : Rect;

	begin
	SetRect(rectToErase, 166, 172, 330, 278);
	EraseRect(rectToErase);
	end;
		{of procedure DoEraseRecordDisplay}

{  DoDrawWindow }

procedure DoDrawWindow;

	var
	lastItemRect : Rect;

	begin
	DoTodaysDate;

	MoveTo(31, 34);
	LineTo(331, 34);
	MoveTo(31, 142);
	LineTo(331, 142);

	FrameRect(gItemRect);
	FrameRect(gQuantRect);
	FrameRect(gValueRect);
	FrameRect(gDateRect);

	SetRect(lastItemRect, 31, 171, 331, 279);
	FrameRect(lastItemRect);

	MoveTo(31, 22);
	DrawString('Today''s date is:');
	MoveTo(38, 57);
	DrawString('Item Title:');
	MoveTo(45, 80);
	DrawString('Quantity:');
	MoveTo(36, 103);
	DrawString('Unit Value:');
	MoveTo(65, 126);
	DrawString('Date:');
	MoveTo(248, 57);
	MoveTo(118, 162);
	DrawString('Last Record Entered:');
	MoveTo(248, 57);
	DrawString('Eg: Barometers');
	MoveTo(248, 80);
	DrawString('Eg: 34');
	MoveTo(248, 103);
	DrawString('Eg: 135.58');
	MoveTo(248, 126);
	DrawString('Eg: 24 Jul 95');
	MoveTo(110, 188);
	DrawString('Item Title:');
	MoveTo(116, 208);
	DrawString('Quantity:');
	MoveTo(107, 228);
	DrawString('Unit Value:');
	MoveTo(104, 248);
	DrawString('Total Value:');
	MoveTo(98, 268);
	DrawString('Review Date:');
	end;
		{of procedure DoDrawWindow}

{  DoAdjustCursor }

procedure DoAdjustCursor(myWindowPtr : WindowPtr; mouseRegion : RgnHandle);

	var
	oldPort : GrafPtr;
	arrowRegion, itemRegion, quantRegion : RgnHandle;
	valueRegion, dateRegion, iBeamRegions : RgnHandle;
	itemRect, quantRect, valueRect, dateRect : Rect;
	mouseXY : Point;

	begin
	if (gInBackground) then
		begin
		SetCursor(qd.arrow);
		Exit(DoAdjustCursor);
		end;

	GetPort(oldPort);
	SetPort(myWindowPtr);

	arrowRegion	 := NewRgn;
	itemRegion   := NewRgn;	
	quantRegion  := NewRgn;	
	valueRegion  := NewRgn;	
	dateRegion   := NewRgn;	
	iBeamRegions := NewRgn;

	SetRectRgn(arrowRegion, -32768, -32768, 32766, 32766);

	itemRect  := gItemRect;
	quantRect := gQuantRect;
	valueRect := gValueRect;
	dateRect  := gDateRect;

	LocalToGlobal(itemRect.topLeft);
	LocalToGlobal(itemRect.botRight);	
	RectRgn(itemRegion, itemRect);
	LocalToGlobal(quantRect.topLeft);
	LocalToGlobal(quantRect.botRight);	
	RectRgn(quantRegion, quantRect);
	LocalToGlobal(valueRect.topLeft);
	LocalToGlobal(valueRect.botRight);	
	RectRgn(valueRegion, valueRect);
	LocalToGlobal(dateRect.topLeft);
	LocalToGlobal(dateRect.botRight);	
	RectRgn(dateRegion, dateRect);

	UnionRgn(itemRegion, quantRegion, iBeamRegions);
	UnionRgn(valueRegion, iBeamRegions, iBeamRegions);
	UnionRgn(dateRegion, iBeamRegions, iBeamRegions);

	DiffRgn(arrowRegion, iBeamRegions, arrowRegion);
	
	GetMouse(mouseXY);
	LocalToGlobal(mouseXY);

	if (PtInRgn(mouseXY, iBeamRegions)) then
		begin
		SetCursor(Cursor(GetCursor(iBeamCursor)^^));
		CopyRgn(iBeamRegions, mouseRegion);
		end
	else begin
		SetCursor(qd.arrow);
		CopyRgn(arrowRegion, mouseRegion);
		end;

	DisposeRgn(arrowRegion);
	DisposeRgn(itemRegion);
	DisposeRgn(quantRegion);	
	DisposeRgn(valueRegion);	
	DisposeRgn(dateRegion);	
	DisposeRgn(iBeamRegions);

	SetPort(oldPort);
	end;
		{of procedure DoAdjustCursor}

{  DoEditMenu }

procedure DoEditMenu(menuItem : integer);

	begin
	case (menuItem) of

		iCut:
			begin
			TECut(gCurrentEditRecHdl);
			end;

		iCopy:
			begin
			TECopy(gCurrentEditRecHdl);
			end;

		iPaste:
			begin
			TEPaste(gCurrentEditRecHdl);
			end;

		iClear:
			begin
			TEDelete(gCurrentEditRecHdl);
			end;

		iSelectAll:
			begin
			TESetSelect(0, gCurrentEditRecHdl^^.teLength, gCurrentEditRecHdl);
			end;
		end;
			{of case statement}
	end;
		{of procedure DoEditMenu}

{  DoMenuChoice }

procedure DoMenuChoice(menuChoice : longint);

	var
	menuID, menuItem : integer;
	itemName : string;
	daDriverRefNum : integer;

	begin
	menuID	 := HiWord(menuChoice);
	menuItem := LoWord(menuChoice);

	if (menuID = 0) then
		Exit(DoMenuChoice);

	case (menuID) of

		mApple:
			begin
			if (menuItem = iAbout) then
				SysBeep(10)
			else
				begin
				GetMenuItemText(GetMenuHandle(mApple), menuItem, itemName);
				daDriverRefNum := OpenDeskAcc(itemName);
				end;
			end;
			
		mFile:
			begin
			if (menuItem = iQuit) then
				gDone := true;
			end;
			
		mEdit:
			begin
			DoEditMenu(menuItem);
			end;
		end;
			{of case statement}

	HiliteMenu(0);
	end;
		{of procedure DoMenuChoice}

{  DoAdjustMenus }

procedure DoAdjustMenus;

	var
	fileMenuHdl, editMenuHdl : MenuHandle;
	
	begin
	fileMenuHdl := GetMenuHandle(mFile);
	editMenuHdl := GetMenuHandle(mEdit);
		
	if (gCurrentEditRecHdl^^.selStart < gCurrentEditRecHdl^^.selEnd) then
		begin
		EnableItem(editMenuHdl, iCut);
		EnableItem(editMenuHdl, iCopy);
		EnableItem(editMenuHdl, iClear);
		end
	else begin
		DisableItem(editMenuHdl, iCut);
		DisableItem(editMenuHdl, iCopy);
		DisableItem(editMenuHdl, iClear);
		end;

	if (TEGetScrapLength > 0) then
		EnableItem(editMenuHdl, iPaste)
	else
		DisableItem(editMenuHdl, iPaste);

	if (gCurrentEditRecHdl^^.teLength > 0) then
		EnableItem(editMenuHdl, iSelectAll)
	else
		DisableItem(editMenuHdl, iSelectAll);

	DrawMenuBar;
	end;
		{of procedure DoAdjustMenus}

{  DoAcceptDateField }

procedure DoAcceptDateField(dateEditHdl : TEHandle);

	var
	dateCacheRec : DateCacheRecord;
	textPtr : Ptr;
	textLength, lengthUsed : longint;
	longDateTimeRec : LongDateRec;
	longDateTimeValue : LongDateTime;
	dateString : string;
	ignored : OSErr;

	begin
	ignored := InitDateCache(@dateCacheRec);
	textPtr := dateEditHdl^^.hText^;
	textLength := dateEditHdl^^.teLength;

	ignored := StringToDate(textPtr, textLength, @dateCacheRec, lengthUsed, 
																longDateTimeRec);

	LongDateToSeconds(longDateTimeRec, longDateTimeValue);
	longDateTimeValue.lo := longDateTimeValue.lo + 15724800;
	LongDateString(longDateTimeValue, longDate, dateString, nil);

	MoveTo(166, 268);
	DrawString(dateString);
	end;
		{of procedure DoAcceptDateField}

{  DoAcceptValueField }

procedure DoAcceptValueField(valueEditHdl : TEHandle; quantEditHdl : TEHandle);

	var
	itl4ResourceHdl : Handle;
	numpartsOffset : longint;
	numpartsLength : longint;
	numpartsTable : NumberParts;
	formatString : string;
	formatStringRec : NumFormatString;
	inputNumString : string;
	formattedNumString : string;
	value80Bit : extended80;
	quantityString : string;
	quantity : longint;
	result : FormatResultType;
	ignored : OSErr;				
	valueDouble : Double;                                                    { For PowerPC }

	begin
	formatString := '''$''###,###,###.00;''Valueless'';''Valueless''';
	
	GetIntlResourceTable(smSystemScript, iuNumberPartsTable, itl4ResourceHdl,  
											 		numpartsOffset, numpartsLength);
	numpartsTable := NumberPartsPtr(longint(itl4ResourceHdl^) + numpartsOffset)^;

	ignored := StringToFormatRec(formatString, numpartsTable, formatStringRec);

	GetDialogItemText(valueEditHdl^^.hText, inputNumString);

	ignored := StringToExtended(inputNumString, formatStringRec, numpartsTable, 
																	value80Bit);
	ignored := ExtendedToString(value80Bit, formatStringRec, numpartsTable, 
																formattedNumString);

	MoveTo(166, 228);
	DrawString(formattedNumString);

	GetDialogItemText(quantEditHdl^^.hText, quantityString);
	StringToNum(quantityString, quantity);

	valueDouble := x80tod(value80Bit);                                       { For PowerPC }
	valueDouble := valueDouble * quantity;                                   { For PowerPC }
	dtox80(valueDouble, value80Bit);                                         { For PowerPC }

	result := ExtendedToString(value80Bit, formatStringRec, numpartsTable, 
																	formattedNumString);
	MoveTo(166, 248);
	if (result = fFormatOverflow) then
		DrawString('Too large to display')
	else
		DrawString(formattedNumString);
	end;
		{of procedure DoAcceptValueField}

{  DoAcceptQuantField }

procedure DoAcceptQuantField(quantEditHdl : TEHandle);

	var
	quantString : string;

	begin
	GetDialogItemText(quantEditHdl^^.hText, quantString);
	MoveTo(166, 208);
	DrawString(quantString);
	end;
		{of procedure DoAcceptQuantField}

{  DoAcceptItemField }

procedure DoAcceptItemField(itemEditHdl : TEHandle);

	var
	itemString : string;

	begin
	GetDialogItemText(itemEditHdl^^.hText, itemString);
	MoveTo(166, 188);
	DrawString(itemString);
	end;
		{of procedure DoAcceptItemField}

{  DoAcceptNewRecord }

procedure DoAcceptNewRecord;

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;
	
	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	if ((docRecHdl^^.itemEditHdl^^.teLength = 0) or 
		(docRecHdl^^.quantEditHdl^^.teLength = 0) or
		(docRecHdl^^.valueEditHdl^^.teLength = 0)	or
		(docRecHdl^^.dateEditHdl^^.teLength = 0)) then
		begin
		SysBeep(10);
		Exit(DoAcceptNewRecord);
		end;

	DoEraseRecordDisplay;

	DoAcceptItemField(docRecHdl^^.itemEditHdl);
	DoAcceptQuantField(docRecHdl^^.quantEditHdl);
	DoAcceptValueField(docRecHdl^^.valueEditHdl, docRecHdl^^.quantEditHdl);
	DoAcceptDateField(docRecHdl^^.dateEditHdl);

	TESetSelect(0, 32767, docRecHdl^^.itemEditHdl);
	TEDelete(docRecHdl^^.itemEditHdl);

	TESetSelect(0, 32767, docRecHdl^^.quantEditHdl);
	TEDelete(docRecHdl^^.quantEditHdl);

	TESetSelect(0, 32767, docRecHdl^^.valueEditHdl);
	TEDelete(docRecHdl^^.valueEditHdl);

	TESetSelect(0, 32767, docRecHdl^^.dateEditHdl);
	TEDelete(docRecHdl^^.dateEditHdl);

	TEDeactivate(gCurrentEditRecHdl);
	gCurrentEditRecHdl := docRecHdl^^.itemEditHdl;
	TEActivate(docRecHdl^^.itemEditHdl);

	gMaxCharasThisField := kMaxCharasItem;
	end;
		{of procedure DoAcceptNewRecord}

{  DoChangeCurrentEditRec }

procedure DoChangeCurrentEditRec(newEditRecHdl : TEHandle; teClickFlag : Boolean; 
									mouseXY : Point; maxCharas : integer);

	begin
	TEDeactivate(gCurrentEditRecHdl);
	TEActivate(newEditRecHdl);
	gCurrentEditRecHdl := newEditRecHdl;

	if (teClickFlag) then
		TEClick(mouseXY, false, newEditRecHdl);

	TESetSelect(0, 32767, gCurrentEditRecHdl);

	gMaxCharasThisField := maxCharas;
	end;
		{of procedure DoChangeCurrentEditRec}

{  DoActivate }

procedure DoActivate(theEvent : EventRecord);

	var
	myWindowPtr : WindowPtr;
	becomingActive : Boolean;

	begin
	myWindowPtr := WindowPtr(theEvent.message);

	becomingActive := boolean(BAnd(theEvent.modifiers, activeFlag) = activeFlag);
	
	if (becomingActive) then
		TEActivate(gCurrentEditRecHdl)
	else
		TEDeactivate(gCurrentEditRecHdl);
	end;
		{of procedure DoActivate}

{  DoUpdate }

procedure DoUpdate(theEvent : EventRecord);

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;
	oldPort : GrafPtr;
	
	begin	
	myWindowPtr := WindowPtr(theEvent.message);
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	GetPort(oldPort);
	SetPort(myWindowPtr);

	BeginUpdate(WindowPtr(theEvent.message));

	EraseRect(myWindowPtr^.portRect);
	DoDrawWindow;
	TEUpdate(myWindowPtr^.portRect, docRecHdl^^.itemEditHdl);
	TEUpdate(myWindowPtr^.portRect, docRecHdl^^.quantEditHdl);
	TEUpdate(myWindowPtr^.portRect, docRecHdl^^.valueEditHdl);
	TEUpdate(myWindowPtr^.portRect, docRecHdl^^.dateEditHdl);
	EndUpdate(WindowPtr(theEvent.message));

	SetPort(oldPort);
	end;
		{of procedure DoUpdate}

{  DoInContent }

procedure DoInContent(theEvent : EventRecord);

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;
	mouseXY : Point;
	shiftKeyPosition : Boolean;

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	mouseXY := theEvent.where;
	GlobalToLocal(mouseXY);

	shiftKeyPosition := BAnd(theEvent.modifiers, shiftKey) <> 0;

	if (PtInRect(mouseXY, gItemRect)) then
		begin
		if (gCurrentEditRecHdl = docRecHdl^^.itemEditHdl) then
			TEClick(mouseXY, shiftKeyPosition, docRecHdl^^.itemEditHdl)
		else
			DoChangeCurrentEditRec(docRecHdl^^.itemEditHdl,true,mouseXY,kMaxCharasItem);
		end
	else if (PtInRect(mouseXY, gQuantRect)) then
		begin
		if (gCurrentEditRecHdl = docRecHdl^^.quantEditHdl) then
			TEClick(mouseXY, shiftKeyPosition, docRecHdl^^.quantEditHdl)
		else
			DoChangeCurrentEditRec(docRecHdl^^.quantEditHdl,true,mouseXY,kMaxCharasQuant);
		end
	else if (PtInRect(mouseXY, gValueRect)) then
		begin
		if (gCurrentEditRecHdl = docRecHdl^^.valueEditHdl) then
			TEClick(mouseXY, shiftKeyPosition, docRecHdl^^.valueEditHdl)
		else
			DoChangeCurrentEditRec(docRecHdl^^.valueEditHdl,true,mouseXY,kMaxCharasValue);
		end
	else if (PtInRect(mouseXY, gDateRect)) then
		begin
		if (gCurrentEditRecHdl = docRecHdl^^.dateEditHdl) then
			TEClick(mouseXY, shiftKeyPosition, docRecHdl^^.dateEditHdl)
		else
			DoChangeCurrentEditRec(docRecHdl^^.dateEditHdl, true, mouseXY, kMaxCharasDate);
		end;
	end;
		{of procedure DoInContent}

{  DoMouseDown }

procedure DoMouseDown(theEvent : EventRecord);

	var
	myWindowPtr : WindowPtr;
	partCode : integer;

	begin
	partCode := FindWindow(theEvent.where, myWindowPtr);
	
	case (partCode) of

		inMenuBar:
			begin
			DoAdjustMenus;
			DoMenuChoice(MenuSelect(theEvent.where));
			end;

		inSysWindow:
			begin
			SystemClick(theEvent, myWindowPtr);
			end;

		inContent:
			begin
			if (myWindowPtr <> FrontWindow) then
				SelectWindow(myWindowPtr)
			else
				DoInContent(theEvent);
			end;

		inDrag:
			begin
			DragWindow(myWindowPtr, theEvent.where, qd.screenBits.bounds);
			end;

		inGoAway:
			begin
			if (TrackGoAway(myWindowPtr, theEvent.where)) then
				gDone := true;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoMouseDown}
{  DoHandleDelKey }

procedure DoHandleDelKey;

	var
	selectionLength : integer;
		
	begin
	selectionLength := gCurrentEditRecHdl^^.selEnd - gCurrentEditRecHdl^^.selStart;
	if (selectionLength = 0) then
		gCurrentEditRecHdl^^.selEnd := gCurrentEditRecHdl^^.selEnd + 1;
	TEDelete(gCurrentEditRecHdl);
	end;
		{of procedure DoHandleDelKey}

{  DoHandleTabKey }

procedure DoHandleTabKey;

	var
	myWindowPtr : WindowPtr;
	docRecHdl : DocRecHandle;
	dummy : Point;

	begin
	myWindowPtr := FrontWindow;
	docRecHdl := DocRecHandle(GetWRefCon(myWindowPtr));

	if (gCurrentEditRecHdl = docRecHdl^^.itemEditHdl) then
		DoChangeCurrentEditRec(docRecHdl^^.quantEditHdl, false, dummy, kMaxCharasQuant)
	else if (gCurrentEditRecHdl = docRecHdl^^.quantEditHdl) then
		DoChangeCurrentEditRec(docRecHdl^^.valueEditHdl, false, dummy, kMaxCharasValue)	
	else if (gCurrentEditRecHdl = docRecHdl^^.valueEditHdl) then
		DoChangeCurrentEditRec(docRecHdl^^.dateEditHdl, false, dummy, kMaxCharasDate)	
	else if (gCurrentEditRecHdl = docRecHdl^^.dateEditHdl) then
		DoChangeCurrentEditRec(docRecHdl^^.itemEditHdl, false, dummy, kMaxCharasItem);	
	end;
		{of procedure DoHandleTabKey}

{  DoKeyEvent }

procedure DoKeyEvent(charCode : char);

	begin
	if (charCode = char(kTab)) then
		DoHandleTabKey
	else if (charCode = char(kDel)) then
		DoHandleDelKey
	else if (charCode = char(kReturn)) then
		DoAcceptNewRecord	
	else begin
		if ((gCurrentEditRecHdl^^.teLength < gMaxCharasThisField) or
		                (integer(charCode) < $20))
			then TEKey(charCode, gCurrentEditRecHdl)
			else SysBeep(10);
		end;
	end;
		{of procedure DoKeyEvent}

{  DoEvents }

procedure DoEvents (theEvent : EventRecord);

	var
	charCode : char;

	begin
	case (theEvent.what) of

		mouseDown:
			begin
			DoMouseDown(theEvent);
			end;

		keyDown, autoKey:
			begin
			charCode := chr(BAnd(theEvent.message, charCodeMask));
			if (BAnd(theEvent.modifiers, cmdKey) <> 0) then
				begin
				DoAdjustMenus;
				DoMenuChoice(MenuKey(charCode));
				end
			else
				DoKeyEvent(charCode);
			end;

		updateEvt:
			begin
			DoUpdate(theEvent);
			end;

		activateEvt:
			begin
			DoActivate(theEvent);
			end;

		osEvt:
			begin
			if (BAnd(BSR(theEvent.message, 24), $000000FF) = suspendResumeMessage) then
				if (BAnd(theEvent.message, resumeFlag) = 0) then
					gInBackground := true
				else gInBackground := false;
			end;
		end;
			{of case statement}
	end;
		{of procedure DoEvents}

{  DoIdle }

procedure DoIdle;

	var
	rawSeconds : UInt32;
	theTimeString : string;
	theEraseRect : Rect;

	begin
	if (gCurrentEditRecHdl <> nil) then
		TEIdle(gCurrentEditRecHdl);

	GetDateTime(rawSeconds);
	if (rawSeconds > gOldRawSeconds) then
		begin
		TimeString(rawSeconds, true, theTimeString, nil);
		MoveTo(285, 22);
		SetRect(theEraseRect, 285, 12, 335, 22);
		EraseRect(theEraseRect);
		DrawString(theTimeString);
		gOldRawSeconds := rawSeconds;
		end;
	end;
		{of procedure DoIdle}


{  EventLoop }

procedure EventLoop;

	var
	eventRec : EventRecord;
	gotEvent : Boolean;
	sleepTime : longint;

	begin
	gDone := false;
	gCursorRegion := NewRgn;
	sleepTime := LMGetCaretTime;

	while not (gDone) do
		begin
		gotEvent := WaitNextEvent(everyEvent, eventRec, sleepTime, gCursorRegion);

		if not (gInBackground) then
			DoAdjustCursor(FrontWindow, gCursorRegion);

		if (gotEvent) then
			DoEvents(eventRec)
		else
			DoIdle;
		end;
		
	end;
		{of procedure EventLoop}

{  start of main program }

begin

gItemRect.top := 45;
gItemRect.left := 92;
gItemRect.bottom := 62;
gItemRect.right := 242;
gQuantRect.top := 68;
gQuantRect.left := 92;
gQuantRect.bottom := 85;
gQuantRect.right := 242;
gValueRect.top := 91;
gValueRect.left := 92;
gValueRect.bottom := 108;
gValueRect.right := 242;
gDateRect.top := 114;
gDateRect.left := 92;
gDateRect.bottom := 131;
gDateRect.right := 242;
GetDateTime(gOldRawSeconds);

	{  initialize managers }

	DoInitManagers;

	{  set up menu bar and menus }

	menubarHdl := GetNewMBar(rMenubar);
	if (menubarHdl = nil) then
		ExitToShell;
	SetMenuBar(menubarHdl);
	DrawMenuBar;

	menuHdl := GetMenuHandle(mApple);
	if (menuHdl = nil) then
		ExitToShell
	else
		AppendResMenu(menuHdl, 'DRVR');

	{  open window and attach document record }

	myWindowPtr := GetNewWindow(rWindow, nil, WindowPtr(-1));
	if (myWindowPtr = nil) then
		ExitToShell;
		
	docRecHdl := DocRecHandle(NewHandle(sizeof(DocRec)));
	if (docRecHdl = nil) then
		ExitToShell;

	SetWRefCon(myWindowPtr, longint(docRecHdl));
	SetPort(myWindowPtr);

	TextSize(10);
	TextFont(geneva);

	{  set up edit records }

	DoSetUpEditRecords(myWindowPtr);

	{  enter eventLoop }

	EventLoop;

end.
	
{  }
